/*
Copyright (C) 2018-2019 de4dot@gmail.com

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

using Generator.Enums;
using Generator.Enums.Formatter;
using Generator.Enums.Formatter.Masm;

namespace Generator.Formatters.Masm {
	static class CtorInfosData {
		public static object[][] GetData(GenTypes genTypes) {
			var ctorKind = genTypes[TypeIds.MasmCtorKind];
			var code = genTypes[TypeIds.Code];
			var codeSize = genTypes[TypeIds.CodeSize];
			var instrOpInfoFlags = genTypes[TypeIds.MasmInstrOpInfoFlags];
			var register = genTypes[TypeIds.Register];
			var pseudoOpsKind = genTypes[TypeIds.PseudoOpsKind];

			return Utils.Sort(new object[][] {
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.INVALID)], FormatterConstants.InvalidMnemonicName },
				new object[] { ctorKind[nameof(CtorKind.DeclareData)], code[nameof(Code.DeclareByte)], "db" },
				new object[] { ctorKind[nameof(CtorKind.DeclareData)], code[nameof(Code.DeclareWord)], "dw" },
				new object[] { ctorKind[nameof(CtorKind.DeclareData)], code[nameof(Code.DeclareDword)], "dd" },
				new object[] { ctorKind[nameof(CtorKind.DeclareData)], code[nameof(Code.DeclareQword)], "dq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_rm8_r8)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_rm16_r16)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_rm32_r32)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_rm64_r64)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_r8_rm8)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_r16_rm16)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_r32_rm32)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_r64_rm64)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_AL_imm8)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_AX_imm16)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_EAX_imm32)], "add" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Add_RAX_imm32)], "add" },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushw_ES)], "push", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushd_ES)], "push", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popw_ES)], "pop", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popd_ES)], "pop", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_rm8_r8)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_rm16_r16)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_rm32_r32)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_rm64_r64)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_r8_rm8)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_r16_rm16)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_r32_rm32)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_r64_rm64)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_AL_imm8)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_AX_imm16)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_EAX_imm32)], "or" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Or_RAX_imm32)], "or" },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushw_CS)], "push", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushd_CS)], "push", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popw_CS)], "pop", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_rm8_r8)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_rm16_r16)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_rm32_r32)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_rm64_r64)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_r8_rm8)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_r16_rm16)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_r32_rm32)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_r64_rm64)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_AL_imm8)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_AX_imm16)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_EAX_imm32)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adc_RAX_imm32)], "adc" },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushw_SS)], "push", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushd_SS)], "push", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popw_SS)], "pop", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popd_SS)], "pop", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_rm8_r8)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_rm16_r16)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_rm32_r32)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_rm64_r64)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_r8_rm8)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_r16_rm16)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_r32_rm32)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_r64_rm64)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_AL_imm8)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_AX_imm16)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_EAX_imm32)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sbb_RAX_imm32)], "sbb" },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushw_DS)], "push", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushd_DS)], "push", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popw_DS)], "pop", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popd_DS)], "pop", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_rm8_r8)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_rm16_r16)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_rm32_r32)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_rm64_r64)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_r8_rm8)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_r16_rm16)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_r32_rm32)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_r64_rm64)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_AL_imm8)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_AX_imm16)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_EAX_imm32)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.And_RAX_imm32)], "and" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Daa)], "daa" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_rm8_r8)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_rm16_r16)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_rm32_r32)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_rm64_r64)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_r8_rm8)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_r16_rm16)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_r32_rm32)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_r64_rm64)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_AL_imm8)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_AX_imm16)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_EAX_imm32)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sub_RAX_imm32)], "sub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Das)], "das" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_rm8_r8)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_rm16_r16)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_rm32_r32)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_rm64_r64)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_r8_rm8)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_r16_rm16)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_r32_rm32)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_r64_rm64)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_AL_imm8)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_AX_imm16)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_EAX_imm32)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xor_RAX_imm32)], "xor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Aaa)], "aaa" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_rm8_r8)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_rm16_r16)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_rm32_r32)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_rm64_r64)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_r8_rm8)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_r16_rm16)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_r32_rm32)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_r64_rm64)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_AL_imm8)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_AX_imm16)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_EAX_imm32)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmp_RAX_imm32)], "cmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Aas)], "aas" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Inc_r16)], "inc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Inc_r32)], "inc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Dec_r16)], "dec" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Dec_r32)], "dec" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Push_r16)], "push" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Push_r32)], "push" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Push_r64)], "push" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pop_r16)], "pop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pop_r32)], "pop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pop_r64)], "pop" },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushaw)], "pusha", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushad)], "pusha", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popaw)], "popa", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popad)], "popa", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bound_r16_m1616)], "bound", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bound_r32_m3232)], "bound", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Arpl_rm16_r16)], "arpl", instrOpInfoFlags[nameof(InstrOpInfoFlags.None)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Arpl_r32m16_r32)], "arpl", instrOpInfoFlags[nameof(InstrOpInfoFlags.None)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsxd_r16_rm16)], "movsxd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsxd_r32_rm32)], "movsxd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsxd_r64_rm32)], "movsxd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Push_imm16)], "push", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushd_imm32)], "push", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushq_imm32)], "push", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.imul)], code[nameof(Code.Imul_r16_rm16_imm16)], "imul" },
				new object[] { ctorKind[nameof(CtorKind.imul)], code[nameof(Code.Imul_r32_rm32_imm32)], "imul" },
				new object[] { ctorKind[nameof(CtorKind.imul)], code[nameof(Code.Imul_r64_rm64_imm32)], "imul" },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushw_imm8)], "push", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushd_imm8)], "push", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushq_imm8)], "push", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.imul)], code[nameof(Code.Imul_r16_rm16_imm8)], "imul" },
				new object[] { ctorKind[nameof(CtorKind.imul)], code[nameof(Code.Imul_r32_rm32_imm8)], "imul" },
				new object[] { ctorKind[nameof(CtorKind.imul)], code[nameof(Code.Imul_r64_rm64_imm8)], "imul" },
				new object[] { ctorKind[nameof(CtorKind.YD)], code[nameof(Code.Insb_m8_DX)], "ins", 'b', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.YD)], code[nameof(Code.Insw_m16_DX)], "ins", 'w', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.YD)], code[nameof(Code.Insd_m32_DX)], "ins", 'd', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.DX)], code[nameof(Code.Outsb_DX_m8)], "outs", 'b', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.DX)], code[nameof(Code.Outsw_DX_m16)], "outs", 'w', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.DX)], code[nameof(Code.Outsd_DX_m32)], "outs", 'd', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jo_rel8_16)], "jo", 0 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jo_rel8_32)], "jo", 0 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jo_rel8_64)], "jo", 0 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jno_rel8_16)], "jno", 1 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jno_rel8_32)], "jno", 1 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jno_rel8_64)], "jno", 1 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jb_rel8_16)], "jb", "jc", "jnae", 2 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jb_rel8_32)], "jb", "jc", "jnae", 2 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jb_rel8_64)], "jb", "jc", "jnae", 2 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jae_rel8_16)], "jae", "jnb", "jnc", 3 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jae_rel8_32)], "jae", "jnb", "jnc", 3 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jae_rel8_64)], "jae", "jnb", "jnc", 3 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Je_rel8_16)], "je", "jz", 4 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Je_rel8_32)], "je", "jz", 4 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Je_rel8_64)], "je", "jz", 4 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jne_rel8_16)], "jne", "jnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jne_rel8_32)], "jne", "jnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jne_rel8_64)], "jne", "jnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jbe_rel8_16)], "jbe", "jna", 6 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jbe_rel8_32)], "jbe", "jna", 6 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jbe_rel8_64)], "jbe", "jna", 6 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Ja_rel8_16)], "ja", "jnbe", 7 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Ja_rel8_32)], "ja", "jnbe", 7 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Ja_rel8_64)], "ja", "jnbe", 7 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Js_rel8_16)], "js", 8 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Js_rel8_32)], "js", 8 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Js_rel8_64)], "js", 8 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jns_rel8_16)], "jns", 9 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jns_rel8_32)], "jns", 9 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jns_rel8_64)], "jns", 9 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jp_rel8_16)], "jp", "jpe", 10 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jp_rel8_32)], "jp", "jpe", 10 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jp_rel8_64)], "jp", "jpe", 10 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jnp_rel8_16)], "jnp", "jpo", 11 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jnp_rel8_32)], "jnp", "jpo", 11 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jnp_rel8_64)], "jnp", "jpo", 11 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jl_rel8_16)], "jl", "jnge", 12 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jl_rel8_32)], "jl", "jnge", 12 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jl_rel8_64)], "jl", "jnge", 12 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jge_rel8_16)], "jge", "jnl", 13 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jge_rel8_32)], "jge", "jnl", 13 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jge_rel8_64)], "jge", "jnl", 13 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jle_rel8_16)], "jle", "jng", 14 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jle_rel8_32)], "jle", "jng", 14 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jle_rel8_64)], "jle", "jng", 14 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jg_rel8_16)], "jg", "jnle", 15 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jg_rel8_32)], "jg", "jnle", 15 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jg_rel8_64)], "jg", "jnle", 15 },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Add_rm8_imm8)], "add", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Or_rm8_imm8)], "or", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Adc_rm8_imm8)], "adc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sbb_rm8_imm8)], "sbb", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.And_rm8_imm8)], "and", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sub_rm8_imm8)], "sub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Xor_rm8_imm8)], "xor", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmp_rm8_imm8)], "cmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Add_rm16_imm16)], "add", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Add_rm32_imm32)], "add", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Add_rm64_imm32)], "add", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Or_rm16_imm16)], "or", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Or_rm32_imm32)], "or", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Or_rm64_imm32)], "or", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Adc_rm16_imm16)], "adc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Adc_rm32_imm32)], "adc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Adc_rm64_imm32)], "adc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sbb_rm16_imm16)], "sbb", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sbb_rm32_imm32)], "sbb", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sbb_rm64_imm32)], "sbb", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.And_rm16_imm16)], "and", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.And_rm32_imm32)], "and", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.And_rm64_imm32)], "and", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sub_rm16_imm16)], "sub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sub_rm32_imm32)], "sub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sub_rm64_imm32)], "sub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Xor_rm16_imm16)], "xor", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Xor_rm32_imm32)], "xor", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Xor_rm64_imm32)], "xor", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmp_rm16_imm16)], "cmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmp_rm32_imm32)], "cmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmp_rm64_imm32)], "cmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Add_rm8_imm8_82)], "add", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Or_rm8_imm8_82)], "or", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Adc_rm8_imm8_82)], "adc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sbb_rm8_imm8_82)], "sbb", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.And_rm8_imm8_82)], "and", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sub_rm8_imm8_82)], "sub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Xor_rm8_imm8_82)], "xor", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmp_rm8_imm8_82)], "cmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Add_rm16_imm8)], "add", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Add_rm32_imm8)], "add", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Add_rm64_imm8)], "add", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Or_rm16_imm8)], "or", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Or_rm32_imm8)], "or", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Or_rm64_imm8)], "or", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Adc_rm16_imm8)], "adc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Adc_rm32_imm8)], "adc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Adc_rm64_imm8)], "adc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sbb_rm16_imm8)], "sbb", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sbb_rm32_imm8)], "sbb", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sbb_rm64_imm8)], "sbb", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.And_rm16_imm8)], "and", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.And_rm32_imm8)], "and", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.And_rm64_imm8)], "and", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sub_rm16_imm8)], "sub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sub_rm32_imm8)], "sub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sub_rm64_imm8)], "sub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Xor_rm16_imm8)], "xor", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Xor_rm32_imm8)], "xor", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Xor_rm64_imm8)], "xor", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmp_rm16_imm8)], "cmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmp_rm32_imm8)], "cmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmp_rm64_imm8)], "cmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Test_rm8_r8)], "test" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Test_rm16_r16)], "test" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Test_rm32_r32)], "test" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Test_rm64_r64)], "test" },
				new object[] { ctorKind[nameof(CtorKind.reverse2)], code[nameof(Code.Xchg_rm8_r8)], "xchg" },
				new object[] { ctorKind[nameof(CtorKind.reverse2)], code[nameof(Code.Xchg_rm16_r16)], "xchg" },
				new object[] { ctorKind[nameof(CtorKind.reverse2)], code[nameof(Code.Xchg_rm32_r32)], "xchg" },
				new object[] { ctorKind[nameof(CtorKind.reverse2)], code[nameof(Code.Xchg_rm64_r64)], "xchg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_rm8_r8)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_rm16_r16)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_rm32_r32)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_rm64_r64)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r8_rm8)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r16_rm16)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r32_rm32)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r64_rm64)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_rm16_Sreg)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r32m16_Sreg)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r64m16_Sreg)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lea_r16_m)], "lea" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lea_r32_m)], "lea" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lea_r64_m)], "lea" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_Sreg_rm16)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_Sreg_r32m16)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_Sreg_r64m16)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.pushm)], code[nameof(Code.Pop_rm16)], "pop", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.pushm)], code[nameof(Code.Pop_rm32)], "pop", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.pushm)], code[nameof(Code.Pop_rm64)], "pop", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.nop)], code[nameof(Code.Nopw)], "nop", 16, register[nameof(Register.AX)] },
				new object[] { ctorKind[nameof(CtorKind.nop)], code[nameof(Code.Nopd)], "nop", 32 | 64, register[nameof(Register.EAX)] },
				new object[] { ctorKind[nameof(CtorKind.nop)], code[nameof(Code.Nopq)], "nop", 0, register[nameof(Register.RAX)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xchg_r16_AX)], "xchg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xchg_r32_EAX)], "xchg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xchg_r64_RAX)], "xchg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pause)], "pause" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cbw)], "cbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cwde)], "cwde" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cdqe)], "cdqe" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cwd)], "cwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cdq)], "cdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cqo)], "cqo" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Call_ptr1616)], "call" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Call_ptr1632)], "call" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wait)], "wait" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pushfw)], "pushf" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pushfd)], "pushfd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pushfq)], "pushfq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Popfw)], "popf" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Popfd)], "popfd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Popfq)], "popfq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sahf)], "sahf" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lahf)], "lahf" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_AL_moffs8)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_AX_moffs16)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_EAX_moffs32)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_RAX_moffs64)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_moffs8_AL)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_moffs16_AX)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_moffs32_EAX)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_moffs64_RAX)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.YX)], code[nameof(Code.Movsb_m8_m8)], "movs", 'b', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.YX)], code[nameof(Code.Movsw_m16_m16)], "movs", 'w', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.YX)], code[nameof(Code.Movsd_m32_m32)], "movs", 'd', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.YX)], code[nameof(Code.Movsq_m64_m64)], "movs", 'q', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.XY)], code[nameof(Code.Cmpsb_m8_m8)], "cmps", 'b', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.XY)], code[nameof(Code.Cmpsw_m16_m16)], "cmps", 'w', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.XY)], code[nameof(Code.Cmpsd_m32_m32)], "cmps", 'd', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.XY)], code[nameof(Code.Cmpsq_m64_m64)], "cmps", 'q', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Test_AL_imm8)], "test" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Test_AX_imm16)], "test" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Test_EAX_imm32)], "test" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Test_RAX_imm32)], "test" },
				new object[] { ctorKind[nameof(CtorKind.YA)], code[nameof(Code.Stosb_m8_AL)], "stos", 'b', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.YA)], code[nameof(Code.Stosw_m16_AX)], "stos", 'w', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.YA)], code[nameof(Code.Stosd_m32_EAX)], "stos", 'd', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.YA)], code[nameof(Code.Stosq_m64_RAX)], "stos", 'q', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.AX)], code[nameof(Code.Lodsb_AL_m8)], "lods", 'b', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.AX)], code[nameof(Code.Lodsw_AX_m16)], "lods", 'w', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.AX)], code[nameof(Code.Lodsd_EAX_m32)], "lods", 'd', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.AX)], code[nameof(Code.Lodsq_RAX_m64)], "lods", 'q', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.AY)], code[nameof(Code.Scasb_AL_m8)], "scas", 'b', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.AY)], code[nameof(Code.Scasw_AX_m16)], "scas", 'w', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.AY)], code[nameof(Code.Scasd_EAX_m32)], "scas", 'd', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.AY)], code[nameof(Code.Scasq_RAX_m64)], "scas", 'q', new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r8_imm8)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r16_imm16)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r32_imm32)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r64_imm64)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm8_imm8)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm8_imm8)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm8_imm8)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm8_imm8)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm8_imm8)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm8_imm8)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm8_imm8)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm8_imm8)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm16_imm8)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm32_imm8)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm64_imm8)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm16_imm8)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm32_imm8)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm64_imm8)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm16_imm8)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm32_imm8)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm64_imm8)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm16_imm8)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm32_imm8)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm64_imm8)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm16_imm8)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm32_imm8)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm64_imm8)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm16_imm8)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm32_imm8)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm64_imm8)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm16_imm8)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm32_imm8)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm64_imm8)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm16_imm8)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm32_imm8)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm64_imm8)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.OpSize2_bnd)], code[nameof(Code.Retnw_imm16)], "ret", "ret", "retw", "retw" },
				new object[] { ctorKind[nameof(CtorKind.OpSize2_bnd)], code[nameof(Code.Retnd_imm16)], "ret", "retnd", "ret", "retnd" },
				new object[] { ctorKind[nameof(CtorKind.bnd_1)], code[nameof(Code.Retnq_imm16)], "ret" },
				new object[] { ctorKind[nameof(CtorKind.OpSize2_bnd)], code[nameof(Code.Retnw)], "ret", "ret", "retw", "retw" },
				new object[] { ctorKind[nameof(CtorKind.OpSize2_bnd)], code[nameof(Code.Retnd)], "ret", "retnd", "ret", "retnd" },
				new object[] { ctorKind[nameof(CtorKind.bnd_1)], code[nameof(Code.Retnq)], "ret" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Les_r16_m1616)], "les" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Les_r32_m1632)], "les" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lds_r16_m1616)], "lds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lds_r32_m1632)], "lds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mov_rm8_imm8)], "mov", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xabort_imm8)], "xabort" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mov_rm16_imm16)], "mov", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mov_rm32_imm32)], "mov", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mov_rm64_imm32)], "mov", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xbegin_rel16)], "xbegin" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xbegin_rel32)], "xbegin" },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Enterw_imm16_imm8)], "enter", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Enterd_imm16_imm8)], "enter", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Enterq_imm16_imm8)], "enter", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Leavew)], "leave", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Leaved)], "leave", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Leaveq)], "leave", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize2)], code[nameof(Code.Retfw_imm16)], "retf", "retf", "retfw", "retfw" },
				new object[] { ctorKind[nameof(CtorKind.OpSize2)], code[nameof(Code.Retfd_imm16)], "retf", "retfd", "retf", "retf" },
				new object[] { ctorKind[nameof(CtorKind.OpSize2)], code[nameof(Code.Retfq_imm16)], "retfq", "retf", "retf", "retfq" },
				new object[] { ctorKind[nameof(CtorKind.OpSize2)], code[nameof(Code.Retfw)], "retf", "retf", "retfw", "retfw" },
				new object[] { ctorKind[nameof(CtorKind.OpSize2)], code[nameof(Code.Retfd)], "retf", "retfd", "retf", "retf" },
				new object[] { ctorKind[nameof(CtorKind.OpSize2)], code[nameof(Code.Retfq)], "retfq", "retf", "retf", "retfq" },
				new object[] { ctorKind[nameof(CtorKind.Ib)], code[nameof(Code.Int3)], "int" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Int_imm8)], "int" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Into)], "into" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Iretw)], "iret" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Iretd)], "iretd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Iretq)], "iretq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm8_1)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm8_1)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm8_1)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm8_1)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm8_1)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm8_1)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm8_1)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm8_1)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm16_1)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm32_1)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm64_1)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm16_1)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm32_1)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm64_1)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm16_1)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm32_1)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm64_1)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm16_1)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm32_1)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm64_1)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm16_1)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm32_1)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm64_1)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm16_1)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm32_1)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm64_1)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm16_1)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm32_1)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm64_1)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm16_1)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm32_1)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm64_1)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm8_CL)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm8_CL)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm8_CL)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm8_CL)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm8_CL)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm8_CL)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm8_CL)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm8_CL)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm16_CL)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm32_CL)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rol_rm64_CL)], "rol", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm16_CL)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm32_CL)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ror_rm64_CL)], "ror", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm16_CL)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm32_CL)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcl_rm64_CL)], "rcl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm16_CL)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm32_CL)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcr_rm64_CL)], "rcr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm16_CL)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm32_CL)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shl_rm64_CL)], "shl", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm16_CL)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm32_CL)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Shr_rm64_CL)], "shr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm16_CL)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm32_CL)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sal_rm64_CL)], "sal", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm16_CL)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm32_CL)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sar_rm64_CL)], "sar", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.AamAad)], code[nameof(Code.Aam_imm8)], "aam" },
				new object[] { ctorKind[nameof(CtorKind.AamAad)], code[nameof(Code.Aad_imm8)], "aad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Salc)], "salc" },
				new object[] { ctorKind[nameof(CtorKind.XLAT)], code[nameof(Code.Xlat_m8)], "xlat", 'b' },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fadd_m32fp)], "fadd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fmul_m32fp)], "fmul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fcom_m32fp)], "fcom", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fcomp_m32fp)], "fcomp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fsub_m32fp)], "fsub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fsubr_m32fp)], "fsubr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fdiv_m32fp)], "fdiv", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fdivr_m32fp)], "fdivr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fadd_st0_sti)], "fadd" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fmul_st0_sti)], "fmul" },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fcom_st0_sti)], "fcom", true },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fcomp_st0_sti)], "fcomp", true },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fsub_st0_sti)], "fsub" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fsubr_st0_sti)], "fsubr" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fdiv_st0_sti)], "fdiv" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fdivr_st0_sti)], "fdivr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fld_m32fp)], "fld", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fst_m32fp)], "fst", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fstp_m32fp)], "fstp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fldenv_m14byte)], "fldenv", 16 },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fldenv_m28byte)], "fldenv", 32 | 64 },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fldcw_m2byte)], "fldcw" },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fnstenv_m14byte)], "fnstenv", 16 },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fstenv_m14byte)], "fstenv", 16 },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fnstenv_m28byte)], "fnstenv", 32 | 64 },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fstenv_m28byte)], "fstenv", 32 | 64 },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fnstcw_m2byte)], "fnstcw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fstcw_m2byte)], "fstcw" },
				new object[] { ctorKind[nameof(CtorKind.STIG1_1)], code[nameof(Code.Fld_st0_sti)], "fld" },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fxch_st0_sti)], "fxch", true },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fnop)], "fnop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fstpnce_sti)], "fstp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fchs)], "fchs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fabs)], "fabs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ftst)], "ftst" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fxam)], "fxam" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fld1)], "fld1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fldl2t)], "fldl2t" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fldl2e)], "fldl2e" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fldpi)], "fldpi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fldlg2)], "fldlg2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fldln2)], "fldln2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fldz)], "fldz" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.F2xm1)], "f2xm1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fyl2x)], "fyl2x" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fptan)], "fptan" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fpatan)], "fpatan" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fxtract)], "fxtract" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fprem1)], "fprem1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fdecstp)], "fdecstp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fincstp)], "fincstp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fprem)], "fprem" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fyl2xp1)], "fyl2xp1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fsqrt)], "fsqrt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fsincos)], "fsincos" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Frndint)], "frndint" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fscale)], "fscale" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fsin)], "fsin" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fcos)], "fcos" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fiadd_m32int)], "fiadd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fimul_m32int)], "fimul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ficom_m32int)], "ficom", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ficomp_m32int)], "ficomp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fisub_m32int)], "fisub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fisubr_m32int)], "fisubr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fidiv_m32int)], "fidiv", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fidivr_m32int)], "fidivr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcmovb_st0_sti)], "fcmovb" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcmove_st0_sti)], "fcmove" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcmovbe_st0_sti)], "fcmovbe" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcmovu_st0_sti)], "fcmovu" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fucompp)], "fucompp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fild_m32int)], "fild", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fisttp_m32int)], "fisttp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fist_m32int)], "fist", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fistp_m32int)], "fistp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fld_m80fp)], "fld", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fstp_m80fp)], "fstp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcmovnb_st0_sti)], "fcmovnb" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcmovne_st0_sti)], "fcmovne" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcmovnbe_st0_sti)], "fcmovnbe" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcmovnu_st0_sti)], "fcmovnu" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fneni)], "fneni" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Feni)], "feni" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fndisi)], "fndisi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fdisi)], "fdisi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fnclex)], "fnclex" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fclex)], "fclex" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fninit)], "fninit" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Finit)], "finit" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fnsetpm)], "fnsetpm" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fsetpm)], "fsetpm" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Frstpm)], "frstpm" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fucomi_st0_sti)], "fucomi" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcomi_st0_sti)], "fcomi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fadd_m64fp)], "fadd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fmul_m64fp)], "fmul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fcom_m64fp)], "fcom", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fcomp_m64fp)], "fcomp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fsub_m64fp)], "fsub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fsubr_m64fp)], "fsubr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fdiv_m64fp)], "fdiv", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fdivr_m64fp)], "fdivr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.STi_ST)], code[nameof(Code.Fadd_sti_st0)], "fadd" },
				new object[] { ctorKind[nameof(CtorKind.STi_ST)], code[nameof(Code.Fmul_sti_st0)], "fmul" },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fcom_st0_sti_DCD0)], "fcom", true },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fcomp_st0_sti_DCD8)], "fcomp", true },
				new object[] { ctorKind[nameof(CtorKind.STi_ST)], code[nameof(Code.Fsubr_sti_st0)], "fsubr" },
				new object[] { ctorKind[nameof(CtorKind.STi_ST)], code[nameof(Code.Fsub_sti_st0)], "fsub" },
				new object[] { ctorKind[nameof(CtorKind.STi_ST)], code[nameof(Code.Fdivr_sti_st0)], "fdivr" },
				new object[] { ctorKind[nameof(CtorKind.STi_ST)], code[nameof(Code.Fdiv_sti_st0)], "fdiv" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fld_m64fp)], "fld", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fisttp_m64int)], "fisttp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fst_m64fp)], "fst", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fstp_m64fp)], "fstp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Frstor_m94byte)], "frstor", 16 },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Frstor_m108byte)], "frstor", 32 | 64 },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fnsave_m94byte)], "fnsave", 16 },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fsave_m94byte)], "fsave", 16 },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fnsave_m108byte)], "fnsave", 32 | 64 },
				new object[] { ctorKind[nameof(CtorKind.memsize)], code[nameof(Code.Fsave_m108byte)], "fsave", 32 | 64 },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fnstsw_m2byte)], "fnstsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fstsw_m2byte)], "fstsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ffree_sti)], "ffree" },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fxch_st0_sti_DDC8)], "fxch", true },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fst_sti)], "fst" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fstp_sti)], "fstp" },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fucom_st0_sti)], "fucom", true },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fucomp_st0_sti)], "fucomp", true },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fiadd_m16int)], "fiadd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fimul_m16int)], "fimul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ficom_m16int)], "ficom", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ficomp_m16int)], "ficomp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fisub_m16int)], "fisub", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fisubr_m16int)], "fisubr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fidiv_m16int)], "fidiv", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fidivr_m16int)], "fidivr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.STi_ST2)], code[nameof(Code.Faddp_sti_st0)], "faddp" },
				new object[] { ctorKind[nameof(CtorKind.STi_ST2)], code[nameof(Code.Fmulp_sti_st0)], "fmulp" },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fcomp_st0_sti_DED0)], "fcomp", true },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fcompp)], "fcompp" },
				new object[] { ctorKind[nameof(CtorKind.STi_ST2)], code[nameof(Code.Fsubrp_sti_st0)], "fsubrp" },
				new object[] { ctorKind[nameof(CtorKind.STi_ST2)], code[nameof(Code.Fsubp_sti_st0)], "fsubp" },
				new object[] { ctorKind[nameof(CtorKind.STi_ST2)], code[nameof(Code.Fdivrp_sti_st0)], "fdivrp" },
				new object[] { ctorKind[nameof(CtorKind.STi_ST2)], code[nameof(Code.Fdivp_sti_st0)], "fdivp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fild_m16int)], "fild", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fisttp_m16int)], "fisttp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fist_m16int)], "fist", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fistp_m16int)], "fistp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fbld_m80bcd)], "fbld", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fild_m64int)], "fild", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fbstp_m80bcd)], "fbstp", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Fistp_m64int)], "fistp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ffreep_sti)], "ffreep" },
				new object[] { ctorKind[nameof(CtorKind.STIG1_2)], code[nameof(Code.Fxch_st0_sti_DFC8)], "fxch", true },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fstp_sti_DFD0)], "fstp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fstp_sti_DFD8)], "fstp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fnstsw_AX)], "fnstsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fstsw_AX)], "fstsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fstdw_AX)], "fstdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fstsg_AX)], "fstsg" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fucomip_st0_sti)], "fucomip" },
				new object[] { ctorKind[nameof(CtorKind.ST_STi)], code[nameof(Code.Fcomip_st0_sti)], "fcomip" },
				new object[] { ctorKind[nameof(CtorKind.Loopcc2)], code[nameof(Code.Loopne_rel8_16_CX)], "loopne", "loopnz", 'w', 5, codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.Loopcc2)], code[nameof(Code.Loopne_rel8_32_CX)], "loopne", "loopnz", 'w', 5, codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.Loopcc2)], code[nameof(Code.Loopne_rel8_16_ECX)], "loopne", "loopnz", 'd', 5, codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.Loopcc2)], code[nameof(Code.Loopne_rel8_32_ECX)], "loopne", "loopnz", 'd', 5, codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.Loopcc1)], code[nameof(Code.Loopne_rel8_64_ECX)], "loopned", "loopnzd", 5 },
				new object[] { ctorKind[nameof(CtorKind.Loopcc1)], code[nameof(Code.Loopne_rel8_16_RCX)], "loopne", "loopnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.Loopcc1)], code[nameof(Code.Loopne_rel8_64_RCX)], "loopne", "loopnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.Loopcc2)], code[nameof(Code.Loope_rel8_16_CX)], "loope", "loopz", 'w', 4, codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.Loopcc2)], code[nameof(Code.Loope_rel8_32_CX)], "loope", "loopz", 'w', 4, codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.Loopcc2)], code[nameof(Code.Loope_rel8_16_ECX)], "loope", "loopz", 'd', 4, codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.Loopcc2)], code[nameof(Code.Loope_rel8_32_ECX)], "loope", "loopz", 'd', 4, codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.Loopcc1)], code[nameof(Code.Loope_rel8_64_ECX)], "looped", "loopzd", 4 },
				new object[] { ctorKind[nameof(CtorKind.Loopcc1)], code[nameof(Code.Loope_rel8_16_RCX)], "loope", "loopz", 4 },
				new object[] { ctorKind[nameof(CtorKind.Loopcc1)], code[nameof(Code.Loope_rel8_64_RCX)], "loope", "loopz", 4 },
				new object[] { ctorKind[nameof(CtorKind.OpSize_2)], code[nameof(Code.Loop_rel8_16_CX)], "loop", 'w', codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_2)], code[nameof(Code.Loop_rel8_32_CX)], "loop", 'w', codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_2)], code[nameof(Code.Loop_rel8_16_ECX)], "loop", 'd', codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_2)], code[nameof(Code.Loop_rel8_32_ECX)], "loop", 'd', codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Loop_rel8_64_ECX)], "loopd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Loop_rel8_16_RCX)], "loop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Loop_rel8_64_RCX)], "loop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jcxz_rel8_16)], "jcxz" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jcxz_rel8_32)], "jcxz" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jecxz_rel8_16)], "jecxz" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jecxz_rel8_32)], "jecxz" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jecxz_rel8_64)], "jecxz" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jrcxz_rel8_16)], "jrcxz" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jrcxz_rel8_64)], "jrcxz" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.In_AL_imm8)], "in" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.In_AX_imm8)], "in" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.In_EAX_imm8)], "in" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Out_imm8_AL)], "out" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Out_imm8_AX)], "out" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Out_imm8_EAX)], "out" },
				new object[] { ctorKind[nameof(CtorKind.bnd_1)], code[nameof(Code.Call_rel16)], "call" },
				new object[] { ctorKind[nameof(CtorKind.bnd_1)], code[nameof(Code.Call_rel32_32)], "call" },
				new object[] { ctorKind[nameof(CtorKind.bnd_1)], code[nameof(Code.Call_rel32_64)], "call" },
				new object[] { ctorKind[nameof(CtorKind.bnd_1)], code[nameof(Code.Jmp_rel16)], "jmp" },
				new object[] { ctorKind[nameof(CtorKind.bnd_1)], code[nameof(Code.Jmp_rel32_32)], "jmp" },
				new object[] { ctorKind[nameof(CtorKind.bnd_1)], code[nameof(Code.Jmp_rel32_64)], "jmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jmp_ptr1616)], "jmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jmp_ptr1632)], "jmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jmp_rel8_16)], "jmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jmp_rel8_32)], "jmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jmp_rel8_64)], "jmp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.In_AL_DX)], "in" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.In_AX_DX)], "in" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.In_EAX_DX)], "in" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Out_DX_AL)], "out" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Out_DX_AX)], "out" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Out_DX_EAX)], "out" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Int1)], "int1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Hlt)], "hlt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmc)], "cmc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Test_rm8_imm8)], "test", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Test_rm8_imm8_F6r1)], "test", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Not_rm8)], "not", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Neg_rm8)], "neg", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mul_rm8)], "mul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Imul_rm8)], "imul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Div_rm8)], "div", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Idiv_rm8)], "idiv", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Test_rm16_imm16)], "test", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Test_rm32_imm32)], "test", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Test_rm64_imm32)], "test", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Test_rm16_imm16_F7r1)], "test", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Test_rm32_imm32_F7r1)], "test", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Test_rm64_imm32_F7r1)], "test", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Not_rm16)], "not", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Not_rm32)], "not", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Not_rm64)], "not", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Neg_rm16)], "neg", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Neg_rm32)], "neg", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Neg_rm64)], "neg", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mul_rm16)], "mul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mul_rm32)], "mul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mul_rm64)], "mul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Imul_rm16)], "imul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Imul_rm32)], "imul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Imul_rm64)], "imul", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Div_rm16)], "div", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Div_rm32)], "div", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Div_rm64)], "div", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Idiv_rm16)], "idiv", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Idiv_rm32)], "idiv", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Idiv_rm64)], "idiv", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clc)], "clc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Stc)], "stc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cli)], "cli" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sti)], "sti" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cld)], "cld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Std)], "std" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Inc_rm8)], "inc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Dec_rm8)], "dec", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Inc_rm16)], "inc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Inc_rm32)], "inc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Inc_rm64)], "inc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Dec_rm16)], "dec", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Dec_rm32)], "dec", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Dec_rm64)], "dec", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.bnd_2)], code[nameof(Code.Call_rm16)], "call", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.bnd_2)], code[nameof(Code.Call_rm32)], "call", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.bnd_2)], code[nameof(Code.Call_rm64)], "call", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Call_m1616)], "call", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Call_m1632)], "call", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Call_m1664)], "call", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.bnd_2)], code[nameof(Code.Jmp_rm16)], "jmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.bnd_2)], code[nameof(Code.Jmp_rm32)], "jmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.bnd_2)], code[nameof(Code.Jmp_rm64)], "jmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Jmp_m1616)], "jmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Jmp_m1632)], "jmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Jmp_m1664)], "jmp", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.pushm)], code[nameof(Code.Push_rm16)], "push", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.pushm)], code[nameof(Code.Push_rm32)], "push", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.pushm)], code[nameof(Code.Push_rm64)], "push", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sldt_rm16)], "sldt", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sldt_r32m16)], "sldt", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sldt_r64m16)], "sldt", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Str_rm16)], "str", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Str_r32m16)], "str", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Str_r64m16)], "str", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Lldt_rm16)], "lldt", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Lldt_r32m16)], "lldt", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Lldt_r64m16)], "lldt", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ltr_rm16)], "ltr", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Ltr_r32m16)], "ltr", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Ltr_r64m16)], "ltr", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Verr_rm16)], "verr", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Verr_r32m16)], "verr", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Verr_r64m16)], "verr", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Verw_rm16)], "verw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Verw_r32m16)], "verw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Verw_r64m16)], "verw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Jmpe_rm16)], "jmpe", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Jmpe_rm32)], "jmpe", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Sgdt_m1632_16)], "sgdt", 'w', codeSize[nameof(CodeSize.Code16)], true },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Sgdt_m1632)], "sgdt", 'd', codeSize[nameof(CodeSize.Code32)], true },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Sgdt_m1664)], "sgdt", 'q', codeSize[nameof(CodeSize.Code64)], true },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Sidt_m1632_16)], "sidt", 'w', codeSize[nameof(CodeSize.Code16)], true },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Sidt_m1632)], "sidt", 'd', codeSize[nameof(CodeSize.Code32)], true },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Sidt_m1664)], "sidt", 'q', codeSize[nameof(CodeSize.Code64)], true },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Lgdt_m1632_16)], "lgdt", 'w', codeSize[nameof(CodeSize.Code16)], false },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Lgdt_m1632)], "lgdt", 'd', codeSize[nameof(CodeSize.Code32)], false },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Lgdt_m1664)], "lgdt", 'q', codeSize[nameof(CodeSize.Code64)], false },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Lidt_m1632_16)], "lidt", 'w', codeSize[nameof(CodeSize.Code16)], false },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Lidt_m1632)], "lidt", 'd', codeSize[nameof(CodeSize.Code32)], false },
				new object[] { ctorKind[nameof(CtorKind.fword)], code[nameof(Code.Lidt_m1664)], "lidt", 'q', codeSize[nameof(CodeSize.Code64)], false },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Smsw_rm16)], "smsw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Smsw_r32m16)], "smsw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Smsw_r64m16)], "smsw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rstorssp_m64)], "rstorssp", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Lmsw_rm16)], "lmsw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Lmsw_r32m16)], "lmsw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg16)], code[nameof(Code.Lmsw_r64m16)], "lmsw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Invlpg_m)], "invlpg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Enclv)], "enclv" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Vmcall)], "vmcall" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Vmlaunch)], "vmlaunch" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Vmresume)], "vmresume" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Vmxoff)], "vmxoff" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pconfig)], "pconfig" },
				new object[] { ctorKind[nameof(CtorKind.monitor)], code[nameof(Code.Monitorw)], "monitor", register[nameof(Register.AX)], register[nameof(Register.ECX)], register[nameof(Register.EDX)] },
				new object[] { ctorKind[nameof(CtorKind.monitor)], code[nameof(Code.Monitord)], "monitor", register[nameof(Register.EAX)], register[nameof(Register.ECX)], register[nameof(Register.EDX)] },
				new object[] { ctorKind[nameof(CtorKind.monitor)], code[nameof(Code.Monitorq)], "monitor", register[nameof(Register.RAX)], register[nameof(Register.RCX)], register[nameof(Register.RDX)] },
				new object[] { ctorKind[nameof(CtorKind.mwait)], code[nameof(Code.Mwait)], "mwait" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clac)], "clac" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Stac)], "stac" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Encls)], "encls" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xgetbv)], "xgetbv" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsetbv)], "xsetbv" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Vmfunc)], "vmfunc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xend)], "xend" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xtest)], "xtest" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Enclu)], "enclu" },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Vmrunw)], "vmrun", register[nameof(Register.AX)] },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Vmrund)], "vmrun", register[nameof(Register.EAX)] },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Vmrunq)], "vmrun", register[nameof(Register.RAX)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Vmmcall)], "vmmcall" },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Vmloadw)], "vmload", register[nameof(Register.AX)] },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Vmloadd)], "vmload", register[nameof(Register.EAX)] },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Vmloadq)], "vmload", register[nameof(Register.RAX)] },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Vmsavew)], "vmsave", register[nameof(Register.AX)] },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Vmsaved)], "vmsave", register[nameof(Register.EAX)] },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Vmsaveq)], "vmsave", register[nameof(Register.RAX)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Stgi)], "stgi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clgi)], "clgi" },
				new object[] { ctorKind[nameof(CtorKind.reg)], code[nameof(Code.Skinit)], "skinit", register[nameof(Register.EAX)] },
				new object[] { ctorKind[nameof(CtorKind.invlpga)], code[nameof(Code.Invlpgaw)], "invlpga", 16 },
				new object[] { ctorKind[nameof(CtorKind.invlpga)], code[nameof(Code.Invlpgad)], "invlpga", 32 },
				new object[] { ctorKind[nameof(CtorKind.invlpga)], code[nameof(Code.Invlpgaq)], "invlpga", 64 },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Setssbsy)], "setssbsy" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Saveprevssp)], "saveprevssp" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdpkru)], "rdpkru" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrpkru)], "wrpkru" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Swapgs)], "swapgs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdtscp)], "rdtscp" },
				new object[] { ctorKind[nameof(CtorKind.monitor)], code[nameof(Code.Monitorxw)], "monitorx", register[nameof(Register.AX)], register[nameof(Register.ECX)], register[nameof(Register.EDX)] },
				new object[] { ctorKind[nameof(CtorKind.monitor)], code[nameof(Code.Monitorxd)], "monitorx", register[nameof(Register.EAX)], register[nameof(Register.ECX)], register[nameof(Register.EDX)] },
				new object[] { ctorKind[nameof(CtorKind.monitor)], code[nameof(Code.Monitorxq)], "monitorx", register[nameof(Register.RAX)], register[nameof(Register.RCX)], register[nameof(Register.RDX)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mcommit)], "mcommit" },
				new object[] { ctorKind[nameof(CtorKind.mwaitx)], code[nameof(Code.Mwaitx)], "mwaitx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clzerow)], "clzero" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clzerod)], "clzero" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clzeroq)], "clzero" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdpru)], "rdpru" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Lar_r16_rm16)], "lar", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Lar_r32_r32m16)], "lar", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Lar_r64_r64m16)], "lar", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Lsl_r16_rm16)], "lsl", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Lsl_r32_r32m16)], "lsl", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Lsl_r64_r64m16)], "lsl", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Loadallreset286)], "loadallreset286" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Loadall286)], "loadall286" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Syscall)], "syscall" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clts)], "clts" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Loadall386)], "loadall386" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sysretd)], "sysret" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sysretq)], "sysretq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Invd)], "invd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wbinvd)], "wbinvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wbnoinvd)], "wbnoinvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cl1invmb)], "cl1invmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ud2)], "ud2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm16_r16_0F0D)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm32_r32_0F0D)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm64_r64_0F0D)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Prefetch_m8)], "prefetch" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Prefetchw_m8)], "prefetchw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Prefetchwt1_m8)], "prefetchwt1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Femms)], "femms" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umov_rm8_r8)], "umov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umov_rm16_r16)], "umov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umov_rm32_r32)], "umov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umov_r8_rm8)], "umov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umov_r16_rm16)], "umov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umov_r32_rm32)], "umov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movups_xmm_xmmm128)], "movups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovups_xmm_xmmm128)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovups_ymm_ymmm256)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovups_xmm_k1z_xmmm128)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovups_ymm_k1z_ymmm256)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovups_zmm_k1z_zmmm512)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movupd_xmm_xmmm128)], "movupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovupd_xmm_xmmm128)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovupd_ymm_ymmm256)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovupd_xmm_k1z_xmmm128)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovupd_ymm_k1z_ymmm256)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovupd_zmm_k1z_zmmm512)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movss_xmm_xmmm32)], "movss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm)], "vmovss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovss_xmm_m32)], "vmovss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm)], "vmovss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovss_xmm_k1z_m32)], "vmovss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsd_xmm_xmmm64)], "movsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm)], "vmovsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovsd_xmm_m64)], "vmovsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm)], "vmovsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovsd_xmm_k1z_m64)], "vmovsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movups_xmmm128_xmm)], "movups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovups_xmmm128_xmm)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovups_ymmm256_ymm)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovups_xmmm128_k1z_xmm)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovups_ymmm256_k1z_ymm)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovups_zmmm512_k1z_zmm)], "vmovups" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movupd_xmmm128_xmm)], "movupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovupd_xmmm128_xmm)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovupd_ymmm256_ymm)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovupd_xmmm128_k1z_xmm)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovupd_ymmm256_k1z_ymm)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovupd_zmmm512_k1z_zmm)], "vmovupd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movss_xmmm32_xmm)], "movss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm_0F11)], "vmovss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovss_m32_xmm)], "vmovss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11)], "vmovss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovss_m32_k1_xmm)], "vmovss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsd_xmmm64_xmm)], "movsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm_0F11)], "vmovsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovsd_m64_xmm)], "vmovsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11)], "vmovsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovsd_m64_k1_xmm)], "vmovsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movhlps_xmm_xmm)], "movhlps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movlps_xmm_m64)], "movlps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovhlps_xmm_xmm_xmm)], "vmovhlps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovlps_xmm_xmm_m64)], "vmovlps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovhlps_xmm_xmm_xmm)], "vmovhlps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovlps_xmm_xmm_m64)], "vmovlps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movlpd_xmm_m64)], "movlpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovlpd_xmm_xmm_m64)], "vmovlpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovlpd_xmm_xmm_m64)], "vmovlpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movsldup_xmm_xmmm128)], "movsldup" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovsldup_xmm_xmmm128)], "vmovsldup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovsldup_ymm_ymmm256)], "vmovsldup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovsldup_xmm_k1z_xmmm128)], "vmovsldup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovsldup_ymm_k1z_ymmm256)], "vmovsldup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovsldup_zmm_k1z_zmmm512)], "vmovsldup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movddup_xmm_xmmm64)], "movddup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovddup_xmm_xmmm64)], "vmovddup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovddup_ymm_ymmm256)], "vmovddup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovddup_xmm_k1z_xmmm64)], "vmovddup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovddup_ymm_k1z_ymmm256)], "vmovddup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovddup_zmm_k1z_zmmm512)], "vmovddup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movlps_m64_xmm)], "movlps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovlps_m64_xmm)], "vmovlps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovlps_m64_xmm)], "vmovlps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movlpd_m64_xmm)], "movlpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovlpd_m64_xmm)], "vmovlpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovlpd_m64_xmm)], "vmovlpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Unpcklps_xmm_xmmm128)], "unpcklps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vunpcklps_xmm_xmm_xmmm128)], "vunpcklps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vunpcklps_ymm_ymm_ymmm256)], "vunpcklps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32)], "vunpcklps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32)], "vunpcklps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32)], "vunpcklps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Unpcklpd_xmm_xmmm128)], "unpcklpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vunpcklpd_xmm_xmm_xmmm128)], "vunpcklpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vunpcklpd_ymm_ymm_ymmm256)], "vunpcklpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64)], "vunpcklpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64)], "vunpcklpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64)], "vunpcklpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Unpckhps_xmm_xmmm128)], "unpckhps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vunpckhps_xmm_xmm_xmmm128)], "vunpckhps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vunpckhps_ymm_ymm_ymmm256)], "vunpckhps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32)], "vunpckhps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32)], "vunpckhps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32)], "vunpckhps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Unpckhpd_xmm_xmmm128)], "unpckhpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vunpckhpd_xmm_xmm_xmmm128)], "vunpckhpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vunpckhpd_ymm_ymm_ymmm256)], "vunpckhpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64)], "vunpckhpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64)], "vunpckhpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64)], "vunpckhpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movlhps_xmm_xmm)], "movlhps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovlhps_xmm_xmm_xmm)], "vmovlhps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovlhps_xmm_xmm_xmm)], "vmovlhps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movhps_xmm_m64)], "movhps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovhps_xmm_xmm_m64)], "vmovhps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovhps_xmm_xmm_m64)], "vmovhps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movhpd_xmm_m64)], "movhpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovhpd_xmm_xmm_m64)], "vmovhpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovhpd_xmm_xmm_m64)], "vmovhpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movshdup_xmm_xmmm128)], "movshdup" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovshdup_xmm_xmmm128)], "vmovshdup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovshdup_ymm_ymmm256)], "vmovshdup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovshdup_xmm_k1z_xmmm128)], "vmovshdup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovshdup_ymm_k1z_ymmm256)], "vmovshdup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovshdup_zmm_k1z_zmmm512)], "vmovshdup", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movhps_m64_xmm)], "movhps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovhps_m64_xmm)], "vmovhps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovhps_m64_xmm)], "vmovhps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movhpd_m64_xmm)], "movhpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovhpd_m64_xmm)], "vmovhpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovhpd_m64_xmm)], "vmovhpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm16_r16_0F18)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm32_r32_0F18)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm64_r64_0F18)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm16_r16_0F19)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm32_r32_0F19)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm64_r64_0F19)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm16_r16_0F1A)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm32_r32_0F1A)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm64_r64_0F1A)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm16_r16_0F1B)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm32_r32_0F1B)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm64_r64_0F1B)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm16_r16_0F1C)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm32_r32_0F1C)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm64_r64_0F1C)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm16_r16_0F1D)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm32_r32_0F1D)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm64_r64_0F1D)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm16_r16_0F1E)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm32_r32_0F1E)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm64_r64_0F1E)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm16_r16_0F1F)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm32_r32_0F1F)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.ReservedNop_rm64_r64_0F1F)], "nop" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Prefetchnta_m8)], "prefetchnta" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Prefetcht0_m8)], "prefetcht0" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Prefetcht1_m8)], "prefetcht1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Prefetcht2_m8)], "prefetcht2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bndldx_bnd_mib)], "bndldx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.MemSize_DwordOrQword)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bndmov_bnd_bndm64)], "bndmov", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bndmov_bnd_bndm128)], "bndmov", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bndcl_bnd_rm32)], "bndcl" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bndcl_bnd_rm64)], "bndcl" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bndcu_bnd_rm32)], "bndcu" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bndcu_bnd_rm64)], "bndcu" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bndstx_mib_bnd)], "bndstx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.MemSize_DwordOrQword)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bndmov_bndm64_bnd)], "bndmov", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bndmov_bndm128_bnd)], "bndmov", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bndmk_bnd_m32)], "bndmk", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bndmk_bnd_m64)], "bndmk", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bndcn_bnd_rm32)], "bndcn" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bndcn_bnd_rm64)], "bndcn" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cldemote_m8)], "cldemote", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdsspd_r32)], "rdsspd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdsspq_r64)], "rdsspq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Endbr64)], "endbr64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Endbr32)], "endbr32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Nop_rm16)], "nop", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Nop_rm32)], "nop", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Nop_rm64)], "nop", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r32_cr)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r64_cr)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r32_dr)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r64_dr)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_cr_r32)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_cr_r64)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_dr_r32)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_dr_r64)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_r32_tr)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mov_tr_r32)], "mov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movaps_xmm_xmmm128)], "movaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovaps_xmm_xmmm128)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovaps_ymm_ymmm256)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovaps_xmm_k1z_xmmm128)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovaps_ymm_k1z_ymmm256)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovaps_zmm_k1z_zmmm512)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movapd_xmm_xmmm128)], "movapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovapd_xmm_xmmm128)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovapd_ymm_ymmm256)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovapd_xmm_k1z_xmmm128)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovapd_ymm_k1z_ymmm256)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovapd_zmm_k1z_zmmm512)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movaps_xmmm128_xmm)], "movaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovaps_xmmm128_xmm)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovaps_ymmm256_ymm)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovaps_xmmm128_k1z_xmm)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovaps_ymmm256_k1z_ymm)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovaps_zmmm512_k1z_zmm)], "vmovaps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movapd_xmmm128_xmm)], "movapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovapd_xmmm128_xmm)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovapd_ymmm256_ymm)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovapd_xmmm128_k1z_xmm)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovapd_ymmm256_k1z_ymm)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovapd_zmmm512_k1z_zmm)], "vmovapd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtpi2ps_xmm_mmm64)], "cvtpi2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtpi2pd_xmm_mmm64)], "cvtpi2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtsi2ss_xmm_rm32)], "cvtsi2ss", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtsi2ss_xmm_rm64)], "cvtsi2ss", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm32)], "vcvtsi2ss", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm64)], "vcvtsi2ss", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm32_er)], "vcvtsi2ss", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm64_er)], "vcvtsi2ss", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtsi2sd_xmm_rm32)], "cvtsi2sd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtsi2sd_xmm_rm64)], "cvtsi2sd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm32)], "vcvtsi2sd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm64)], "vcvtsi2sd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm32_er)], "vcvtsi2sd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm64_er)], "vcvtsi2sd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movntps_m128_xmm)], "movntps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovntps_m128_xmm)], "vmovntps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovntps_m256_ymm)], "vmovntps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntps_m128_xmm)], "vmovntps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntps_m256_ymm)], "vmovntps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntps_m512_zmm)], "vmovntps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movntpd_m128_xmm)], "movntpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovntpd_m128_xmm)], "vmovntpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovntpd_m256_ymm)], "vmovntpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntpd_m128_xmm)], "vmovntpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntpd_m256_ymm)], "vmovntpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntpd_m512_zmm)], "vmovntpd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movntss_m32_xmm)], "movntss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movntsd_m64_xmm)], "movntsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_2)], code[nameof(Code.Cvttps2pi_mm_xmmm64)], "cvttps2pi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Cvttpd2pi_mm_xmmm128)], "cvttpd2pi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvttss2si_r32_xmmm32)], "cvttss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvttss2si_r64_xmmm32)], "cvttss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvttss2si_r32_xmmm32)], "vcvttss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvttss2si_r64_xmmm32)], "vcvttss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttss2si_r32_xmmm32_sae)], "vcvttss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttss2si_r64_xmmm32_sae)], "vcvttss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvttsd2si_r32_xmmm64)], "cvttsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvttsd2si_r64_xmmm64)], "cvttsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvttsd2si_r32_xmmm64)], "vcvttsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvttsd2si_r64_xmmm64)], "vcvttsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttsd2si_r32_xmmm64_sae)], "vcvttsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttsd2si_r64_xmmm64_sae)], "vcvttsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_2)], code[nameof(Code.Cvtps2pi_mm_xmmm64)], "cvtps2pi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Cvtpd2pi_mm_xmmm128)], "cvtpd2pi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtss2si_r32_xmmm32)], "cvtss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtss2si_r64_xmmm32)], "cvtss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtss2si_r32_xmmm32)], "vcvtss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtss2si_r64_xmmm32)], "vcvtss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtss2si_r32_xmmm32_er)], "vcvtss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtss2si_r64_xmmm32_er)], "vcvtss2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtsd2si_r32_xmmm64)], "cvtsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtsd2si_r64_xmmm64)], "cvtsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtsd2si_r32_xmmm64)], "vcvtsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtsd2si_r64_xmmm64)], "vcvtsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtsd2si_r32_xmmm64_er)], "vcvtsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtsd2si_r64_xmmm64_er)], "vcvtsd2si", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ucomiss_xmm_xmmm32)], "ucomiss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vucomiss_xmm_xmmm32)], "vucomiss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vucomiss_xmm_xmmm32_sae)], "vucomiss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ucomisd_xmm_xmmm64)], "ucomisd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vucomisd_xmm_xmmm64)], "vucomisd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vucomisd_xmm_xmmm64_sae)], "vucomisd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Comiss_xmm_xmmm32)], "comiss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Comisd_xmm_xmmm64)], "comisd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcomiss_xmm_xmmm32)], "vcomiss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcomisd_xmm_xmmm64)], "vcomisd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcomiss_xmm_xmmm32_sae)], "vcomiss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcomisd_xmm_xmmm64_sae)], "vcomisd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrmsr)], "wrmsr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdtsc)], "rdtsc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdmsr)], "rdmsr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdpmc)], "rdpmc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sysenter)], "sysenter" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sysexitd)], "sysexit" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sysexitq)], "sysexitq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Getsecd)], "getsec" },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovo_r16_rm16)], "cmovo", 0 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovo_r32_rm32)], "cmovo", 0 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovo_r64_rm64)], "cmovo", 0 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovno_r16_rm16)], "cmovno", 1 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovno_r32_rm32)], "cmovno", 1 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovno_r64_rm64)], "cmovno", 1 },
				new object[] { ctorKind[nameof(CtorKind.CCa_3)], code[nameof(Code.Cmovb_r16_rm16)], "cmovb", "cmovc", "cmovnae", 2 },
				new object[] { ctorKind[nameof(CtorKind.CCa_3)], code[nameof(Code.Cmovb_r32_rm32)], "cmovb", "cmovc", "cmovnae", 2 },
				new object[] { ctorKind[nameof(CtorKind.CCa_3)], code[nameof(Code.Cmovb_r64_rm64)], "cmovb", "cmovc", "cmovnae", 2 },
				new object[] { ctorKind[nameof(CtorKind.CCa_3)], code[nameof(Code.Cmovae_r16_rm16)], "cmovae", "cmovnb", "cmovnc", 3 },
				new object[] { ctorKind[nameof(CtorKind.CCa_3)], code[nameof(Code.Cmovae_r32_rm32)], "cmovae", "cmovnb", "cmovnc", 3 },
				new object[] { ctorKind[nameof(CtorKind.CCa_3)], code[nameof(Code.Cmovae_r64_rm64)], "cmovae", "cmovnb", "cmovnc", 3 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmove_r16_rm16)], "cmove", "cmovz", 4 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmove_r32_rm32)], "cmove", "cmovz", 4 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmove_r64_rm64)], "cmove", "cmovz", 4 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovne_r16_rm16)], "cmovne", "cmovnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovne_r32_rm32)], "cmovne", "cmovnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovne_r64_rm64)], "cmovne", "cmovnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovbe_r16_rm16)], "cmovbe", "cmovna", 6 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovbe_r32_rm32)], "cmovbe", "cmovna", 6 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovbe_r64_rm64)], "cmovbe", "cmovna", 6 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmova_r16_rm16)], "cmova", "cmovnbe", 7 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmova_r32_rm32)], "cmova", "cmovnbe", 7 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmova_r64_rm64)], "cmova", "cmovnbe", 7 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovs_r16_rm16)], "cmovs", 8 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovs_r32_rm32)], "cmovs", 8 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovs_r64_rm64)], "cmovs", 8 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovns_r16_rm16)], "cmovns", 9 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovns_r32_rm32)], "cmovns", 9 },
				new object[] { ctorKind[nameof(CtorKind.CCa_1)], code[nameof(Code.Cmovns_r64_rm64)], "cmovns", 9 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovp_r16_rm16)], "cmovp", "cmovpe", 10 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovp_r32_rm32)], "cmovp", "cmovpe", 10 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovp_r64_rm64)], "cmovp", "cmovpe", 10 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovnp_r16_rm16)], "cmovnp", "cmovpo", 11 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovnp_r32_rm32)], "cmovnp", "cmovpo", 11 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovnp_r64_rm64)], "cmovnp", "cmovpo", 11 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovl_r16_rm16)], "cmovl", "cmovnge", 12 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovl_r32_rm32)], "cmovl", "cmovnge", 12 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovl_r64_rm64)], "cmovl", "cmovnge", 12 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovge_r16_rm16)], "cmovge", "cmovnl", 13 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovge_r32_rm32)], "cmovge", "cmovnl", 13 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovge_r64_rm64)], "cmovge", "cmovnl", 13 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovle_r16_rm16)], "cmovle", "cmovng", 14 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovle_r32_rm32)], "cmovle", "cmovng", 14 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovle_r64_rm64)], "cmovle", "cmovng", 14 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovg_r16_rm16)], "cmovg", "cmovnle", 15 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovg_r32_rm32)], "cmovg", "cmovnle", 15 },
				new object[] { ctorKind[nameof(CtorKind.CCa_2)], code[nameof(Code.Cmovg_r64_rm64)], "cmovg", "cmovnle", 15 },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kandw_kr_kr_kr)], "kandw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kandq_kr_kr_kr)], "kandq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kandb_kr_kr_kr)], "kandb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kandd_kr_kr_kr)], "kandd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kandnw_kr_kr_kr)], "kandnw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kandnq_kr_kr_kr)], "kandnq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kandnb_kr_kr_kr)], "kandnb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kandnd_kr_kr_kr)], "kandnd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Knotw_kr_kr)], "knotw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Knotq_kr_kr)], "knotq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Knotb_kr_kr)], "knotb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Knotd_kr_kr)], "knotd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Korw_kr_kr_kr)], "korw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Korq_kr_kr_kr)], "korq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Korb_kr_kr_kr)], "korb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kord_kr_kr_kr)], "kord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kxnorw_kr_kr_kr)], "kxnorw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kxnorq_kr_kr_kr)], "kxnorq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kxnorb_kr_kr_kr)], "kxnorb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kxnord_kr_kr_kr)], "kxnord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kxorw_kr_kr_kr)], "kxorw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kxorq_kr_kr_kr)], "kxorq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kxorb_kr_kr_kr)], "kxorb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kxord_kr_kr_kr)], "kxord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kaddw_kr_kr_kr)], "kaddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kaddq_kr_kr_kr)], "kaddq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kaddb_kr_kr_kr)], "kaddb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kaddd_kr_kr_kr)], "kaddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kunpckwd_kr_kr_kr)], "kunpckwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kunpckdq_kr_kr_kr)], "kunpckdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kunpckbw_kr_kr_kr)], "kunpckbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movmskps_r32_xmm)], "movmskps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movmskps_r64_xmm)], "movmskps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovmskps_r32_xmm)], "vmovmskps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovmskps_r64_xmm)], "vmovmskps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovmskps_r32_ymm)], "vmovmskps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovmskps_r64_ymm)], "vmovmskps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movmskpd_r32_xmm)], "movmskpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movmskpd_r64_xmm)], "movmskpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovmskpd_r32_xmm)], "vmovmskpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovmskpd_r64_xmm)], "vmovmskpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovmskpd_r32_ymm)], "vmovmskpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmovmskpd_r64_ymm)], "vmovmskpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sqrtps_xmm_xmmm128)], "sqrtps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vsqrtps_xmm_xmmm128)], "vsqrtps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vsqrtps_ymm_ymmm256)], "vsqrtps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsqrtps_xmm_k1z_xmmm128b32)], "vsqrtps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsqrtps_ymm_k1z_ymmm256b32)], "vsqrtps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er)], "vsqrtps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sqrtpd_xmm_xmmm128)], "sqrtpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vsqrtpd_xmm_xmmm128)], "vsqrtpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vsqrtpd_ymm_ymmm256)], "vsqrtpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsqrtpd_xmm_k1z_xmmm128b64)], "vsqrtpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsqrtpd_ymm_k1z_ymmm256b64)], "vsqrtpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er)], "vsqrtpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sqrtss_xmm_xmmm32)], "sqrtss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vsqrtss_xmm_xmm_xmmm32)], "vsqrtss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er)], "vsqrtss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Sqrtsd_xmm_xmmm64)], "sqrtsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vsqrtsd_xmm_xmm_xmmm64)], "vsqrtsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er)], "vsqrtsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rsqrtps_xmm_xmmm128)], "rsqrtps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vrsqrtps_xmm_xmmm128)], "vrsqrtps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vrsqrtps_ymm_ymmm256)], "vrsqrtps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rsqrtss_xmm_xmmm32)], "rsqrtss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vrsqrtss_xmm_xmm_xmmm32)], "vrsqrtss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rcpps_xmm_xmmm128)], "rcpps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vrcpps_xmm_xmmm128)], "vrcpps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vrcpps_ymm_ymmm256)], "vrcpps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Rcpss_xmm_xmmm32)], "rcpss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vrcpss_xmm_xmm_xmmm32)], "vrcpss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Andps_xmm_xmmm128)], "andps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vandps_xmm_xmm_xmmm128)], "vandps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vandps_ymm_ymm_ymmm256)], "vandps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandps_xmm_k1z_xmm_xmmm128b32)], "vandps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandps_ymm_k1z_ymm_ymmm256b32)], "vandps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandps_zmm_k1z_zmm_zmmm512b32)], "vandps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Andpd_xmm_xmmm128)], "andpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vandpd_xmm_xmm_xmmm128)], "vandpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vandpd_ymm_ymm_ymmm256)], "vandpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64)], "vandpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64)], "vandpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64)], "vandpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Andnps_xmm_xmmm128)], "andnps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vandnps_xmm_xmm_xmmm128)], "vandnps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vandnps_ymm_ymm_ymmm256)], "vandnps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32)], "vandnps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32)], "vandnps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32)], "vandnps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Andnpd_xmm_xmmm128)], "andnpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vandnpd_xmm_xmm_xmmm128)], "vandnpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vandnpd_ymm_ymm_ymmm256)], "vandnpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64)], "vandnpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64)], "vandnpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64)], "vandnpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Orps_xmm_xmmm128)], "orps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vorps_xmm_xmm_xmmm128)], "vorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vorps_ymm_ymm_ymmm256)], "vorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vorps_xmm_k1z_xmm_xmmm128b32)], "vorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vorps_ymm_k1z_ymm_ymmm256b32)], "vorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vorps_zmm_k1z_zmm_zmmm512b32)], "vorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Orpd_xmm_xmmm128)], "orpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vorpd_xmm_xmm_xmmm128)], "vorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vorpd_ymm_ymm_ymmm256)], "vorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64)], "vorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64)], "vorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64)], "vorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xorps_xmm_xmmm128)], "xorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vxorps_xmm_xmm_xmmm128)], "vxorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vxorps_ymm_ymm_ymmm256)], "vxorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32)], "vxorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32)], "vxorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32)], "vxorps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xorpd_xmm_xmmm128)], "xorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vxorpd_xmm_xmm_xmmm128)], "vxorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vxorpd_ymm_ymm_ymmm256)], "vxorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64)], "vxorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64)], "vxorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64)], "vxorpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Addps_xmm_xmmm128)], "addps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaddps_xmm_xmm_xmmm128)], "vaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaddps_ymm_ymm_ymmm256)], "vaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32)], "vaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32)], "vaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er)], "vaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Addpd_xmm_xmmm128)], "addpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaddpd_xmm_xmm_xmmm128)], "vaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaddpd_ymm_ymm_ymmm256)], "vaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64)], "vaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64)], "vaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er)], "vaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Addss_xmm_xmmm32)], "addss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vaddss_xmm_xmm_xmmm32)], "vaddss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er)], "vaddss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Addsd_xmm_xmmm64)], "addsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vaddsd_xmm_xmm_xmmm64)], "vaddsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er)], "vaddsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mulps_xmm_xmmm128)], "mulps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmulps_xmm_xmm_xmmm128)], "vmulps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmulps_ymm_ymm_ymmm256)], "vmulps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32)], "vmulps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32)], "vmulps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er)], "vmulps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mulpd_xmm_xmmm128)], "mulpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmulpd_xmm_xmm_xmmm128)], "vmulpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmulpd_ymm_ymm_ymmm256)], "vmulpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64)], "vmulpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64)], "vmulpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er)], "vmulpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mulss_xmm_xmmm32)], "mulss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmulss_xmm_xmm_xmmm32)], "vmulss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er)], "vmulss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Mulsd_xmm_xmmm64)], "mulsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmulsd_xmm_xmm_xmmm64)], "vmulsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er)], "vmulsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtps2pd_xmm_xmmm64)], "cvtps2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtps2pd_xmm_xmmm64)], "vcvtps2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtps2pd_ymm_xmmm128)], "vcvtps2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32)], "vcvtps2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32)], "vcvtps2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae)], "vcvtps2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cvtpd2ps_xmm_xmmm128)], "cvtpd2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtpd2ps_xmm_xmmm128)], "vcvtpd2ps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtpd2ps_xmm_ymmm256)], "vcvtpd2ps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64)], "vcvtpd2ps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64)], "vcvtpd2ps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er)], "vcvtpd2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtss2sd_xmm_xmmm32)], "cvtss2sd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtss2sd_xmm_xmm_xmmm32)], "vcvtss2sd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae)], "vcvtss2sd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtsd2ss_xmm_xmmm64)], "cvtsd2ss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtsd2ss_xmm_xmm_xmmm64)], "vcvtsd2ss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er)], "vcvtsd2ss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cvtdq2ps_xmm_xmmm128)], "cvtdq2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtdq2ps_xmm_xmmm128)], "vcvtdq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtdq2ps_ymm_ymmm256)], "vcvtdq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32)], "vcvtdq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32)], "vcvtdq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er)], "vcvtdq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64)], "vcvtqq2ps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64)], "vcvtqq2ps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er)], "vcvtqq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cvtps2dq_xmm_xmmm128)], "cvtps2dq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtps2dq_xmm_xmmm128)], "vcvtps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtps2dq_ymm_ymmm256)], "vcvtps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32)], "vcvtps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32)], "vcvtps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er)], "vcvtps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cvttps2dq_xmm_xmmm128)], "cvttps2dq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvttps2dq_xmm_xmmm128)], "vcvttps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvttps2dq_ymm_ymmm256)], "vcvttps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32)], "vcvttps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32)], "vcvttps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae)], "vcvttps2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Subps_xmm_xmmm128)], "subps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vsubps_xmm_xmm_xmmm128)], "vsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vsubps_ymm_ymm_ymmm256)], "vsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32)], "vsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32)], "vsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er)], "vsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Subpd_xmm_xmmm128)], "subpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vsubpd_xmm_xmm_xmmm128)], "vsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vsubpd_ymm_ymm_ymmm256)], "vsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64)], "vsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64)], "vsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er)], "vsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Subss_xmm_xmmm32)], "subss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vsubss_xmm_xmm_xmmm32)], "vsubss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er)], "vsubss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Subsd_xmm_xmmm64)], "subsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vsubsd_xmm_xmm_xmmm64)], "vsubsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er)], "vsubsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Minps_xmm_xmmm128)], "minps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vminps_xmm_xmm_xmmm128)], "vminps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vminps_ymm_ymm_ymmm256)], "vminps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vminps_xmm_k1z_xmm_xmmm128b32)], "vminps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vminps_ymm_k1z_ymm_ymmm256b32)], "vminps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae)], "vminps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Minpd_xmm_xmmm128)], "minpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vminpd_xmm_xmm_xmmm128)], "vminpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vminpd_ymm_ymm_ymmm256)], "vminpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64)], "vminpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64)], "vminpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae)], "vminpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Minss_xmm_xmmm32)], "minss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vminss_xmm_xmm_xmmm32)], "vminss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae)], "vminss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Minsd_xmm_xmmm64)], "minsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vminsd_xmm_xmm_xmmm64)], "vminsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae)], "vminsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Divps_xmm_xmmm128)], "divps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vdivps_xmm_xmm_xmmm128)], "vdivps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vdivps_ymm_ymm_ymmm256)], "vdivps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32)], "vdivps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32)], "vdivps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er)], "vdivps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Divpd_xmm_xmmm128)], "divpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vdivpd_xmm_xmm_xmmm128)], "vdivpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vdivpd_ymm_ymm_ymmm256)], "vdivpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64)], "vdivpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64)], "vdivpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er)], "vdivpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Divss_xmm_xmmm32)], "divss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vdivss_xmm_xmm_xmmm32)], "vdivss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er)], "vdivss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Divsd_xmm_xmmm64)], "divsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vdivsd_xmm_xmm_xmmm64)], "vdivsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er)], "vdivsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Maxps_xmm_xmmm128)], "maxps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaxps_xmm_xmm_xmmm128)], "vmaxps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaxps_ymm_ymm_ymmm256)], "vmaxps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32)], "vmaxps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32)], "vmaxps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae)], "vmaxps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Maxpd_xmm_xmmm128)], "maxpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaxpd_xmm_xmm_xmmm128)], "vmaxpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaxpd_ymm_ymm_ymmm256)], "vmaxpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64)], "vmaxpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64)], "vmaxpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae)], "vmaxpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Maxss_xmm_xmmm32)], "maxss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmaxss_xmm_xmm_xmmm32)], "vmaxss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae)], "vmaxss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Maxsd_xmm_xmmm64)], "maxsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmaxsd_xmm_xmm_xmmm64)], "vmaxsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae)], "vmaxsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Punpcklbw_mm_mmm32)], "punpcklbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Punpcklbw_xmm_xmmm128)], "punpcklbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpcklbw_xmm_xmm_xmmm128)], "vpunpcklbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpcklbw_ymm_ymm_ymmm256)], "vpunpcklbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128)], "vpunpcklbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256)], "vpunpcklbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512)], "vpunpcklbw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Punpcklwd_mm_mmm32)], "punpcklwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Punpcklwd_xmm_xmmm128)], "punpcklwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpcklwd_xmm_xmm_xmmm128)], "vpunpcklwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpcklwd_ymm_ymm_ymmm256)], "vpunpcklwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128)], "vpunpcklwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256)], "vpunpcklwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512)], "vpunpcklwd" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Punpckldq_mm_mmm32)], "punpckldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Punpckldq_xmm_xmmm128)], "punpckldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckldq_xmm_xmm_xmmm128)], "vpunpckldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckldq_ymm_ymm_ymmm256)], "vpunpckldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32)], "vpunpckldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32)], "vpunpckldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32)], "vpunpckldq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Packsswb_mm_mmm64)], "packsswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Packsswb_xmm_xmmm128)], "packsswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpacksswb_xmm_xmm_xmmm128)], "vpacksswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpacksswb_ymm_ymm_ymmm256)], "vpacksswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128)], "vpacksswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256)], "vpacksswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512)], "vpacksswb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pcmpgtb_mm_mmm64)], "pcmpgtb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpgtb_xmm_xmmm128)], "pcmpgtb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpgtb_xmm_xmm_xmmm128)], "vpcmpgtb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpgtb_ymm_ymm_ymmm256)], "vpcmpgtb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128)], "vpcmpgtb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256)], "vpcmpgtb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512)], "vpcmpgtb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pcmpgtw_mm_mmm64)], "pcmpgtw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpgtw_xmm_xmmm128)], "pcmpgtw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpgtw_xmm_xmm_xmmm128)], "vpcmpgtw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpgtw_ymm_ymm_ymmm256)], "vpcmpgtw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128)], "vpcmpgtw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256)], "vpcmpgtw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512)], "vpcmpgtw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pcmpgtd_mm_mmm64)], "pcmpgtd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpgtd_xmm_xmmm128)], "pcmpgtd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpgtd_xmm_xmm_xmmm128)], "vpcmpgtd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpgtd_ymm_ymm_ymmm256)], "vpcmpgtd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32)], "vpcmpgtd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32)], "vpcmpgtd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32)], "vpcmpgtd" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Packuswb_mm_mmm64)], "packuswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Packuswb_xmm_xmmm128)], "packuswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpackuswb_xmm_xmm_xmmm128)], "vpackuswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpackuswb_ymm_ymm_ymmm256)], "vpackuswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128)], "vpackuswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256)], "vpackuswb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512)], "vpackuswb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Punpckhbw_mm_mmm64)], "punpckhbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Punpckhbw_xmm_xmmm128)], "punpckhbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckhbw_xmm_xmm_xmmm128)], "vpunpckhbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckhbw_ymm_ymm_ymmm256)], "vpunpckhbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128)], "vpunpckhbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256)], "vpunpckhbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512)], "vpunpckhbw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Punpckhwd_mm_mmm64)], "punpckhwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Punpckhwd_xmm_xmmm128)], "punpckhwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckhwd_xmm_xmm_xmmm128)], "vpunpckhwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckhwd_ymm_ymm_ymmm256)], "vpunpckhwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128)], "vpunpckhwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256)], "vpunpckhwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512)], "vpunpckhwd" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Punpckhdq_mm_mmm64)], "punpckhdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Punpckhdq_xmm_xmmm128)], "punpckhdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckhdq_xmm_xmm_xmmm128)], "vpunpckhdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckhdq_ymm_ymm_ymmm256)], "vpunpckhdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32)], "vpunpckhdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32)], "vpunpckhdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32)], "vpunpckhdq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Packssdw_mm_mmm64)], "packssdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Packssdw_xmm_xmmm128)], "packssdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpackssdw_xmm_xmm_xmmm128)], "vpackssdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpackssdw_ymm_ymm_ymmm256)], "vpackssdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32)], "vpackssdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32)], "vpackssdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32)], "vpackssdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Punpcklqdq_xmm_xmmm128)], "punpcklqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpcklqdq_xmm_xmm_xmmm128)], "vpunpcklqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpcklqdq_ymm_ymm_ymmm256)], "vpunpcklqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64)], "vpunpcklqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64)], "vpunpcklqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64)], "vpunpcklqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Punpckhqdq_xmm_xmmm128)], "punpckhqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckhqdq_xmm_xmm_xmmm128)], "vpunpckhqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpunpckhqdq_ymm_ymm_ymmm256)], "vpunpckhqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64)], "vpunpckhqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64)], "vpunpckhqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64)], "vpunpckhqdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movd_mm_rm32)], "movd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movq_mm_rm64)], "movq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movd_xmm_rm32)], "movd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movq_xmm_rm64)], "movq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovd_xmm_rm32)], "vmovd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovq_xmm_rm64)], "vmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovd_xmm_rm32)], "vmovd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovq_xmm_rm64)], "vmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movq_mm_mmm64)], "movq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movdqa_xmm_xmmm128)], "movdqa" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovdqa_xmm_xmmm128)], "vmovdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovdqa_ymm_ymmm256)], "vmovdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa32_xmm_k1z_xmmm128)], "vmovdqa32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa32_ymm_k1z_ymmm256)], "vmovdqa32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa32_zmm_k1z_zmmm512)], "vmovdqa32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa64_xmm_k1z_xmmm128)], "vmovdqa64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa64_ymm_k1z_ymmm256)], "vmovdqa64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa64_zmm_k1z_zmmm512)], "vmovdqa64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movdqu_xmm_xmmm128)], "movdqu" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovdqu_xmm_xmmm128)], "vmovdqu", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovdqu_ymm_ymmm256)], "vmovdqu", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu32_xmm_k1z_xmmm128)], "vmovdqu32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu32_ymm_k1z_ymmm256)], "vmovdqu32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu32_zmm_k1z_zmmm512)], "vmovdqu32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu64_xmm_k1z_xmmm128)], "vmovdqu64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu64_ymm_k1z_ymmm256)], "vmovdqu64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu64_zmm_k1z_zmmm512)], "vmovdqu64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu8_xmm_k1z_xmmm128)], "vmovdqu8" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu8_ymm_k1z_ymmm256)], "vmovdqu8" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu8_zmm_k1z_zmmm512)], "vmovdqu8" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu16_xmm_k1z_xmmm128)], "vmovdqu16" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu16_ymm_k1z_ymmm256)], "vmovdqu16" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu16_zmm_k1z_zmmm512)], "vmovdqu16" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pshufw_mm_mmm64_imm8)], "pshufw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pshufd_xmm_xmmm128_imm8)], "pshufd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpshufd_xmm_xmmm128_imm8)], "vpshufd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpshufd_ymm_ymmm256_imm8)], "vpshufd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8)], "vpshufd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8)], "vpshufd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8)], "vpshufd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pshufhw_xmm_xmmm128_imm8)], "pshufhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpshufhw_xmm_xmmm128_imm8)], "vpshufhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpshufhw_ymm_ymmm256_imm8)], "vpshufhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8)], "vpshufhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8)], "vpshufhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8)], "vpshufhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pshuflw_xmm_xmmm128_imm8)], "pshuflw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpshuflw_xmm_xmmm128_imm8)], "vpshuflw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpshuflw_ymm_ymmm256_imm8)], "vpshuflw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8)], "vpshuflw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8)], "vpshuflw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8)], "vpshuflw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrlw_mm_imm8)], "psrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrlw_xmm_imm8)], "psrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlw_xmm_xmm_imm8)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlw_ymm_ymm_imm8)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psraw_mm_imm8)], "psraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psraw_xmm_imm8)], "psraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsraw_xmm_xmm_imm8)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsraw_ymm_ymm_imm8)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmmm128_imm8)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymmm256_imm8)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmmm512_imm8)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psllw_mm_imm8)], "psllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psllw_xmm_imm8)], "psllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllw_xmm_xmm_imm8)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllw_ymm_ymm_imm8)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmmm128_imm8)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymmm256_imm8)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmmm512_imm8)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprord_xmm_k1z_xmmm128b32_imm8)], "vprord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprord_ymm_k1z_ymmm256b32_imm8)], "vprord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprord_zmm_k1z_zmmm512b32_imm8)], "vprord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8)], "vprorq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8)], "vprorq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8)], "vprorq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprold_xmm_k1z_xmmm128b32_imm8)], "vprold" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprold_ymm_k1z_ymmm256b32_imm8)], "vprold" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprold_zmm_k1z_zmmm512b32_imm8)], "vprold" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8)], "vprolq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8)], "vprolq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8)], "vprolq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrld_mm_imm8)], "psrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrld_xmm_imm8)], "psrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrld_xmm_xmm_imm8)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrld_ymm_ymm_imm8)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrad_mm_imm8)], "psrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrad_xmm_imm8)], "psrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrad_xmm_xmm_imm8)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrad_ymm_ymm_imm8)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8)], "vpsraq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8)], "vpsraq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8)], "vpsraq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pslld_mm_imm8)], "pslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pslld_xmm_imm8)], "pslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpslld_xmm_xmm_imm8)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpslld_ymm_ymm_imm8)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrlq_mm_imm8)], "psrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrlq_xmm_imm8)], "psrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlq_xmm_xmm_imm8)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlq_ymm_ymm_imm8)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrldq_xmm_imm8)], "psrldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrldq_xmm_xmm_imm8)], "vpsrldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrldq_ymm_ymm_imm8)], "vpsrldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrldq_xmm_xmmm128_imm8)], "vpsrldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrldq_ymm_ymmm256_imm8)], "vpsrldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrldq_zmm_zmmm512_imm8)], "vpsrldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psllq_mm_imm8)], "psllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psllq_xmm_imm8)], "psllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllq_xmm_xmm_imm8)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllq_ymm_ymm_imm8)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pslldq_xmm_imm8)], "pslldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpslldq_xmm_xmm_imm8)], "vpslldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpslldq_ymm_ymm_imm8)], "vpslldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpslldq_xmm_xmmm128_imm8)], "vpslldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpslldq_ymm_ymmm256_imm8)], "vpslldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpslldq_zmm_zmmm512_imm8)], "vpslldq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pcmpeqb_mm_mmm64)], "pcmpeqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpeqb_xmm_xmmm128)], "pcmpeqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpeqb_xmm_xmm_xmmm128)], "vpcmpeqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpeqb_ymm_ymm_ymmm256)], "vpcmpeqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128)], "vpcmpeqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256)], "vpcmpeqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512)], "vpcmpeqb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pcmpeqw_mm_mmm64)], "pcmpeqw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpeqw_xmm_xmmm128)], "pcmpeqw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpeqw_xmm_xmm_xmmm128)], "vpcmpeqw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpeqw_ymm_ymm_ymmm256)], "vpcmpeqw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128)], "vpcmpeqw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256)], "vpcmpeqw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512)], "vpcmpeqw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pcmpeqd_mm_mmm64)], "pcmpeqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpeqd_xmm_xmmm128)], "pcmpeqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpeqd_xmm_xmm_xmmm128)], "vpcmpeqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpeqd_ymm_ymm_ymmm256)], "vpcmpeqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32)], "vpcmpeqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32)], "vpcmpeqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32)], "vpcmpeqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Emms)], "emms" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vzeroupper)], "vzeroupper" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vzeroall)], "vzeroall" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Vmread_rm32_r32)], "vmread", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Vmread_rm64_r64)], "vmread", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32)], "vcvttps2udq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32)], "vcvttps2udq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae)], "vcvttps2udq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64)], "vcvttpd2udq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64)], "vcvttpd2udq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae)], "vcvttpd2udq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Extrq_xmm_imm8_imm8)], "extrq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32)], "vcvttps2uqq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32)], "vcvttps2uqq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae)], "vcvttps2uqq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64)], "vcvttpd2uqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64)], "vcvttpd2uqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae)], "vcvttpd2uqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttss2usi_r32_xmmm32_sae)], "vcvttss2usi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttss2usi_r64_xmmm32_sae)], "vcvttss2usi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Insertq_xmm_xmm_imm8_imm8)], "insertq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttsd2usi_r32_xmmm64_sae)], "vcvttsd2usi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttsd2usi_r64_xmmm64_sae)], "vcvttsd2usi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Vmwrite_r32_rm32)], "vmwrite", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Vmwrite_r64_rm64)], "vmwrite", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32)], "vcvtps2udq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32)], "vcvtps2udq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er)], "vcvtps2udq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64)], "vcvtpd2udq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64)], "vcvtpd2udq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er)], "vcvtpd2udq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Extrq_xmm_xmm)], "extrq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32)], "vcvtps2uqq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32)], "vcvtps2uqq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er)], "vcvtps2uqq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64)], "vcvtpd2uqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64)], "vcvtpd2uqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er)], "vcvtpd2uqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtss2usi_r32_xmmm32_er)], "vcvtss2usi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtss2usi_r64_xmmm32_er)], "vcvtss2usi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Insertq_xmm_xmm)], "insertq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtsd2usi_r32_xmmm64_er)], "vcvtsd2usi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtsd2usi_r64_xmmm64_er)], "vcvtsd2usi", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32)], "vcvttps2qq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32)], "vcvttps2qq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae)], "vcvttps2qq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64)], "vcvttpd2qq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64)], "vcvttpd2qq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae)], "vcvttpd2qq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32)], "vcvtudq2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32)], "vcvtudq2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er)], "vcvtudq2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64)], "vcvtuqq2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64)], "vcvtuqq2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er)], "vcvtuqq2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32)], "vcvtudq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32)], "vcvtudq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er)], "vcvtudq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64)], "vcvtuqq2ps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64)], "vcvtuqq2ps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er)], "vcvtuqq2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32)], "vcvtps2qq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32)], "vcvtps2qq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er)], "vcvtps2qq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64)], "vcvtpd2qq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64)], "vcvtpd2qq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er)], "vcvtpd2qq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm32_er)], "vcvtusi2ss", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm64_er)], "vcvtusi2ss", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm32_er)], "vcvtusi2sd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm64_er)], "vcvtusi2sd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Haddpd_xmm_xmmm128)], "haddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vhaddpd_xmm_xmm_xmmm128)], "vhaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vhaddpd_ymm_ymm_ymmm256)], "vhaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Haddps_xmm_xmmm128)], "haddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vhaddps_xmm_xmm_xmmm128)], "vhaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vhaddps_ymm_ymm_ymmm256)], "vhaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Hsubpd_xmm_xmmm128)], "hsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vhsubpd_xmm_xmm_xmmm128)], "vhsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vhsubpd_ymm_ymm_ymmm256)], "vhsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Hsubps_xmm_xmmm128)], "hsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vhsubps_xmm_xmm_xmmm128)], "vhsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vhsubps_ymm_ymm_ymmm256)], "vhsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movd_rm32_mm)], "movd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movq_rm64_mm)], "movq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movd_rm32_xmm)], "movd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movq_rm64_xmm)], "movq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovd_rm32_xmm)], "vmovd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovq_rm64_xmm)], "vmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovd_rm32_xmm)], "vmovd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovq_rm64_xmm)], "vmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movq_xmm_xmmm64)], "movq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovq_xmm_xmmm64)], "vmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovq_xmm_xmmm64)], "vmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movq_mmm64_mm)], "movq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movdqa_xmmm128_xmm)], "movdqa" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovdqa_xmmm128_xmm)], "vmovdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovdqa_ymmm256_ymm)], "vmovdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa32_xmmm128_k1z_xmm)], "vmovdqa32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa32_ymmm256_k1z_ymm)], "vmovdqa32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa32_zmmm512_k1z_zmm)], "vmovdqa32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa64_xmmm128_k1z_xmm)], "vmovdqa64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa64_ymmm256_k1z_ymm)], "vmovdqa64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqa64_zmmm512_k1z_zmm)], "vmovdqa64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movdqu_xmmm128_xmm)], "movdqu" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovdqu_xmmm128_xmm)], "vmovdqu", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovdqu_ymmm256_ymm)], "vmovdqu", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu32_xmmm128_k1z_xmm)], "vmovdqu32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu32_ymmm256_k1z_ymm)], "vmovdqu32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu32_zmmm512_k1z_zmm)], "vmovdqu32" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu64_xmmm128_k1z_xmm)], "vmovdqu64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu64_ymmm256_k1z_ymm)], "vmovdqu64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu64_zmmm512_k1z_zmm)], "vmovdqu64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu8_xmmm128_k1z_xmm)], "vmovdqu8" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu8_ymmm256_k1z_ymm)], "vmovdqu8" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu8_zmmm512_k1z_zmm)], "vmovdqu8" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu16_xmmm128_k1z_xmm)], "vmovdqu16" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu16_ymmm256_k1z_ymm)], "vmovdqu16" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vmovdqu16_zmmm512_k1z_zmm)], "vmovdqu16" },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jo_rel16)], "jo", 0 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jo_rel32_32)], "jo", 0 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jo_rel32_64)], "jo", 0 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jno_rel16)], "jno", 1 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jno_rel32_32)], "jno", 1 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jno_rel32_64)], "jno", 1 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jb_rel16)], "jb", "jc", "jnae", 2 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jb_rel32_32)], "jb", "jc", "jnae", 2 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jb_rel32_64)], "jb", "jc", "jnae", 2 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jae_rel16)], "jae", "jnb", "jnc", 3 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jae_rel32_32)], "jae", "jnb", "jnc", 3 },
				new object[] { ctorKind[nameof(CtorKind.jcc_3)], code[nameof(Code.Jae_rel32_64)], "jae", "jnb", "jnc", 3 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Je_rel16)], "je", "jz", 4 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Je_rel32_32)], "je", "jz", 4 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Je_rel32_64)], "je", "jz", 4 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jne_rel16)], "jne", "jnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jne_rel32_32)], "jne", "jnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jne_rel32_64)], "jne", "jnz", 5 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jbe_rel16)], "jbe", "jna", 6 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jbe_rel32_32)], "jbe", "jna", 6 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jbe_rel32_64)], "jbe", "jna", 6 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Ja_rel16)], "ja", "jnbe", 7 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Ja_rel32_32)], "ja", "jnbe", 7 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Ja_rel32_64)], "ja", "jnbe", 7 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Js_rel16)], "js", 8 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Js_rel32_32)], "js", 8 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Js_rel32_64)], "js", 8 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jns_rel16)], "jns", 9 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jns_rel32_32)], "jns", 9 },
				new object[] { ctorKind[nameof(CtorKind.jcc_1)], code[nameof(Code.Jns_rel32_64)], "jns", 9 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jp_rel16)], "jp", "jpe", 10 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jp_rel32_32)], "jp", "jpe", 10 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jp_rel32_64)], "jp", "jpe", 10 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jnp_rel16)], "jnp", "jpo", 11 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jnp_rel32_32)], "jnp", "jpo", 11 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jnp_rel32_64)], "jnp", "jpo", 11 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jl_rel16)], "jl", "jnge", 12 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jl_rel32_32)], "jl", "jnge", 12 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jl_rel32_64)], "jl", "jnge", 12 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jge_rel16)], "jge", "jnl", 13 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jge_rel32_32)], "jge", "jnl", 13 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jge_rel32_64)], "jge", "jnl", 13 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jle_rel16)], "jle", "jng", 14 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jle_rel32_32)], "jle", "jng", 14 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jle_rel32_64)], "jle", "jng", 14 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jg_rel16)], "jg", "jnle", 15 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jg_rel32_32)], "jg", "jnle", 15 },
				new object[] { ctorKind[nameof(CtorKind.jcc_2)], code[nameof(Code.Jg_rel32_64)], "jg", "jnle", 15 },
				new object[] { ctorKind[nameof(CtorKind.CCb_1)], code[nameof(Code.Seto_rm8)], "seto", 0, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_1)], code[nameof(Code.Setno_rm8)], "setno", 1, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_3)], code[nameof(Code.Setb_rm8)], "setb", "setc", "setnae", 2, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_3)], code[nameof(Code.Setae_rm8)], "setae", "setnb", "setnc", 3, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Sete_rm8)], "sete", "setz", 4, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Setne_rm8)], "setne", "setnz", 5, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Setbe_rm8)], "setbe", "setna", 6, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Seta_rm8)], "seta", "setnbe", 7, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_1)], code[nameof(Code.Sets_rm8)], "sets", 8, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_1)], code[nameof(Code.Setns_rm8)], "setns", 9, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Setp_rm8)], "setp", "setpe", 10, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Setnp_rm8)], "setnp", "setpo", 11, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Setl_rm8)], "setl", "setnge", 12, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Setge_rm8)], "setge", "setnl", 13, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Setle_rm8)], "setle", "setng", 14, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.CCb_2)], code[nameof(Code.Setg_rm8)], "setg", "setnle", 15, instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Kmovw_kr_km16)], "kmovw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Kmovq_kr_km64)], "kmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Kmovb_kr_km8)], "kmovb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Kmovd_kr_km32)], "kmovd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Kmovw_m16_kr)], "kmovw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Kmovq_m64_kr)], "kmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Kmovb_m8_kr)], "kmovb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Kmovd_m32_kr)], "kmovd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kmovw_kr_r32)], "kmovw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kmovb_kr_r32)], "kmovb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kmovd_kr_r32)], "kmovd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kmovq_kr_r64)], "kmovq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kmovw_r32_kr)], "kmovw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kmovb_r32_kr)], "kmovb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kmovd_r32_kr)], "kmovd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kmovq_r64_kr)], "kmovq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kortestw_kr_kr)], "kortestw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kortestq_kr_kr)], "kortestq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kortestb_kr_kr)], "kortestb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kortestd_kr_kr)], "kortestd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Ktestw_kr_kr)], "ktestw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Ktestq_kr_kr)], "ktestq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Ktestb_kr_kr)], "ktestb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Ktestd_kr_kr)], "ktestd" },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushw_FS)], "push", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushd_FS)], "push", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushq_FS)], "push", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popw_FS)], "pop", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popd_FS)], "pop", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popq_FS)], "pop", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cpuid)], "cpuid" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bt_rm16_r16)], "bt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bt_rm32_r32)], "bt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bt_rm64_r64)], "bt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shld_rm16_r16_imm8)], "shld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shld_rm32_r32_imm8)], "shld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shld_rm64_r64_imm8)], "shld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shld_rm16_r16_CL)], "shld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shld_rm32_r32_CL)], "shld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shld_rm64_r64_CL)], "shld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Montmul_16)], "montmul" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Montmul_32)], "montmul" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Montmul_64)], "montmul" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsha1_16)], "xsha1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsha1_32)], "xsha1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsha1_64)], "xsha1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsha256_16)], "xsha256" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsha256_32)], "xsha256" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsha256_64)], "xsha256" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xbts_r16_rm16)], "xbts" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xbts_r32_rm32)], "xbts" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xstore_16)], "xstore" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xstore_32)], "xstore" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xstore_64)], "xstore" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptEcb_16)], "xcryptecb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptEcb_32)], "xcryptecb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptEcb_64)], "xcryptecb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptCbc_16)], "xcryptcbc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptCbc_32)], "xcryptcbc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptCbc_64)], "xcryptcbc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptCtr_16)], "xcryptctr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptCtr_32)], "xcryptctr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptCtr_64)], "xcryptctr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptCfb_16)], "xcryptcfb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptCfb_32)], "xcryptcfb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptCfb_64)], "xcryptcfb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptOfb_16)], "xcryptofb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptOfb_32)], "xcryptofb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XcryptOfb_64)], "xcryptofb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ibts_rm16_r16)], "ibts" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ibts_rm32_r32)], "ibts" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmpxchg486_rm8_r8)], "cmpxchg486" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmpxchg486_rm16_r16)], "cmpxchg486" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmpxchg486_rm32_r32)], "cmpxchg486" },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushw_GS)], "push", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushd_GS)], "push", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Pushq_GS)], "push", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popw_GS)], "pop", codeSize[nameof(CodeSize.Code16)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popd_GS)], "pop", codeSize[nameof(CodeSize.Code32)] },
				new object[] { ctorKind[nameof(CtorKind.OpSize_1)], code[nameof(Code.Popq_GS)], "pop", codeSize[nameof(CodeSize.Code64)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rsm)], "rsm" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bts_rm16_r16)], "bts" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bts_rm32_r32)], "bts" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bts_rm64_r64)], "bts" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shrd_rm16_r16_imm8)], "shrd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shrd_rm32_r32_imm8)], "shrd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shrd_rm64_r64_imm8)], "shrd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shrd_rm16_r16_CL)], "shrd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shrd_rm32_r32_CL)], "shrd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shrd_rm64_r64_CL)], "shrd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fxsave_m512byte)], "fxsave" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fxsave64_m512byte)], "fxsave64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdfsbase_r32)], "rdfsbase" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdfsbase_r64)], "rdfsbase" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fxrstor_m512byte)], "fxrstor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Fxrstor64_m512byte)], "fxrstor64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdgsbase_r32)], "rdgsbase" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdgsbase_r64)], "rdgsbase" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ldmxcsr_m32)], "ldmxcsr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrfsbase_r32)], "wrfsbase" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrfsbase_r64)], "wrfsbase" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vldmxcsr_m32)], "vldmxcsr", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Stmxcsr_m32)], "stmxcsr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrgsbase_r32)], "wrgsbase" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrgsbase_r64)], "wrgsbase" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vstmxcsr_m32)], "vstmxcsr", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsave_mem)], "xsave" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsave64_mem)], "xsave64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ptwrite_rm32)], "ptwrite", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ptwrite_rm64)], "ptwrite", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xrstor_mem)], "xrstor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xrstor64_mem)], "xrstor64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Incsspd_r32)], "incsspd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Incsspq_r64)], "incsspq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsaveopt_mem)], "xsaveopt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsaveopt64_mem)], "xsaveopt64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clwb_m8)], "clwb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Tpause_r32)], "tpause" },
				new object[] { ctorKind[nameof(CtorKind.Reg32)], code[nameof(Code.Tpause_r64)], "tpause", instrOpInfoFlags[nameof(InstrOpInfoFlags.None)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Clrssbsy_m64)], "clrssbsy", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umonitor_r16)], "umonitor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umonitor_r32)], "umonitor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umonitor_r64)], "umonitor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Umwait_r32)], "umwait" },
				new object[] { ctorKind[nameof(CtorKind.Reg32)], code[nameof(Code.Umwait_r64)], "umwait", instrOpInfoFlags[nameof(InstrOpInfoFlags.None)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clflush_m8)], "clflush" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Clflushopt_m8)], "clflushopt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfence)], "lfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfence_E9)], "lfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfence_EA)], "lfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfence_EB)], "lfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfence_EC)], "lfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfence_ED)], "lfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfence_EE)], "lfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfence_EF)], "lfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mfence)], "mfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mfence_F1)], "mfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mfence_F2)], "mfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mfence_F3)], "mfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mfence_F4)], "mfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mfence_F5)], "mfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mfence_F6)], "mfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mfence_F7)], "mfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sfence)], "sfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sfence_F9)], "sfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sfence_FA)], "sfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sfence_FB)], "sfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sfence_FC)], "sfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sfence_FD)], "sfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sfence_FE)], "sfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sfence_FF)], "sfence" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcommit)], "pcommit" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Imul_r16_rm16)], "imul" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Imul_r32_rm32)], "imul" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Imul_r64_rm64)], "imul" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmpxchg_rm8_r8)], "cmpxchg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmpxchg_rm16_r16)], "cmpxchg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmpxchg_rm32_r32)], "cmpxchg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cmpxchg_rm64_r64)], "cmpxchg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lss_r16_m1616)], "lss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lss_r32_m1632)], "lss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lss_r64_m1664)], "lss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Btr_rm16_r16)], "btr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Btr_rm32_r32)], "btr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Btr_rm64_r64)], "btr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfs_r16_m1616)], "lfs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfs_r32_m1632)], "lfs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lfs_r64_m1664)], "lfs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lgs_r16_m1616)], "lgs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lgs_r32_m1632)], "lgs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lgs_r64_m1664)], "lgs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movzx_r16_rm8)], "movzx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movzx_r32_rm8)], "movzx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movzx_r64_rm8)], "movzx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movzx_r16_rm16)], "movzx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movzx_r32_rm16)], "movzx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movzx_r64_rm16)], "movzx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jmpe_disp16)], "jmpe" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Jmpe_disp32)], "jmpe" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Popcnt_r16_rm16)], "popcnt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Popcnt_r32_rm32)], "popcnt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Popcnt_r64_rm64)], "popcnt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ud1_r16_rm16)], "ud1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ud1_r32_rm32)], "ud1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ud1_r64_rm64)], "ud1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bt_rm16_imm8)], "bt", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bt_rm32_imm8)], "bt", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bt_rm64_imm8)], "bt", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bts_rm16_imm8)], "bts", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bts_rm32_imm8)], "bts", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Bts_rm64_imm8)], "bts", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Btr_rm16_imm8)], "btr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Btr_rm32_imm8)], "btr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Btr_rm64_imm8)], "btr", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Btc_rm16_imm8)], "btc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Btc_rm32_imm8)], "btc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Btc_rm64_imm8)], "btc", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Btc_rm16_r16)], "btc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Btc_rm32_r32)], "btc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Btc_rm64_r64)], "btc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bsf_r16_rm16)], "bsf" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bsf_r32_rm32)], "bsf" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bsf_r64_rm64)], "bsf" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Tzcnt_r16_rm16)], "tzcnt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Tzcnt_r32_rm32)], "tzcnt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Tzcnt_r64_rm64)], "tzcnt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bsr_r16_rm16)], "bsr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bsr_r32_rm32)], "bsr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bsr_r64_rm64)], "bsr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lzcnt_r16_rm16)], "lzcnt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lzcnt_r32_rm32)], "lzcnt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Lzcnt_r64_rm64)], "lzcnt" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsx_r16_rm8)], "movsx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsx_r32_rm8)], "movsx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsx_r64_rm8)], "movsx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsx_r16_rm16)], "movsx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsx_r32_rm16)], "movsx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movsx_r64_rm16)], "movsx", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xadd_rm8_r8)], "xadd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xadd_rm16_r16)], "xadd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xadd_rm32_r32)], "xadd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xadd_rm64_r64)], "xadd" },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.Cmpps_xmm_xmmm128_imm8)], "cmpps", pseudoOpsKind[nameof(PseudoOpsKind.cmpps)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8)], "vcmpps", pseudoOpsKind[nameof(PseudoOpsKind.vcmpps)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8)], "vcmpps", pseudoOpsKind[nameof(PseudoOpsKind.vcmpps)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8)], "vcmpps", pseudoOpsKind[nameof(PseudoOpsKind.vcmpps)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8)], "vcmpps", pseudoOpsKind[nameof(PseudoOpsKind.vcmpps)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae)], "vcmpps", pseudoOpsKind[nameof(PseudoOpsKind.vcmpps)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.Cmppd_xmm_xmmm128_imm8)], "cmppd", pseudoOpsKind[nameof(PseudoOpsKind.cmppd)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8)], "vcmppd", pseudoOpsKind[nameof(PseudoOpsKind.vcmppd)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8)], "vcmppd", pseudoOpsKind[nameof(PseudoOpsKind.vcmppd)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8)], "vcmppd", pseudoOpsKind[nameof(PseudoOpsKind.vcmppd)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8)], "vcmppd", pseudoOpsKind[nameof(PseudoOpsKind.vcmppd)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae)], "vcmppd", pseudoOpsKind[nameof(PseudoOpsKind.vcmppd)] },
				new object[] { ctorKind[nameof(CtorKind.pops_3)], code[nameof(Code.Cmpss_xmm_xmmm32_imm8)], "cmpss", pseudoOpsKind[nameof(PseudoOpsKind.cmpss)], instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.pops_3)], code[nameof(Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8)], "vcmpss", pseudoOpsKind[nameof(PseudoOpsKind.vcmpss)], instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.pops_3)], code[nameof(Code.EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae)], "vcmpss", pseudoOpsKind[nameof(PseudoOpsKind.vcmpss)], instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.pops_3)], code[nameof(Code.Cmpsd_xmm_xmmm64_imm8)], "cmpsd", pseudoOpsKind[nameof(PseudoOpsKind.cmpsd)], instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.pops_3)], code[nameof(Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8)], "vcmpsd", pseudoOpsKind[nameof(PseudoOpsKind.vcmpsd)], instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.pops_3)], code[nameof(Code.EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae)], "vcmpsd", pseudoOpsKind[nameof(PseudoOpsKind.vcmpsd)], instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movnti_m32_r32)], "movnti" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movnti_m64_r64)], "movnti" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_2)], code[nameof(Code.Pinsrw_mm_r32m16_imm8)], "pinsrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_2)], code[nameof(Code.Pinsrw_mm_r64m16_imm8)], "pinsrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pinsrw_xmm_r32m16_imm8)], "pinsrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pinsrw_xmm_r64m16_imm8)], "pinsrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r32m16_imm8)], "vpinsrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg32)], code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r64m16_imm8)], "vpinsrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r32m16_imm8)], "vpinsrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg32)], code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r64m16_imm8)], "vpinsrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pextrw_r32_mm_imm8)], "pextrw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pextrw_r64_mm_imm8)], "pextrw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pextrw_r32_xmm_imm8)], "pextrw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pextrw_r64_xmm_imm8)], "pextrw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpextrw_r32_xmm_imm8)], "vpextrw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpextrw_r64_xmm_imm8)], "vpextrw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpextrw_r32_xmm_imm8)], "vpextrw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpextrw_r64_xmm_imm8)], "vpextrw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shufps_xmm_xmmm128_imm8)], "shufps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vshufps_xmm_xmm_xmmm128_imm8)], "vshufps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vshufps_ymm_ymm_ymmm256_imm8)], "vshufps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8)], "vshufps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8)], "vshufps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8)], "vshufps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Shufpd_xmm_xmmm128_imm8)], "shufpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vshufpd_xmm_xmm_xmmm128_imm8)], "vshufpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vshufpd_ymm_ymm_ymmm256_imm8)], "vshufpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8)], "vshufpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8)], "vshufpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8)], "vshufpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmpxchg8b_m64)], "cmpxchg8b", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cmpxchg16b_m128)], "cmpxchg16b", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Normal)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xrstors_mem)], "xrstors" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xrstors64_mem)], "xrstors64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsavec_mem)], "xsavec" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsavec64_mem)], "xsavec64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsaves_mem)], "xsaves" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsaves64_mem)], "xsaves64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Vmptrld_m64)], "vmptrld", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Vmclear_m64)], "vmclear", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Vmxon_m64)], "vmxon", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdrand_r16)], "rdrand" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdrand_r32)], "rdrand" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdrand_r64)], "rdrand" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Vmptrst_m64)], "vmptrst", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdseed_r16)], "rdseed" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdseed_r32)], "rdseed" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdseed_r64)], "rdseed" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdpid_r32)], "rdpid" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rdpid_r64)], "rdpid" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bswap_r16)], "bswap" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bswap_r32)], "bswap" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Bswap_r64)], "bswap" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Addsubpd_xmm_xmmm128)], "addsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaddsubpd_xmm_xmm_xmmm128)], "vaddsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaddsubpd_ymm_ymm_ymmm256)], "vaddsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Addsubps_xmm_xmmm128)], "addsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaddsubps_xmm_xmm_xmmm128)], "vaddsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaddsubps_ymm_ymm_ymmm256)], "vaddsubps" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psrlw_mm_mmm64)], "psrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrlw_xmm_xmmm128)], "psrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlw_xmm_xmm_xmmm128)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlw_ymm_ymm_xmmm128)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128)], "vpsrlw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psrld_mm_mmm64)], "psrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrld_xmm_xmmm128)], "psrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrld_xmm_xmm_xmmm128)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrld_ymm_ymm_xmmm128)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmm_xmmm128)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymm_xmmm128)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmm_xmmm128)], "vpsrld" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psrlq_mm_mmm64)], "psrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrlq_xmm_xmmm128)], "psrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlq_xmm_xmm_xmmm128)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlq_ymm_ymm_xmmm128)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128)], "vpsrlq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Paddq_mm_mmm64)], "paddq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Paddq_xmm_xmmm128)], "paddq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddq_xmm_xmm_xmmm128)], "vpaddq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddq_ymm_ymm_ymmm256)], "vpaddq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64)], "vpaddq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64)], "vpaddq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64)], "vpaddq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pmullw_mm_mmm64)], "pmullw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmullw_xmm_xmmm128)], "pmullw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmullw_xmm_xmm_xmmm128)], "vpmullw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmullw_ymm_ymm_ymmm256)], "vpmullw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmullw_xmm_k1z_xmm_xmmm128)], "vpmullw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmullw_ymm_k1z_ymm_ymmm256)], "vpmullw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmullw_zmm_k1z_zmm_zmmm512)], "vpmullw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movq_xmmm64_xmm)], "movq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovq_xmmm64_xmm)], "vmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovq_xmmm64_xmm)], "vmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movq2dq_xmm_mm)], "movq2dq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Movdq2q_mm_xmm)], "movdq2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovmskb_r32_mm)], "pmovmskb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovmskb_r64_mm)], "pmovmskb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovmskb_r32_xmm)], "pmovmskb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovmskb_r64_xmm)], "pmovmskb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmovmskb_r32_xmm)], "vpmovmskb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmovmskb_r64_xmm)], "vpmovmskb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmovmskb_r32_ymm)], "vpmovmskb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmovmskb_r64_ymm)], "vpmovmskb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psubusb_mm_mmm64)], "psubusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psubusb_xmm_xmmm128)], "psubusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubusb_xmm_xmm_xmmm128)], "vpsubusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubusb_ymm_ymm_ymmm256)], "vpsubusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128)], "vpsubusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256)], "vpsubusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512)], "vpsubusb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psubusw_mm_mmm64)], "psubusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psubusw_xmm_xmmm128)], "psubusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubusw_xmm_xmm_xmmm128)], "vpsubusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubusw_ymm_ymm_ymmm256)], "vpsubusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128)], "vpsubusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256)], "vpsubusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512)], "vpsubusw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pminub_mm_mmm64)], "pminub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pminub_xmm_xmmm128)], "pminub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminub_xmm_xmm_xmmm128)], "vpminub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminub_ymm_ymm_ymmm256)], "vpminub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminub_xmm_k1z_xmm_xmmm128)], "vpminub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminub_ymm_k1z_ymm_ymmm256)], "vpminub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminub_zmm_k1z_zmm_zmmm512)], "vpminub" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pand_mm_mmm64)], "pand" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pand_xmm_xmmm128)], "pand" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpand_xmm_xmm_xmmm128)], "vpand" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpand_ymm_ymm_ymmm256)], "vpand" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32)], "vpandd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32)], "vpandd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32)], "vpandd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64)], "vpandq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64)], "vpandq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64)], "vpandq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Paddusb_mm_mmm64)], "paddusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Paddusb_xmm_xmmm128)], "paddusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddusb_xmm_xmm_xmmm128)], "vpaddusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddusb_ymm_ymm_ymmm256)], "vpaddusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128)], "vpaddusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256)], "vpaddusb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512)], "vpaddusb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Paddusw_mm_mmm64)], "paddusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Paddusw_xmm_xmmm128)], "paddusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddusw_xmm_xmm_xmmm128)], "vpaddusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddusw_ymm_ymm_ymmm256)], "vpaddusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128)], "vpaddusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256)], "vpaddusw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512)], "vpaddusw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pmaxub_mm_mmm64)], "pmaxub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmaxub_xmm_xmmm128)], "pmaxub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxub_xmm_xmm_xmmm128)], "vpmaxub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxub_ymm_ymm_ymmm256)], "vpmaxub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128)], "vpmaxub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256)], "vpmaxub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512)], "vpmaxub" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pandn_mm_mmm64)], "pandn" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pandn_xmm_xmmm128)], "pandn" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpandn_xmm_xmm_xmmm128)], "vpandn" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpandn_ymm_ymm_ymmm256)], "vpandn" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32)], "vpandnd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32)], "vpandnd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32)], "vpandnd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64)], "vpandnq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64)], "vpandnq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64)], "vpandnq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pavgb_mm_mmm64)], "pavgb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pavgb_xmm_xmmm128)], "pavgb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpavgb_xmm_xmm_xmmm128)], "vpavgb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpavgb_ymm_ymm_ymmm256)], "vpavgb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpavgb_xmm_k1z_xmm_xmmm128)], "vpavgb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpavgb_ymm_k1z_ymm_ymmm256)], "vpavgb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpavgb_zmm_k1z_zmm_zmmm512)], "vpavgb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psraw_mm_mmm64)], "psraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psraw_xmm_xmmm128)], "psraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsraw_xmm_xmm_xmmm128)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsraw_ymm_ymm_xmmm128)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmm_xmmm128)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymm_xmmm128)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmm_xmmm128)], "vpsraw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psrad_mm_mmm64)], "psrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psrad_xmm_xmmm128)], "psrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrad_xmm_xmm_xmmm128)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrad_ymm_ymm_xmmm128)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmm_xmmm128)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymm_xmmm128)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmm_xmmm128)], "vpsrad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmm_xmmm128)], "vpsraq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymm_xmmm128)], "vpsraq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmm_xmmm128)], "vpsraq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pavgw_mm_mmm64)], "pavgw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pavgw_xmm_xmmm128)], "pavgw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpavgw_xmm_xmm_xmmm128)], "vpavgw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpavgw_ymm_ymm_ymmm256)], "vpavgw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpavgw_xmm_k1z_xmm_xmmm128)], "vpavgw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpavgw_ymm_k1z_ymm_ymmm256)], "vpavgw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpavgw_zmm_k1z_zmm_zmmm512)], "vpavgw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pmulhuw_mm_mmm64)], "pmulhuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmulhuw_xmm_xmmm128)], "pmulhuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmulhuw_xmm_xmm_xmmm128)], "vpmulhuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmulhuw_ymm_ymm_ymmm256)], "vpmulhuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128)], "vpmulhuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256)], "vpmulhuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512)], "vpmulhuw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pmulhw_mm_mmm64)], "pmulhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmulhw_xmm_xmmm128)], "pmulhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmulhw_xmm_xmm_xmmm128)], "vpmulhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmulhw_ymm_ymm_ymmm256)], "vpmulhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128)], "vpmulhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256)], "vpmulhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512)], "vpmulhw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cvttpd2dq_xmm_xmmm128)], "cvttpd2dq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvttpd2dq_xmm_xmmm128)], "vcvttpd2dq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvttpd2dq_xmm_ymmm256)], "vcvttpd2dq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64)], "vcvttpd2dq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64)], "vcvttpd2dq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae)], "vcvttpd2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Cvtdq2pd_xmm_xmmm64)], "cvtdq2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtdq2pd_xmm_xmmm64)], "vcvtdq2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtdq2pd_ymm_xmmm128)], "vcvtdq2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32)], "vcvtdq2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32)], "vcvtdq2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er)], "vcvtdq2pd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64)], "vcvtqq2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64)], "vcvtqq2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er)], "vcvtqq2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Cvtpd2dq_xmm_xmmm128)], "cvtpd2dq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtpd2dq_xmm_xmmm128)], "vcvtpd2dq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtpd2dq_xmm_ymmm256)], "vcvtpd2dq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64)], "vcvtpd2dq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64)], "vcvtpd2dq", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er)], "vcvtpd2dq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Movntq_m64_mm)], "movntq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movntdq_m128_xmm)], "movntdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovntdq_m128_xmm)], "vmovntdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovntdq_m256_ymm)], "vmovntdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntdq_m128_xmm)], "vmovntdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntdq_m256_ymm)], "vmovntdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntdq_m512_zmm)], "vmovntdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psubsb_mm_mmm64)], "psubsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psubsb_xmm_xmmm128)], "psubsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubsb_xmm_xmm_xmmm128)], "vpsubsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubsb_ymm_ymm_ymmm256)], "vpsubsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128)], "vpsubsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256)], "vpsubsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512)], "vpsubsb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psubsw_mm_mmm64)], "psubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psubsw_xmm_xmmm128)], "psubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubsw_xmm_xmm_xmmm128)], "vpsubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubsw_ymm_ymm_ymmm256)], "vpsubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128)], "vpsubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256)], "vpsubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512)], "vpsubsw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pminsw_mm_mmm64)], "pminsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pminsw_xmm_xmmm128)], "pminsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminsw_xmm_xmm_xmmm128)], "vpminsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminsw_ymm_ymm_ymmm256)], "vpminsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsw_xmm_k1z_xmm_xmmm128)], "vpminsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsw_ymm_k1z_ymm_ymmm256)], "vpminsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsw_zmm_k1z_zmm_zmmm512)], "vpminsw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Por_mm_mmm64)], "por" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Por_xmm_xmmm128)], "por" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpor_xmm_xmm_xmmm128)], "vpor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpor_ymm_ymm_ymmm256)], "vpor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpord_xmm_k1z_xmm_xmmm128b32)], "vpord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpord_ymm_k1z_ymm_ymmm256b32)], "vpord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpord_zmm_k1z_zmm_zmmm512b32)], "vpord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vporq_xmm_k1z_xmm_xmmm128b64)], "vporq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vporq_ymm_k1z_ymm_ymmm256b64)], "vporq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vporq_zmm_k1z_zmm_zmmm512b64)], "vporq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Paddsb_mm_mmm64)], "paddsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Paddsb_xmm_xmmm128)], "paddsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddsb_xmm_xmm_xmmm128)], "vpaddsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddsb_ymm_ymm_ymmm256)], "vpaddsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128)], "vpaddsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256)], "vpaddsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512)], "vpaddsb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Paddsw_mm_mmm64)], "paddsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Paddsw_xmm_xmmm128)], "paddsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddsw_xmm_xmm_xmmm128)], "vpaddsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddsw_ymm_ymm_ymmm256)], "vpaddsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128)], "vpaddsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256)], "vpaddsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512)], "vpaddsw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pmaxsw_mm_mmm64)], "pmaxsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmaxsw_xmm_xmmm128)], "pmaxsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxsw_xmm_xmm_xmmm128)], "vpmaxsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxsw_ymm_ymm_ymmm256)], "vpmaxsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128)], "vpmaxsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256)], "vpmaxsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512)], "vpmaxsw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pxor_mm_mmm64)], "pxor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pxor_xmm_xmmm128)], "pxor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpxor_xmm_xmm_xmmm128)], "vpxor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpxor_ymm_ymm_ymmm256)], "vpxor" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32)], "vpxord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32)], "vpxord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32)], "vpxord" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64)], "vpxorq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64)], "vpxorq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64)], "vpxorq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Lddqu_xmm_m128)], "lddqu", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vlddqu_xmm_m128)], "vlddqu", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vlddqu_ymm_m256)], "vlddqu", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psllw_mm_mmm64)], "psllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psllw_xmm_xmmm128)], "psllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllw_xmm_xmm_xmmm128)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllw_ymm_ymm_xmmm128)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmm_xmmm128)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymm_xmmm128)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmm_xmmm128)], "vpsllw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pslld_mm_mmm64)], "pslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pslld_xmm_xmmm128)], "pslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpslld_xmm_xmm_xmmm128)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpslld_ymm_ymm_xmmm128)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmm_xmmm128)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymm_xmmm128)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmm_xmmm128)], "vpslld" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psllq_mm_mmm64)], "psllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psllq_xmm_xmmm128)], "psllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllq_xmm_xmm_xmmm128)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllq_ymm_ymm_xmmm128)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmm_xmmm128)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymm_xmmm128)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmm_xmmm128)], "vpsllq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pmuludq_mm_mmm64)], "pmuludq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmuludq_xmm_xmmm128)], "pmuludq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmuludq_xmm_xmm_xmmm128)], "vpmuludq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmuludq_ymm_ymm_ymmm256)], "vpmuludq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64)], "vpmuludq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64)], "vpmuludq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64)], "vpmuludq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pmaddwd_mm_mmm64)], "pmaddwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmaddwd_xmm_xmmm128)], "pmaddwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaddwd_xmm_xmm_xmmm128)], "vpmaddwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaddwd_ymm_ymm_ymmm256)], "vpmaddwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128)], "vpmaddwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256)], "vpmaddwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512)], "vpmaddwd" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psadbw_mm_mmm64)], "psadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psadbw_xmm_xmmm128)], "psadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsadbw_xmm_xmm_xmmm128)], "vpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsadbw_ymm_ymm_ymmm256)], "vpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsadbw_xmm_xmm_xmmm128)], "vpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsadbw_ymm_ymm_ymmm256)], "vpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsadbw_zmm_zmm_zmmm512)], "vpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.maskmovq)], code[nameof(Code.Maskmovq_rDI_mm_mm)], "maskmovq", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.maskmovq)], code[nameof(Code.Maskmovdqu_rDI_xmm_xmm)], "maskmovdqu", instrOpInfoFlags[nameof(InstrOpInfoFlags.None)] },
				new object[] { ctorKind[nameof(CtorKind.maskmovq)], code[nameof(Code.VEX_Vmaskmovdqu_rDI_xmm_xmm)], "vmaskmovdqu", instrOpInfoFlags[nameof(InstrOpInfoFlags.None)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psubb_mm_mmm64)], "psubb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psubb_xmm_xmmm128)], "psubb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubb_xmm_xmm_xmmm128)], "vpsubb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubb_ymm_ymm_ymmm256)], "vpsubb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubb_xmm_k1z_xmm_xmmm128)], "vpsubb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubb_ymm_k1z_ymm_ymmm256)], "vpsubb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubb_zmm_k1z_zmm_zmmm512)], "vpsubb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psubw_mm_mmm64)], "psubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psubw_xmm_xmmm128)], "psubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubw_xmm_xmm_xmmm128)], "vpsubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubw_ymm_ymm_ymmm256)], "vpsubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubw_xmm_k1z_xmm_xmmm128)], "vpsubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubw_ymm_k1z_ymm_ymmm256)], "vpsubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubw_zmm_k1z_zmm_zmmm512)], "vpsubw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psubd_mm_mmm64)], "psubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psubd_xmm_xmmm128)], "psubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubd_xmm_xmm_xmmm128)], "vpsubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubd_ymm_ymm_ymmm256)], "vpsubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32)], "vpsubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32)], "vpsubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32)], "vpsubd" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psubq_mm_mmm64)], "psubq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psubq_xmm_xmmm128)], "psubq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubq_xmm_xmm_xmmm128)], "vpsubq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsubq_ymm_ymm_ymmm256)], "vpsubq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64)], "vpsubq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64)], "vpsubq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64)], "vpsubq" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Paddb_mm_mmm64)], "paddb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Paddb_xmm_xmmm128)], "paddb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddb_xmm_xmm_xmmm128)], "vpaddb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddb_ymm_ymm_ymmm256)], "vpaddb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddb_xmm_k1z_xmm_xmmm128)], "vpaddb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddb_ymm_k1z_ymm_ymmm256)], "vpaddb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddb_zmm_k1z_zmm_zmmm512)], "vpaddb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Paddw_mm_mmm64)], "paddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Paddw_xmm_xmmm128)], "paddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddw_xmm_xmm_xmmm128)], "vpaddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddw_ymm_ymm_ymmm256)], "vpaddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddw_xmm_k1z_xmm_xmmm128)], "vpaddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddw_ymm_k1z_ymm_ymmm256)], "vpaddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddw_zmm_k1z_zmm_zmmm512)], "vpaddw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Paddd_mm_mmm64)], "paddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Paddd_xmm_xmmm128)], "paddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddd_xmm_xmm_xmmm128)], "vpaddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpaddd_ymm_ymm_ymmm256)], "vpaddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32)], "vpaddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32)], "vpaddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32)], "vpaddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ud0_r16_rm16)], "ud0" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ud0_r32_rm32)], "ud0" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ud0_r64_rm64)], "ud0" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pshufb_mm_mmm64)], "pshufb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pshufb_xmm_xmmm128)], "pshufb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpshufb_xmm_xmm_xmmm128)], "vpshufb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpshufb_ymm_ymm_ymmm256)], "vpshufb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufb_xmm_k1z_xmm_xmmm128)], "vpshufb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufb_ymm_k1z_ymm_ymmm256)], "vpshufb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufb_zmm_k1z_zmm_zmmm512)], "vpshufb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Phaddw_mm_mmm64)], "phaddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Phaddw_xmm_xmmm128)], "phaddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphaddw_xmm_xmm_xmmm128)], "vphaddw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphaddw_ymm_ymm_ymmm256)], "vphaddw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Phaddd_mm_mmm64)], "phaddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Phaddd_xmm_xmmm128)], "phaddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphaddd_xmm_xmm_xmmm128)], "vphaddd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphaddd_ymm_ymm_ymmm256)], "vphaddd" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Phaddsw_mm_mmm64)], "phaddsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Phaddsw_xmm_xmmm128)], "phaddsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphaddsw_xmm_xmm_xmmm128)], "vphaddsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphaddsw_ymm_ymm_ymmm256)], "vphaddsw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pmaddubsw_mm_mmm64)], "pmaddubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmaddubsw_xmm_xmmm128)], "pmaddubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaddubsw_xmm_xmm_xmmm128)], "vpmaddubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaddubsw_ymm_ymm_ymmm256)], "vpmaddubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128)], "vpmaddubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256)], "vpmaddubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512)], "vpmaddubsw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Phsubw_mm_mmm64)], "phsubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Phsubw_xmm_xmmm128)], "phsubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphsubw_xmm_xmm_xmmm128)], "vphsubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphsubw_ymm_ymm_ymmm256)], "vphsubw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Phsubd_mm_mmm64)], "phsubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Phsubd_xmm_xmmm128)], "phsubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphsubd_xmm_xmm_xmmm128)], "vphsubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphsubd_ymm_ymm_ymmm256)], "vphsubd" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Phsubsw_mm_mmm64)], "phsubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Phsubsw_xmm_xmmm128)], "phsubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphsubsw_xmm_xmm_xmmm128)], "vphsubsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphsubsw_ymm_ymm_ymmm256)], "vphsubsw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psignb_mm_mmm64)], "psignb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psignb_xmm_xmmm128)], "psignb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsignb_xmm_xmm_xmmm128)], "vpsignb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsignb_ymm_ymm_ymmm256)], "vpsignb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psignw_mm_mmm64)], "psignw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psignw_xmm_xmmm128)], "psignw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsignw_xmm_xmm_xmmm128)], "vpsignw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsignw_ymm_ymm_ymmm256)], "vpsignw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Psignd_mm_mmm64)], "psignd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psignd_xmm_xmmm128)], "psignd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsignd_xmm_xmm_xmmm128)], "vpsignd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsignd_ymm_ymm_ymmm256)], "vpsignd" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pmulhrsw_mm_mmm64)], "pmulhrsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmulhrsw_xmm_xmmm128)], "pmulhrsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmulhrsw_xmm_xmm_xmmm128)], "vpmulhrsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmulhrsw_ymm_ymm_ymmm256)], "vpmulhrsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128)], "vpmulhrsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256)], "vpmulhrsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512)], "vpmulhrsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermilps_xmm_xmm_xmmm128)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermilps_ymm_ymm_ymmm256)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermilpd_xmm_xmm_xmmm128)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermilpd_ymm_ymm_ymmm256)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vtestps_xmm_xmmm128)], "vtestps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vtestps_ymm_ymmm256)], "vtestps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vtestpd_xmm_xmmm128)], "vtestpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vtestpd_ymm_ymmm256)], "vtestpd" },
				new object[] { ctorKind[nameof(CtorKind.pblendvb)], code[nameof(Code.Pblendvb_xmm_xmmm128)], "pblendvb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128)], "vpsrlvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256)], "vpsrlvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512)], "vpsrlvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovuswb_xmmm64_k1z_xmm)], "vpmovuswb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovuswb_xmmm128_k1z_ymm)], "vpmovuswb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovuswb_ymmm256_k1z_zmm)], "vpmovuswb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsravw_xmm_k1z_xmm_xmmm128)], "vpsravw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsravw_ymm_k1z_ymm_ymmm256)], "vpsravw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsravw_zmm_k1z_zmm_zmmm512)], "vpsravw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusdb_xmmm32_k1z_xmm)], "vpmovusdb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusdb_xmmm64_k1z_ymm)], "vpmovusdb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusdb_xmmm128_k1z_zmm)], "vpmovusdb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128)], "vpsllvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256)], "vpsllvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512)], "vpsllvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusqb_xmmm16_k1z_xmm)], "vpmovusqb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusqb_xmmm32_k1z_ymm)], "vpmovusqb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusqb_xmmm64_k1z_zmm)], "vpmovusqb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtph2ps_xmm_xmmm64)], "vcvtph2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtph2ps_ymm_xmmm128)], "vcvtph2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtph2ps_xmm_k1z_xmmm64)], "vcvtph2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtph2ps_ymm_k1z_xmmm128)], "vcvtph2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae)], "vcvtph2ps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusdw_xmmm64_k1z_xmm)], "vpmovusdw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusdw_xmmm128_k1z_ymm)], "vpmovusdw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusdw_ymmm256_k1z_zmm)], "vpmovusdw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.pblendvb)], code[nameof(Code.Blendvps_xmm_xmmm128)], "blendvps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32)], "vprorvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32)], "vprorvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32)], "vprorvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64)], "vprorvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64)], "vprorvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64)], "vprorvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusqw_xmmm32_k1z_xmm)], "vpmovusqw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusqw_xmmm64_k1z_ymm)], "vpmovusqw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusqw_xmmm128_k1z_zmm)], "vpmovusqw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.pblendvb)], code[nameof(Code.Blendvpd_xmm_xmmm128)], "blendvpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32)], "vprolvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32)], "vprolvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32)], "vprolvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64)], "vprolvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64)], "vprolvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64)], "vprolvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusqd_xmmm64_k1z_xmm)], "vpmovusqd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusqd_xmmm128_k1z_ymm)], "vpmovusqd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovusqd_ymmm256_k1z_zmm)], "vpmovusqd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermps_ymm_ymm_ymmm256)], "vpermps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32)], "vpermps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32)], "vpermps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64)], "vpermpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64)], "vpermpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Ptest_xmm_xmmm128)], "ptest", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vptest_xmm_xmmm128)], "vptest", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vptest_ymm_ymmm256)], "vptest", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vbroadcastss_xmm_xmmm32)], "vbroadcastss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vbroadcastss_ymm_xmmm32)], "vbroadcastss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastss_xmm_k1z_xmmm32)], "vbroadcastss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastss_ymm_k1z_xmmm32)], "vbroadcastss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastss_zmm_k1z_xmmm32)], "vbroadcastss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vbroadcastsd_ymm_xmmm64)], "vbroadcastsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64)], "vbroadcastf32x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64)], "vbroadcastf32x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastsd_ymm_k1z_xmmm64)], "vbroadcastsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastsd_zmm_k1z_xmmm64)], "vbroadcastsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vbroadcastf128_ymm_m128)], "vbroadcastf128", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastf32x4_ymm_k1z_m128)], "vbroadcastf32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastf32x4_zmm_k1z_m128)], "vbroadcastf32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastf64x2_ymm_k1z_m128)], "vbroadcastf64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastf64x2_zmm_k1z_m128)], "vbroadcastf64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastf32x8_zmm_k1z_m256)], "vbroadcastf32x8", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcastf64x4_zmm_k1z_m256)], "vbroadcastf64x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pabsb_mm_mmm64)], "pabsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pabsb_xmm_xmmm128)], "pabsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpabsb_xmm_xmmm128)], "vpabsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpabsb_ymm_ymmm256)], "vpabsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsb_xmm_k1z_xmmm128)], "vpabsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsb_ymm_k1z_ymmm256)], "vpabsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsb_zmm_k1z_zmmm512)], "vpabsb" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pabsw_mm_mmm64)], "pabsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pabsw_xmm_xmmm128)], "pabsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpabsw_xmm_xmmm128)], "vpabsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpabsw_ymm_ymmm256)], "vpabsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsw_xmm_k1z_xmmm128)], "vpabsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsw_ymm_k1z_ymmm256)], "vpabsw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsw_zmm_k1z_zmmm512)], "vpabsw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Pabsd_mm_mmm64)], "pabsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pabsd_xmm_xmmm128)], "pabsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpabsd_xmm_xmmm128)], "vpabsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpabsd_ymm_ymmm256)], "vpabsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsd_xmm_k1z_xmmm128b32)], "vpabsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsd_ymm_k1z_ymmm256b32)], "vpabsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsd_zmm_k1z_zmmm512b32)], "vpabsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsq_xmm_k1z_xmmm128b64)], "vpabsq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsq_ymm_k1z_ymmm256b64)], "vpabsq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpabsq_zmm_k1z_zmmm512b64)], "vpabsq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovsxbw_xmm_xmmm64)], "pmovsxbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxbw_xmm_xmmm64)], "vpmovsxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxbw_ymm_xmmm128)], "vpmovsxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxbw_xmm_k1z_xmmm64)], "vpmovsxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxbw_ymm_k1z_xmmm128)], "vpmovsxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxbw_zmm_k1z_ymmm256)], "vpmovsxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovswb_xmmm64_k1z_xmm)], "vpmovswb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovswb_xmmm128_k1z_ymm)], "vpmovswb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovswb_ymmm256_k1z_zmm)], "vpmovswb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovsxbd_xmm_xmmm32)], "pmovsxbd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxbd_xmm_xmmm32)], "vpmovsxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxbd_ymm_xmmm64)], "vpmovsxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxbd_xmm_k1z_xmmm32)], "vpmovsxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxbd_ymm_k1z_xmmm64)], "vpmovsxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxbd_zmm_k1z_xmmm128)], "vpmovsxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsdb_xmmm32_k1z_xmm)], "vpmovsdb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsdb_xmmm64_k1z_ymm)], "vpmovsdb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsdb_xmmm128_k1z_zmm)], "vpmovsdb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovsxbq_xmm_xmmm16)], "pmovsxbq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxbq_xmm_xmmm16)], "vpmovsxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxbq_ymm_xmmm32)], "vpmovsxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxbq_xmm_k1z_xmmm16)], "vpmovsxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxbq_ymm_k1z_xmmm32)], "vpmovsxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxbq_zmm_k1z_xmmm64)], "vpmovsxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsqb_xmmm16_k1z_xmm)], "vpmovsqb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsqb_xmmm32_k1z_ymm)], "vpmovsqb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsqb_xmmm64_k1z_zmm)], "vpmovsqb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovsxwd_xmm_xmmm64)], "pmovsxwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxwd_xmm_xmmm64)], "vpmovsxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxwd_ymm_xmmm128)], "vpmovsxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxwd_xmm_k1z_xmmm64)], "vpmovsxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxwd_ymm_k1z_xmmm128)], "vpmovsxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxwd_zmm_k1z_ymmm256)], "vpmovsxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsdw_xmmm64_k1z_xmm)], "vpmovsdw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsdw_xmmm128_k1z_ymm)], "vpmovsdw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsdw_ymmm256_k1z_zmm)], "vpmovsdw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovsxwq_xmm_xmmm32)], "pmovsxwq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxwq_xmm_xmmm32)], "vpmovsxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxwq_ymm_xmmm64)], "vpmovsxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxwq_xmm_k1z_xmmm32)], "vpmovsxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxwq_ymm_k1z_xmmm64)], "vpmovsxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxwq_zmm_k1z_xmmm128)], "vpmovsxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsqw_xmmm32_k1z_xmm)], "vpmovsqw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsqw_xmmm64_k1z_ymm)], "vpmovsqw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsqw_xmmm128_k1z_zmm)], "vpmovsqw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovsxdq_xmm_xmmm64)], "pmovsxdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxdq_xmm_xmmm64)], "vpmovsxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovsxdq_ymm_xmmm128)], "vpmovsxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxdq_xmm_k1z_xmmm64)], "vpmovsxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxdq_ymm_k1z_xmmm128)], "vpmovsxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsxdq_zmm_k1z_ymmm256)], "vpmovsxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsqd_xmmm64_k1z_xmm)], "vpmovsqd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsqd_xmmm128_k1z_ymm)], "vpmovsqd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovsqd_ymmm256_k1z_zmm)], "vpmovsqd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmb_kr_k1_xmm_xmmm128)], "vptestmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmb_kr_k1_ymm_ymmm256)], "vptestmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmb_kr_k1_zmm_zmmm512)], "vptestmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmw_kr_k1_xmm_xmmm128)], "vptestmw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmw_kr_k1_ymm_ymmm256)], "vptestmw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmw_kr_k1_zmm_zmmm512)], "vptestmw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmb_kr_k1_xmm_xmmm128)], "vptestnmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmb_kr_k1_ymm_ymmm256)], "vptestnmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmb_kr_k1_zmm_zmmm512)], "vptestnmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmw_kr_k1_xmm_xmmm128)], "vptestnmw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmw_kr_k1_ymm_ymmm256)], "vptestnmw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmw_kr_k1_zmm_zmmm512)], "vptestnmw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmd_kr_k1_xmm_xmmm128b32)], "vptestmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmd_kr_k1_ymm_ymmm256b32)], "vptestmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmd_kr_k1_zmm_zmmm512b32)], "vptestmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmq_kr_k1_xmm_xmmm128b64)], "vptestmq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmq_kr_k1_ymm_ymmm256b64)], "vptestmq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestmq_kr_k1_zmm_zmmm512b64)], "vptestmq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32)], "vptestnmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32)], "vptestnmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32)], "vptestnmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64)], "vptestnmq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64)], "vptestnmq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64)], "vptestnmq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmuldq_xmm_xmmm128)], "pmuldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmuldq_xmm_xmm_xmmm128)], "vpmuldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmuldq_ymm_ymm_ymmm256)], "vpmuldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64)], "vpmuldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64)], "vpmuldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64)], "vpmuldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2b_xmm_kr)], "vpmovm2b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2b_ymm_kr)], "vpmovm2b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2b_zmm_kr)], "vpmovm2b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2w_xmm_kr)], "vpmovm2w" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2w_ymm_kr)], "vpmovm2w" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2w_zmm_kr)], "vpmovm2w" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpeqq_xmm_xmmm128)], "pcmpeqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpeqq_xmm_xmm_xmmm128)], "vpcmpeqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpeqq_ymm_ymm_ymmm256)], "vpcmpeqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64)], "vpcmpeqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64)], "vpcmpeqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64)], "vpcmpeqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovb2m_kr_xmm)], "vpmovb2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovb2m_kr_ymm)], "vpmovb2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovb2m_kr_zmm)], "vpmovb2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovw2m_kr_xmm)], "vpmovw2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovw2m_kr_ymm)], "vpmovw2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovw2m_kr_zmm)], "vpmovw2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Movntdqa_xmm_m128)], "movntdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovntdqa_xmm_m128)], "vmovntdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vmovntdqa_ymm_m256)], "vmovntdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntdqa_xmm_m128)], "vmovntdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntdqa_ymm_m256)], "vmovntdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vmovntdqa_zmm_m512)], "vmovntdqa", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastmb2q_xmm_kr)], "vpbroadcastmb2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastmb2q_ymm_kr)], "vpbroadcastmb2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastmb2q_zmm_kr)], "vpbroadcastmb2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Packusdw_xmm_xmmm128)], "packusdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpackusdw_xmm_xmm_xmmm128)], "vpackusdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpackusdw_ymm_ymm_ymmm256)], "vpackusdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32)], "vpackusdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32)], "vpackusdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32)], "vpackusdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaskmovps_xmm_xmm_m128)], "vmaskmovps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaskmovps_ymm_ymm_m256)], "vmaskmovps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32)], "vscalefps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32)], "vscalefps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er)], "vscalefps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64)], "vscalefpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64)], "vscalefpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er)], "vscalefpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaskmovpd_xmm_xmm_m128)], "vmaskmovpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaskmovpd_ymm_ymm_m256)], "vmaskmovpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er)], "vscalefss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er)], "vscalefsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaskmovps_m128_xmm_xmm)], "vmaskmovps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaskmovps_m256_ymm_ymm)], "vmaskmovps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaskmovpd_m128_xmm_xmm)], "vmaskmovpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmaskmovpd_m256_ymm_ymm)], "vmaskmovpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovzxbw_xmm_xmmm64)], "pmovzxbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxbw_xmm_xmmm64)], "vpmovzxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxbw_ymm_xmmm128)], "vpmovzxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxbw_xmm_k1z_xmmm64)], "vpmovzxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxbw_ymm_k1z_xmmm128)], "vpmovzxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxbw_zmm_k1z_ymmm256)], "vpmovzxbw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovwb_xmmm64_k1z_xmm)], "vpmovwb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovwb_xmmm128_k1z_ymm)], "vpmovwb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovwb_ymmm256_k1z_zmm)], "vpmovwb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovzxbd_xmm_xmmm32)], "pmovzxbd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxbd_xmm_xmmm32)], "vpmovzxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxbd_ymm_xmmm64)], "vpmovzxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxbd_xmm_k1z_xmmm32)], "vpmovzxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxbd_ymm_k1z_xmmm64)], "vpmovzxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxbd_zmm_k1z_xmmm128)], "vpmovzxbd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovdb_xmmm32_k1z_xmm)], "vpmovdb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovdb_xmmm64_k1z_ymm)], "vpmovdb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovdb_xmmm128_k1z_zmm)], "vpmovdb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovzxbq_xmm_xmmm16)], "pmovzxbq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxbq_xmm_xmmm16)], "vpmovzxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxbq_ymm_xmmm32)], "vpmovzxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxbq_xmm_k1z_xmmm16)], "vpmovzxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxbq_ymm_k1z_xmmm32)], "vpmovzxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxbq_zmm_k1z_xmmm64)], "vpmovzxbq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovqb_xmmm16_k1z_xmm)], "vpmovqb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovqb_xmmm32_k1z_ymm)], "vpmovqb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovqb_xmmm64_k1z_zmm)], "vpmovqb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovzxwd_xmm_xmmm64)], "pmovzxwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxwd_xmm_xmmm64)], "vpmovzxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxwd_ymm_xmmm128)], "vpmovzxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxwd_xmm_k1z_xmmm64)], "vpmovzxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxwd_ymm_k1z_xmmm128)], "vpmovzxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxwd_zmm_k1z_ymmm256)], "vpmovzxwd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovdw_xmmm64_k1z_xmm)], "vpmovdw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovdw_xmmm128_k1z_ymm)], "vpmovdw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovdw_ymmm256_k1z_zmm)], "vpmovdw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovzxwq_xmm_xmmm32)], "pmovzxwq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxwq_xmm_xmmm32)], "vpmovzxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxwq_ymm_xmmm64)], "vpmovzxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxwq_xmm_k1z_xmmm32)], "vpmovzxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxwq_ymm_k1z_xmmm64)], "vpmovzxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxwq_zmm_k1z_xmmm128)], "vpmovzxwq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovqw_xmmm32_k1z_xmm)], "vpmovqw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovqw_xmmm64_k1z_ymm)], "vpmovqw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovqw_xmmm128_k1z_zmm)], "vpmovqw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmovzxdq_xmm_xmmm64)], "pmovzxdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxdq_xmm_xmmm64)], "vpmovzxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpmovzxdq_ymm_xmmm128)], "vpmovzxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxdq_xmm_k1z_xmmm64)], "vpmovzxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxdq_ymm_k1z_xmmm128)], "vpmovzxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovzxdq_zmm_k1z_ymmm256)], "vpmovzxdq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovqd_xmmm64_k1z_xmm)], "vpmovqd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovqd_xmmm128_k1z_ymm)], "vpmovqd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpmovqd_ymmm256_k1z_zmm)], "vpmovqd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermd_ymm_ymm_ymmm256)], "vpermd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32)], "vpermd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32)], "vpermd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64)], "vpermq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64)], "vpermq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpgtq_xmm_xmmm128)], "pcmpgtq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpgtq_xmm_xmm_xmmm128)], "vpcmpgtq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpgtq_ymm_ymm_ymmm256)], "vpcmpgtq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64)], "vpcmpgtq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64)], "vpcmpgtq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64)], "vpcmpgtq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pminsb_xmm_xmmm128)], "pminsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminsb_xmm_xmm_xmmm128)], "vpminsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminsb_ymm_ymm_ymmm256)], "vpminsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsb_xmm_k1z_xmm_xmmm128)], "vpminsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsb_ymm_k1z_ymm_ymmm256)], "vpminsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsb_zmm_k1z_zmm_zmmm512)], "vpminsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2d_xmm_kr)], "vpmovm2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2d_ymm_kr)], "vpmovm2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2d_zmm_kr)], "vpmovm2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2q_xmm_kr)], "vpmovm2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2q_ymm_kr)], "vpmovm2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovm2q_zmm_kr)], "vpmovm2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pminsd_xmm_xmmm128)], "pminsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminsd_xmm_xmm_xmmm128)], "vpminsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminsd_ymm_ymm_ymmm256)], "vpminsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32)], "vpminsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32)], "vpminsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32)], "vpminsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64)], "vpminsq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64)], "vpminsq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64)], "vpminsq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovd2m_kr_xmm)], "vpmovd2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovd2m_kr_ymm)], "vpmovd2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovd2m_kr_zmm)], "vpmovd2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovq2m_kr_xmm)], "vpmovq2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovq2m_kr_ymm)], "vpmovq2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmovq2m_kr_zmm)], "vpmovq2m" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pminuw_xmm_xmmm128)], "pminuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminuw_xmm_xmm_xmmm128)], "vpminuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminuw_ymm_ymm_ymmm256)], "vpminuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminuw_xmm_k1z_xmm_xmmm128)], "vpminuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminuw_ymm_k1z_ymm_ymmm256)], "vpminuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminuw_zmm_k1z_zmm_zmmm512)], "vpminuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastmw2d_xmm_kr)], "vpbroadcastmw2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastmw2d_ymm_kr)], "vpbroadcastmw2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastmw2d_zmm_kr)], "vpbroadcastmw2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pminud_xmm_xmmm128)], "pminud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminud_xmm_xmm_xmmm128)], "vpminud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpminud_ymm_ymm_ymmm256)], "vpminud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32)], "vpminud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32)], "vpminud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32)], "vpminud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64)], "vpminuq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64)], "vpminuq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64)], "vpminuq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmaxsb_xmm_xmmm128)], "pmaxsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxsb_xmm_xmm_xmmm128)], "vpmaxsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxsb_ymm_ymm_ymmm256)], "vpmaxsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128)], "vpmaxsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256)], "vpmaxsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512)], "vpmaxsb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmaxsd_xmm_xmmm128)], "pmaxsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxsd_xmm_xmm_xmmm128)], "vpmaxsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxsd_ymm_ymm_ymmm256)], "vpmaxsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32)], "vpmaxsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32)], "vpmaxsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32)], "vpmaxsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64)], "vpmaxsq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64)], "vpmaxsq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64)], "vpmaxsq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmaxuw_xmm_xmmm128)], "pmaxuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxuw_xmm_xmm_xmmm128)], "vpmaxuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxuw_ymm_ymm_ymmm256)], "vpmaxuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128)], "vpmaxuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256)], "vpmaxuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512)], "vpmaxuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmaxud_xmm_xmmm128)], "pmaxud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxud_xmm_xmm_xmmm128)], "vpmaxud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaxud_ymm_ymm_ymmm256)], "vpmaxud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32)], "vpmaxud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32)], "vpmaxud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32)], "vpmaxud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64)], "vpmaxuq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64)], "vpmaxuq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64)], "vpmaxuq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pmulld_xmm_xmmm128)], "pmulld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmulld_xmm_xmm_xmmm128)], "vpmulld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmulld_ymm_ymm_ymmm256)], "vpmulld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32)], "vpmulld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32)], "vpmulld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32)], "vpmulld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64)], "vpmullq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64)], "vpmullq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64)], "vpmullq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Phminposuw_xmm_xmmm128)], "phminposuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vphminposuw_xmm_xmmm128)], "vphminposuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetexpps_xmm_k1z_xmmm128b32)], "vgetexpps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetexpps_ymm_k1z_ymmm256b32)], "vgetexpps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae)], "vgetexpps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetexppd_xmm_k1z_xmmm128b64)], "vgetexppd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetexppd_ymm_k1z_ymmm256b64)], "vgetexppd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae)], "vgetexppd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae)], "vgetexpss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae)], "vgetexpsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vplzcntd_xmm_k1z_xmmm128b32)], "vplzcntd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vplzcntd_ymm_k1z_ymmm256b32)], "vplzcntd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vplzcntd_zmm_k1z_zmmm512b32)], "vplzcntd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vplzcntq_xmm_k1z_xmmm128b64)], "vplzcntq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vplzcntq_ymm_k1z_ymmm256b64)], "vplzcntq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vplzcntq_zmm_k1z_zmmm512b64)], "vplzcntq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlvd_xmm_xmm_xmmm128)], "vpsrlvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlvd_ymm_ymm_ymmm256)], "vpsrlvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlvq_xmm_xmm_xmmm128)], "vpsrlvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsrlvq_ymm_ymm_ymmm256)], "vpsrlvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32)], "vpsrlvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32)], "vpsrlvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32)], "vpsrlvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64)], "vpsrlvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64)], "vpsrlvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64)], "vpsrlvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsravd_xmm_xmm_xmmm128)], "vpsravd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsravd_ymm_ymm_ymmm256)], "vpsravd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32)], "vpsravd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32)], "vpsravd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32)], "vpsravd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64)], "vpsravq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64)], "vpsravq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64)], "vpsravq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllvd_xmm_xmm_xmmm128)], "vpsllvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllvd_ymm_ymm_ymmm256)], "vpsllvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllvq_xmm_xmm_xmmm128)], "vpsllvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpsllvq_ymm_ymm_ymmm256)], "vpsllvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32)], "vpsllvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32)], "vpsllvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32)], "vpsllvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64)], "vpsllvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64)], "vpsllvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64)], "vpsllvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrcp14ps_xmm_k1z_xmmm128b32)], "vrcp14ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrcp14ps_ymm_k1z_ymmm256b32)], "vrcp14ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrcp14ps_zmm_k1z_zmmm512b32)], "vrcp14ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrcp14pd_xmm_k1z_xmmm128b64)], "vrcp14pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrcp14pd_ymm_k1z_ymmm256b64)], "vrcp14pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrcp14pd_zmm_k1z_zmmm512b64)], "vrcp14pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32)], "vrcp14ss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64)], "vrcp14sd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32)], "vrsqrt14ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32)], "vrsqrt14ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32)], "vrsqrt14ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64)], "vrsqrt14pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64)], "vrsqrt14pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64)], "vrsqrt14pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32)], "vrsqrt14ss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64)], "vrsqrt14sd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32)], "vpdpbusd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32)], "vpdpbusd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32)], "vpdpbusd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32)], "vpdpbusds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32)], "vpdpbusds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32)], "vpdpbusds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32)], "vpdpwssd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32)], "vpdpwssd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32)], "vpdpwssd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32)], "vdpbf16ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32)], "vdpbf16ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32)], "vdpbf16ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128)], "vp4dpwssd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32)], "vpdpwssds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32)], "vpdpwssds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32)], "vpdpwssds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128)], "vp4dpwssds", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntb_xmm_k1z_xmmm128)], "vpopcntb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntb_ymm_k1z_ymmm256)], "vpopcntb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntb_zmm_k1z_zmmm512)], "vpopcntb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntw_xmm_k1z_xmmm128)], "vpopcntw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntw_ymm_k1z_ymmm256)], "vpopcntw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntw_zmm_k1z_zmmm512)], "vpopcntw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntd_xmm_k1z_xmmm128b32)], "vpopcntd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntd_ymm_k1z_ymmm256b32)], "vpopcntd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntd_zmm_k1z_zmmm512b32)], "vpopcntd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntq_xmm_k1z_xmmm128b64)], "vpopcntq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntq_ymm_k1z_ymmm256b64)], "vpopcntq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpopcntq_zmm_k1z_zmmm512b64)], "vpopcntq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpbroadcastd_xmm_xmmm32)], "vpbroadcastd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpbroadcastd_ymm_xmmm32)], "vpbroadcastd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_xmmm32)], "vpbroadcastd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_xmmm32)], "vpbroadcastd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_xmmm32)], "vpbroadcastd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpbroadcastq_xmm_xmmm64)], "vpbroadcastq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpbroadcastq_ymm_xmmm64)], "vpbroadcastq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64)], "vbroadcasti32x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64)], "vbroadcasti32x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64)], "vbroadcasti32x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_xmmm64)], "vpbroadcastq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_xmmm64)], "vpbroadcastq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_xmmm64)], "vpbroadcastq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vbroadcasti128_ymm_m128)], "vbroadcasti128", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcasti32x4_ymm_k1z_m128)], "vbroadcasti32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcasti32x4_zmm_k1z_m128)], "vbroadcasti32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcasti64x2_ymm_k1z_m128)], "vbroadcasti64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcasti64x2_zmm_k1z_m128)], "vbroadcasti64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcasti32x8_zmm_k1z_m256)], "vbroadcasti32x8", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vbroadcasti64x4_zmm_k1z_m256)], "vbroadcasti64x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandb_xmm_k1z_xmmm128)], "vpexpandb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandb_ymm_k1z_ymmm256)], "vpexpandb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandb_zmm_k1z_zmmm512)], "vpexpandb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandw_xmm_k1z_xmmm128)], "vpexpandw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandw_ymm_k1z_ymmm256)], "vpexpandw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandw_zmm_k1z_zmmm512)], "vpexpandw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressb_xmmm128_k1z_xmm)], "vpcompressb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressb_ymmm256_k1z_ymm)], "vpcompressb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressb_zmmm512_k1z_zmm)], "vpcompressb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressw_xmmm128_k1z_xmm)], "vpcompressw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressw_ymmm256_k1z_ymm)], "vpcompressw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressw_zmmm512_k1z_zmm)], "vpcompressw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32)], "vpblendmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32)], "vpblendmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32)], "vpblendmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64)], "vpblendmq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64)], "vpblendmq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64)], "vpblendmq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32)], "vblendmps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32)], "vblendmps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32)], "vblendmps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64)], "vblendmpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64)], "vblendmpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64)], "vblendmpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128)], "vpblendmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256)], "vpblendmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512)], "vpblendmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128)], "vpblendmw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256)], "vpblendmw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512)], "vpblendmw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vp2intersectd_kp1_xmm_xmmm128b32)], "vp2intersectd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vp2intersectd_kp1_ymm_ymmm256b32)], "vp2intersectd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vp2intersectd_kp1_zmm_zmmm512b32)], "vp2intersectd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vp2intersectq_kp1_xmm_xmmm128b64)], "vp2intersectq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vp2intersectq_kp1_ymm_ymmm256b64)], "vp2intersectq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vp2intersectq_kp1_zmm_zmmm512b64)], "vp2intersectq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128)], "vpshldvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256)], "vpshldvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512)], "vpshldvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32)], "vpshldvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32)], "vpshldvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32)], "vpshldvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64)], "vpshldvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64)], "vpshldvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64)], "vpshldvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128)], "vpshrdvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256)], "vpshrdvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512)], "vpshrdvw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32)], "vcvtneps2bf16", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32)], "vcvtneps2bf16", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32)], "vcvtneps2bf16", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32)], "vcvtne2ps2bf16" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32)], "vcvtne2ps2bf16" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32)], "vcvtne2ps2bf16" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32)], "vpshrdvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32)], "vpshrdvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32)], "vpshrdvd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64)], "vpshrdvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64)], "vpshrdvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64)], "vpshrdvq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128)], "vpermi2b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256)], "vpermi2b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512)], "vpermi2b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128)], "vpermi2w" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256)], "vpermi2w" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512)], "vpermi2w" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32)], "vpermi2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32)], "vpermi2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32)], "vpermi2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64)], "vpermi2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64)], "vpermi2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64)], "vpermi2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32)], "vpermi2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32)], "vpermi2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32)], "vpermi2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64)], "vpermi2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64)], "vpermi2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64)], "vpermi2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpbroadcastb_xmm_xmmm8)], "vpbroadcastb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpbroadcastb_ymm_xmmm8)], "vpbroadcastb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_xmmm8)], "vpbroadcastb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_xmmm8)], "vpbroadcastb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_xmmm8)], "vpbroadcastb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpbroadcastw_xmm_xmmm16)], "vpbroadcastw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpbroadcastw_ymm_xmmm16)], "vpbroadcastw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_xmmm16)], "vpbroadcastw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_xmmm16)], "vpbroadcastw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_xmmm16)], "vpbroadcastw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_r32)], "vpbroadcastb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_r32)], "vpbroadcastb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_r32)], "vpbroadcastb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_r32)], "vpbroadcastw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_r32)], "vpbroadcastw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_r32)], "vpbroadcastw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_r32)], "vpbroadcastd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_r32)], "vpbroadcastd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_r32)], "vpbroadcastd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_r64)], "vpbroadcastq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_r64)], "vpbroadcastq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_r64)], "vpbroadcastq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128)], "vpermt2b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256)], "vpermt2b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512)], "vpermt2b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128)], "vpermt2w" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256)], "vpermt2w" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512)], "vpermt2w" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32)], "vpermt2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32)], "vpermt2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32)], "vpermt2d" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64)], "vpermt2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64)], "vpermt2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64)], "vpermt2q" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32)], "vpermt2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32)], "vpermt2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32)], "vpermt2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64)], "vpermt2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64)], "vpermt2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64)], "vpermt2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Invept_r32_m128)], "invept", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.MemSize_Normal), nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Invept_r64_m128)], "invept", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.MemSize_Normal), nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Invvpid_r32_m128)], "invvpid", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.MemSize_Normal), nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Invvpid_r64_m128)], "invvpid", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.MemSize_Normal), nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Invpcid_r32_m128)], "invpcid", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.MemSize_Normal), nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Invpcid_r64_m128)], "invpcid", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.MemSize_Normal), nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64)], "vpmultishiftqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64)], "vpmultishiftqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64)], "vpmultishiftqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vexpandps_xmm_k1z_xmmm128)], "vexpandps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vexpandps_ymm_k1z_ymmm256)], "vexpandps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vexpandps_zmm_k1z_zmmm512)], "vexpandps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vexpandpd_xmm_k1z_xmmm128)], "vexpandpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vexpandpd_ymm_k1z_ymmm256)], "vexpandpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vexpandpd_zmm_k1z_zmmm512)], "vexpandpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandd_xmm_k1z_xmmm128)], "vpexpandd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandd_ymm_k1z_ymmm256)], "vpexpandd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandd_zmm_k1z_zmmm512)], "vpexpandd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandq_xmm_k1z_xmmm128)], "vpexpandq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandq_ymm_k1z_ymmm256)], "vpexpandq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpexpandq_zmm_k1z_zmmm512)], "vpexpandq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcompressps_xmmm128_k1z_xmm)], "vcompressps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcompressps_ymmm256_k1z_ymm)], "vcompressps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcompressps_zmmm512_k1z_zmm)], "vcompressps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcompresspd_xmmm128_k1z_xmm)], "vcompresspd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcompresspd_ymmm256_k1z_ymm)], "vcompresspd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vcompresspd_zmmm512_k1z_zmm)], "vcompresspd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressd_xmmm128_k1z_xmm)], "vpcompressd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressd_ymmm256_k1z_ymm)], "vpcompressd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressd_zmmm512_k1z_zmm)], "vpcompressd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressq_xmmm128_k1z_xmm)], "vpcompressq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressq_ymmm256_k1z_ymm)], "vpcompressq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcompressq_zmmm512_k1z_zmm)], "vpcompressq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaskmovd_xmm_xmm_m128)], "vpmaskmovd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaskmovd_ymm_ymm_m256)], "vpmaskmovd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaskmovq_xmm_xmm_m128)], "vpmaskmovq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaskmovq_ymm_ymm_m256)], "vpmaskmovq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermb_xmm_k1z_xmm_xmmm128)], "vpermb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermb_ymm_k1z_ymm_ymmm256)], "vpermb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermb_zmm_k1z_zmm_zmmm512)], "vpermb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermw_xmm_k1z_xmm_xmmm128)], "vpermw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermw_ymm_k1z_ymm_ymmm256)], "vpermw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermw_zmm_k1z_zmm_zmmm512)], "vpermw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaskmovd_m128_xmm_xmm)], "vpmaskmovd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaskmovd_m256_ymm_ymm)], "vpmaskmovd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaskmovq_m128_xmm_xmm)], "vpmaskmovq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpmaskmovq_m256_ymm_ymm)], "vpmaskmovq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128)], "vpshufbitqmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256)], "vpshufbitqmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512)], "vpshufbitqmb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpgatherdd_xmm_vm32x_xmm)], "vpgatherdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpgatherdd_ymm_vm32y_ymm)], "vpgatherdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpgatherdq_xmm_vm32x_xmm)], "vpgatherdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpgatherdq_ymm_vm32x_ymm)], "vpgatherdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherdd_xmm_k1_vm32x)], "vpgatherdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherdd_ymm_k1_vm32y)], "vpgatherdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherdd_zmm_k1_vm32z)], "vpgatherdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherdq_xmm_k1_vm32x)], "vpgatherdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherdq_ymm_k1_vm32x)], "vpgatherdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherdq_zmm_k1_vm32y)], "vpgatherdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpgatherqd_xmm_vm64x_xmm)], "vpgatherqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpgatherqd_xmm_vm64y_xmm)], "vpgatherqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpgatherqq_xmm_vm64x_xmm)], "vpgatherqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpgatherqq_ymm_vm64y_ymm)], "vpgatherqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64x)], "vpgatherqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64y)], "vpgatherqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherqd_ymm_k1_vm64z)], "vpgatherqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherqq_xmm_k1_vm64x)], "vpgatherqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherqq_ymm_k1_vm64y)], "vpgatherqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpgatherqq_zmm_k1_vm64z)], "vpgatherqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgatherdps_xmm_vm32x_xmm)], "vgatherdps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgatherdps_ymm_vm32y_ymm)], "vgatherdps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgatherdpd_xmm_vm32x_xmm)], "vgatherdpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgatherdpd_ymm_vm32x_ymm)], "vgatherdpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherdps_xmm_k1_vm32x)], "vgatherdps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherdps_ymm_k1_vm32y)], "vgatherdps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherdps_zmm_k1_vm32z)], "vgatherdps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherdpd_xmm_k1_vm32x)], "vgatherdpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherdpd_ymm_k1_vm32x)], "vgatherdpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherdpd_zmm_k1_vm32y)], "vgatherdpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgatherqps_xmm_vm64x_xmm)], "vgatherqps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgatherqps_xmm_vm64y_xmm)], "vgatherqps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgatherqpd_xmm_vm64x_xmm)], "vgatherqpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgatherqpd_ymm_vm64y_ymm)], "vgatherqpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64x)], "vgatherqps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64y)], "vgatherqps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherqps_ymm_k1_vm64z)], "vgatherqps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherqpd_xmm_k1_vm64x)], "vgatherqpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherqpd_ymm_k1_vm64y)], "vgatherqpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherqpd_zmm_k1_vm64z)], "vgatherqpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub132ps_xmm_xmm_xmmm128)], "vfmaddsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub132ps_ymm_ymm_ymmm256)], "vfmaddsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub132pd_xmm_xmm_xmmm128)], "vfmaddsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub132pd_ymm_ymm_ymmm256)], "vfmaddsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32)], "vfmaddsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32)], "vfmaddsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmaddsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64)], "vfmaddsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64)], "vfmaddsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmaddsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd132ps_xmm_xmm_xmmm128)], "vfmsubadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd132ps_ymm_ymm_ymmm256)], "vfmsubadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd132pd_xmm_xmm_xmmm128)], "vfmsubadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd132pd_ymm_ymm_ymmm256)], "vfmsubadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32)], "vfmsubadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32)], "vfmsubadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmsubadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64)], "vfmsubadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64)], "vfmsubadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmsubadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd132ps_xmm_xmm_xmmm128)], "vfmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd132ps_ymm_ymm_ymmm256)], "vfmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd132pd_xmm_xmm_xmmm128)], "vfmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd132pd_ymm_ymm_ymmm256)], "vfmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32)], "vfmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32)], "vfmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64)], "vfmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64)], "vfmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd132ss_xmm_xmm_xmmm32)], "vfmadd132ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd132sd_xmm_xmm_xmmm64)], "vfmadd132sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er)], "vfmadd132ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er)], "vfmadd132sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub132ps_xmm_xmm_xmmm128)], "vfmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub132ps_ymm_ymm_ymmm256)], "vfmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub132pd_xmm_xmm_xmmm128)], "vfmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub132pd_ymm_ymm_ymmm256)], "vfmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32)], "vfmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32)], "vfmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64)], "vfmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64)], "vfmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_V4fmaddps_zmm_k1z_zmmp3_m128)], "v4fmaddps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub132ss_xmm_xmm_xmmm32)], "vfmsub132ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub132sd_xmm_xmm_xmmm64)], "vfmsub132sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er)], "vfmsub132ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er)], "vfmsub132sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_V4fmaddss_xmm_k1z_xmmp3_m128)], "v4fmaddss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd132ps_xmm_xmm_xmmm128)], "vfnmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd132ps_ymm_ymm_ymmm256)], "vfnmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd132pd_xmm_xmm_xmmm128)], "vfnmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd132pd_ymm_ymm_ymmm256)], "vfnmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32)], "vfnmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32)], "vfnmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], "vfnmadd132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64)], "vfnmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64)], "vfnmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], "vfnmadd132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd132ss_xmm_xmm_xmmm32)], "vfnmadd132ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd132sd_xmm_xmm_xmmm64)], "vfnmadd132sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er)], "vfnmadd132ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er)], "vfnmadd132sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub132ps_xmm_xmm_xmmm128)], "vfnmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub132ps_ymm_ymm_ymmm256)], "vfnmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub132pd_xmm_xmm_xmmm128)], "vfnmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub132pd_ymm_ymm_ymmm256)], "vfnmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32)], "vfnmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32)], "vfnmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], "vfnmsub132ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64)], "vfnmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64)], "vfnmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], "vfnmsub132pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub132ss_xmm_xmm_xmmm32)], "vfnmsub132ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub132sd_xmm_xmm_xmmm64)], "vfnmsub132sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er)], "vfnmsub132ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er)], "vfnmsub132sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterdd_vm32x_k1_xmm)], "vpscatterdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterdd_vm32y_k1_ymm)], "vpscatterdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterdd_vm32z_k1_zmm)], "vpscatterdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_xmm)], "vpscatterdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_ymm)], "vpscatterdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterdq_vm32y_k1_zmm)], "vpscatterdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterqd_vm64x_k1_xmm)], "vpscatterqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterqd_vm64y_k1_xmm)], "vpscatterqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterqd_vm64z_k1_ymm)], "vpscatterqd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterqq_vm64x_k1_xmm)], "vpscatterqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterqq_vm64y_k1_ymm)], "vpscatterqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpscatterqq_vm64z_k1_zmm)], "vpscatterqq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterdps_vm32x_k1_xmm)], "vscatterdps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterdps_vm32y_k1_ymm)], "vscatterdps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterdps_vm32z_k1_zmm)], "vscatterdps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_xmm)], "vscatterdpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_ymm)], "vscatterdpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterdpd_vm32y_k1_zmm)], "vscatterdpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterqps_vm64x_k1_xmm)], "vscatterqps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterqps_vm64y_k1_xmm)], "vscatterqps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterqps_vm64z_k1_ymm)], "vscatterqps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterqpd_vm64x_k1_xmm)], "vscatterqpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterqpd_vm64y_k1_ymm)], "vscatterqpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterqpd_vm64z_k1_zmm)], "vscatterqpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub213ps_xmm_xmm_xmmm128)], "vfmaddsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub213ps_ymm_ymm_ymmm256)], "vfmaddsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub213pd_xmm_xmm_xmmm128)], "vfmaddsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub213pd_ymm_ymm_ymmm256)], "vfmaddsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32)], "vfmaddsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32)], "vfmaddsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmaddsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64)], "vfmaddsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64)], "vfmaddsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmaddsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd213ps_xmm_xmm_xmmm128)], "vfmsubadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd213ps_ymm_ymm_ymmm256)], "vfmsubadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd213pd_xmm_xmm_xmmm128)], "vfmsubadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd213pd_ymm_ymm_ymmm256)], "vfmsubadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32)], "vfmsubadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32)], "vfmsubadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmsubadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64)], "vfmsubadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64)], "vfmsubadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmsubadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd213ps_xmm_xmm_xmmm128)], "vfmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd213ps_ymm_ymm_ymmm256)], "vfmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd213pd_xmm_xmm_xmmm128)], "vfmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd213pd_ymm_ymm_ymmm256)], "vfmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32)], "vfmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32)], "vfmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64)], "vfmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64)], "vfmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd213ss_xmm_xmm_xmmm32)], "vfmadd213ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd213sd_xmm_xmm_xmmm64)], "vfmadd213sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er)], "vfmadd213ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er)], "vfmadd213sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub213ps_xmm_xmm_xmmm128)], "vfmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub213ps_ymm_ymm_ymmm256)], "vfmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub213pd_xmm_xmm_xmmm128)], "vfmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub213pd_ymm_ymm_ymmm256)], "vfmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32)], "vfmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32)], "vfmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64)], "vfmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64)], "vfmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128)], "v4fnmaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub213ss_xmm_xmm_xmmm32)], "vfmsub213ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub213sd_xmm_xmm_xmmm64)], "vfmsub213sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er)], "vfmsub213ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er)], "vfmsub213sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128)], "v4fnmaddss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd213ps_xmm_xmm_xmmm128)], "vfnmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd213ps_ymm_ymm_ymmm256)], "vfnmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd213pd_xmm_xmm_xmmm128)], "vfnmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd213pd_ymm_ymm_ymmm256)], "vfnmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32)], "vfnmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32)], "vfnmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], "vfnmadd213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64)], "vfnmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64)], "vfnmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], "vfnmadd213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd213ss_xmm_xmm_xmmm32)], "vfnmadd213ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd213sd_xmm_xmm_xmmm64)], "vfnmadd213sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er)], "vfnmadd213ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er)], "vfnmadd213sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub213ps_xmm_xmm_xmmm128)], "vfnmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub213ps_ymm_ymm_ymmm256)], "vfnmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub213pd_xmm_xmm_xmmm128)], "vfnmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub213pd_ymm_ymm_ymmm256)], "vfnmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32)], "vfnmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32)], "vfnmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], "vfnmsub213ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64)], "vfnmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64)], "vfnmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], "vfnmsub213pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub213ss_xmm_xmm_xmmm32)], "vfnmsub213ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub213sd_xmm_xmm_xmmm64)], "vfnmsub213sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er)], "vfnmsub213ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er)], "vfnmsub213sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64)], "vpmadd52luq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64)], "vpmadd52luq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64)], "vpmadd52luq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64)], "vpmadd52huq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64)], "vpmadd52huq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64)], "vpmadd52huq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub231ps_xmm_xmm_xmmm128)], "vfmaddsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub231ps_ymm_ymm_ymmm256)], "vfmaddsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub231pd_xmm_xmm_xmmm128)], "vfmaddsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsub231pd_ymm_ymm_ymmm256)], "vfmaddsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32)], "vfmaddsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32)], "vfmaddsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmaddsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64)], "vfmaddsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64)], "vfmaddsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmaddsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd231ps_xmm_xmm_xmmm128)], "vfmsubadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd231ps_ymm_ymm_ymmm256)], "vfmsubadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd231pd_xmm_xmm_xmmm128)], "vfmsubadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubadd231pd_ymm_ymm_ymmm256)], "vfmsubadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32)], "vfmsubadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32)], "vfmsubadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmsubadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64)], "vfmsubadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64)], "vfmsubadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmsubadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd231ps_xmm_xmm_xmmm128)], "vfmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd231ps_ymm_ymm_ymmm256)], "vfmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd231pd_xmm_xmm_xmmm128)], "vfmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd231pd_ymm_ymm_ymmm256)], "vfmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32)], "vfmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32)], "vfmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64)], "vfmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64)], "vfmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd231ss_xmm_xmm_xmmm32)], "vfmadd231ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmadd231sd_xmm_xmm_xmmm64)], "vfmadd231sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er)], "vfmadd231ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er)], "vfmadd231sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub231ps_xmm_xmm_xmmm128)], "vfmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub231ps_ymm_ymm_ymmm256)], "vfmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub231pd_xmm_xmm_xmmm128)], "vfmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub231pd_ymm_ymm_ymmm256)], "vfmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32)], "vfmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32)], "vfmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], "vfmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64)], "vfmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64)], "vfmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], "vfmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub231ss_xmm_xmm_xmmm32)], "vfmsub231ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsub231sd_xmm_xmm_xmmm64)], "vfmsub231sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er)], "vfmsub231ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er)], "vfmsub231sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd231ps_xmm_xmm_xmmm128)], "vfnmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd231ps_ymm_ymm_ymmm256)], "vfnmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd231pd_xmm_xmm_xmmm128)], "vfnmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd231pd_ymm_ymm_ymmm256)], "vfnmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32)], "vfnmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32)], "vfnmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], "vfnmadd231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64)], "vfnmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64)], "vfnmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], "vfnmadd231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd231ss_xmm_xmm_xmmm32)], "vfnmadd231ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmadd231sd_xmm_xmm_xmmm64)], "vfnmadd231sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er)], "vfnmadd231ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er)], "vfnmadd231sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub231ps_xmm_xmm_xmmm128)], "vfnmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub231ps_ymm_ymm_ymmm256)], "vfnmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub231pd_xmm_xmm_xmmm128)], "vfnmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub231pd_ymm_ymm_ymmm256)], "vfnmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32)], "vfnmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32)], "vfnmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], "vfnmsub231ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64)], "vfnmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64)], "vfnmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], "vfnmsub231pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub231ss_xmm_xmm_xmmm32)], "vfnmsub231ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsub231sd_xmm_xmm_xmmm64)], "vfnmsub231sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er)], "vfnmsub231ss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er)], "vfnmsub231sd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpconflictd_xmm_k1z_xmmm128b32)], "vpconflictd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpconflictd_ymm_k1z_ymmm256b32)], "vpconflictd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpconflictd_zmm_k1z_zmmm512b32)], "vpconflictd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpconflictq_xmm_k1z_xmmm128b64)], "vpconflictq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpconflictq_ymm_k1z_ymmm256b64)], "vpconflictq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpconflictq_zmm_k1z_zmmm512b64)], "vpconflictq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherpf0dps_vm32z_k1)], "vgatherpf0dps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherpf0dpd_vm32y_k1)], "vgatherpf0dpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherpf1dps_vm32z_k1)], "vgatherpf1dps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherpf1dpd_vm32y_k1)], "vgatherpf1dpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterpf0dps_vm32z_k1)], "vscatterpf0dps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterpf0dpd_vm32y_k1)], "vscatterpf0dpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterpf1dps_vm32z_k1)], "vscatterpf1dps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterpf1dpd_vm32y_k1)], "vscatterpf1dpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherpf0qps_vm64z_k1)], "vgatherpf0qps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherpf0qpd_vm64z_k1)], "vgatherpf0qpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherpf1qps_vm64z_k1)], "vgatherpf1qps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgatherpf1qpd_vm64z_k1)], "vgatherpf1qpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterpf0qps_vm64z_k1)], "vscatterpf0qps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterpf0qpd_vm64z_k1)], "vscatterpf0qpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterpf1qps_vm64z_k1)], "vscatterpf1qps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vscatterpf1qpd_vm64z_k1)], "vscatterpf1qpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sha1nexte_xmm_xmmm128)], "sha1nexte" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae)], "vexp2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae)], "vexp2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sha1msg1_xmm_xmmm128)], "sha1msg1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sha1msg2_xmm_xmmm128)], "sha1msg2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae)], "vrcp28ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae)], "vrcp28pd" },
				new object[] { ctorKind[nameof(CtorKind.pblendvb)], code[nameof(Code.Sha256rnds2_xmm_xmmm128)], "sha256rnds2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae)], "vrcp28ss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae)], "vrcp28sd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sha256msg1_xmm_xmmm128)], "sha256msg1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae)], "vrsqrt28ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae)], "vrsqrt28pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sha256msg2_xmm_xmmm128)], "sha256msg2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae)], "vrsqrt28ss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae)], "vrsqrt28sd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Gf2p8mulb_xmm_xmmm128)], "gf2p8mulb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgf2p8mulb_xmm_xmm_xmmm128)], "vgf2p8mulb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgf2p8mulb_ymm_ymm_ymmm256)], "vgf2p8mulb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128)], "vgf2p8mulb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256)], "vgf2p8mulb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512)], "vgf2p8mulb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Aesimc_xmm_xmmm128)], "aesimc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaesimc_xmm_xmmm128)], "vaesimc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Aesenc_xmm_xmmm128)], "aesenc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaesenc_xmm_xmm_xmmm128)], "vaesenc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaesenc_ymm_ymm_ymmm256)], "vaesenc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesenc_xmm_xmm_xmmm128)], "vaesenc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesenc_ymm_ymm_ymmm256)], "vaesenc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesenc_zmm_zmm_zmmm512)], "vaesenc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Aesenclast_xmm_xmmm128)], "aesenclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaesenclast_xmm_xmm_xmmm128)], "vaesenclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaesenclast_ymm_ymm_ymmm256)], "vaesenclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesenclast_xmm_xmm_xmmm128)], "vaesenclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesenclast_ymm_ymm_ymmm256)], "vaesenclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesenclast_zmm_zmm_zmmm512)], "vaesenclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Aesdec_xmm_xmmm128)], "aesdec" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaesdec_xmm_xmm_xmmm128)], "vaesdec" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaesdec_ymm_ymm_ymmm256)], "vaesdec" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesdec_xmm_xmm_xmmm128)], "vaesdec" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesdec_ymm_ymm_ymmm256)], "vaesdec" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesdec_zmm_zmm_zmmm512)], "vaesdec" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Aesdeclast_xmm_xmmm128)], "aesdeclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaesdeclast_xmm_xmm_xmmm128)], "vaesdeclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaesdeclast_ymm_ymm_ymmm256)], "vaesdeclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesdeclast_xmm_xmm_xmmm128)], "vaesdeclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesdeclast_ymm_ymm_ymmm256)], "vaesdeclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vaesdeclast_zmm_zmm_zmmm512)], "vaesdeclast" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movbe_r16_m16)], "movbe" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movbe_r32_m32)], "movbe" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movbe_r64_m64)], "movbe" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Crc32_r32_rm8)], "crc32", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Crc32_r64_rm8)], "crc32", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movbe_m16_r16)], "movbe" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movbe_m32_r32)], "movbe" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movbe_m64_r64)], "movbe" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Crc32_r32_rm16)], "crc32", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Crc32_r32_rm32)], "crc32", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Crc32_r64_rm64)], "crc32", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Andn_r32_r32_rm32)], "andn" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Andn_r64_r64_rm64)], "andn" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Blsr_r32_rm32)], "blsr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Blsr_r64_rm64)], "blsr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Blsmsk_r32_rm32)], "blsmsk" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Blsmsk_r64_rm64)], "blsmsk" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Blsi_r32_rm32)], "blsi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Blsi_r64_rm64)], "blsi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Bzhi_r32_rm32_r32)], "bzhi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Bzhi_r64_rm64_r64)], "bzhi" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrussd_m32_r32)], "wrussd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrussq_m64_r64)], "wrussq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Pext_r32_r32_rm32)], "pext" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Pext_r64_r64_rm64)], "pext" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Pdep_r32_r32_rm32)], "pdep" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Pdep_r64_r64_rm64)], "pdep" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrssd_m32_r32)], "wrssd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Wrssq_m64_r64)], "wrssq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adcx_r32_rm32)], "adcx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adcx_r64_rm64)], "adcx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adox_r32_rm32)], "adox" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Adox_r64_rm64)], "adox" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Mulx_r32_r32_rm32)], "mulx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Mulx_r64_r64_rm64)], "mulx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Bextr_r32_rm32_r32)], "bextr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Bextr_r64_rm64_r64)], "bextr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Shlx_r32_rm32_r32)], "shlx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Shlx_r64_rm64_r64)], "shlx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Sarx_r32_rm32_r32)], "sarx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Sarx_r64_rm64_r64)], "sarx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Shrx_r32_rm32_r32)], "shrx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Shrx_r64_rm64_r64)], "shrx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movdir64b_r16_m512)], "movdir64b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movdir64b_r32_m512)], "movdir64b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movdir64b_r64_m512)], "movdir64b" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Enqcmds_r16_m512)], "enqcmds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Enqcmds_r32_m512)], "enqcmds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Enqcmds_r64_m512)], "enqcmds" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Enqcmd_r16_m512)], "enqcmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Enqcmd_r32_m512)], "enqcmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Enqcmd_r64_m512)], "enqcmd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movdiri_m32_r32)], "movdiri" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Movdiri_m64_r64)], "movdiri" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermq_ymm_ymmm256_imm8)], "vpermq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8)], "vpermq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8)], "vpermq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermpd_ymm_ymmm256_imm8)], "vpermpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8)], "vpermpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8)], "vpermpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpblendd_xmm_xmm_xmmm128_imm8)], "vpblendd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpblendd_ymm_ymm_ymmm256_imm8)], "vpblendd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8)], "valignd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8)], "valignd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8)], "valignd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8)], "valignq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8)], "valignq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8)], "valignq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermilps_xmm_xmmm128_imm8)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermilps_ymm_ymmm256_imm8)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8)], "vpermilps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermilpd_xmm_xmmm128_imm8)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermilpd_ymm_ymmm256_imm8)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8)], "vpermilpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vperm2f128_ymm_ymm_ymmm256_imm8)], "vperm2f128" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Roundps_xmm_xmmm128_imm8)], "roundps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vroundps_xmm_xmmm128_imm8)], "vroundps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vroundps_ymm_ymmm256_imm8)], "vroundps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8)], "vrndscaleps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8)], "vrndscaleps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae)], "vrndscaleps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Roundpd_xmm_xmmm128_imm8)], "roundpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vroundpd_xmm_xmmm128_imm8)], "vroundpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vroundpd_ymm_ymmm256_imm8)], "vroundpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8)], "vrndscalepd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8)], "vrndscalepd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae)], "vrndscalepd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Roundss_xmm_xmmm32_imm8)], "roundss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vroundss_xmm_xmm_xmmm32_imm8)], "vroundss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae)], "vrndscaless" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Roundsd_xmm_xmmm64_imm8)], "roundsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vroundsd_xmm_xmm_xmmm64_imm8)], "vroundsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae)], "vrndscalesd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Blendps_xmm_xmmm128_imm8)], "blendps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vblendps_xmm_xmm_xmmm128_imm8)], "vblendps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vblendps_ymm_ymm_ymmm256_imm8)], "vblendps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Blendpd_xmm_xmmm128_imm8)], "blendpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vblendpd_xmm_xmm_xmmm128_imm8)], "vblendpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vblendpd_ymm_ymm_ymmm256_imm8)], "vblendpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pblendw_xmm_xmmm128_imm8)], "pblendw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpblendw_xmm_xmm_xmmm128_imm8)], "vpblendw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpblendw_ymm_ymm_ymmm256_imm8)], "vpblendw" },
				new object[] { ctorKind[nameof(CtorKind.mmxmem_1)], code[nameof(Code.Palignr_mm_mmm64_imm8)], "palignr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Palignr_xmm_xmmm128_imm8)], "palignr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpalignr_xmm_xmm_xmmm128_imm8)], "vpalignr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpalignr_ymm_ymm_ymmm256_imm8)], "vpalignr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8)], "vpalignr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8)], "vpalignr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8)], "vpalignr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pextrb_r32m8_xmm_imm8)], "pextrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pextrb_r64m8_xmm_imm8)], "pextrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpextrb_r32m8_xmm_imm8)], "vpextrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpextrb_r64m8_xmm_imm8)], "vpextrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpextrb_r32m8_xmm_imm8)], "vpextrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpextrb_r64m8_xmm_imm8)], "vpextrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pextrw_r32m16_xmm_imm8)], "pextrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pextrw_r64m16_xmm_imm8)], "pextrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpextrw_r32m16_xmm_imm8)], "vpextrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpextrw_r64m16_xmm_imm8)], "vpextrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpextrw_r32m16_xmm_imm8)], "vpextrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpextrw_r64m16_xmm_imm8)], "vpextrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pextrd_rm32_xmm_imm8)], "pextrd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pextrq_rm64_xmm_imm8)], "pextrq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpextrd_rm32_xmm_imm8)], "vpextrd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpextrq_rm64_xmm_imm8)], "vpextrq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpextrd_rm32_xmm_imm8)], "vpextrd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpextrq_rm64_xmm_imm8)], "vpextrq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Extractps_rm32_xmm_imm8)], "extractps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Reg32)], code[nameof(Code.Extractps_r64m32_xmm_imm8)], "extractps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vextractps_rm32_xmm_imm8)], "vextractps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Reg32)], code[nameof(Code.VEX_Vextractps_r64m32_xmm_imm8)], "vextractps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextractps_rm32_xmm_imm8)], "vextractps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Reg32)], code[nameof(Code.EVEX_Vextractps_r64m32_xmm_imm8)], "vextractps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vinsertf128_ymm_ymm_xmmm128_imm8)], "vinsertf128", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8)], "vinsertf32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8)], "vinsertf32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8)], "vinsertf64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8)], "vinsertf64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vextractf128_xmmm128_ymm_imm8)], "vextractf128", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8)], "vextractf32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8)], "vextractf32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8)], "vextractf64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8)], "vextractf64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8)], "vinsertf32x8" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8)], "vinsertf64x4" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8)], "vextractf32x8", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8)], "vextractf64x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtps2ph_xmmm64_xmm_imm8)], "vcvtps2ph", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vcvtps2ph_xmmm128_ymm_imm8)], "vcvtps2ph", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8)], "vcvtps2ph", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8)], "vcvtps2ph", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae)], "vcvtps2ph", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8)], "vpcmpud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8)], "vpcmpud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8)], "vpcmpud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8)], "vpcmpuq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8)], "vpcmpuq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8)], "vpcmpuq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8)], "vpcmpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8)], "vpcmpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8)], "vpcmpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8)], "vpcmpq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8)], "vpcmpq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8)], "vpcmpq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pinsrb_xmm_r32m8_imm8)], "pinsrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pinsrb_xmm_r64m8_imm8)], "pinsrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r32m8_imm8)], "vpinsrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg32)], code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r64m8_imm8)], "vpinsrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r32m8_imm8)], "vpinsrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Reg32)], code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r64m8_imm8)], "vpinsrb", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Insertps_xmm_xmmm32_imm8)], "insertps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vinsertps_xmm_xmm_xmmm32_imm8)], "vinsertps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinsertps_xmm_xmm_xmmm32_imm8)], "vinsertps", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pinsrd_xmm_rm32_imm8)], "pinsrd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.Pinsrq_xmm_rm64_imm8)], "pinsrq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpinsrd_xmm_xmm_rm32_imm8)], "vpinsrd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vpinsrq_xmm_xmm_rm64_imm8)], "vpinsrq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpinsrd_xmm_xmm_rm32_imm8)], "vpinsrd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vpinsrq_xmm_xmm_rm64_imm8)], "vpinsrq", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8)], "vshuff32x4" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8)], "vshuff32x4" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8)], "vshuff64x2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8)], "vshuff64x2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8)], "vpternlogd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8)], "vpternlogd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8)], "vpternlogd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8)], "vpternlogq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8)], "vpternlogq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8)], "vpternlogq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8)], "vgetmantps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8)], "vgetmantps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae)], "vgetmantps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8)], "vgetmantpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8)], "vgetmantpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae)], "vgetmantpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae)], "vgetmantss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae)], "vgetmantsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kshiftrb_kr_kr_imm8)], "kshiftrb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kshiftrw_kr_kr_imm8)], "kshiftrw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kshiftrd_kr_kr_imm8)], "kshiftrd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kshiftrq_kr_kr_imm8)], "kshiftrq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kshiftlb_kr_kr_imm8)], "kshiftlb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kshiftlw_kr_kr_imm8)], "kshiftlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kshiftld_kr_kr_imm8)], "kshiftld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Kshiftlq_kr_kr_imm8)], "kshiftlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vinserti128_ymm_ymm_xmmm128_imm8)], "vinserti128", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8)], "vinserti32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8)], "vinserti32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8)], "vinserti64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8)], "vinserti64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vextracti128_xmmm128_ymm_imm8)], "vextracti128", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8)], "vextracti32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8)], "vextracti32x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8)], "vextracti64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8)], "vextracti64x2", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8)], "vinserti32x8", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8)], "vinserti64x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8)], "vextracti32x8", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8)], "vextracti64x4", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8)], "vpcmpub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8)], "vpcmpub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8)], "vpcmpub" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8)], "vpcmpuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8)], "vpcmpuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8)], "vpcmpuw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8)], "vpcmpb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8)], "vpcmpb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8)], "vpcmpb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8)], "vpcmpw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8)], "vpcmpw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8)], "vpcmpw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Dpps_xmm_xmmm128_imm8)], "dpps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vdpps_xmm_xmm_xmmm128_imm8)], "vdpps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vdpps_ymm_ymm_ymmm256_imm8)], "vdpps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Dppd_xmm_xmmm128_imm8)], "dppd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vdppd_xmm_xmm_xmmm128_imm8)], "vdppd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Mpsadbw_xmm_xmmm128_imm8)], "mpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8)], "vmpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8)], "vmpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8)], "vdbpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8)], "vdbpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8)], "vdbpsadbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8)], "vshufi32x4" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8)], "vshufi32x4" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8)], "vshufi64x2" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8)], "vshufi64x2" },
				new object[] { ctorKind[nameof(CtorKind.pclmulqdq)], code[nameof(Code.Pclmulqdq_xmm_xmmm128_imm8)], "pclmulqdq", pseudoOpsKind[nameof(PseudoOpsKind.pclmulqdq)] },
				new object[] { ctorKind[nameof(CtorKind.pclmulqdq)], code[nameof(Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], "vpclmulqdq", pseudoOpsKind[nameof(PseudoOpsKind.vpclmulqdq)] },
				new object[] { ctorKind[nameof(CtorKind.pclmulqdq)], code[nameof(Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], "vpclmulqdq", pseudoOpsKind[nameof(PseudoOpsKind.vpclmulqdq)] },
				new object[] { ctorKind[nameof(CtorKind.pclmulqdq)], code[nameof(Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], "vpclmulqdq", pseudoOpsKind[nameof(PseudoOpsKind.vpclmulqdq)] },
				new object[] { ctorKind[nameof(CtorKind.pclmulqdq)], code[nameof(Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], "vpclmulqdq", pseudoOpsKind[nameof(PseudoOpsKind.vpclmulqdq)] },
				new object[] { ctorKind[nameof(CtorKind.pclmulqdq)], code[nameof(Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8)], "vpclmulqdq", pseudoOpsKind[nameof(PseudoOpsKind.vpclmulqdq)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vperm2i128_ymm_ymm_ymmm256_imm8)], "vperm2i128" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm2)], "vpermil2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm2)], "vpermil2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm2)], "vpermil2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm2)], "vpermil2ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm2)], "vpermil2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm2)], "vpermil2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm2)], "vpermil2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm2)], "vpermil2pd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vblendvps_xmm_xmm_xmmm128_xmm)], "vblendvps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vblendvps_ymm_ymm_ymmm256_ymm)], "vblendvps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vblendvpd_xmm_xmm_xmmm128_xmm)], "vblendvpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vblendvpd_ymm_ymm_ymmm256_ymm)], "vblendvpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpblendvb_xmm_xmm_xmmm128_xmm)], "vpblendvb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpblendvb_ymm_ymm_ymmm256_ymm)], "vpblendvb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8)], "vrangeps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8)], "vrangeps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], "vrangeps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8)], "vrangepd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8)], "vrangepd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], "vrangepd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae)], "vrangess" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae)], "vrangesd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8)], "vfixupimmps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8)], "vfixupimmps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], "vfixupimmps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8)], "vfixupimmpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8)], "vfixupimmpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], "vfixupimmpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae)], "vfixupimmss" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae)], "vfixupimmsd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8)], "vreduceps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8)], "vreduceps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae)], "vreduceps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8)], "vreducepd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8)], "vreducepd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae)], "vreducepd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae)], "vreducess" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae)], "vreducesd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm)], "vfmaddsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm)], "vfmaddsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128)], "vfmaddsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256)], "vfmaddsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm)], "vfmaddsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm)], "vfmaddsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128)], "vfmaddsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256)], "vfmaddsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm)], "vfmsubaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm)], "vfmsubaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128)], "vfmsubaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256)], "vfmsubaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm)], "vfmsubaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm)], "vfmsubaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128)], "vfmsubaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256)], "vfmsubaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpestrm_xmm_xmmm128_imm8)], "pcmpestrm" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpestrm64_xmm_xmmm128_imm8)], "pcmpestrm64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpestrm_xmm_xmmm128_imm8)], "vpcmpestrm" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpestrm64_xmm_xmmm128_imm8)], "vpcmpestrm64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpestri_xmm_xmmm128_imm8)], "pcmpestri" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpestri64_xmm_xmmm128_imm8)], "pcmpestri64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpestri_xmm_xmmm128_imm8)], "vpcmpestri" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpestri64_xmm_xmmm128_imm8)], "vpcmpestri64" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpistrm_xmm_xmmm128_imm8)], "pcmpistrm" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpistrm_xmm_xmmm128_imm8)], "vpcmpistrm" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pcmpistri_xmm_xmmm128_imm8)], "pcmpistri" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vpcmpistri_xmm_xmmm128_imm8)], "vpcmpistri" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8)], "vfpclassps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8)], "vfpclassps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8)], "vfpclassps", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8)], "vfpclasspd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8)], "vfpclasspd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8)], "vfpclasspd", new OrEnumValue(instrOpInfoFlags, nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize), nameof(InstrOpInfoFlags.ShowMinMemSize_ForceSize)) },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vfpclassss_kr_k1_xmmm32_imm8)], "vfpclassss", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.EVEX_Vfpclasssd_kr_k1_xmmm64_imm8)], "vfpclasssd", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmmm128_xmm)], "vfmaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymmm256_ymm)], "vfmaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmm_xmmm128)], "vfmaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymm_ymmm256)], "vfmaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm)], "vfmaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm)], "vfmaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128)], "vfmaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256)], "vfmaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmmm32_xmm)], "vfmaddss", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmm_xmmm32)], "vfmaddss", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm)], "vfmaddsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64)], "vfmaddsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmmm128_xmm)], "vfmsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymmm256_ymm)], "vfmsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmm_xmmm128)], "vfmsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymm_ymmm256)], "vfmsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm)], "vfmsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm)], "vfmsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128)], "vfmsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256)], "vfmsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmmm32_xmm)], "vfmsubss", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmm_xmmm32)], "vfmsubss", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm)], "vfmsubsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64)], "vfmsubsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8)], "vpshldw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8)], "vpshldw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8)], "vpshldw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8)], "vpshldd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8)], "vpshldd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8)], "vpshldd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8)], "vpshldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8)], "vpshldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8)], "vpshldq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8)], "vpshrdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8)], "vpshrdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8)], "vpshrdw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8)], "vpshrdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8)], "vpshrdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8)], "vpshrdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8)], "vpshrdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8)], "vpshrdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8)], "vpshrdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm)], "vfnmaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm)], "vfnmaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128)], "vfnmaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256)], "vfnmaddps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm)], "vfnmaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm)], "vfnmaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128)], "vfnmaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256)], "vfnmaddpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm)], "vfnmaddss", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32)], "vfnmaddss", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm)], "vfnmaddsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64)], "vfnmaddsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm)], "vfnmsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm)], "vfnmsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128)], "vfnmsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256)], "vfnmsubps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm)], "vfnmsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm)], "vfnmsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128)], "vfnmsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256)], "vfnmsubpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm)], "vfnmsubss", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32)], "vfnmsubss", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm)], "vfnmsubsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64)], "vfnmsubsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Sha1rnds4_xmm_xmmm128_imm8)], "sha1rnds4" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Gf2p8affineqb_xmm_xmmm128_imm8)], "gf2p8affineqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8)], "vgf2p8affineqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8)], "vgf2p8affineqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8)], "vgf2p8affineqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8)], "vgf2p8affineqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8)], "vgf2p8affineqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Gf2p8affineinvqb_xmm_xmmm128_imm8)], "gf2p8affineinvqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8)], "vgf2p8affineinvqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8)], "vgf2p8affineinvqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8)], "vgf2p8affineinvqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8)], "vgf2p8affineinvqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8)], "vgf2p8affineinvqb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Aeskeygenassist_xmm_xmmm128_imm8)], "aeskeygenassist" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Vaeskeygenassist_xmm_xmmm128_imm8)], "vaeskeygenassist" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Rorx_r32_rm32_imm8)], "rorx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Rorx_r64_rm64_imm8)], "rorx" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacssww_xmm_xmm_xmmm128_xmm)], "vpmacssww" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm)], "vpmacsswd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm)], "vpmacssdql" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm)], "vpmacssdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm)], "vpmacssdqh" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacsww_xmm_xmm_xmmm128_xmm)], "vpmacsww" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacswd_xmm_xmm_xmmm128_xmm)], "vpmacswd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm)], "vpmacsdql" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm)], "vpmacsdd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm)], "vpmacsdqh" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmmm128_xmm)], "vpcmov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymmm256_ymm)], "vpcmov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmm_xmmm128)], "vpcmov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymm_ymmm256)], "vpcmov" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpperm_xmm_xmm_xmmm128_xmm)], "vpperm" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpperm_xmm_xmm_xmm_xmmm128)], "vpperm" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm)], "vpmadcsswd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm)], "vpmadcswd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotb_xmm_xmmm128_imm8)], "vprotb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotw_xmm_xmmm128_imm8)], "vprotw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotd_xmm_xmmm128_imm8)], "vprotd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotq_xmm_xmmm128_imm8)], "vprotq" },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8)], "vpcomb", pseudoOpsKind[nameof(PseudoOpsKind.vpcomb)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8)], "vpcomw", pseudoOpsKind[nameof(PseudoOpsKind.vpcomw)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8)], "vpcomd", pseudoOpsKind[nameof(PseudoOpsKind.vpcomd)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8)], "vpcomq", pseudoOpsKind[nameof(PseudoOpsKind.vpcomq)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8)], "vpcomub", pseudoOpsKind[nameof(PseudoOpsKind.vpcomub)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8)], "vpcomuw", pseudoOpsKind[nameof(PseudoOpsKind.vpcomuw)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8)], "vpcomud", pseudoOpsKind[nameof(PseudoOpsKind.vpcomud)] },
				new object[] { ctorKind[nameof(CtorKind.pops_2)], code[nameof(Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8)], "vpcomuq", pseudoOpsKind[nameof(PseudoOpsKind.vpcomuq)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blcfill_r32_rm32)], "blcfill" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blcfill_r64_rm64)], "blcfill" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blsfill_r32_rm32)], "blsfill" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blsfill_r64_rm64)], "blsfill" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blcs_r32_rm32)], "blcs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blcs_r64_rm64)], "blcs" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Tzmsk_r32_rm32)], "tzmsk" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Tzmsk_r64_rm64)], "tzmsk" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blcic_r32_rm32)], "blcic" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blcic_r64_rm64)], "blcic" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blsic_r32_rm32)], "blsic" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blsic_r64_rm64)], "blsic" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_T1mskc_r32_rm32)], "t1mskc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_T1mskc_r64_rm64)], "t1mskc" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blcmsk_r32_rm32)], "blcmsk" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blcmsk_r64_rm64)], "blcmsk" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blci_r32_rm32)], "blci" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Blci_r64_rm64)], "blci" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Llwpcb_r32)], "llwpcb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Llwpcb_r64)], "llwpcb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Slwpcb_r32)], "slwpcb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Slwpcb_r64)], "slwpcb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vfrczps_xmm_xmmm128)], "vfrczps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vfrczps_ymm_ymmm256)], "vfrczps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vfrczpd_xmm_xmmm128)], "vfrczpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vfrczpd_ymm_ymmm256)], "vfrczpd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.XOP_Vfrczss_xmm_xmmm32)], "vfrczss", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.XOP_Vfrczsd_xmm_xmmm64)], "vfrczsd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_XmmwordPtr)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotb_xmm_xmmm128_xmm)], "vprotb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotb_xmm_xmm_xmmm128)], "vprotb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotw_xmm_xmmm128_xmm)], "vprotw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotw_xmm_xmm_xmmm128)], "vprotw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotd_xmm_xmmm128_xmm)], "vprotd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotd_xmm_xmm_xmmm128)], "vprotd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotq_xmm_xmmm128_xmm)], "vprotq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vprotq_xmm_xmm_xmmm128)], "vprotq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshlb_xmm_xmmm128_xmm)], "vpshlb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshlb_xmm_xmm_xmmm128)], "vpshlb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshlw_xmm_xmmm128_xmm)], "vpshlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshlw_xmm_xmm_xmmm128)], "vpshlw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshld_xmm_xmmm128_xmm)], "vpshld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshld_xmm_xmm_xmmm128)], "vpshld" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshlq_xmm_xmmm128_xmm)], "vpshlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshlq_xmm_xmm_xmmm128)], "vpshlq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshab_xmm_xmmm128_xmm)], "vpshab" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshab_xmm_xmm_xmmm128)], "vpshab" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshaw_xmm_xmmm128_xmm)], "vpshaw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshaw_xmm_xmm_xmmm128)], "vpshaw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshad_xmm_xmmm128_xmm)], "vpshad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshad_xmm_xmm_xmmm128)], "vpshad" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshaq_xmm_xmmm128_xmm)], "vpshaq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vpshaq_xmm_xmm_xmmm128)], "vpshaq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphaddbw_xmm_xmmm128)], "vphaddbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphaddbd_xmm_xmmm128)], "vphaddbd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphaddbq_xmm_xmmm128)], "vphaddbq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphaddwd_xmm_xmmm128)], "vphaddwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphaddwq_xmm_xmmm128)], "vphaddwq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphadddq_xmm_xmmm128)], "vphadddq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphaddubw_xmm_xmmm128)], "vphaddubw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphaddubd_xmm_xmmm128)], "vphaddubd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphaddubq_xmm_xmmm128)], "vphaddubq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphadduwd_xmm_xmmm128)], "vphadduwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphadduwq_xmm_xmmm128)], "vphadduwq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphaddudq_xmm_xmmm128)], "vphaddudq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphsubbw_xmm_xmmm128)], "vphsubbw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphsubwd_xmm_xmmm128)], "vphsubwd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Vphsubdq_xmm_xmmm128)], "vphsubdq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Bextr_r32_rm32_imm32)], "bextr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.XOP_Bextr_r64_rm64_imm32)], "bextr" },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.XOP_Lwpins_r32_rm32_imm32)], "lwpins", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.XOP_Lwpins_r64_rm32_imm32)], "lwpins", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.XOP_Lwpval_r32_rm32_imm32)], "lwpval", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.XOP_Lwpval_r64_rm32_imm32)], "lwpval", instrOpInfoFlags[nameof(InstrOpInfoFlags.ShowNoMemSize_ForceSize)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pi2fw_mm_mmm64)], "pi2fw", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pi2fd_mm_mmm64)], "pi2fd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pf2iw_mm_mmm64)], "pf2iw", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pf2id_mm_mmm64)], "pf2id", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfrcpv_mm_mmm64)], "pfrcpv", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfrsqrtv_mm_mmm64)], "pfrsqrtv", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfnacc_mm_mmm64)], "pfnacc", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfpnacc_mm_mmm64)], "pfpnacc", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfcmpge_mm_mmm64)], "pfcmpge", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfmin_mm_mmm64)], "pfmin", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfrcp_mm_mmm64)], "pfrcp", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfrsqrt_mm_mmm64)], "pfrsqrt", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfsub_mm_mmm64)], "pfsub", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfadd_mm_mmm64)], "pfadd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfcmpgt_mm_mmm64)], "pfcmpgt", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfmax_mm_mmm64)], "pfmax", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfrcpit1_mm_mmm64)], "pfrcpit1", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfrsqit1_mm_mmm64)], "pfrsqit1", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfsubr_mm_mmm64)], "pfsubr", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfacc_mm_mmm64)], "pfacc", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfcmpeq_mm_mmm64)], "pfcmpeq", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfmul_mm_mmm64)], "pfmul", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pfrcpit2_mm_mmm64)], "pfrcpit2", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pmulhrw_mm_mmm64)], "pmulhrw", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pswapd_mm_mmm64)], "pswapd", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_2)], code[nameof(Code.D3NOW_Pavgusb_mm_mmm64)], "pavgusb", instrOpInfoFlags[nameof(InstrOpInfoFlags.MemSize_Mmx)] },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rmpadjust)], "rmpadjust" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Rmpupdate)], "rmpupdate" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Psmash)], "psmash" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pvalidatew)], "pvalidate" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pvalidated)], "pvalidate" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Pvalidateq)], "pvalidate" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Serialize)], "serialize" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xsusldtrk)], "xsusldtrk" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Xresldtrk)], "xresldtrk" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Invlpgbw)], "invlpgb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Invlpgbd)], "invlpgb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Invlpgbq)], "invlpgb" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Tlbsync)], "tlbsync" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.PrefetchReserved3_m8)], "prefetchw" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.PrefetchReserved4_m8)], "prefetch" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.PrefetchReserved5_m8)], "prefetch" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.PrefetchReserved6_m8)], "prefetch" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.PrefetchReserved7_m8)], "prefetch" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Ud0)], "ud0" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Vmgexit)], "vmgexit" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.Getsecq)], "getsecq" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Ldtilecfg_m512)], "ldtilecfg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tilerelease)], "tilerelease" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Sttilecfg_m512)], "sttilecfg" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tilezero_tmm)], "tilezero" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tileloaddt1_tmm_sibmem)], "tileloaddt1" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tilestored_sibmem_tmm)], "tilestored" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tileloadd_tmm_sibmem)], "tileloadd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tdpbf16ps_tmm_tmm_tmm)], "tdpbf16ps" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tdpbuud_tmm_tmm_tmm)], "tdpbuud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tdpbusd_tmm_tmm_tmm)], "tdpbusd" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tdpbsud_tmm_tmm_tmm)], "tdpbsud" },
				new object[] { ctorKind[nameof(CtorKind.Normal_1)], code[nameof(Code.VEX_Tdpbssd_tmm_tmm_tmm)], "tdpbssd" },
			});
		}
	}
}
