/*
Copyright (C) 2018-2019 de4dot@gmail.com

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Linq;
using Generator.Encoder;
using Generator.InstructionInfo;
using Generator.Enums;
using Generator.Enums.Encoder;
using Generator.Enums.InstructionInfo;

namespace Generator.Tables {
	static class InstructionDefsData {
		public static InstructionDef[] CreateTable(GenTypes genTypes) {
			var code = genTypes[TypeIds.Code];
			var mnemonic = genTypes[TypeIds.Mnemonic];
			var memSize = genTypes[TypeIds.MemorySize];
			var encoding = genTypes[TypeIds.EncodingKind];
			var flowControl = genTypes[TypeIds.FlowControl];
			var cpuid = genTypes[TypeIds.CpuidFeature];
			var result = new (int opCount, EnumValue mnemonic, EnumValue mem, EnumValue bcst, OpCodeInfo opCodeInfo, InstrInfo instrInfo)[] {
				(0, mnemonic[nameof(Mnemonic.INVALID)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.INVALID)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x00, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.NoInstruction, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.INVALID)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Db)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.DeclareByte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x00, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.NoInstruction, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.DeclareByte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Dw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.DeclareWord)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x00, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.NoInstruction, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.DeclareWord)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Dd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.DeclareDword)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x00, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.NoInstruction, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.DeclareDword)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Dq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.DeclareQword)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x00, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.NoInstruction, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.DeclareQword)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x00, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Add_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x01, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Add_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x01, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Add_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x01, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Add_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x02, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Add_r8_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x03, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Add_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x03, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Add_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x03, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Add_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x04, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Add_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_AX_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x05, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Add_AX_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_EAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x05, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Add_EAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_RAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x05, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Add_RAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushw_ES)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x06, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.ES }),
				new InstrInfo(code[nameof(Code.Pushw_ES)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushd_ES)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x06, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.ES }),
				new InstrInfo(code[nameof(Code.Pushd_ES)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popw_ES)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x07, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.ES }),
				new InstrInfo(code[nameof(Code.Popw_ES)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popd_ES)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x07, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.ES }),
				new InstrInfo(code[nameof(Code.Popd_ES)], CodeInfo.Pop_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x08, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Or_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x09, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Or_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x09, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Or_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x09, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Or_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Or_r8_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Or_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Or_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0B, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Or_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Or_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_AX_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Or_AX_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_EAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Or_EAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_RAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Or_RAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushw_CS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0E, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.CS }),
				new InstrInfo(code[nameof(Code.Pushw_CS)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushd_CS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0E, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.CS }),
				new InstrInfo(code[nameof(Code.Pushd_CS)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popw_CS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x0F, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.CS }),
				new InstrInfo(code[nameof(Code.Popw_CS)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086_ONLY)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x10, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Adc_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x11, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Adc_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x11, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Adc_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x11, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Adc_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x12, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Adc_r8_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x13, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Adc_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x13, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Adc_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x13, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Adc_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x14, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Adc_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_AX_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x15, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Adc_AX_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_EAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x15, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Adc_EAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_RAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x15, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Adc_RAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushw_SS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x16, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.SS }),
				new InstrInfo(code[nameof(Code.Pushw_SS)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushd_SS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x16, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.SS }),
				new InstrInfo(code[nameof(Code.Pushd_SS)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popw_SS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x17, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.SS }),
				new InstrInfo(code[nameof(Code.Popw_SS)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popd_SS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x17, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.SS }),
				new InstrInfo(code[nameof(Code.Popd_SS)], CodeInfo.Pop_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x18, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Sbb_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x19, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Sbb_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x19, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Sbb_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x19, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Sbb_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Sbb_r8_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Sbb_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Sbb_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1B, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Sbb_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sbb_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_AX_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Sbb_AX_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_EAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Sbb_EAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_RAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Sbb_RAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushw_DS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1E, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.DS }),
				new InstrInfo(code[nameof(Code.Pushw_DS)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushd_DS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1E, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.DS }),
				new InstrInfo(code[nameof(Code.Pushd_DS)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popw_DS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1F, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.DS }),
				new InstrInfo(code[nameof(Code.Popw_DS)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popd_DS)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x1F, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.DS }),
				new InstrInfo(code[nameof(Code.Popd_DS)], CodeInfo.Pop_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x20, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.And_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x21, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.And_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x21, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.And_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x21, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.And_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x22, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.And_r8_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x23, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.And_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x23, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.And_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x23, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.And_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x24, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.And_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_AX_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x25, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.And_AX_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_EAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x25, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.And_EAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_RAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x25, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.And_RAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Daa)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Daa)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x27, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Daa)], CodeInfo.RW_AL, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.AF | RflagsBits.CF, RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x28, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Sub_rm8_r8)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x29, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Sub_rm16_r16)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x29, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Sub_rm32_r32)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x29, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Sub_rm64_r64)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x2A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Sub_r8_rm8)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x2B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Sub_r16_rm16)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x2B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Sub_r32_rm32)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x2B, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Sub_r64_rm64)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x2C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sub_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_AX_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x2D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Sub_AX_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_EAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x2D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Sub_EAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_RAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x2D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Sub_RAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Das)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Das)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x2F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Das)], CodeInfo.RW_AL, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.AF | RflagsBits.CF, RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x30, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Xor_rm8_r8)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x31, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Xor_rm16_r16)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x31, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Xor_rm32_r32)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x31, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Xor_rm64_r64)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x32, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Xor_r8_rm8)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x33, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Xor_r16_rm16)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x33, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Xor_r32_rm32)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x33, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Xor_r64_rm64)], CodeInfo.Clear_rflags, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x34, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Xor_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_AX_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x35, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Xor_AX_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_EAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x35, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Xor_EAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_RAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x35, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Xor_RAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Aaa)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aaa)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x37, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Aaa)], CodeInfo.RW_AX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.AF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x38, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Cmp_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x39, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Cmp_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x39, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Cmp_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x39, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Cmp_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x3A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Cmp_r8_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x3B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmp_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x3B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmp_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x3B, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmp_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x3C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Cmp_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_AX_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x3D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Cmp_AX_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_EAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x3D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Cmp_EAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_RAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x3D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Cmp_RAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Aas)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aas)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x3F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Aas)], CodeInfo.RW_AX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.AF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Inc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Inc_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x40, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.r16_rw }),
				new InstrInfo(code[nameof(Code.Inc_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Inc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Inc_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x40, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.r32_rd }),
				new InstrInfo(code[nameof(Code.Inc_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Dec)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Dec_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x48, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.r16_rw }),
				new InstrInfo(code[nameof(Code.Dec_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Dec)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Dec_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x48, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.r32_rd }),
				new InstrInfo(code[nameof(Code.Dec_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Push_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x50, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.r16_rw }),
				new InstrInfo(code[nameof(Code.Push_r16)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Push_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x50, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.r32_rd }),
				new InstrInfo(code[nameof(Code.Push_r32)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Push_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x50, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.r64_ro }),
				new InstrInfo(code[nameof(Code.Push_r64)], CodeInfo.Push_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pop_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x58, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.r16_rw }),
				new InstrInfo(code[nameof(Code.Pop_r16)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pop_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x58, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.r32_rd }),
				new InstrInfo(code[nameof(Code.Pop_r32)], CodeInfo.Pop_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pop_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x58, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.r64_ro }),
				new InstrInfo(code[nameof(Code.Pop_r64)], CodeInfo.Pop_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Pusha)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushaw)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x60, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pushaw)], CodeInfo.Pusha, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Pusha)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushad)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x60, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pushad)], CodeInfo.Pusha, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Popa)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popaw)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x61, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Popaw)], CodeInfo.Popa, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Popa)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popad)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x61, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Popad)], CodeInfo.Popa, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(2, mnemonic[nameof(Mnemonic.Bound)], memSize[nameof(MemorySize.Bound16_WordWord)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bound_r16_m1616)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x62, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gw, LegacyOpKind.Mw2 }),
				new InstrInfo(code[nameof(Code.Bound_r16_m1616)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bound)], memSize[nameof(MemorySize.Bound32_DwordDword)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bound_r32_m3232)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x62, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gd, LegacyOpKind.Md2 }),
				new InstrInfo(code[nameof(Code.Bound_r32_m3232)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Arpl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Arpl_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x63, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Arpl_rm16_r16)], CodeInfo.Arpl, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.ReadCondWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Arpl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Arpl_r32m16_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x63, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.RdMw, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Arpl_r32m16_r32)], CodeInfo.Arpl, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadCondWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movsxd)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsxd_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x63, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Movsxd_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsxd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsxd_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x63, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Movsxd_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsxd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsxd_r64_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x63, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Movsxd_r64_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Push_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x68, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Push_imm16)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushd_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x68, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Pushd_imm32)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushq_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x68, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Pushq_imm32)], CodeInfo.Push_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(3, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_r16_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x69, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Imul_r16_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_r32_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x69, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Imul_r32_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_r64_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x69, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Imul_r64_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushw_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6A, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.Pushw_imm8)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushd_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6A, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.Pushd_imm8)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushq_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.Pushq_imm8)], CodeInfo.Push_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(3, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_r16_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew, LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.Imul_r16_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_r32_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed, LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.Imul_r32_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_r64_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6B, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq, LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.Imul_r64_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Insb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Insb_m8_DX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yb, LegacyOpKind.DX }),
				new InstrInfo(code[nameof(Code.Insb_m8_DX)], CodeInfo.Ins, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Insw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Insw_m16_DX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yw, LegacyOpKind.DX }),
				new InstrInfo(code[nameof(Code.Insw_m16_DX)], CodeInfo.Ins, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Insd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Insd_m32_DX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yd, LegacyOpKind.DX }),
				new InstrInfo(code[nameof(Code.Insd_m32_DX)], CodeInfo.Ins, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Outsb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Outsb_DX_m8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.DX, LegacyOpKind.Xb }),
				new InstrInfo(code[nameof(Code.Outsb_DX_m8)], CodeInfo.Outs, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Outsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Outsw_DX_m16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6F, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.DX, LegacyOpKind.Xw }),
				new InstrInfo(code[nameof(Code.Outsw_DX_m16)], CodeInfo.Outs, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Outsd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Outsd_DX_m32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x6F, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.DX, LegacyOpKind.Xd }),
				new InstrInfo(code[nameof(Code.Outsd_DX_m32)], CodeInfo.Outs, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Jo)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jo_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x70, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jo_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jo)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jo_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x70, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jo_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jo)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jo_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x70, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jo_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jno)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jno_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x71, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jno_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jno)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jno_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x71, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jno_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jno)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jno_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x71, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jno_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jb_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x72, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jb_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jb_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x72, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jb_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jb_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x72, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jb_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jae)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jae_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x73, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jae_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jae)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jae_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x73, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jae_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jae)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jae_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x73, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jae_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Je)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Je_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x74, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Je_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Je)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Je_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x74, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Je_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Je)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Je_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x74, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Je_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jne_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x75, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jne_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jne_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x75, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jne_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jne_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x75, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jne_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jbe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jbe_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x76, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jbe_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jbe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jbe_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x76, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jbe_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jbe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jbe_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x76, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jbe_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ja)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ja_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x77, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Ja_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ja)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ja_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x77, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Ja_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ja)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ja_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x77, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Ja_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Js)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Js_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x78, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Js_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Js)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Js_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x78, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Js_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Js)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Js_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x78, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Js_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jns)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jns_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x79, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jns_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jns)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jns_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x79, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jns_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jns)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jns_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x79, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jns_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jp_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7A, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jp_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jp_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7A, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jp_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jp_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jp_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jnp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jnp_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jnp_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jnp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jnp_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jnp_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jnp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jnp_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jnp_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jl)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jl_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7C, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jl_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jl)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jl_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7C, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jl_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jl)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jl_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jl_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jge)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jge_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jge_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jge)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jge_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jge_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jge)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jge_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jge_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jle)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jle_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7E, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jle_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jle)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jle_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7E, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jle_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jle)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jle_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jle_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jg_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7F, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jg_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jg_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7F, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jg_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jg_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x7F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jg_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x80, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Add_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x80, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Or_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x80, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Adc_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x80, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sbb_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x80, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.And_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x80, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sub_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x80, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Xor_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x80, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Cmp_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Add_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Add_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Add_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Or_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Or_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Or_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 2, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Adc_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 2, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Adc_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 2, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Adc_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 3, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Sbb_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 3, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Sbb_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Sbb_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.And_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.And_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.And_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 5, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Sub_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 5, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Sub_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Sub_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Xor_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Xor_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Xor_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 7, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Cmp_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 7, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Cmp_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x81, 7, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Cmp_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm8_imm8_82)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x82, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Add_rm8_imm8_82)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm8_imm8_82)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x82, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Or_rm8_imm8_82)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm8_imm8_82)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x82, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Adc_rm8_imm8_82)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm8_imm8_82)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x82, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sbb_rm8_imm8_82)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm8_imm8_82)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x82, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.And_rm8_imm8_82)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm8_imm8_82)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x82, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sub_rm8_imm8_82)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm8_imm8_82)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x82, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Xor_rm8_imm8_82)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm8_imm8_82)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x82, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Cmp_rm8_imm8_82)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.Add_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.Add_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Add)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Add_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.Add_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.Or_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.Or_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Or)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Or_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.Or_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 2, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.Adc_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 2, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.Adc_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adc)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adc_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 2, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.Adc_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 3, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.Sbb_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 3, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.Sbb_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sbb)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sbb_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.Sbb_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.And_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.And_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.And)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.And_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.And_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 5, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.Sub_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 5, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.Sub_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sub)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sub_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.Sub_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.Xor_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.Xor_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xor)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xor_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.Xor_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 7, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib16 }),
				new InstrInfo(code[nameof(Code.Cmp_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 7, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib32 }),
				new InstrInfo(code[nameof(Code.Cmp_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmp)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmp_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x83, 7, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib64 }),
				new InstrInfo(code[nameof(Code.Cmp_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x84, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Test_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x85, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Test_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x85, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Test_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x85, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Test_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xchg)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xchg_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x86, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Xchg_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xchg)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xchg_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x87, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Xchg_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xchg)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xchg_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x87, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Xchg_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xchg)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xchg_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x87, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Xchg_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x88, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Mov_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x89, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Mov_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x89, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Mov_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x89, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Mov_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Mov_r8_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Mov_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Mov_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8B, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Mov_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_rm16_Sreg)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8C, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Sw }),
				new InstrInfo(code[nameof(Code.Mov_rm16_Sreg)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r32m16_Sreg)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8C, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Sw }),
				new InstrInfo(code[nameof(Code.Mov_r32m16_Sreg)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r64m16_Sreg)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8C, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Sw }),
				new InstrInfo(code[nameof(Code.Mov_r64m16_Sreg)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lea)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lea_r16_m)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Lea_r16_m)], CodeInfo.Lea, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Lea)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lea_r32_m)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Lea_r32_m)], CodeInfo.Lea, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Lea)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lea_r64_m)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Lea_r64_m)], CodeInfo.Lea, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_Sreg_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8E, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Sw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Mov_Sreg_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_Sreg_r32m16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8E, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Sw, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Mov_Sreg_r32m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_Sreg_r64m16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8E, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Sw, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Mov_Sreg_r64m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pop_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8F, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Pop_rm16)], CodeInfo.Pop_Ev, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pop_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8F, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Pop_rm32)], CodeInfo.Pop_Ev, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pop_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x8F, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Pop_rm64)], CodeInfo.Pop_Ev, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Nop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Nopw)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x90, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Nopw)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Nop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Nopd)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x90, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Nopd)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Nop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Nopq)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x90, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Nopq)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xchg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xchg_r16_AX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x90, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.r16_rw, LegacyOpKind.AX }),
				new InstrInfo(code[nameof(Code.Xchg_r16_AX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xchg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xchg_r32_EAX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x90, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.r32_rd, LegacyOpKind.EAX }),
				new InstrInfo(code[nameof(Code.Xchg_r32_EAX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xchg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xchg_r64_RAX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x90, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.r64_ro, LegacyOpKind.RAX }),
				new InstrInfo(code[nameof(Code.Xchg_r64_RAX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Pause)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pause)], MandatoryPrefix.PF3, OpCodeTableKind.Normal, 0x90, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pause)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PAUSE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Cbw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cbw)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x98, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cbw)], CodeInfo.R_AL_W_AH, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Cwde)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cwde)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x98, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cwde)], CodeInfo.Cwde, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Cdqe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cdqe)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x98, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cdqe)], CodeInfo.Cdqe, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Cwd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cwd)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x99, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cwd)], CodeInfo.Cwd, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Cdq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cdq)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x99, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cdq)], CodeInfo.Cdq, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Cqo)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cqo)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x99, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cqo)], CodeInfo.Cqo, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_ptr1616)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9A, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Aww }),
				new InstrInfo(code[nameof(Code.Call_ptr1616)], CodeInfo.Push_2_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_ptr1632)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9A, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Adw }),
				new InstrInfo(code[nameof(Code.Call_ptr1632)], CodeInfo.Push_4_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Wait)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wait)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Wait)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Pushf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushfw)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9C, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pushfw)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Pushf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushfd)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9C, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pushfd)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Pushf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushfq)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pushfq)], CodeInfo.Push_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Popf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popfw)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Popfw)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Popf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popfd)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Popfd)], CodeInfo.Pop_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Popf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popfq)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Popfq)], CodeInfo.Pop_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Sahf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sahf)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sahf)], CodeInfo.Lahf, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Lahf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lahf)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0x9F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Lahf)], CodeInfo.Lahf, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_AL_moffs8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ob }),
				new InstrInfo(code[nameof(Code.Mov_AL_moffs8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_AX_moffs16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA1, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Ow }),
				new InstrInfo(code[nameof(Code.Mov_AX_moffs16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_EAX_moffs32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA1, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Od }),
				new InstrInfo(code[nameof(Code.Mov_EAX_moffs32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_RAX_moffs64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA1, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Oq }),
				new InstrInfo(code[nameof(Code.Mov_RAX_moffs64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_moffs8_AL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ob, LegacyOpKind.AL }),
				new InstrInfo(code[nameof(Code.Mov_moffs8_AL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_moffs16_AX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA3, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ow, LegacyOpKind.AX }),
				new InstrInfo(code[nameof(Code.Mov_moffs16_AX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_moffs32_EAX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA3, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Od, LegacyOpKind.EAX }),
				new InstrInfo(code[nameof(Code.Mov_moffs32_EAX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_moffs64_RAX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA3, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Oq, LegacyOpKind.RAX }),
				new InstrInfo(code[nameof(Code.Mov_moffs64_RAX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsb_m8_m8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yb, LegacyOpKind.Xb }),
				new InstrInfo(code[nameof(Code.Movsb_m8_m8)], CodeInfo.Movs, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsw_m16_m16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA5, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yw, LegacyOpKind.Xw }),
				new InstrInfo(code[nameof(Code.Movsw_m16_m16)], CodeInfo.Movs, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsd_m32_m32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA5, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yd, LegacyOpKind.Xd }),
				new InstrInfo(code[nameof(Code.Movsd_m32_m32)], CodeInfo.Movs, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsq_m64_m64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA5, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yq, LegacyOpKind.Xq }),
				new InstrInfo(code[nameof(Code.Movsq_m64_m64)], CodeInfo.Movs, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpsb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpsb_m8_m8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix | OpCodeFlags.RepnePrefix, new[] { LegacyOpKind.Xb, LegacyOpKind.Yb }),
				new InstrInfo(code[nameof(Code.Cmpsb_m8_m8)], CodeInfo.Cmps, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpsw_m16_m16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA7, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix | OpCodeFlags.RepnePrefix, new[] { LegacyOpKind.Xw, LegacyOpKind.Yw }),
				new InstrInfo(code[nameof(Code.Cmpsw_m16_m16)], CodeInfo.Cmps, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpsd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpsd_m32_m32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA7, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix | OpCodeFlags.RepnePrefix, new[] { LegacyOpKind.Xd, LegacyOpKind.Yd }),
				new InstrInfo(code[nameof(Code.Cmpsd_m32_m32)], CodeInfo.Cmps, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpsq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpsq_m64_m64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA7, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix | OpCodeFlags.RepnePrefix, new[] { LegacyOpKind.Xq, LegacyOpKind.Yq }),
				new InstrInfo(code[nameof(Code.Cmpsq_m64_m64)], CodeInfo.Cmps, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Test_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_AX_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA9, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Test_AX_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_EAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA9, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Test_EAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_RAX_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xA9, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RAX, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Test_RAX_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Stosb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Stosb_m8_AL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yb, LegacyOpKind.AL }),
				new InstrInfo(code[nameof(Code.Stosb_m8_AL)], CodeInfo.Stos, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Stosw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Stosw_m16_AX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAB, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yw, LegacyOpKind.AX }),
				new InstrInfo(code[nameof(Code.Stosw_m16_AX)], CodeInfo.Stos, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Stosd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Stosd_m32_EAX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAB, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yd, LegacyOpKind.EAX }),
				new InstrInfo(code[nameof(Code.Stosd_m32_EAX)], CodeInfo.Stos, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Stosq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Stosq_m64_RAX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAB, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.Yq, LegacyOpKind.RAX }),
				new InstrInfo(code[nameof(Code.Stosq_m64_RAX)], CodeInfo.Stos, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lodsb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lodsb_AL_m8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.AL, LegacyOpKind.Xb }),
				new InstrInfo(code[nameof(Code.Lodsb_AL_m8)], CodeInfo.Lods, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lodsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lodsw_AX_m16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAD, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.AX, LegacyOpKind.Xw }),
				new InstrInfo(code[nameof(Code.Lodsw_AX_m16)], CodeInfo.Lods, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lodsd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lodsd_EAX_m32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAD, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.EAX, LegacyOpKind.Xd }),
				new InstrInfo(code[nameof(Code.Lodsd_EAX_m32)], CodeInfo.Lods, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lodsq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lodsq_RAX_m64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAD, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, new[] { LegacyOpKind.RAX, LegacyOpKind.Xq }),
				new InstrInfo(code[nameof(Code.Lodsq_RAX_m64)], CodeInfo.Lods, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Scasb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Scasb_AL_m8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix | OpCodeFlags.RepnePrefix, new[] { LegacyOpKind.AL, LegacyOpKind.Yb }),
				new InstrInfo(code[nameof(Code.Scasb_AL_m8)], CodeInfo.Scas, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Scasw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Scasw_AX_m16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAF, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix | OpCodeFlags.RepnePrefix, new[] { LegacyOpKind.AX, LegacyOpKind.Yw }),
				new InstrInfo(code[nameof(Code.Scasw_AX_m16)], CodeInfo.Scas, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Scasd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Scasd_EAX_m32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAF, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix | OpCodeFlags.RepnePrefix, new[] { LegacyOpKind.EAX, LegacyOpKind.Yd }),
				new InstrInfo(code[nameof(Code.Scasd_EAX_m32)], CodeInfo.Scas, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Scasq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Scasq_RAX_m64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xAF, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix | OpCodeFlags.RepnePrefix, new[] { LegacyOpKind.RAX, LegacyOpKind.Yq }),
				new InstrInfo(code[nameof(Code.Scasq_RAX_m64)], CodeInfo.Scas, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.DF, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xB0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.r8_rb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Mov_r8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xB8, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.r16_rw, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Mov_r16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xB8, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.r32_rd, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Mov_r32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r64_imm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xB8, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.r64_ro, LegacyOpKind.Iq }),
				new InstrInfo(code[nameof(Code.Mov_r64_imm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC0, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rol_rm8_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC0, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Ror_rm8_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC0, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rcl_rm8_imm8)], CodeInfo.Shift_Ib_MASK1FMOD9, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC0, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rcr_rm8_imm8)], CodeInfo.Shift_Ib_MASK1FMOD9, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC0, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shl_rm8_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC0, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shr_rm8_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC0, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sal_rm8_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC0, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sar_rm8_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rol_rm16_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rol_rm32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rol_rm64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Ror_rm16_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Ror_rm32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Ror_rm64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 2, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rcl_rm16_imm8)], CodeInfo.Shift_Ib_MASK1FMOD11, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 2, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rcl_rm32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 2, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rcl_rm64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 3, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rcr_rm16_imm8)], CodeInfo.Shift_Ib_MASK1FMOD11, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 3, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rcr_rm32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Rcr_rm64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shl_rm16_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shl_rm32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shl_rm64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 5, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shr_rm16_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 5, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shr_rm32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shr_rm64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sal_rm16_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sal_rm32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sal_rm64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 7, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sar_rm16_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 7, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sar_rm32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC1, 7, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sar_rm64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retnw_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC2, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix, new[] { LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Retnw_imm16)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Ret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retnd_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC2, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix, new[] { LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Retnd_imm16)], CodeInfo.Pop_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Ret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retnq_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix, new[] { LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Retnq_imm16)], CodeInfo.Pop_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Ret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retnw)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC3, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Retnw)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Ret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retnd)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC3, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Retnd)], CodeInfo.Pop_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Ret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retnq)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Retnq)], CodeInfo.Pop_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(2, mnemonic[nameof(Mnemonic.Les)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Les_r16_m1616)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC4, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gw, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Les_r16_m1616)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Les)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Les_r32_m1632)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC4, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gd, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Les_r32_m1632)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lds)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lds_r16_m1616)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC5, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gw, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lds_r16_m1616)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lds)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lds_r32_m1632)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC5, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gd, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lds_r32_m1632)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC6, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Mov_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Xabort)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xabort_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC6F8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Xabort_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.XbeginXabortXend)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RTM)] }, new[] { OpInfo.Read }, InstrInfoFlags.SaveRestore)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC7, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Mov_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC7, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Mov_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC7, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Mov_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Xbegin)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xbegin_rel16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC7F8, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Jxw }),
				new InstrInfo(code[nameof(Code.Xbegin_rel16)], CodeInfo.CW_EAX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.XbeginXabortXend)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RTM)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Xbegin)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xbegin_rel32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC7F8, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Jxd }),
				new InstrInfo(code[nameof(Code.Xbegin_rel32)], CodeInfo.CW_EAX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.XbeginXabortXend)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RTM)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Enter)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enterw_imm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC8, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Iw, LegacyOpKind.Ib21 }),
				new InstrInfo(code[nameof(Code.Enterw_imm16_imm8)], CodeInfo.Enter, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(2, mnemonic[nameof(Mnemonic.Enter)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enterd_imm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC8, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Iw, LegacyOpKind.Ib21 }),
				new InstrInfo(code[nameof(Code.Enterd_imm16_imm8)], CodeInfo.Enter, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(2, mnemonic[nameof(Mnemonic.Enter)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enterq_imm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Iw, LegacyOpKind.Ib21 }),
				new InstrInfo(code[nameof(Code.Enterq_imm16_imm8)], CodeInfo.Enter, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Leave)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Leavew)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC9, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Leavew)], CodeInfo.Leave, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL186)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Leave)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Leaved)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC9, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Leaved)], CodeInfo.Leave, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Leave)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Leaveq)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xC9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Leaveq)], CodeInfo.Leave, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Retf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retfw_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCA, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Retfw_imm16)], CodeInfo.Pop_2_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Retf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retfd_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCA, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Retfd_imm16)], CodeInfo.Pop_4_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Retf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retfq_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCA, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Retfq_imm16)], CodeInfo.Pop_8_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Retf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retfw)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCB, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Retfw)], CodeInfo.Pop_2_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Retf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retfd)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCB, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Retfd)], CodeInfo.Pop_4_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Retf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Retfq)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCB, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Retfq)], CodeInfo.Pop_8_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Int)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Int3)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Int3)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Interrupt)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Int)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Int_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Int_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Interrupt)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Into)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Into)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Into)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Interrupt)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Iret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Iretw)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCF, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Iretw)], CodeInfo.Iret, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Iret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Iretd)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCF, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Iretd)], CodeInfo.Iret, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Iret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Iretq)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xCF, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Iretq)], CodeInfo.Iret, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.StackInstruction)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm8_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD0, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rol_rm8_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm8_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD0, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Ror_rm8_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm8_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD0, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rcl_rm8_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm8_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD0, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rcr_rm8_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm8_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD0, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Shl_rm8_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm8_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD0, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Shr_rm8_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm8_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD0, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Sal_rm8_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm8_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD0, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Sar_rm8_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm16_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rol_rm16_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm32_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rol_rm32_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm64_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rol_rm64_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm16_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Ror_rm16_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm32_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Ror_rm32_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm64_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Ror_rm64_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm16_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 2, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rcl_rm16_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm32_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 2, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rcl_rm32_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm64_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 2, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rcl_rm64_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm16_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 3, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rcr_rm16_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm32_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 3, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rcr_rm32_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm64_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Rcr_rm64_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm16_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Shl_rm16_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm32_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Shl_rm32_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm64_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Shl_rm64_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm16_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 5, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Shr_rm16_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm32_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 5, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Shr_rm32_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm64_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Shr_rm64_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm16_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Sal_rm16_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm32_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Sal_rm32_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm64_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Sal_rm64_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm16_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 7, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Sar_rm16_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm32_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 7, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Sar_rm32_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm64_1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD1, 7, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Imm1 }),
				new InstrInfo(code[nameof(Code.Sar_rm64_1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm8_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD2, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rol_rm8_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm8_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD2, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Ror_rm8_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm8_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD2, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rcl_rm8_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm8_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD2, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rcr_rm8_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm8_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD2, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shl_rm8_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm8_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD2, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shr_rm8_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm8_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD2, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Sal_rm8_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm8_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD2, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Sar_rm8_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm16_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rol_rm16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm32_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rol_rm32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rol)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rol_rm64_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rol_rm64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm16_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Ror_rm16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm32_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Ror_rm32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ror)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ror_rm64_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Ror_rm64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm16_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 2, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rcl_rm16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm32_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 2, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rcl_rm32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcl)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcl_rm64_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 2, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rcl_rm64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm16_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 3, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rcr_rm16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm32_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 3, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rcr_rm32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Rcr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcr_rm64_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Rcr_rm64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.OF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm16_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shl_rm16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm32_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shl_rm32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shl)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shl_rm64_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shl_rm64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm16_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 5, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shr_rm16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm32_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 5, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shr_rm32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Shr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shr_rm64_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shr_rm64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm16_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Sal_rm16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm32_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Sal_rm32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sal)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sal_rm64_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Sal_rm64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm16_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 7, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Sar_rm16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm32_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 7, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Sar_rm32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sar)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sar_rm64_CL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD3, 7, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Sar_rm64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Aam)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aam_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Aam_imm8)], CodeInfo.R_AL_W_AX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.AF | RflagsBits.CF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Aad)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aad_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Aad_imm8)], CodeInfo.RW_AX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.AF | RflagsBits.CF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Salc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Salc)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Salc)], CodeInfo.Salc, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Xlatb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xlat_m8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.MRBX }),
				new InstrInfo(code[nameof(Code.Xlat_m8)], CodeInfo.RW_AL, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fadd)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fadd_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fadd_m32fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fmul)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fmul_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fmul_m32fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fcom)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcom_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fcom_m32fp)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fcomp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcomp_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fcomp_m32fp)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fsub)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsub_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fsub_m32fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fsubr)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsubr_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fsubr_m32fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fdiv)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdiv_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fdiv_m32fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fdivr)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdivr_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fdivr_m32fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fadd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fadd_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8C0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fadd_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fmul)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fmul_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8C8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fmul_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcom)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcom_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8D0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcom_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcomp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcomp_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8D8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcomp_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fsub)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsub_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8E0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fsub_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fsubr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsubr_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8E8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fsubr_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fdiv)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdiv_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8F0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fdiv_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fdivr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdivr_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD8F8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fdivr_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fld)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fld_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fld_m32fp)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fst)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fst_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fst_m32fp)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstp_m32fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf32 }),
				new InstrInfo(code[nameof(Code.Fstp_m32fp)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fldenv)], memSize[nameof(MemorySize.FpuEnv14)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fldenv_m14byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M14 }),
				new InstrInfo(code[nameof(Code.Fldenv_m14byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fldenv)], memSize[nameof(MemorySize.FpuEnv28)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fldenv_m28byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M28 }),
				new InstrInfo(code[nameof(Code.Fldenv_m28byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fldcw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fldcw_m2byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mw }),
				new InstrInfo(code[nameof(Code.Fldcw_m2byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fnstenv)], memSize[nameof(MemorySize.FpuEnv14)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnstenv_m14byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M14 }),
				new InstrInfo(code[nameof(Code.Fnstenv_m14byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstenv)], memSize[nameof(MemorySize.FpuEnv14)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstenv_m14byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, new[] { LegacyOpKind.M14 }),
				new InstrInfo(code[nameof(Code.Fstenv_m14byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fnstenv)], memSize[nameof(MemorySize.FpuEnv28)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnstenv_m28byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M28 }),
				new InstrInfo(code[nameof(Code.Fnstenv_m28byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstenv)], memSize[nameof(MemorySize.FpuEnv28)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstenv_m28byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, new[] { LegacyOpKind.M28 }),
				new InstrInfo(code[nameof(Code.Fstenv_m28byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fnstcw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnstcw_m2byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mw }),
				new InstrInfo(code[nameof(Code.Fnstcw_m2byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstcw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstcw_m2byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, new[] { LegacyOpKind.Mw }),
				new InstrInfo(code[nameof(Code.Fstcw_m2byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fld_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9C0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fld_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fxch)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fxch_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9C8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fxch_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fnop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnop)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9D0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fnop)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstpnce)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstpnce_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9D8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fstpnce_sti)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fchs)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fchs)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9E0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fchs)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fabs)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fabs)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9E1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fabs)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Ftst)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ftst)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9E4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Ftst)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fxam)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fxam)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9E5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fxam)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fld1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fld1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9E8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fld1)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fldl2t)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fldl2t)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9E9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fldl2t)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fldl2e)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fldl2e)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9EA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fldl2e)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fldpi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fldpi)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9EB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fldpi)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fldlg2)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fldlg2)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9EC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fldlg2)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fldln2)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fldln2)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9ED, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fldln2)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fldz)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fldz)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9EE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fldz)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.F2xm1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.F2xm1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.F2xm1)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fyl2x)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fyl2x)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fyl2x)], CodeInfo.R_ST0_RW_ST1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fptan)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fptan)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fptan)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fpatan)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fpatan)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fpatan)], CodeInfo.R_ST0_RW_ST1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fxtract)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fxtract)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fxtract)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fprem1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fprem1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fprem1)], CodeInfo.RW_ST0_R_ST1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU387)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fdecstp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdecstp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fdecstp)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fincstp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fincstp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fincstp)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fprem)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fprem)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fprem)], CodeInfo.RW_ST0_R_ST1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fyl2xp1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fyl2xp1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9F9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fyl2xp1)], CodeInfo.R_ST0_RW_ST1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fsqrt)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsqrt)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9FA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fsqrt)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fsincos)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsincos)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9FB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fsincos)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU387)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Frndint)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Frndint)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9FC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Frndint)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fscale)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fscale)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9FD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fscale)], CodeInfo.RW_ST0_R_ST1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fsin)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsin)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9FE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fsin)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU387)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fcos)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcos)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xD9FF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fcos)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU387)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fiadd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fiadd_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDA, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fiadd_m32int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fimul)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fimul_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDA, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fimul_m32int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ficom)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ficom_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDA, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Ficom_m32int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ficomp)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ficomp_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDA, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Ficomp_m32int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fisub)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fisub_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDA, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fisub_m32int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fisubr)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fisubr_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDA, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fisubr_m32int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fidiv)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fidiv_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDA, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fidiv_m32int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fidivr)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fidivr_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDA, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fidivr_m32int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcmovb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcmovb_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDAC0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcmovb_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.CondRead }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcmove)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcmove_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDAC8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcmove_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.CondRead }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcmovbe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcmovbe_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDAD0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcmovbe_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.CondRead }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcmovu)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcmovu_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDAD8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcmovu_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.CondRead }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fucompp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fucompp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDAE9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fucompp)], CodeInfo.R_ST0_R_ST1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU387)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fild)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fild_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDB, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fild_m32int)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fisttp)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fisttp_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDB, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fisttp_m32int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fist)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fist_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDB, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fist_m32int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fistp)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fistp_m32int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDB, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi32 }),
				new InstrInfo(code[nameof(Code.Fistp_m32int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fld)], memSize[nameof(MemorySize.Float80)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fld_m80fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDB, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf80 }),
				new InstrInfo(code[nameof(Code.Fld_m80fp)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstp)], memSize[nameof(MemorySize.Float80)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstp_m80fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDB, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf80 }),
				new InstrInfo(code[nameof(Code.Fstp_m80fp)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcmovnb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcmovnb_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBC0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcmovnb_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.CondRead }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcmovne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcmovne_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBC8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcmovne_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.CondRead }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcmovnbe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcmovnbe_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBD0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcmovnbe_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.CondRead }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcmovnu)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcmovnu_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBD8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcmovnu_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.CondRead }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fneni)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fneni)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fneni)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Feni)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Feni)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Feni)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fndisi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fndisi)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fndisi)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fdisi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdisi)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fdisi)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fnclex)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnclex)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fnclex)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fclex)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fclex)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fclex)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fninit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fninit)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fninit)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Finit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Finit)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Finit)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fnsetpm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnsetpm)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fnsetpm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU287)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fsetpm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsetpm)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fsetpm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU287)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Frstpm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Frstpm)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Frstpm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU287XL_ONLY)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fucomi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fucomi_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBE8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fucomi_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcomi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcomi_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDBF0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcomi_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fadd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fadd_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDC, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fadd_m64fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fmul)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fmul_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDC, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fmul_m64fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fcom)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcom_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDC, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fcom_m64fp)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fcomp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcomp_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDC, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fcomp_m64fp)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fsub)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsub_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDC, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fsub_m64fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fsubr)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsubr_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDC, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fsubr_m64fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fdiv)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdiv_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDC, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fdiv_m64fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fdivr)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdivr_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDC, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fdivr_m64fp)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fadd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fadd_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDCC0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fadd_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fmul)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fmul_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDCC8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fmul_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcom)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcom_st0_sti_DCD0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDCD0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcom_st0_sti_DCD0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcomp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcomp_st0_sti_DCD8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDCD8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcomp_st0_sti_DCD8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fsubr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsubr_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDCE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fsubr_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fsub)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsub_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDCE8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fsub_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fdivr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdivr_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDCF0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fdivr_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fdiv)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdiv_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDCF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fdiv_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fld)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fld_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fld_m64fp)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fisttp)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fisttp_m64int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fisttp_m64int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fst)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fst_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fst_m64fp)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstp_m64fp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mf64 }),
				new InstrInfo(code[nameof(Code.Fstp_m64fp)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Frstor)], memSize[nameof(MemorySize.FpuState94)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Frstor_m94byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M98 }),
				new InstrInfo(code[nameof(Code.Frstor_m94byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Frstor)], memSize[nameof(MemorySize.FpuState108)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Frstor_m108byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M108 }),
				new InstrInfo(code[nameof(Code.Frstor_m108byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Fnsave)], memSize[nameof(MemorySize.FpuState94)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnsave_m94byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M98 }),
				new InstrInfo(code[nameof(Code.Fnsave_m94byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Fsave)], memSize[nameof(MemorySize.FpuState94)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsave_m94byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, new[] { LegacyOpKind.M98 }),
				new InstrInfo(code[nameof(Code.Fsave_m94byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Fnsave)], memSize[nameof(MemorySize.FpuState108)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnsave_m108byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M108 }),
				new InstrInfo(code[nameof(Code.Fnsave_m108byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Fsave)], memSize[nameof(MemorySize.FpuState108)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsave_m108byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, new[] { LegacyOpKind.M108 }),
				new InstrInfo(code[nameof(Code.Fsave_m108byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Fnstsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnstsw_m2byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mw }),
				new InstrInfo(code[nameof(Code.Fnstsw_m2byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstsw_m2byte)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDD, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, new[] { LegacyOpKind.Mw }),
				new InstrInfo(code[nameof(Code.Fstsw_m2byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ffree)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ffree_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDDC0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Ffree_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fxch)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fxch_st0_sti_DDC8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDDC8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fxch_st0_sti_DDC8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fst)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fst_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDDD0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fst_sti)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstp_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDDD8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fstp_sti)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fucom)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fucom_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDDE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fucom_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fucomp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fucomp_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDDE8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fucomp_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fiadd)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fiadd_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDE, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fiadd_m16int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fimul)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fimul_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDE, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fimul_m16int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ficom)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ficom_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDE, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Ficom_m16int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ficomp)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ficomp_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDE, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Ficomp_m16int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fisub)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fisub_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDE, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fisub_m16int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fisubr)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fisubr_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDE, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fisubr_m16int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fidiv)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fidiv_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDE, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fidiv_m16int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fidivr)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fidivr_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDE, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fidivr_m16int)], CodeInfo.RW_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Faddp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Faddp_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDEC0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Faddp_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fmulp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fmulp_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDEC8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fmulp_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcomp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcomp_st0_sti_DED0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDED0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcomp_st0_sti_DED0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Fcompp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcompp)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDED9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Fcompp)], CodeInfo.R_ST0_ST1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fsubrp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsubrp_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDEE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fsubrp_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fsubp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fsubp_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDEE8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fsubp_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fdivrp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdivrp_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDEF0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fdivrp_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fdivp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fdivp_sti_st0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDEF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi, LegacyOpKind.ST }),
				new InstrInfo(code[nameof(Code.Fdivp_sti_st0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fild)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fild_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDF, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fild_m16int)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fisttp)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fisttp_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDF, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fisttp_m16int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fist)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fist_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDF, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fist_m16int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fistp)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fistp_m16int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDF, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi16 }),
				new InstrInfo(code[nameof(Code.Fistp_m16int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fbld)], memSize[nameof(MemorySize.Bcd)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fbld_m80bcd)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDF, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfbcd }),
				new InstrInfo(code[nameof(Code.Fbld_m80bcd)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fild)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fild_m64int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDF, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi64 }),
				new InstrInfo(code[nameof(Code.Fild_m64int)], CodeInfo.W_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fbstp)], memSize[nameof(MemorySize.Bcd)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fbstp_m80bcd)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDF, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfbcd }),
				new InstrInfo(code[nameof(Code.Fbstp_m80bcd)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fistp)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fistp_m64int)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDF, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mfi64 }),
				new InstrInfo(code[nameof(Code.Fistp_m64int)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ffreep)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ffreep_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFC0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Ffreep_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fxch)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fxch_st0_sti_DFC8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFC8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fxch_st0_sti_DFC8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstp_sti_DFD0)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFD0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fstp_sti_DFD0)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstp_sti_DFD8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFD8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fstp_sti_DFD8)], CodeInfo.R_ST0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fnstsw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fnstsw_AX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX }),
				new InstrInfo(code[nameof(Code.Fnstsw_AX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU287)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstsw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstsw_AX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Fwait, new[] { LegacyOpKind.AX }),
				new InstrInfo(code[nameof(Code.Fstsw_AX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU287)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstdw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstdw_AX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFE1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.AX }),
				new InstrInfo(code[nameof(Code.Fstdw_AX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU387SL_ONLY)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fstsg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fstsg_AX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFE2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.AX }),
				new InstrInfo(code[nameof(Code.Fstsg_AX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU387SL_ONLY)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fucomip)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fucomip_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFE8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fucomip_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Fcomip)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fcomip_st0_sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xDFF0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.ST, LegacyOpKind.STi }),
				new InstrInfo(code[nameof(Code.Fcomip_st0_sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FPU)], cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loopne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loopne_rel8_16_CX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE0, -1, OperandSize.Size16, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Loopne_rel8_16_CX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loopne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loopne_rel8_32_CX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE0, -1, OperandSize.Size32, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Loopne_rel8_32_CX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loopne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loopne_rel8_16_ECX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE0, -1, OperandSize.Size16, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Loopne_rel8_16_ECX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loopne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loopne_rel8_32_ECX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE0, -1, OperandSize.Size32, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Loopne_rel8_32_ECX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loopne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loopne_rel8_64_ECX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE0, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode64, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Loopne_rel8_64_ECX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loopne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loopne_rel8_16_RCX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE0, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Loopne_rel8_16_RCX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loopne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loopne_rel8_64_RCX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Loopne_rel8_64_RCX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loope)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loope_rel8_16_CX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE1, -1, OperandSize.Size16, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Loope_rel8_16_CX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loope)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loope_rel8_32_CX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE1, -1, OperandSize.Size32, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Loope_rel8_32_CX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loope)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loope_rel8_16_ECX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE1, -1, OperandSize.Size16, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Loope_rel8_16_ECX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loope)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loope_rel8_32_ECX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE1, -1, OperandSize.Size32, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Loope_rel8_32_ECX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loope)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loope_rel8_64_ECX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE1, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode64, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Loope_rel8_64_ECX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loope)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loope_rel8_16_RCX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE1, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Loope_rel8_16_RCX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loope)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loope_rel8_64_RCX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Loope_rel8_64_RCX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loop_rel8_16_CX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE2, -1, OperandSize.Size16, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Loop_rel8_16_CX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loop_rel8_32_CX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE2, -1, OperandSize.Size32, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Loop_rel8_32_CX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loop_rel8_16_ECX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE2, -1, OperandSize.Size16, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Loop_rel8_16_ECX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loop_rel8_32_ECX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE2, -1, OperandSize.Size32, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Loop_rel8_32_ECX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loop_rel8_64_ECX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE2, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode64, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Loop_rel8_64_ECX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loop_rel8_16_RCX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE2, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Loop_rel8_16_RCX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Loop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loop_rel8_64_RCX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Loop_rel8_64_RCX)], CodeInfo.Loop, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jcxz)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jcxz_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE3, -1, OperandSize.Size16, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jcxz_rel8_16)], CodeInfo.Jrcxz, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jcxz)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jcxz_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE3, -1, OperandSize.Size32, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jcxz_rel8_32)], CodeInfo.Jrcxz, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jecxz)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jecxz_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE3, -1, OperandSize.Size16, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jecxz_rel8_16)], CodeInfo.Jrcxz, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jecxz)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jecxz_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE3, -1, OperandSize.Size32, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jecxz_rel8_32)], CodeInfo.Jrcxz, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jecxz)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jecxz_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE3, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode64, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jecxz_rel8_64)], CodeInfo.Jrcxz, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jrcxz)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jrcxz_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE3, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jrcxz_rel8_16)], CodeInfo.Jrcxz, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jrcxz)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jrcxz_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jrcxz_rel8_64)], CodeInfo.Jrcxz, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.In)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.In_AL_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.In_AL_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.In)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.In_AX_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE5, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.In_AX_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.In)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.In_EAX_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE5, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.In_EAX_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Out)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Out_imm8_AL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ib, LegacyOpKind.AL }),
				new InstrInfo(code[nameof(Code.Out_imm8_AL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Out)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Out_imm8_AX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE7, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ib, LegacyOpKind.AX }),
				new InstrInfo(code[nameof(Code.Out_imm8_AX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Out)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Out_imm8_EAX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE7, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ib, LegacyOpKind.EAX }),
				new InstrInfo(code[nameof(Code.Out_imm8_EAX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_rel16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE8, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Call_rel16)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE8, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Call_rel32_32)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Call_rel32_64)], CodeInfo.Push_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_rel16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE9, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jmp_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE9, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jmp_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xE9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jmp_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_ptr1616)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xEA, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Aww }),
				new InstrInfo(code[nameof(Code.Jmp_ptr1616)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_ptr1632)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xEA, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Adw }),
				new InstrInfo(code[nameof(Code.Jmp_ptr1632)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_rel8_16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xEB, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.wJb }),
				new InstrInfo(code[nameof(Code.Jmp_rel8_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_rel8_32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xEB, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.dJb }),
				new InstrInfo(code[nameof(Code.Jmp_rel8_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_rel8_64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xEB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.qJb }),
				new InstrInfo(code[nameof(Code.Jmp_rel8_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.In)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.In_AL_DX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xEC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AL, LegacyOpKind.DX }),
				new InstrInfo(code[nameof(Code.In_AL_DX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.In)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.In_AX_DX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xED, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.AX, LegacyOpKind.DX }),
				new InstrInfo(code[nameof(Code.In_AX_DX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.In)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.In_EAX_DX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xED, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.EAX, LegacyOpKind.DX }),
				new InstrInfo(code[nameof(Code.In_EAX_DX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Out)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Out_DX_AL)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xEE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.DX, LegacyOpKind.AL }),
				new InstrInfo(code[nameof(Code.Out_DX_AL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Out)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Out_DX_AX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xEF, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.DX, LegacyOpKind.AX }),
				new InstrInfo(code[nameof(Code.Out_DX_AX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Out)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Out_DX_EAX)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xEF, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.DX, LegacyOpKind.EAX }),
				new InstrInfo(code[nameof(Code.Out_DX_EAX)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Int1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Int1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Int1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Interrupt)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Hlt)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Hlt)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Hlt)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Cmc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmc)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cmc)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm8_imm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF6, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Test_rm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm8_imm8_F6r1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF6, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Test_rm8_imm8_F6r1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Not)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Not_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF6, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Not_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Neg)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Neg_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF6, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Neg_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Mul)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mul_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF6, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Mul_rm8)], CodeInfo.Mul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF6, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Imul_rm8)], CodeInfo.Mul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Div)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Div_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF6, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Div_rm8)], CodeInfo.Div, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Idiv)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Idiv_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF6, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Idiv_rm8)], CodeInfo.Div, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm16_imm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Test_rm16_imm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm32_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Test_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm64_imm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Test_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm16_imm16_F7r1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Iw }),
				new InstrInfo(code[nameof(Code.Test_rm16_imm16_F7r1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm32_imm32_F7r1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Id }),
				new InstrInfo(code[nameof(Code.Test_rm32_imm32_F7r1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Test)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Test_rm64_imm32_F7r1)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Id64 }),
				new InstrInfo(code[nameof(Code.Test_rm64_imm32_F7r1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Not)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Not_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 2, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Not_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Not)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Not_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 2, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Not_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Not)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Not_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 2, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Not_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Neg)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Neg_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 3, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Neg_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Neg)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Neg_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 3, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Neg_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Neg)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Neg_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Neg_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Mul)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mul_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Mul_rm16)], CodeInfo.Mul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Mul)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mul_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Mul_rm32)], CodeInfo.Mul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Mul)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mul_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Mul_rm64)], CodeInfo.Mul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 5, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Imul_rm16)], CodeInfo.Mul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 5, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Imul_rm32)], CodeInfo.Mul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Imul_rm64)], CodeInfo.Mul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Div)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Div_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Div_rm16)], CodeInfo.Div, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Div)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Div_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Div_rm32)], CodeInfo.Div, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Div)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Div_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Div_rm64)], CodeInfo.Div, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Idiv)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Idiv_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 7, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Idiv_rm16)], CodeInfo.Div, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Idiv)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Idiv_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 7, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Idiv_rm32)], CodeInfo.Div, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Idiv)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Idiv_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF7, 7, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Idiv_rm64)], CodeInfo.Div, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Clc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clc)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Clc)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Stc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Stc)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xF9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Stc)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.CF, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Cli)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cli)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cli)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.IF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sti)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sti)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.IF, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Cld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cld)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cld)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.DF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Std)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Std)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Std)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.DF, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Inc)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Inc_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFE, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Inc_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Dec)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Dec_rm8)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFE, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Dec_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Inc)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Inc_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Inc_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Inc)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Inc_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Inc_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Inc)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Inc_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Inc_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Dec)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Dec_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Dec_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Dec)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Dec_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Dec_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Dec)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Dec_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Dec_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.WordOffset)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 2, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.NotrackPrefix, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Call_rm16)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectCall)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.DwordOffset)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 2, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.NotrackPrefix, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Call_rm32)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectCall)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.QwordOffset)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.NotrackPrefix, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Call_rm64)], CodeInfo.Push_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectCall)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_m1616)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 3, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eww }),
				new InstrInfo(code[nameof(Code.Call_m1616)], CodeInfo.Push_2_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectCall)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_m1632)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 3, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Edw }),
				new InstrInfo(code[nameof(Code.Call_m1632)], CodeInfo.Push_4_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectCall)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Call)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Call_m1664)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eqw }),
				new InstrInfo(code[nameof(Code.Call_m1664)], CodeInfo.Push_8_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectCall)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.WordOffset)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.NotrackPrefix, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Jmp_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.DwordOffset)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.NotrackPrefix, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Jmp_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.QwordOffset)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.NotrackPrefix, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Jmp_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_m1616)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 5, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eww }),
				new InstrInfo(code[nameof(Code.Jmp_m1616)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_m1632)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 5, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Edw }),
				new InstrInfo(code[nameof(Code.Jmp_m1632)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmp)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmp_m1664)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eqw }),
				new InstrInfo(code[nameof(Code.Jmp_m1664)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Push_rm16)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Push_rm16)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL8086)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Push_rm32)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Push_rm32)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Push_rm64)], MandatoryPrefix.None, OpCodeTableKind.Normal, 0xFF, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Push_rm64)], CodeInfo.Push_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Sldt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sldt_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Sldt_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Sldt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sldt_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_d }),
				new InstrInfo(code[nameof(Code.Sldt_r32m16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Sldt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sldt_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_q }),
				new InstrInfo(code[nameof(Code.Sldt_r64m16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Str)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Str_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Str_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Str)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Str_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_d }),
				new InstrInfo(code[nameof(Code.Str_r32m16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Str)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Str_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_q }),
				new InstrInfo(code[nameof(Code.Str_r64m16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Lldt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lldt_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 2, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Lldt_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Lldt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lldt_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 2, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_d }),
				new InstrInfo(code[nameof(Code.Lldt_r32m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Lldt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lldt_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 2, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_q }),
				new InstrInfo(code[nameof(Code.Lldt_r64m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Ltr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ltr_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 3, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Ltr_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Ltr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ltr_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 3, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_d }),
				new InstrInfo(code[nameof(Code.Ltr_r32m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Ltr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ltr_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_q }),
				new InstrInfo(code[nameof(Code.Ltr_r64m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Verr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Verr_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Verr_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Verr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Verr_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_d }),
				new InstrInfo(code[nameof(Code.Verr_r32m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Verr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Verr_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_q }),
				new InstrInfo(code[nameof(Code.Verr_r64m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Verw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Verw_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 5, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Verw_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Verw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Verw_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 5, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_d }),
				new InstrInfo(code[nameof(Code.Verw_r32m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Verw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Verw_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_q }),
				new InstrInfo(code[nameof(Code.Verw_r64m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Jmpe)], memSize[nameof(MemorySize.WordOffset)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmpe_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Jmpe_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.IA64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmpe)], memSize[nameof(MemorySize.DwordOffset)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmpe_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x00, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Jmpe_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.IndirectBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.IA64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Sgdt)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sgdt_m1632_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Sgdt_m1632_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Sgdt)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sgdt_m1632)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Sgdt_m1632)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Sgdt)], memSize[nameof(MemorySize.Fword10)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sgdt_m1664)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Sgdt_m1664)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Sidt)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sidt_m1632_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Sidt_m1632_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Sidt)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sidt_m1632)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Sidt_m1632)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Sidt)], memSize[nameof(MemorySize.Fword10)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sidt_m1664)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Sidt_m1664)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Lgdt)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lgdt_m1632_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 2, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Lgdt_m1632_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Lgdt)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lgdt_m1632)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 2, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Lgdt_m1632)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Lgdt)], memSize[nameof(MemorySize.Fword10)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lgdt_m1664)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Lgdt_m1664)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Lidt)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lidt_m1632_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 3, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Lidt_m1632_16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Lidt)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lidt_m1632)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 3, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Lidt_m1632)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Lidt)], memSize[nameof(MemorySize.Fword10)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lidt_m1664)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ms }),
				new InstrInfo(code[nameof(Code.Lidt_m1664)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Smsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Smsw_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Smsw_rm16)], CodeInfo.R_CR0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Smsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Smsw_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_d }),
				new InstrInfo(code[nameof(Code.Smsw_r32m16)], CodeInfo.R_CR0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Smsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Smsw_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_q }),
				new InstrInfo(code[nameof(Code.Smsw_r64m16)], CodeInfo.R_CR0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rstorssp)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rstorssp_m64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x01, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mq }),
				new InstrInfo(code[nameof(Code.Rstorssp_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Lmsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lmsw_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Lmsw_rm16)], CodeInfo.RW_CR0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Lmsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lmsw_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_d }),
				new InstrInfo(code[nameof(Code.Lmsw_r32m16)], CodeInfo.RW_CR0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Lmsw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lmsw_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew_q }),
				new InstrInfo(code[nameof(Code.Lmsw_r64m16)], CodeInfo.RW_CR0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Invlpg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invlpg_m)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Invlpg_m)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Enclv)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enclv)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Enclv)], CodeInfo.Encls, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ENCLV)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmcall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmcall)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmcall)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Vmlaunch)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmlaunch)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmlaunch)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmresume)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmresume)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmresume)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmxoff)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmxoff)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmxoff)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Pconfig)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pconfig)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pconfig)], CodeInfo.Pconfig, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PCONFIG)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Monitor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Monitorw)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C8, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Monitorw)], CodeInfo.Monitor, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MONITOR)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Monitor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Monitord)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C8, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Monitord)], CodeInfo.Monitor, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MONITOR)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Monitor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Monitorq)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Monitorq)], CodeInfo.Monitor, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MONITOR)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Mwait)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mwait)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01C9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mwait)], CodeInfo.Mwait, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MONITOR)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Clac)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clac)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01CA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Clac)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.AC, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SMAP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Stac)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Stac)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01CB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Stac)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.AC, new[] { cpuid[nameof(CpuidFeature.SMAP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Encls)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Encls)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01CF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Encls)], CodeInfo.Encls, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SGX1)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Xgetbv)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xgetbv)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01D0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xgetbv)], CodeInfo.R_ECX_W_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xsetbv)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsetbv)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01D1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xsetbv)], CodeInfo.R_EAX_ECX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmfunc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmfunc)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01D4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmfunc)], CodeInfo.Vmfunc, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xend)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xend)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01D5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xend)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.XbeginXabortXend)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RTM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xtest)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xtest)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01D6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xtest)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.HLE_or_RTM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Enclu)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enclu)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01D7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Enclu)], CodeInfo.Encls, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SGX1)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode)),
				(0, mnemonic[nameof(Mnemonic.Vmrun)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmrunw)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01D8, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmrunw)], CodeInfo.Vmload, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmrun)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmrund)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01D8, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmrund)], CodeInfo.Vmload, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmrun)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmrunq)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01D8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmrunq)], CodeInfo.Vmload, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmmcall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmmcall)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01D9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmmcall)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Vmload)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmloadw)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DA, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmloadw)], CodeInfo.Vmload, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmload)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmloadd)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DA, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmloadd)], CodeInfo.Vmload, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmload)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmloadq)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmloadq)], CodeInfo.Vmload, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmsave)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmsavew)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DB, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmsavew)], CodeInfo.Vmload, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmsave)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmsaved)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DB, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmsaved)], CodeInfo.Vmload, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Vmsave)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmsaveq)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmsaveq)], CodeInfo.Vmload, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Stgi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Stgi)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Stgi)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SKINIT_or_SVML)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Clgi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clgi)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Clgi)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Skinit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Skinit)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Skinit)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SKINIT_or_SVML)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Invlpga)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invlpgaw)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DF, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Invlpgaw)], CodeInfo.Invlpga, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Invlpga)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invlpgad)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DF, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Invlpgad)], CodeInfo.Invlpga, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Invlpga)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invlpgaq)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01DF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Invlpgaq)], CodeInfo.Invlpga, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Setssbsy)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setssbsy)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x01E8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Setssbsy)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Saveprevssp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Saveprevssp)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x01EA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Saveprevssp)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode)),
				(0, mnemonic[nameof(Mnemonic.Rdpkru)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdpkru)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01EE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Rdpkru)], CodeInfo.R_ECX_W_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PKU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Wrpkru)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrpkru)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01EF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Wrpkru)], CodeInfo.R_EAX_ECX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PKU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Swapgs)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Swapgs)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01F8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Swapgs)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Rdtscp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdtscp)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01F9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Rdtscp)], CodeInfo.W_EAX_ECX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDTSCP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Monitorx)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Monitorxw)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01FA, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Monitorxw)], CodeInfo.Monitor, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MONITORX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Monitorx)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Monitorxd)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01FA, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Monitorxd)], CodeInfo.Monitor, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MONITORX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Monitorx)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Monitorxq)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01FA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Monitorxq)], CodeInfo.Monitor, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MONITORX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mcommit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mcommit)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x01FA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mcommit)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MCOMMIT)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mwaitx)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mwaitx)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01FB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mwaitx)], CodeInfo.Mwaitx, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MONITORX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Clzero)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clzerow)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01FC, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Clzerow)], CodeInfo.Clzero, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CLZERO)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Clzero)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clzerod)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01FC, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Clzerod)], CodeInfo.Clzero, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CLZERO)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Clzero)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clzeroq)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01FC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Clzeroq)], CodeInfo.Clzero, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CLZERO)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Rdpru)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdpru)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x01FD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Rdpru)], CodeInfo.R_ECX_W_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDPRU)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lar)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lar_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x02, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Lar_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Lar)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lar_r32_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x02, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Lar_r32_r32m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Lar)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lar_r64_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x02, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Lar_r64_r64m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Lsl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lsl_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x03, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Lsl_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Lsl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lsl_r32_r32m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x03, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Lsl_r32_r32m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Lsl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lsl_r64_r64m16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x03, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Lsl_r64_r64m16)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(0, mnemonic[nameof(Mnemonic.Loadall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loadallreset286)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x04, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Loadallreset286)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286_ONLY)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Loadall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loadall286)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x05, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Loadall286)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286_ONLY)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Syscall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Syscall)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x05, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Syscall)], CodeInfo.Syscall, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SYSCALL)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Clts)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clts)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x06, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Clts)], CodeInfo.RW_CR0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Loadall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Loadall386)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x07, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Loadall386)], CodeInfo.Loadall386, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_ONLY)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Sysret)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sysretd)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x07, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sysretd)], CodeInfo.Syscall, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SYSCALL)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Sysretq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sysretq)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x07, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sysretq)], CodeInfo.Syscall, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SYSCALL)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Invd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invd)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x08, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Invd)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Wbinvd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wbinvd)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x09, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Wbinvd)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Wbnoinvd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wbnoinvd)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x09, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Wbnoinvd)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.WBNOINVD)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Cl1invmb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cl1invmb)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cl1invmb)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CL1INVMB)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Ud2)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ud2)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Ud2)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm16_r16_0F0D)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm16_r16_0F0D)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm32_r32_0F0D)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm32_r32_0F0D)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm64_r64_0F0D)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm64_r64_0F0D)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetch)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Prefetch_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Prefetch_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PREFETCHW)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetchw)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Prefetchw_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Prefetchw_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PREFETCHW)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetchwt1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Prefetchwt1_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Prefetchwt1_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PREFETCHWT1)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Femms)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Femms)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Femms)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Umov)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umov_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x10, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Umov_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_486_ONLY)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Umov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umov_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x11, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Umov_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_486_ONLY)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Umov)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umov_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x11, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Umov_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_486_ONLY)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Umov)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umov_r8_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x12, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gb, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Umov_r8_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_486_ONLY)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Umov)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umov_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x13, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Umov_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_486_ONLY)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Umov)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umov_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x13, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Umov_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_486_ONLY)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movups)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movups_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x10, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movups_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovups_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x10, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovups_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovups_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x10, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovups_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovups_xmm_k1z_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovups_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovups_ymm_k1z_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovups_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovups_zmm_k1z_zmmm512)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovups_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movupd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movupd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x10, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movupd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovupd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x10, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovupd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovupd_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x10, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovupd_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovupd_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovupd_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovupd_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovupd_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovupd_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovupd_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x10, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.WriteMem_ReadWriteReg, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmovss)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x10, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovss_xmm_m32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x10, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmovss_xmm_m32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovss)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.RX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovss_xmm_k1z_m32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovss_xmm_k1z_m32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsd_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x10, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movsd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.WriteMem_ReadWriteReg, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmovsd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x10, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovsd_xmm_m64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x10, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmovsd_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovsd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.RX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_m64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x10, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_m64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movups)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movups_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x11, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.WX, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movups_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovups_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x11, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovups_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovups_ymmm256_ymm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x11, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovups_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovups_xmmm128_k1z_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovups_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovups_ymmm256_k1z_ymm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovups_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovups)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovups_zmmm512_k1z_zmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovups_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movupd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movupd_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x11, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.WX, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movupd_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovupd_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x11, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovupd_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovupd_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x11, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovupd_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovupd_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovupd_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovupd_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovupd_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovupd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovupd_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovupd_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movss_xmmm32_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x11, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.WX, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movss_xmmm32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.WriteMem_ReadWriteReg, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmovss)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm_0F11)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x11, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.RX, VexOpKind.HX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm_0F11)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovss_m32_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x11, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovss_m32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovss)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.RX, EvexOpKind.HX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovss_m32_k1_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.M, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovss_m32_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsd_xmmm64_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x11, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.WX, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movsd_xmmm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.WriteMem_ReadWriteReg, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmovsd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm_0F11)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x11, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.RX, VexOpKind.HX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm_0F11)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovsd_m64_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x11, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovsd_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovsd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.RX, EvexOpKind.HX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovsd_m64_k1_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x11, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.M, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovsd_m64_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movhlps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movhlps_xmm_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x12, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Movhlps_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movlps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movlps_xmm_m64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x12, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Movlps_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmovhlps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovhlps_xmm_xmm_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x12, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovhlps_xmm_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovlps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovlps_xmm_xmm_m64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x12, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmovlps_xmm_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovhlps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovhlps_xmm_xmm_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x12, -1, EvexVectorLength.L128, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.RX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovhlps_xmm_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovlps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovlps_xmm_xmm_m64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x12, -1, EvexVectorLength.L128, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovlps_xmm_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movlpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movlpd_xmm_m64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x12, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Movlpd_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmovlpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovlpd_xmm_xmm_m64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x12, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmovlpd_xmm_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovlpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovlpd_xmm_xmm_m64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x12, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovlpd_xmm_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movsldup)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsldup_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x12, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movsldup_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovsldup)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovsldup_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x12, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovsldup_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovsldup)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovsldup_ymm_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x12, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovsldup_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovsldup)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovsldup_xmm_k1z_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x12, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovsldup_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovsldup)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovsldup_ymm_k1z_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x12, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovsldup_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovsldup)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovsldup_zmm_k1z_zmmm512)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x12, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovsldup_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movddup)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movddup_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x12, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movddup_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovddup)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovddup_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x12, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovddup_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovddup)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovddup_ymm_ymmm256)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x12, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovddup_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovddup)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovddup_xmm_k1z_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x12, -1, EvexVectorLength.L128, TupleType.MOVDDUP_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovddup_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovddup)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovddup_ymm_k1z_ymmm256)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x12, -1, EvexVectorLength.L256, TupleType.MOVDDUP_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovddup_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovddup)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovddup_zmm_k1z_zmmm512)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x12, -1, EvexVectorLength.L512, TupleType.MOVDDUP_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovddup_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movlps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movlps_m64_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x13, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movlps_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovlps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovlps_m64_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x13, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovlps_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovlps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovlps_m64_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x13, -1, EvexVectorLength.L128, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.M, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovlps_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movlpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movlpd_m64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x13, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movlpd_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovlpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovlpd_m64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x13, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovlpd_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovlpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovlpd_m64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x13, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.M, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovlpd_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Unpcklps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Unpcklps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x14, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Unpcklps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vunpcklps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x14, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vunpcklps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vunpcklps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x14, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vunpcklps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x14, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x14, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x14, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Unpcklpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Unpcklpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x14, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Unpcklpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vunpcklpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x14, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vunpcklpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vunpcklpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x14, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vunpcklpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x14, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x14, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpcklpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x14, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Unpckhps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Unpckhps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x15, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Unpckhps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vunpckhps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x15, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vunpckhps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vunpckhps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x15, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vunpckhps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x15, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x15, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x15, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Unpckhpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Unpckhpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x15, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Unpckhpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vunpckhpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x15, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vunpckhpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vunpckhpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x15, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vunpckhpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x15, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x15, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vunpckhpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x15, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movlhps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movlhps_xmm_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x16, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Movlhps_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmovlhps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovlhps_xmm_xmm_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x16, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovlhps_xmm_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovlhps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovlhps_xmm_xmm_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x16, -1, EvexVectorLength.L128, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.RX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovlhps_xmm_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movhps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movhps_xmm_m64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x16, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Movhps_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmovhps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovhps_xmm_xmm_m64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x16, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmovhps_xmm_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovhps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovhps_xmm_xmm_m64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x16, -1, EvexVectorLength.L128, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovhps_xmm_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movhpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movhpd_xmm_m64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x16, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Movhpd_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmovhpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovhpd_xmm_xmm_m64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x16, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmovhpd_xmm_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmovhpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovhpd_xmm_xmm_m64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x16, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovhpd_xmm_xmm_m64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movshdup)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movshdup_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x16, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movshdup_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovshdup)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovshdup_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x16, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovshdup_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovshdup)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovshdup_ymm_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x16, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovshdup_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovshdup)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovshdup_xmm_k1z_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x16, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovshdup_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovshdup)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovshdup_ymm_k1z_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x16, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovshdup_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovshdup)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovshdup_zmm_k1z_zmmm512)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x16, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovshdup_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movhps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movhps_m64_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x17, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movhps_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovhps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovhps_m64_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x17, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovhps_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovhps)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovhps_m64_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x17, -1, EvexVectorLength.L128, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.M, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovhps_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movhpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movhpd_m64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x17, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movhpd_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovhpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovhpd_m64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x17, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovhpd_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovhpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovhpd_m64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x17, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.M, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovhpd_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm16_r16_0F18)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x18, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm16_r16_0F18)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm32_r32_0F18)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x18, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm32_r32_0F18)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm64_r64_0F18)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x18, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm64_r64_0F18)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm16_r16_0F19)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x19, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm16_r16_0F19)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm32_r32_0F19)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x19, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm32_r32_0F19)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm64_r64_0F19)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x19, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm64_r64_0F19)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1A)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1A)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1A)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1A)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1A)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1A)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1B)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1B)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1B)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1B)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1B)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1B)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1C)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1C, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1C)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1C)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1C, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1C)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1C)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1C, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1C)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1D)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1D)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1D)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1D)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1D)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1D)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1E)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1E, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1E)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1E)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1E, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1E)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1E)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1E, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1E)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1F)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1F, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm16_r16_0F1F)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1F)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1F, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm32_r32_0F1F)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.ReservedNop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1F)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1F, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.ReservedNop_rm64_r64_0F1F)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetchnta)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Prefetchnta_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x18, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Prefetchnta_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetcht0)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Prefetcht0_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x18, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Prefetcht0_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetcht1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Prefetcht1_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x18, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Prefetcht1_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetcht2)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Prefetcht2_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x18, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Prefetcht2_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bndldx)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndldx_bnd_mib)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.B, LegacyOpKind.MIB }),
				new InstrInfo(code[nameof(Code.Bndldx_bnd_mib)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Write, OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bndmov)], memSize[nameof(MemorySize.Bnd32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndmov_bnd_bndm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.B, LegacyOpKind.BMq }),
				new InstrInfo(code[nameof(Code.Bndmov_bnd_bndm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bndmov)], memSize[nameof(MemorySize.Bnd64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndmov_bnd_bndm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.B, LegacyOpKind.BMo }),
				new InstrInfo(code[nameof(Code.Bndmov_bnd_bndm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bndcl)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndcl_bnd_rm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.B, LegacyOpKind.Ed_MPX }),
				new InstrInfo(code[nameof(Code.Bndcl_bnd_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Read, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Bndcl)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndcl_bnd_rm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.B, LegacyOpKind.Eq_MPX }),
				new InstrInfo(code[nameof(Code.Bndcl_bnd_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Read, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Bndcu)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndcu_bnd_rm32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.B, LegacyOpKind.Ed_MPX }),
				new InstrInfo(code[nameof(Code.Bndcu_bnd_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Read, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Bndcu)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndcu_bnd_rm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x1A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.B, LegacyOpKind.Eq_MPX }),
				new InstrInfo(code[nameof(Code.Bndcu_bnd_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Read, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Bndstx)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndstx_mib_bnd)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.MIB, LegacyOpKind.B }),
				new InstrInfo(code[nameof(Code.Bndstx_mib_bnd)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.NoMemAccess, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bndmov)], memSize[nameof(MemorySize.Bnd32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndmov_bndm64_bnd)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.BMq, LegacyOpKind.B }),
				new InstrInfo(code[nameof(Code.Bndmov_bndm64_bnd)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bndmov)], memSize[nameof(MemorySize.Bnd64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndmov_bndm128_bnd)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.BMo, LegacyOpKind.B }),
				new InstrInfo(code[nameof(Code.Bndmov_bndm128_bnd)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bndmk)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndmk_bnd_m32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.B, LegacyOpKind.Md_MPX }),
				new InstrInfo(code[nameof(Code.Bndmk_bnd_m32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Write, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Bndmk)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndmk_bnd_m64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.B, LegacyOpKind.Mq_MPX }),
				new InstrInfo(code[nameof(Code.Bndmk_bnd_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Write, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Bndcn)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndcn_bnd_rm32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.B, LegacyOpKind.Ed_MPX }),
				new InstrInfo(code[nameof(Code.Bndcn_bnd_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Read, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(2, mnemonic[nameof(Mnemonic.Bndcn)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bndcn_bnd_rm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x1B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.B, LegacyOpKind.Eq_MPX }),
				new InstrInfo(code[nameof(Code.Bndcn_bnd_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MPX)] }, new[] { OpInfo.Read, OpInfo.NoMemAccess }, InstrInfoFlags.NoSegmentRead)),
				(1, mnemonic[nameof(Mnemonic.Cldemote)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cldemote_m8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x1C, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Cldemote_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CLDEMOTE)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdsspd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdsspd_r32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x1E, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Rdsspd_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdsspq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdsspq_r64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x1E, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Rdsspq_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Endbr64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Endbr64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x1EFA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Endbr64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_IBT)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Endbr32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Endbr32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x1EFB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Endbr32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_IBT)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Nop)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Nop_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1F, 0, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Nop_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Nop)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Nop_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1F, 0, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Nop_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Nop)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Nop_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x1F, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Nop_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MULTIBYTENOP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r32_cr)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x20, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Rd, LegacyOpKind.Cd }),
				new InstrInfo(code[nameof(Code.Mov_r32_cr)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r64_cr)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x20, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq, LegacyOpKind.Cq }),
				new InstrInfo(code[nameof(Code.Mov_r64_cr)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r32_dr)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x21, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Rd, LegacyOpKind.Dd }),
				new InstrInfo(code[nameof(Code.Mov_r32_dr)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r64_dr)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x21, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq, LegacyOpKind.Dq }),
				new InstrInfo(code[nameof(Code.Mov_r64_dr)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_cr_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x22, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Cd, LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Mov_cr_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_cr_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x22, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Cq, LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Mov_cr_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_dr_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x23, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Dd, LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Mov_dr_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_dr_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x23, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Dq, LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Mov_dr_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_r32_tr)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x24, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Rd, LegacyOpKind.Td }),
				new InstrInfo(code[nameof(Code.Mov_r32_tr)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_486_ONLY)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Mov)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mov_tr_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x26, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Td, LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Mov_tr_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_486_ONLY)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Movaps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movaps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x28, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movaps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovaps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x28, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovaps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovaps_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x28, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovaps_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovaps_xmm_k1z_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x28, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovaps_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovaps_ymm_k1z_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x28, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovaps_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovaps_zmm_k1z_zmmm512)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x28, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovaps_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movapd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movapd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x28, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movapd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovapd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x28, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovapd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovapd_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x28, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovapd_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovapd_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x28, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovapd_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovapd_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x28, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovapd_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovapd_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x28, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovapd_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movaps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movaps_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x29, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.WX, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movaps_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovaps_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x29, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovaps_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovaps_ymmm256_ymm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x29, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovaps_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovaps_xmmm128_k1z_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x29, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovaps_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovaps_ymmm256_k1z_ymm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x29, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovaps_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovaps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovaps_zmmm512_k1z_zmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x29, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovaps_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movapd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movapd_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x29, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.WX, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movapd_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovapd_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x29, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovapd_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovapd_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x29, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovapd_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovapd_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x29, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovapd_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovapd_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x29, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovapd_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovapd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovapd_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x29, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovapd_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtpi2ps)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtpi2ps_xmm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Cvtpi2ps_xmm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvtpi2pd)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtpi2pd_xmm_mmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Cvtpi2pd_xmm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvtsi2ss)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtsi2ss_xmm_rm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cvtsi2ss_xmm_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvtsi2ss)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtsi2ss_xmm_rm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2A, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cvtsi2ss_xmm_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsi2ss)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2A, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsi2ss)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2A, -1, VexVectorLength.LIG, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsi2ss)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2A, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.RoundingControl, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsi2ss)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm64_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2A, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtsi2sd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtsi2sd_xmm_rm32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cvtsi2sd_xmm_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvtsi2sd)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtsi2sd_xmm_rm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2A, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cvtsi2sd_xmm_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsi2sd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2A, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsi2sd)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2A, -1, VexVectorLength.LIG, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsi2sd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm32_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2A, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.RoundingControl, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsi2sd)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2A, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movntps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movntps_m128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movntps_m128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovntps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovntps_m128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovntps_m128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovntps_m256_ymm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovntps_m256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntps_m128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2B, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.M, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntps_m128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntps_m256_ymm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2B, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.M, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntps_m256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntps_m512_zmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2B, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.M, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntps_m512_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movntpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movntpd_m128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movntpd_m128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovntpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovntpd_m128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovntpd_m128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovntpd_m256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovntpd_m256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntpd_m128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2B, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.M, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntpd_m128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntpd_m256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2B, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.M, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntpd_m256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntpd_m512_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2B, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.M, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntpd_m512_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movntss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movntss_m32_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movntss_m32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4A)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movntsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movntsd_m64_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movntsd_m64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4A)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvttps2pi)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvttps2pi_mm_xmmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvttps2pi_mm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvttpd2pi)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvttpd2pi_mm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvttpd2pi_mm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvttss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvttss2si_r32_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvttss2si_r32_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvttss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvttss2si_r64_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2C, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvttss2si_r64_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvttss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvttss2si_r32_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2C, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvttss2si_r32_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvttss2si_r64_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2C, -1, VexVectorLength.LIG, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvttss2si_r64_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttss2si_r32_xmmm32_sae)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2C, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.Gd, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttss2si_r32_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttss2si_r64_xmmm32_sae)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2C, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_4, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.Gq, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttss2si_r64_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvttsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvttsd2si_r32_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvttsd2si_r32_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvttsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvttsd2si_r64_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2C, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvttsd2si_r64_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvttsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvttsd2si_r32_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2C, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvttsd2si_r32_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvttsd2si_r64_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2C, -1, VexVectorLength.LIG, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvttsd2si_r64_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttsd2si_r32_xmmm64_sae)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2C, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.Gd, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttsd2si_r32_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttsd2si_r64_xmmm64_sae)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2C, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_8, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.Gq, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttsd2si_r64_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtps2pi)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtps2pi_mm_xmmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtps2pi_mm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvtpd2pi)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtpd2pi_mm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtpd2pi_mm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvtss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtss2si_r32_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtss2si_r32_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvtss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtss2si_r64_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtss2si_r64_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtss2si_r32_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2D, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtss2si_r32_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtss2si_r64_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2D, -1, VexVectorLength.LIG, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtss2si_r64_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtss2si_r32_xmmm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.RoundingControl, new[] { EvexOpKind.Gd, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtss2si_r32_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtss2si)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtss2si_r64_xmmm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x2D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_4, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl, new[] { EvexOpKind.Gq, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtss2si_r64_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtsd2si_r32_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtsd2si_r32_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cvtsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtsd2si_r64_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtsd2si_r64_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtsd2si_r32_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2D, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtsd2si_r32_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtsd2si_r64_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2D, -1, VexVectorLength.LIG, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtsd2si_r64_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtsd2si_r32_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.RoundingControl, new[] { EvexOpKind.Gd, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtsd2si_r32_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtsd2si)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtsd2si_r64_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x2D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_8, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl, new[] { EvexOpKind.Gq, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtsd2si_r64_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Ucomiss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ucomiss_xmm_xmmm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Ucomiss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vucomiss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vucomiss_xmm_xmmm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2E, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vucomiss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vucomiss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vucomiss_xmm_xmmm32_sae)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2E, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vucomiss_xmm_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Ucomisd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ucomisd_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Ucomisd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vucomisd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vucomisd_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2E, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vucomisd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vucomisd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vucomisd_xmm_xmmm64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2E, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vucomisd_xmm_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Comiss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Comiss_xmm_xmmm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Comiss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Comisd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Comisd_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Comisd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcomiss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcomiss_xmm_xmmm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcomiss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcomisd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcomisd_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcomisd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcomiss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcomiss_xmm_xmmm32_sae)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x2F, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcomiss_xmm_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcomisd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcomisd_xmm_xmmm64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x2F, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcomisd_xmm_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(0, mnemonic[nameof(Mnemonic.Wrmsr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrmsr)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x30, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Wrmsr)], CodeInfo.R_EAX_ECX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MSR)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Rdtsc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdtsc)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x31, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Rdtsc)], CodeInfo.W_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TSC)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Rdmsr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdmsr)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x32, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Rdmsr)], CodeInfo.R_ECX_W_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MSR)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Rdpmc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdpmc)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x33, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Rdpmc)], CodeInfo.R_ECX_W_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDPMC)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sysenter)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sysenter)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x34, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sysenter)], CodeInfo.Syscall, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.IF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SEP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sysexit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sysexitd)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x35, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sysexitd)], CodeInfo.Syscall, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SEP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Sysexitq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sysexitq)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x35, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sysexitq)], CodeInfo.Syscall, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SEP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Getsec)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Getsecd)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x37, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Getsecd)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SMX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore)),
				(2, mnemonic[nameof(Mnemonic.Cmovo)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovo_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x40, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovo_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovo)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovo_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x40, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovo_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovo)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovo_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x40, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovo_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovno)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovno_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x41, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovno_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovno)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovno_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x41, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovno_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovno)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovno_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x41, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovno_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovb)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovb_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x42, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovb_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovb)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovb_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x42, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovb_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovb)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovb_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x42, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovb_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovae)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovae_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x43, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovae_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovae)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovae_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x43, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovae_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovae)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovae_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x43, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovae_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmove)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmove_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x44, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmove_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmove)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmove_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x44, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmove_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmove)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmove_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x44, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmove_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovne)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovne_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x45, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovne_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovne)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovne_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x45, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovne_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovne)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovne_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x45, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovne_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovbe)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovbe_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x46, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovbe_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovbe)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovbe_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x46, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovbe_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovbe)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovbe_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x46, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovbe_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmova)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmova_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x47, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmova_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmova)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmova_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x47, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmova_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmova)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmova_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x47, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmova_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovs)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovs_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x48, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovs_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovs)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovs_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x48, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovs_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovs)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovs_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x48, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovs_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovns)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovns_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x49, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovns_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovns)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovns_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x49, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovns_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovns)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovns_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x49, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovns_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovp)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovp_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4A, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovp_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovp)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovp_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4A, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovp_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovp)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovp_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4A, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovp_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovnp)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovnp_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovnp_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovnp)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovnp_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovnp_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovnp)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovnp_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4B, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovnp_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovl)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovl_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4C, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovl_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovl)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovl_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4C, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovl_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovl)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovl_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4C, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovl_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovge)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovge_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovge_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovge)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovge_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovge_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovge)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovge_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4D, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovge_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovle)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovle_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4E, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovle_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovle)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovle_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4E, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovle_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovle)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovle_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4E, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovle_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovg)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovg_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4F, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Cmovg_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovg)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovg_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4F, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Cmovg_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite32_ReadWrite64, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmovg)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmovg_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x4F, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Cmovg_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMOV)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Kandw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kandw_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x41, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kandw_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kandq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kandq_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x41, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kandq_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kandb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kandb_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x41, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kandb_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kandd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kandd_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x41, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kandd_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kandnw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kandnw_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x42, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kandnw_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kandnq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kandnq_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x42, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kandnq_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kandnb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kandnb_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x42, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kandnb_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kandnd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kandnd_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x42, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kandnd_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Knotw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Knotw_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x44, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Knotw_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Knotq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Knotq_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x44, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Knotq_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Knotb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Knotb_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x44, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Knotb_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Knotd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Knotd_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x44, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Knotd_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Korw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Korw_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x45, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Korw_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Korq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Korq_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x45, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Korq_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Korb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Korb_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x45, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Korb_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kord)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kord_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x45, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kord_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kxnorw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kxnorw_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x46, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kxnorw_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kxnorq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kxnorq_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x46, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kxnorq_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kxnorb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kxnorb_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x46, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kxnorb_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kxnord)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kxnord_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x46, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kxnord_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kxorw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kxorw_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x47, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kxorw_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kxorq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kxorq_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x47, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kxorq_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kxorb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kxorb_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x47, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kxorb_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kxord)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kxord_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x47, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kxord_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kaddw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kaddw_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x4A, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kaddw_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kaddq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kaddq_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x4A, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kaddq_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kaddb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kaddb_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x4A, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kaddb_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kaddd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kaddd_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x4A, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kaddd_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kunpckwd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kunpckwd_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x4B, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kunpckwd_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kunpckdq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kunpckdq_kr_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x4B, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kunpckdq_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kunpckbw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kunpckbw_kr_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x4B, -1, VexVectorLength.L1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.HK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kunpckbw_kr_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movmskps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movmskps_r32_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x50, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Movmskps_r32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movmskps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movmskps_r64_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x50, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Movmskps_r64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovmskps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovmskps_r32_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x50, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovmskps_r32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovmskps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovmskps_r64_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x50, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovmskps_r64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovmskps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovmskps_r32_ymm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x50, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.RY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovmskps_r32_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovmskps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovmskps_r64_ymm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x50, -1, VexVectorLength.L256, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.RY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovmskps_r64_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movmskpd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movmskpd_r32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x50, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Movmskpd_r32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movmskpd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movmskpd_r64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x50, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Movmskpd_r64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovmskpd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovmskpd_r32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x50, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovmskpd_r32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovmskpd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovmskpd_r64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x50, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovmskpd_r64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovmskpd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovmskpd_r32_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x50, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.RY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovmskpd_r32_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovmskpd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovmskpd_r64_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x50, -1, VexVectorLength.L256, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.RY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovmskpd_r64_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Sqrtps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sqrtps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x51, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sqrtps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsqrtps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x51, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vsqrtps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsqrtps_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x51, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vsqrtps_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsqrtps_xmm_k1z_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x51, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vsqrtps_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsqrtps_ymm_k1z_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x51, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vsqrtps_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x51, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Sqrtpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sqrtpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x51, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sqrtpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsqrtpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x51, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vsqrtpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsqrtpd_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x51, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vsqrtpd_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsqrtpd_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x51, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vsqrtpd_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsqrtpd_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x51, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vsqrtpd_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vsqrtpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x51, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Sqrtss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sqrtss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x51, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sqrtss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vsqrtss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsqrtss_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x51, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vsqrtss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsqrtss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x51, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Sqrtsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sqrtsd_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x51, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sqrtsd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vsqrtsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsqrtsd_xmm_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x51, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vsqrtsd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsqrtsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x51, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Rsqrtps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rsqrtps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x52, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Rsqrtps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrtps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vrsqrtps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x52, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vrsqrtps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrtps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vrsqrtps_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x52, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vrsqrtps_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Rsqrtss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rsqrtss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x52, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Rsqrtss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vrsqrtss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vrsqrtss_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x52, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vrsqrtss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Rcpps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcpps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x53, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Rcpps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vrcpps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vrcpps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x53, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vrcpps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrcpps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vrcpps_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x53, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vrcpps_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Rcpss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rcpss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x53, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Rcpss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vrcpss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vrcpss_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x53, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vrcpss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Andps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Andps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x54, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Andps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vandps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vandps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x54, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vandps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vandps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x54, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vandps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x54, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vandps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x54, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vandps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x54, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vandps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Andpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Andpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x54, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Andpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vandpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vandpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x54, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vandpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vandpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x54, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vandpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x54, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x54, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x54, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Andnps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Andnps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x55, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Andnps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vandnps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vandnps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x55, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vandnps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandnps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vandnps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x55, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vandnps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandnps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x55, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandnps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x55, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandnps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x55, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Andnpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Andnpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x55, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Andnpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vandnpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vandnpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x55, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vandnpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandnpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vandnpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x55, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vandnpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandnpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x55, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandnpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x55, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vandnpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x55, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Orps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Orps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x56, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Orps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vorps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vorps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x56, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vorps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vorps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vorps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x56, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vorps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vorps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vorps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x56, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vorps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vorps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vorps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x56, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vorps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vorps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vorps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x56, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vorps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Orpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Orpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x56, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Orpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vorpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vorpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x56, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vorpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vorpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vorpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x56, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vorpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vorpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x56, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vorpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x56, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vorpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x56, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Xorps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xorps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x57, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Xorps_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vxorps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vxorps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x57, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vxorps_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vxorps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vxorps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x57, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vxorps_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vxorps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x57, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vxorps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x57, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vxorps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x57, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Xorpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xorpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x57, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Xorpd_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vxorpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vxorpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x57, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vxorpd_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vxorpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vxorpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x57, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vxorpd_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vxorpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x57, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vxorpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x57, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vxorpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x57, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Addps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Addps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x58, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Addps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x58, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaddps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x58, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vaddps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x58, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x58, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x58, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Addpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Addpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x58, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Addpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x58, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaddpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x58, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vaddpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x58, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x58, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x58, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Addss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Addss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x58, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Addss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaddss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddss_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x58, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaddss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x58, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Addsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Addsd_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x58, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Addsd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaddsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddsd_xmm_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x58, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaddsd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x58, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Mulps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mulps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x59, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Mulps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmulps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmulps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x59, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmulps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmulps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x59, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmulps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x59, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x59, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x59, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Mulpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mulpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x59, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Mulpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmulpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmulpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x59, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmulpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmulpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x59, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmulpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x59, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x59, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x59, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Mulss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mulss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x59, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Mulss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmulss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmulss_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x59, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmulss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x59, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Mulsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mulsd_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x59, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Mulsd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmulsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmulsd_xmm_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x59, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmulsd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmulsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x59, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtps2pd)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtps2pd_xmm_xmmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtps2pd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2pd)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtps2pd_xmm_xmmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5A, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtps2pd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2pd)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtps2pd_ymm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtps2pd_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2pd)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5A, -1, EvexVectorLength.L128, TupleType.Half_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2pd)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5A, -1, EvexVectorLength.L256, TupleType.Half_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2pd)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5A, -1, EvexVectorLength.L512, TupleType.Half_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtpd2ps)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtpd2ps_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtpd2ps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2ps)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtpd2ps_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5A, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtpd2ps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2ps)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtpd2ps_xmm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtpd2ps_xmm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2ps)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5A, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2ps)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5A, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2ps)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5A, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtss2sd)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtss2sd_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtss2sd_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vcvtss2sd)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtss2sd_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5A, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtss2sd_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtss2sd)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5A, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtsd2ss)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtsd2ss_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtsd2ss_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsd2ss)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtsd2ss_xmm_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5A, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtsd2ss_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtsd2ss)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5A, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtdq2ps)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtdq2ps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtdq2ps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2ps)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtdq2ps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtdq2ps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2ps)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtdq2ps_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtdq2ps_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2ps)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2ps)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2ps)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtqq2ps)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtqq2ps)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtqq2ps)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtps2dq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtps2dq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtps2dq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2dq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtps2dq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtps2dq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2dq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtps2dq_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtps2dq_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2dq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2dq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2dq)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvttps2dq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvttps2dq_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvttps2dq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2dq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvttps2dq_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvttps2dq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2dq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvttps2dq_ymm_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vcvttps2dq_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2dq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2dq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2dq)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5B, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Subps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Subps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Subps_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsubps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vsubps_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsubps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vsubps_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Subpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Subpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Subpd_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsubpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vsubpd_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsubpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vsubpd_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Subss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Subss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Subss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vsubss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsubss_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5C, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vsubss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5C, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Subsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Subsd_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Subsd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vsubsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vsubsd_xmm_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5C, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vsubsd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vsubsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5C, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Minps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Minps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Minps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vminps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vminps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vminps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vminps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vminps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vminps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5D, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vminps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vminps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5D, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vminps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5D, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Minpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Minpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Minpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vminpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vminpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vminpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vminpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vminpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5D, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5D, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5D, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Minss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Minss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Minss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vminss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vminss_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5D, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vminss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Minsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Minsd_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Minsd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vminsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vminsd_xmm_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5D, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vminsd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vminsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Divps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Divps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Divps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vdivps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vdivps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vdivps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vdivps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vdivps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Divpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Divpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Divpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vdivpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vdivpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vdivpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vdivpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vdivpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Divss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Divss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Divss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vdivss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vdivss_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5E, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vdivss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5E, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Divsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Divsd_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Divsd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vdivsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vdivsd_xmm_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5E, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vdivsd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdivsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5E, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Maxps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Maxps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Maxps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmaxps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaxps_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmaxps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaxps_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmaxps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5F, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5F, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x5F, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Maxpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Maxpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Maxpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmaxpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaxpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmaxpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaxpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmaxpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5F, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5F, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x5F, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Maxss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Maxss_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Maxss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmaxss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaxss_xmm_xmm_xmmm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmaxss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x5F, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Maxsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Maxsd_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Maxsd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmaxsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaxsd_xmm_xmm_xmmm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmaxsd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaxsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x5F, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Punpcklbw)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpcklbw_mm_mmm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x60, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Punpcklbw_mm_mmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Punpcklbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpcklbw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x60, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Punpcklbw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpcklbw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x60, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpcklbw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklbw)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpcklbw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x60, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpcklbw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x60, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklbw)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x60, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklbw)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x60, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Punpcklwd)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpcklwd_mm_mmm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x61, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Punpcklwd_mm_mmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Punpcklwd)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpcklwd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x61, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Punpcklwd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklwd)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpcklwd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x61, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpcklwd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklwd)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpcklwd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x61, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpcklwd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklwd)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x61, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklwd)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x61, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklwd)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x61, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Punpckldq)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpckldq_mm_mmm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x62, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Punpckldq_mm_mmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Punpckldq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpckldq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x62, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Punpckldq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckldq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckldq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x62, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckldq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckldq)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckldq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x62, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckldq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckldq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x62, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckldq)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x62, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckldq)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x62, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Packsswb)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Packsswb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x63, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Packsswb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Packsswb)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Packsswb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x63, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Packsswb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpacksswb)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpacksswb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x63, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpacksswb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpacksswb)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpacksswb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x63, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpacksswb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpacksswb)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x63, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpacksswb)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x63, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpacksswb)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x63, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pcmpgtb)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpgtb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x64, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pcmpgtb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pcmpgtb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpgtb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x64, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pcmpgtb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpgtb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x64, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpgtb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpgtb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x64, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpgtb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x64, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtb_kr_k1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x64, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtb_kr_k1_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtb)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x64, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtb_kr_k1_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pcmpgtw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpgtw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x65, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pcmpgtw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pcmpgtw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpgtw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x65, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pcmpgtw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpgtw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x65, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpgtw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpgtw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x65, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpgtw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x65, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtw_kr_k1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x65, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtw_kr_k1_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x65, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtw_kr_k1_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pcmpgtd)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpgtd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x66, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pcmpgtd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pcmpgtd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpgtd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x66, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pcmpgtd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpgtd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x66, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpgtd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpgtd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x66, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpgtd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x66, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtd_kr_k1_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x66, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtd_kr_k1_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtd)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x66, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtd_kr_k1_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Packuswb)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Packuswb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x67, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Packuswb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Packuswb)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Packuswb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x67, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Packuswb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpackuswb)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpackuswb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x67, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpackuswb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackuswb)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpackuswb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x67, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpackuswb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackuswb)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x67, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackuswb)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x67, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackuswb)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x67, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Punpckhbw)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpckhbw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x68, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Punpckhbw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Punpckhbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpckhbw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x68, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Punpckhbw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckhbw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x68, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckhbw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhbw)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckhbw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x68, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckhbw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x68, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhbw)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x68, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhbw)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x68, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Punpckhwd)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpckhwd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x69, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Punpckhwd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Punpckhwd)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpckhwd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x69, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Punpckhwd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhwd)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckhwd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x69, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckhwd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhwd)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckhwd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x69, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckhwd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhwd)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x69, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhwd)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x69, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhwd)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x69, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Punpckhdq)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpckhdq_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x6A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Punpckhdq_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Punpckhdq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpckhdq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Punpckhdq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhdq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckhdq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6A, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckhdq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhdq)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckhdq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckhdq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhdq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6A, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhdq)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6A, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhdq)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6A, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Packssdw)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Packssdw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x6B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Packssdw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Packssdw)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Packssdw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Packssdw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpackssdw)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpackssdw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpackssdw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackssdw)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpackssdw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpackssdw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackssdw)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6B, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackssdw)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6B, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackssdw)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6B, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Punpcklqdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpcklqdq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Punpcklqdq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklqdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpcklqdq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpcklqdq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklqdq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpcklqdq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpcklqdq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklqdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklqdq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpcklqdq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Punpckhqdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Punpckhqdq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Punpckhqdq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhqdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckhqdq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckhqdq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhqdq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpunpckhqdq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpunpckhqdq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhqdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6D, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhqdq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6D, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpunpckhqdq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6D, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movd_mm_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x6E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Movd_mm_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movq_mm_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x6E, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Movq_mm_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movd_xmm_rm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Movd_xmm_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movq_xmm_rm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6E, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Movq_xmm_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovd_xmm_rm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.VX, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Vmovd_xmm_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovq_xmm_rm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6E, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Vmovq_xmm_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovd_xmm_rm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6E, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.VX, EvexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovd_xmm_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovq_xmm_rm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6E, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovq_xmm_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movq_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x6F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Movq_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movdqa)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdqa_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movdqa_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovdqa_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovdqa_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovdqa_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovdqa_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa32_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa32_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa32_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa32_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa32_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa32_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa64_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa64_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa64_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa64_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa64_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa64_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movdqu)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdqu_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x6F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movdqu_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovdqu_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x6F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovdqu_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovdqu_ymm_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x6F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovdqu_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu32_xmm_k1z_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu32_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu32_ymm_k1z_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu32_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu32_zmm_k1z_zmmm512)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu32_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu64_xmm_k1z_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu64_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu64_ymm_k1z_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu64_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu64_zmm_k1z_zmmm512)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu64_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu8_xmm_k1z_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu8_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu8)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu8_ymm_k1z_ymmm256)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu8_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu8)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu8_zmm_k1z_zmmm512)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu8_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu16)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu16_xmm_k1z_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu16_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu16)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu16_ymm_k1z_ymmm256)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu16_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu16)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu16_zmm_k1z_zmmm512)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x6F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu16_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pshufw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pshufw_mm_mmm64_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x70, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pshufw_mm_mmm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pshufd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pshufd_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x70, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pshufd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpshufd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpshufd_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x70, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpshufd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpshufd_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x70, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpshufd_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x70, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x70, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x70, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pshufhw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pshufhw_xmm_xmmm128_imm8)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x70, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pshufhw_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpshufhw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpshufhw_xmm_xmmm128_imm8)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x70, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpshufhw_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufhw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpshufhw_ymm_ymmm256_imm8)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x70, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpshufhw_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufhw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x70, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufhw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x70, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufhw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x70, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pshuflw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pshuflw_xmm_xmmm128_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x70, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pshuflw_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpshuflw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpshuflw_xmm_xmmm128_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x70, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpshuflw_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshuflw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpshuflw_ymm_ymmm256_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x70, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpshuflw_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshuflw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x70, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshuflw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x70, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshuflw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x70, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psrlw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrlw_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x71, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psrlw_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psrlw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrlw_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psrlw_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlw_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 2, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlw_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlw_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 2, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlw_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 2, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 2, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 2, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psraw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psraw_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x71, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psraw_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psraw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psraw_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psraw_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsraw_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 4, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsraw_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsraw_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 4, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsraw_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 4, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 4, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 4, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psllw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psllw_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x71, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psllw_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psllw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psllw_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psllw_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllw_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 6, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllw_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllw_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 6, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllw_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 6, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 6, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x71, 6, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprord)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprord_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 0, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprord_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprord)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprord_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 0, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprord_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprord)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprord_zmm_k1z_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 0, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprord_zmm_k1z_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprorq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 0, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprorq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 0, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprorq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 0, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprold)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprold_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprold_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprold)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprold_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprold_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprold)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprold_zmm_k1z_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprold_zmm_k1z_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprolq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprolq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprolq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psrld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrld_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x72, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psrld_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psrld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrld_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psrld_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrld_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 2, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrld_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrld_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 2, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrld_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 2, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 2, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 2, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psrad)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrad_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x72, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psrad_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psrad)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrad_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psrad_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrad_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 4, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrad_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrad_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 4, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrad_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 4, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 4, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 4, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 4, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraq)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 4, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraq)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 4, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pslld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pslld_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x72, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pslld_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pslld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pslld_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pslld_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpslld_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 6, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpslld_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpslld_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 6, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpslld_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 6, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 6, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x72, 6, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psrlq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrlq_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x73, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psrlq_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psrlq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrlq_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psrlq_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlq_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 2, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlq_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlq_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 2, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlq_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 2, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 2, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 2, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psrldq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrldq_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psrldq_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrldq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrldq_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 3, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrldq_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrldq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrldq_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 3, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrldq_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrldq)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrldq_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 3, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrldq_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrldq)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrldq_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 3, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrldq_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrldq)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrldq_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 3, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrldq_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psllq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psllq_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x73, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psllq_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psllq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psllq_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Psllq_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllq_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 6, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllq_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllq_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 6, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllq_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 6, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 6, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 6, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pslldq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pslldq_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pslldq_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpslldq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpslldq_xmm_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 7, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HX, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpslldq_xmm_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslldq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpslldq_ymm_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 7, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.HY, VexOpKind.RY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpslldq_ymm_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslldq)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpslldq_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 7, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpslldq_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslldq)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpslldq_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 7, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpslldq_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslldq)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpslldq_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x73, 7, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpslldq_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pcmpeqb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpeqb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x74, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pcmpeqb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pcmpeqb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpeqb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x74, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pcmpeqb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpeqb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x74, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpeqb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpeqb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x74, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpeqb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x74, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqb_kr_k1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x74, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqb_kr_k1_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x74, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqb_kr_k1_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pcmpeqw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpeqw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x75, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pcmpeqw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pcmpeqw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpeqw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x75, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pcmpeqw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpeqw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x75, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpeqw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpeqw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x75, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpeqw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x75, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqw_kr_k1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x75, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqw_kr_k1_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x75, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqw_kr_k1_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pcmpeqd)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpeqd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x76, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pcmpeqd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pcmpeqd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpeqd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x76, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pcmpeqd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpeqd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x76, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpeqd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpeqd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x76, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpeqd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x76, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqd_kr_k1_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x76, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqd_kr_k1_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x76, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqd_kr_k1_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(0, mnemonic[nameof(Mnemonic.Emms)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Emms)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x77, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Emms)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Vzeroupper)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vzeroupper)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x77, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, Array.Empty<VexOpKind>()),
				new InstrInfo(code[nameof(Code.VEX_Vzeroupper)], CodeInfo.Vzeroall, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode)),
				(0, mnemonic[nameof(Mnemonic.Vzeroall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vzeroall)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x77, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, Array.Empty<VexOpKind>()),
				new InstrInfo(code[nameof(Code.VEX_Vzeroall)], CodeInfo.Vzeroall, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmread)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmread_rm32_r32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x78, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Vmread_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Vmread)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmread_rm64_r64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x78, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Vmread_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2udq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2udq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2udq)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2udq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2udq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2udq)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Extrq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Extrq_xmm_imm8_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x78, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RX, LegacyOpKind.Ib, LegacyOpKind.Ib11 }),
				new InstrInfo(code[nameof(Code.Extrq_xmm_imm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4A)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2uqq)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L128, TupleType.Half_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2uqq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L256, TupleType.Half_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2uqq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L512, TupleType.Half_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2uqq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2uqq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2uqq)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttss2usi)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttss2usi_r32_xmmm32_sae)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.Gd, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttss2usi_r32_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttss2usi)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttss2usi_r64_xmmm32_sae)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_4, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.Gq, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttss2usi_r64_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Insertq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Insertq_xmm_xmm_imm8_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x78, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.RX, LegacyOpKind.Ib, LegacyOpKind.Ib11 }),
				new InstrInfo(code[nameof(Code.Insertq_xmm_xmm_imm8_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4A)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvttsd2usi)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttsd2usi_r32_xmmm64_sae)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.Gd, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttsd2usi_r32_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttsd2usi)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttsd2usi_r64_xmmm64_sae)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x78, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_8, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions, new[] { EvexOpKind.Gq, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttsd2usi_r64_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmwrite)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmwrite_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x79, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Vmwrite_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Vmwrite)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmwrite_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x79, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Vmwrite_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2udq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2udq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2udq)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2udq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2udq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2udq)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Extrq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Extrq_xmm_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x79, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Extrq_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4A)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2uqq)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L128, TupleType.Half_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2uqq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L256, TupleType.Half_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2uqq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L512, TupleType.Half_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2uqq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2uqq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2uqq)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtss2usi)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtss2usi_r32_xmmm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.RoundingControl, new[] { EvexOpKind.Gd, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtss2usi_r32_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtss2usi)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtss2usi_r64_xmmm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_4, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl, new[] { EvexOpKind.Gq, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtss2usi_r64_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Insertq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Insertq_xmm_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x79, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Insertq_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4A)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtsd2usi)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtsd2usi_r32_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.RoundingControl, new[] { EvexOpKind.Gd, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtsd2usi_r32_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtsd2usi)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtsd2usi_r64_xmmm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x79, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_8, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl, new[] { EvexOpKind.Gq, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtsd2usi_r64_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2qq)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L128, TupleType.Half_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2qq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L256, TupleType.Half_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttps2qq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L512, TupleType.Half_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2qq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2qq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2qq)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtudq2pd)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Broadcast64_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L128, TupleType.Half_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtudq2pd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L256, TupleType.Half_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtudq2pd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L512, TupleType.Half_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtuqq2pd)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtuqq2pd)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtuqq2pd)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtudq2ps)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtudq2ps)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtudq2ps)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtuqq2ps)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtuqq2ps)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtuqq2ps)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7A, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2qq)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L128, TupleType.Half_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2qq)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L256, TupleType.Half_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtps2qq)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L512, TupleType.Half_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2qq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2qq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2qq)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtusi2ss)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.RoundingControl, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtusi2ss)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm64_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_8, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtusi2sd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm32_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG32 | OpCodeFlags.RoundingControl, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtusi2sd)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7B, -1, EvexVectorLength.L128, TupleType.Tuple1_Fixed_8, OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Haddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Haddpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Haddpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vhaddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vhaddpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vhaddpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vhaddpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vhaddpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vhaddpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Haddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Haddps_xmm_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Haddps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vhaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vhaddps_xmm_xmm_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vhaddps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vhaddps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vhaddps_ymm_ymm_ymmm256)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vhaddps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Hsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Hsubpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Hsubpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vhsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vhsubpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vhsubpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vhsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vhsubpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vhsubpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Hsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Hsubps_xmm_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Hsubps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vhsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vhsubps_xmm_xmm_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vhsubps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vhsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vhsubps_ymm_ymm_ymmm256)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vhsubps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movd_rm32_mm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x7E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.P }),
				new InstrInfo(code[nameof(Code.Movd_rm32_mm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movq_rm64_mm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x7E, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.P }),
				new InstrInfo(code[nameof(Code.Movq_rm64_mm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movd_rm32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movd_rm32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movq_rm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7E, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movq_rm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovd_rm32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Ed, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovd_rm32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovq_rm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7E, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Eq, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovq_rm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovd_rm32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7E, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.Ed, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovd_rm32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovq_rm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7E, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.Eq, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovq_rm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movq_xmm_xmmm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Movq_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovq_xmm_xmmm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovq_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovq_xmm_xmmm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7E, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovq_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movq_mmm64_mm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x7F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Q, LegacyOpKind.P }),
				new InstrInfo(code[nameof(Code.Movq_mmm64_mm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movdqa)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdqa_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.WX, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movdqa_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovdqa_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovdqa_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovdqa_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovdqa_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa32_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa32_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa32_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa32_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa32_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa32_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa64_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa64_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa64_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa64_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqa64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqa64_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqa64_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movdqu)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdqu_xmmm128_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.WX, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movdqu_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovdqu_xmmm128_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovdqu_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovdqu_ymmm256_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovdqu_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu32_xmmm128_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu32_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu32_ymmm256_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu32_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu32_zmmm512_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu32_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu64_xmmm128_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu64_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu64_ymmm256_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu64_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu64_zmmm512_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu64_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu8_xmmm128_k1z_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu8_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu8)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu8_ymmm256_k1z_ymm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu8_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu8)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu8_zmmm512_k1z_zmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu8_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu16)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu16_xmmm128_k1z_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu16_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu16)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu16_ymmm256_k1z_ymm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu16_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovdqu16)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovdqu16_zmmm512_k1z_zmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x7F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovdqu16_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Jo)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jo_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x80, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jo_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jo)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jo_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x80, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jo_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jo)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jo_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x80, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jo_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jno)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jno_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x81, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jno_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jno)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jno_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x81, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jno_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jno)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jno_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x81, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jno_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jb_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x82, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jb_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jb_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x82, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jb_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jb_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x82, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jb_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jae)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jae_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x83, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jae_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jae)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jae_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x83, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jae_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jae)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jae_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x83, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jae_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Je)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Je_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x84, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Je_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Je)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Je_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x84, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Je_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Je)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Je_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x84, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Je_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jne_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x85, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jne_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jne_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x85, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jne_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jne)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jne_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x85, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jne_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jbe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jbe_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x86, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jbe_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jbe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jbe_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x86, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jbe_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jbe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jbe_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x86, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jbe_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ja)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ja_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x87, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Ja_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ja)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ja_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x87, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Ja_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ja)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ja_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x87, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Ja_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Js)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Js_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x88, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Js_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Js)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Js_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x88, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Js_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Js)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Js_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x88, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Js_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jns)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jns_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x89, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jns_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jns)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jns_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x89, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jns_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jns)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jns_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x89, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jns_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jp_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8A, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jp_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jp_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8A, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jp_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jp_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jp_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jnp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jnp_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8B, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jnp_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jnp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jnp_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8B, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jnp_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jnp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jnp_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jnp_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jl)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jl_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8C, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jl_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jl)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jl_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8C, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jl_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jl)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jl_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jl_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jge)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jge_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8D, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jge_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jge)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jge_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8D, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jge_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jge)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jge_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jge_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jle)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jle_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8E, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jle_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jle)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jle_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8E, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jle_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jle)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jle_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jle_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jg_rel16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8F, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.Jw }),
				new InstrInfo(code[nameof(Code.Jg_rel16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jg_rel32_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8F, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.dJd }),
				new InstrInfo(code[nameof(Code.Jg_rel32_32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jg)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jg_rel32_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x8F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.BndPrefix | OpCodeFlags.HintTakenPrefix, new[] { LegacyOpKind.qJd }),
				new InstrInfo(code[nameof(Code.Jg_rel32_64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.ConditionalBranch)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Seto)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Seto_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x90, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Seto_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setno)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setno_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x91, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setno_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setb_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x92, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setb_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setae)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setae_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x93, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setae_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Sete)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sete_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x94, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Sete_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setne)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setne_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x95, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setne_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setbe)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setbe_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x96, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setbe_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Seta)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Seta_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x97, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Seta_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Sets)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sets_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x98, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Sets_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setns)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setns_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x99, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setns_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setp)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setp_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x9A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setp_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setnp)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setnp_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x9B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setnp_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.PF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setl)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setl_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x9C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setl_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setge)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setge_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x9D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setge_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setle)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setle_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x9E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setle_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Setg)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Setg_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x9F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Setg_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Kmovw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovw_kr_km16)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x90, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.WK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovw_kr_km16)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovq_kr_km64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x90, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.WK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovq_kr_km64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovb_kr_km8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x90, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.WK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovb_kr_km8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovd_kr_km32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x90, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.WK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovd_kr_km32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovw_m16_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x91, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.MK, VexOpKind.VK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovw_m16_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovq_m64_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x91, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.MK, VexOpKind.VK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovq_m64_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovb_m8_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x91, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.MK, VexOpKind.VK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovb_m8_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovd_m32_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x91, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.MK, VexOpKind.VK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovd_m32_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovw_kr_r32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x92, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.VEX_Kmovw_kr_r32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovb_kr_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x92, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.VEX_Kmovb_kr_r32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovd_kr_r32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x92, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.VK, VexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.VEX_Kmovd_kr_r32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovq_kr_r64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x92, -1, VexVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.Rq }),
				new InstrInfo(code[nameof(Code.VEX_Kmovq_kr_r64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovw_r32_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x93, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.Gd, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovw_r32_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovb_r32_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x93, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.Gd, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovb_r32_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovd_r32_kr)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x93, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovd_r32_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kmovq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kmovq_r64_kr)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x93, -1, VexVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kmovq_r64_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kortestw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kortestw_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x98, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kortestw_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kortestq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kortestq_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x98, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kortestq_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kortestb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kortestb_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x98, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kortestb_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Kortestd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kortestd_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x98, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Kortestd_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Ktestw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Ktestw_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x99, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Ktestw_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Ktestq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Ktestq_kr_kr)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x99, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Ktestq_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Ktestb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Ktestb_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x99, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Ktestb_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Ktestd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Ktestd_kr_kr)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0x99, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK }),
				new InstrInfo(code[nameof(Code.VEX_Ktestd_kr_kr)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushw_FS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA0, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.FS }),
				new InstrInfo(code[nameof(Code.Pushw_FS)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushd_FS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA0, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.FS }),
				new InstrInfo(code[nameof(Code.Pushd_FS)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushq_FS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.FS }),
				new InstrInfo(code[nameof(Code.Pushq_FS)], CodeInfo.Push_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popw_FS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA1, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.FS }),
				new InstrInfo(code[nameof(Code.Popw_FS)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popd_FS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA1, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.FS }),
				new InstrInfo(code[nameof(Code.Popd_FS)], CodeInfo.Pop_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popq_FS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.FS }),
				new InstrInfo(code[nameof(Code.Popq_FS)], CodeInfo.Pop_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Cpuid)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cpuid)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Cpuid)], CodeInfo.Cpuid, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CPUID)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bt_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA3, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Bt_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bt)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bt_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA3, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Bt_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bt)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bt_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA3, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Bt_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shld)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shld_rm16_r16_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA4, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shld_rm16_r16_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shld)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shld_rm32_r32_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA4, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shld_rm32_r32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shld)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shld_rm64_r64_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA4, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shld_rm64_r64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shld)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shld_rm16_r16_CL)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA5, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shld_rm16_r16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shld)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shld_rm32_r32_CL)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA5, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shld_rm32_r32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shld)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shld_rm64_r64_CL)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA5, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shld_rm64_r64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Montmul)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Montmul_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6C0, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Montmul_16)], CodeInfo.Montmul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_PMM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Montmul)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Montmul_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6C0, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Montmul_32)], CodeInfo.Montmul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_PMM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Montmul)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Montmul_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6C0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Montmul_64)], CodeInfo.Montmul, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_PMM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xsha1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsha1_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6C8, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xsha1_16)], CodeInfo.Xsha, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_PHE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xsha1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsha1_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6C8, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xsha1_32)], CodeInfo.Xsha, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_PHE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xsha1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsha1_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6C8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xsha1_64)], CodeInfo.Xsha, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_PHE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xsha256)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsha256_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6D0, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xsha256_16)], CodeInfo.Xsha, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_PHE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xsha256)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsha256_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6D0, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xsha256_32)], CodeInfo.Xsha, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_PHE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xsha256)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsha256_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6D0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xsha256_64)], CodeInfo.Xsha, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_PHE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xbts)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xbts_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Xbts_r16_rm16)], CodeInfo.Xbts, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_A0_ONLY)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xbts)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xbts_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Xbts_r32_rm32)], CodeInfo.Xbts, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_A0_ONLY)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xstore)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xstore_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7C0, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xstore_16)], CodeInfo.Xstore, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_RNG)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xstore)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xstore_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7C0, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xstore_32)], CodeInfo.Xstore, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_RNG)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xstore)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xstore_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7C0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xstore_64)], CodeInfo.Xstore, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_RNG)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptecb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptEcb_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7C8, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptEcb_16)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptecb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptEcb_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7C8, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptEcb_32)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptecb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptEcb_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7C8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptEcb_64)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptcbc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptCbc_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7D0, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptCbc_16)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptcbc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptCbc_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7D0, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptCbc_32)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptcbc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptCbc_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7D0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptCbc_64)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptctr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptCtr_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7D8, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptCtr_16)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptctr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptCtr_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7D8, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptCtr_32)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptctr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptCtr_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7D8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptCtr_64)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptcfb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptCfb_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7E0, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptCfb_16)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptcfb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptCfb_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7E0, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptCfb_32)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptcfb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptCfb_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7E0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptCfb_64)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptofb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptOfb_16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7E8, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptOfb_16)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptofb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptOfb_32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7E8, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptOfb_32)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xcryptofb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.XcryptOfb_64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7E8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.RepPrefix, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.XcryptOfb_64)], CodeInfo.Xcrypt, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PADLOCK_ACE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ibts)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ibts_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Ibts_rm16_r16)], CodeInfo.Xbts, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_A0_ONLY)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ibts)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ibts_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Ibts_rm32_r32)], CodeInfo.Xbts, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386_A0_ONLY)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpxchg)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpxchg486_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Cmpxchg486_rm8_r8)], CodeInfo.Cmpxchg, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486_A_ONLY)] }, new[] { OpInfo.ReadCondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpxchg)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpxchg486_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Cmpxchg486_rm16_r16)], CodeInfo.Cmpxchg, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486_A_ONLY)] }, new[] { OpInfo.ReadCondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpxchg)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpxchg486_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA7, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Cmpxchg486_rm32_r32)], CodeInfo.Cmpxchg, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486_A_ONLY)] }, new[] { OpInfo.ReadCondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushw_GS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA8, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.GS }),
				new InstrInfo(code[nameof(Code.Pushw_GS)], CodeInfo.Push_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushd_GS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA8, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.GS }),
				new InstrInfo(code[nameof(Code.Pushd_GS)], CodeInfo.Push_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Push)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pushq_GS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.GS }),
				new InstrInfo(code[nameof(Code.Pushq_GS)], CodeInfo.Push_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popw_GS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA9, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.GS }),
				new InstrInfo(code[nameof(Code.Popw_GS)], CodeInfo.Pop_2, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popd_GS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA9, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.GS }),
				new InstrInfo(code[nameof(Code.Popd_GS)], CodeInfo.Pop_4, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(1, mnemonic[nameof(Mnemonic.Pop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popq_GS)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xA9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.GS }),
				new InstrInfo(code[nameof(Code.Popq_GS)], CodeInfo.Pop_8, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write }, InstrInfoFlags.StackInstruction)),
				(0, mnemonic[nameof(Mnemonic.Rsm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rsm)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Rsm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Return)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF | RflagsBits.DF | RflagsBits.IF | RflagsBits.AC, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Bts)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bts_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAB, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Bts_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bts)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bts_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAB, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Bts_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bts)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bts_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAB, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Bts_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shrd)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shrd_rm16_r16_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAC, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shrd_rm16_r16_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shrd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shrd_rm32_r32_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAC, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shrd_rm32_r32_imm8)], CodeInfo.Shift_Ib_MASK1F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shrd)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shrd_rm64_r64_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAC, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shrd_rm64_r64_imm8)], CodeInfo.Shift_Ib_MASK3F, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shrd)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shrd_rm16_r16_CL)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAD, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shrd_rm16_r16_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shrd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shrd_rm32_r32_CL)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAD, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shrd_rm32_r32_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Shrd)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shrd_rm64_r64_CL)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAD, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq, LegacyOpKind.CL }),
				new InstrInfo(code[nameof(Code.Shrd_rm64_r64_CL)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.OF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fxsave)], memSize[nameof(MemorySize.Fxsave_512Byte)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fxsave_m512byte)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Fxsave_m512byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FXSR)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Fxsave64)], memSize[nameof(MemorySize.Fxsave64_512Byte)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fxsave64_m512byte)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Fxsave64_m512byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FXSR)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Rdfsbase)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdfsbase_r32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 0, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Rdfsbase_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FSGSBASE)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdfsbase)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdfsbase_r64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 0, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Rdfsbase_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FSGSBASE)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Fxrstor)], memSize[nameof(MemorySize.Fxsave_512Byte)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fxrstor_m512byte)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Fxrstor_m512byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FXSR)] }, new[] { OpInfo.Read }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Fxrstor64)], memSize[nameof(MemorySize.Fxsave64_512Byte)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Fxrstor64_m512byte)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Fxrstor64_m512byte)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FXSR)] }, new[] { OpInfo.Read }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Rdgsbase)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdgsbase_r32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Rdgsbase_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FSGSBASE)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdgsbase)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdgsbase_r64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Rdgsbase_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FSGSBASE)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ldmxcsr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ldmxcsr_m32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Md }),
				new InstrInfo(code[nameof(Code.Ldmxcsr_m32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Wrfsbase)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrfsbase_r32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 2, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Wrfsbase_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FSGSBASE)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Wrfsbase)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrfsbase_r64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 2, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Wrfsbase_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FSGSBASE)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Vldmxcsr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vldmxcsr_m32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 2, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.Md }),
				new InstrInfo(code[nameof(Code.VEX_Vldmxcsr_m32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Stmxcsr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Stmxcsr_m32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Md }),
				new InstrInfo(code[nameof(Code.Stmxcsr_m32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Wrgsbase)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrgsbase_r32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Wrgsbase_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FSGSBASE)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Wrgsbase)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrgsbase_r64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Wrgsbase_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FSGSBASE)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Vstmxcsr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vstmxcsr_m32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 3, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.Md }),
				new InstrInfo(code[nameof(Code.VEX_Vstmxcsr_m32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Xsave)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsave_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xsave_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVE)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Xsave64)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsave64_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xsave64_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVE)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Ptwrite)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ptwrite_rm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Ptwrite_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PTWRITE)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Ptwrite)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ptwrite_rm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Ptwrite_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PTWRITE)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Xrstor)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xrstor_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xrstor_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVE)] }, new[] { OpInfo.Read }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Xrstor64)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xrstor64_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xrstor64_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVE)] }, new[] { OpInfo.Read }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Incsspd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Incsspd_r32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Incsspd_r32)], CodeInfo.Read_Reg8_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Incsspq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Incsspq_r64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Incsspq_r64)], CodeInfo.Read_Reg8_OpM1, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Xsaveopt)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsaveopt_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xsaveopt_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVEOPT)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Xsaveopt64)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsaveopt64_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xsaveopt64_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVEOPT)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Clwb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clwb_m8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Clwb_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CLWB)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Tpause)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Tpause_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Tpause_r32)], CodeInfo.R_EAX_EDX_Op0_GPR32, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.WAITPKG)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Tpause)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Tpause_r64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Tpause_r64)], CodeInfo.R_EAX_EDX_Op0_GPR32, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.WAITPKG)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Clrssbsy)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clrssbsy_m64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mq }),
				new InstrInfo(code[nameof(Code.Clrssbsy_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Umonitor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umonitor_r16)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Rw }),
				new InstrInfo(code[nameof(Code.Umonitor_r16)], CodeInfo.Umonitor, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.WAITPKG)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Umonitor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umonitor_r32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Umonitor_r32)], CodeInfo.Umonitor, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.WAITPKG)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Umonitor)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umonitor_r64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Umonitor_r64)], CodeInfo.Umonitor, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.WAITPKG)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Umwait)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umwait_r32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Umwait_r32)], CodeInfo.R_EAX_EDX_Op0_GPR32, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.WAITPKG)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Umwait)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Umwait_r64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xAE, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Umwait_r64)], CodeInfo.R_EAX_EDX_Op0_GPR32, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.WAITPKG)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Clflush)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clflush_m8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAE, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Clflush_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CLFSH)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Clflushopt)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Clflushopt_m8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xAE, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.Clflushopt_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CLFLUSHOPT)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Lfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfence)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEE8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Lfence)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Lfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfence_E9)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEE9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Lfence_E9)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Lfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfence_EA)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEEA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Lfence_EA)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Lfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfence_EB)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEEB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Lfence_EB)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Lfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfence_EC)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEEC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Lfence_EC)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Lfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfence_ED)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEED, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Lfence_ED)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Lfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfence_EE)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEEE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Lfence_EE)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Lfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfence_EF)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEEF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Lfence_EF)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mfence)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mfence)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mfence_F1)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mfence_F1)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mfence_F2)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mfence_F2)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mfence_F3)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mfence_F3)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mfence_F4)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mfence_F4)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mfence_F5)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mfence_F5)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mfence_F6)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mfence_F6)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Mfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mfence_F7)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Mfence_F7)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sfence)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sfence)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sfence_F9)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEF9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sfence_F9)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sfence_FA)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEFA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sfence_FA)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sfence_FB)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEFB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sfence_FB)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sfence_FC)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEFC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sfence_FC)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sfence_FD)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEFD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sfence_FD)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sfence_FE)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEFE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sfence_FE)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Sfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sfence_FF)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xAEFF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Sfence_FF)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Pcommit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcommit)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xAEF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pcommit)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PCOMMIT)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAF, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Imul_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAF, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Imul_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Imul)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Imul_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xAF, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Imul_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpxchg)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpxchg_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Cmpxchg_rm8_r8)], CodeInfo.Cmpxchg, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, new[] { OpInfo.ReadCondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpxchg)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpxchg_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB1, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Cmpxchg_rm16_r16)], CodeInfo.Cmpxchg, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, new[] { OpInfo.ReadCondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpxchg)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpxchg_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB1, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Cmpxchg_rm32_r32)], CodeInfo.Cmpxchg, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, new[] { OpInfo.ReadCondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Cmpxchg)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpxchg_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB1, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Cmpxchg_rm64_r64)], CodeInfo.Cmpxchg, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadCondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lss)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lss_r16_m1616)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB2, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lss_r16_m1616)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lss)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lss_r32_m1632)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB2, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lss_r32_m1632)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lss)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lss_r64_m1664)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB2, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lss_r64_m1664)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btr_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB3, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Btr_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btr_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB3, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Btr_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btr_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB3, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Btr_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lfs)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfs_r16_m1616)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB4, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lfs_r16_m1616)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lfs)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfs_r32_m1632)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB4, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lfs_r32_m1632)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lfs)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lfs_r64_m1664)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB4, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lfs_r64_m1664)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lgs)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lgs_r16_m1616)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB5, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lgs_r16_m1616)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lgs)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lgs_r32_m1632)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB5, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lgs_r32_m1632)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lgs)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lgs_r64_m1664)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB5, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Mp }),
				new InstrInfo(code[nameof(Code.Lgs_r64_m1664)], CodeInfo.Lds, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movzx)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movzx_r16_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB6, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Movzx_r16_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movzx)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movzx_r32_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB6, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Movzx_r32_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movzx)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movzx_r64_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB6, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Movzx_r64_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movzx)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movzx_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB7, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Movzx_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movzx)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movzx_r32_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB7, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Movzx_r32_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movzx)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movzx_r64_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB7, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Movzx_r64_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmpe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmpe_disp16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB8, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Jdisp16 }),
				new InstrInfo(code[nameof(Code.Jmpe_disp16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.IA64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Jmpe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Jmpe_disp32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB8, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Jdisp32 }),
				new InstrInfo(code[nameof(Code.Jmpe_disp32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.UnconditionalBranch)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.IA64)] }, new[] { OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Popcnt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popcnt_r16_rm16)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xB8, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Popcnt_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.POPCNT)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Popcnt)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popcnt_r32_rm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xB8, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Popcnt_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.POPCNT)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Popcnt)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Popcnt_r64_rm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xB8, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Popcnt_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.POPCNT)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ud1)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ud1_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB9, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Ud1_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ud1)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ud1_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB9, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Ud1_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ud1)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ud1_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xB9, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Ud1_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bt_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 4, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Bt_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bt)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bt_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 4, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Bt_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bt)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bt_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Bt_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bts)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bts_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 5, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Bts_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bts)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bts_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 5, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Bts_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bts)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bts_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Bts_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btr_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Btr_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btr_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Btr_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btr_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Btr_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btc)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btc_rm16_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 7, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Btc_rm16_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btc)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btc_rm32_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 7, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Btc_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btc)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btc_rm64_imm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBA, 7, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Btc_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btc)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btc_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBB, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Btc_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btc)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btc_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBB, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Btc_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Btc)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Btc_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBB, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Btc_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bsf)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bsf_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBC, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Bsf_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bsf)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bsf_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBC, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Bsf_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bsf)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bsf_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBC, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Bsf_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Tzcnt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Tzcnt_r16_rm16)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xBC, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Tzcnt_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Tzcnt)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Tzcnt_r32_rm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xBC, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Tzcnt_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Tzcnt)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Tzcnt_r64_rm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xBC, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Tzcnt_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bsr)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bsr_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBD, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Bsr_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bsr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bsr_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBD, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Bsr_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Bsr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bsr_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBD, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Bsr_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.CondWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lzcnt)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lzcnt_r16_rm16)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xBD, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Lzcnt_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LZCNT)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lzcnt)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lzcnt_r32_rm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xBD, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Lzcnt_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LZCNT)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Lzcnt)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lzcnt_r64_rm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xBD, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Lzcnt_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LZCNT)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsx)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsx_r16_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBE, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Movsx_r16_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsx)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsx_r32_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBE, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Movsx_r32_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsx)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsx_r64_rm8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBE, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Movsx_r64_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsx)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsx_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBF, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Movsx_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsx)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsx_r32_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBF, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Movsx_r32_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movsx)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movsx_r64_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xBF, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Movsx_r64_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xadd)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xadd_rm8_r8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eb, LegacyOpKind.Gb }),
				new InstrInfo(code[nameof(Code.Xadd_rm8_r8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xadd)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xadd_rm16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC1, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ew, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Xadd_rm16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xadd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xadd_rm32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC1, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Ed, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Xadd_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Xadd)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xadd_rm64_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC1, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Eq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Xadd_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Cmpps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpps_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Cmpps_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vcmpps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC2, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmpps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC2, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmpps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC2, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcmpps_kr_k1_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmpps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC2, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcmpps_kr_k1_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmpps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC2, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcmpps_kr_k1_zmm_zmmm512b32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Cmppd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmppd_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Cmppd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vcmppd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC2, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmppd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC2, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmppd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC2, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcmppd_kr_k1_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmppd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC2, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcmppd_kr_k1_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmppd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC2, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcmppd_kr_k1_zmm_zmmm512b64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Cmpss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpss_xmm_xmmm32_imm8)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xC2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Cmpss_xmm_xmmm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vcmpss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xC2, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmpss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xC2, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcmpss_kr_k1_xmm_xmmm32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Cmpsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpsd_xmm_xmmm64_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xC2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Cmpsd_xmm_xmmm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vcmpsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xC2, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vcmpsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xC2, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcmpsd_kr_k1_xmm_xmmm64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movnti)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movnti_m32_r32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Md, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Movnti_m32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movnti)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movnti_m64_r64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC3, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Mq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Movnti_m64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pinsrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pinsrw_mm_r32m16_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.RdMw, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pinsrw_mm_r32m16_imm8)], CodeInfo.Read_Reg16_OpM1_imm, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pinsrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pinsrw_mm_r64m16_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC4, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.RqMw, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pinsrw_mm_r64m16_imm8)], CodeInfo.Read_Reg16_OpM1_imm, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pinsrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pinsrw_xmm_r32m16_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.RdMw, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pinsrw_xmm_r32m16_imm8)], CodeInfo.Read_Reg16_OpM1_imm, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pinsrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pinsrw_xmm_r64m16_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC4, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.RqMw, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pinsrw_xmm_r64m16_imm8)], CodeInfo.Read_Reg16_OpM1_imm, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r32m16_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC4, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.RdMw, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r32m16_imm8)], CodeInfo.Read_Reg16_OpM1_imm, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r64m16_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC4, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.RqMw, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r64m16_imm8)], CodeInfo.Read_Reg16_OpM1_imm, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r32m16_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC4, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.RdMw, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r32m16_imm8)], CodeInfo.Read_Reg16_OpM1_imm, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r64m16_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC4, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.RqMw, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r64m16_imm8)], CodeInfo.Read_Reg16_OpM1_imm, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pextrw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrw_r32_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrw_r32_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pextrw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrw_r64_mm_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC5, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.N, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrw_r64_mm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pextrw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrw_r32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrw_r32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pextrw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrw_r64_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC5, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.RX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrw_r64_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpextrw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpextrw_r32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC5, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpextrw_r32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpextrw_r64_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC5, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.RX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpextrw_r64_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpextrw_r32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC5, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.Gd, EvexOpKind.RX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpextrw_r32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpextrw_r64_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC5, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.Gq, EvexOpKind.RX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpextrw_r64_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Shufps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shufps_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shufps_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vshufps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vshufps_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vshufps_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vshufps_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vshufps_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC6, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC6, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC6, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Shufpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Shufpd_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Shufpd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vshufpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vshufpd_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vshufpd_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vshufpd_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vshufpd_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC6, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC6, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC6, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Cmpxchg8b)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpxchg8b_m64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC7, 1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix | OpCodeFlags.XacquirePrefix | OpCodeFlags.XreleasePrefix, new[] { LegacyOpKind.Mq }),
				new InstrInfo(code[nameof(Code.Cmpxchg8b_m64)], CodeInfo.Cmpxchg8b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CX8)] }, new[] { OpInfo.ReadCondWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Cmpxchg16b)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cmpxchg16b_m128)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC7, 1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64 | OpCodeFlags.LockPrefix, new[] { LegacyOpKind.Mo }),
				new InstrInfo(code[nameof(Code.Cmpxchg16b_m128)], CodeInfo.Cmpxchg8b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CMPXCHG16B)] }, new[] { OpInfo.ReadCondWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Xrstors)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xrstors_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC7, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xrstors_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVES)] }, new[] { OpInfo.Read }, InstrInfoFlags.SaveRestore | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Xrstors64)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xrstors64_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC7, 3, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xrstors64_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVES)] }, new[] { OpInfo.Read }, InstrInfoFlags.SaveRestore | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Xsavec)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsavec_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC7, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xsavec_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVEC)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Xsavec64)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsavec64_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC7, 4, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xsavec64_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVEC)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Xsaves)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsaves_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC7, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xsaves_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVES)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Xsaves64)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsaves64_mem)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC7, 5, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Xsaves64_mem)], CodeInfo.R_EAX_EDX, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XSAVES)] }, new[] { OpInfo.Write }, InstrInfoFlags.SaveRestore | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Vmptrld)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmptrld_m64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC7, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Vmptrld_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Vmclear)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmclear_m64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xC7, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Vmclear_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Vmxon)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmxon_m64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xC7, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Vmxon_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Rdrand)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdrand_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC7, 6, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Rw }),
				new InstrInfo(code[nameof(Code.Rdrand_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDRAND)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdrand)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdrand_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC7, 6, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Rdrand_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDRAND)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdrand)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdrand_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC7, 6, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Rdrand_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDRAND)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Vmptrst)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmptrst_m64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xC7, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Vmptrst_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VMX)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Rdseed)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdseed_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC7, 7, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Rw }),
				new InstrInfo(code[nameof(Code.Rdseed_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDSEED)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdseed)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdseed_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC7, 7, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Rdseed_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDSEED)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdseed)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdseed_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC7, 7, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Rdseed_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDSEED)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdpid)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdpid_r32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xC7, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Rd }),
				new InstrInfo(code[nameof(Code.Rdpid_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDPID)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Rdpid)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rdpid_r64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xC7, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Rq }),
				new InstrInfo(code[nameof(Code.Rdpid_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.RDPID)] }, new[] { OpInfo.Write }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Bswap)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bswap_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC8, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.r16_rw }),
				new InstrInfo(code[nameof(Code.Bswap_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Bswap)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bswap_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC8, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.r32_rd }),
				new InstrInfo(code[nameof(Code.Bswap_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL486)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Bswap)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Bswap_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xC8, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.r64_ro }),
				new InstrInfo(code[nameof(Code.Bswap_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, new[] { OpInfo.ReadWrite }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Addsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Addsubpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Addsubpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaddsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddsubpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD0, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaddsubpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddsubpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD0, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vaddsubpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Addsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Addsubps_xmm_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xD0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Addsubps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaddsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddsubps_xmm_xmm_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xD0, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaddsubps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaddsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaddsubps_ymm_ymm_ymmm256)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xD0, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vaddsubps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psrlw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrlw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xD1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psrlw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psrlw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrlw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psrlw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD1, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlw_ymm_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD1, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlw_ymm_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD1, -1, EvexVectorLength.L128, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD1, -1, EvexVectorLength.L256, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD1, -1, EvexVectorLength.L512, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psrld)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrld_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xD2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psrld_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psrld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrld_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psrld_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrld_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD2, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrld_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrld_ymm_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD2, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrld_ymm_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD2, -1, EvexVectorLength.L128, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD2, -1, EvexVectorLength.L256, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD2, -1, EvexVectorLength.L512, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psrlq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrlq_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xD3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psrlq_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psrlq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrlq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psrlq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD3, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlq_ymm_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD3, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlq_ymm_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD3, -1, EvexVectorLength.L128, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD3, -1, EvexVectorLength.L256, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD3, -1, EvexVectorLength.L512, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Paddq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddq_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xD4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Paddq_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Paddq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Paddq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpaddq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD4, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD4, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD4, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD4, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD4, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmullw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmullw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xD5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pmullw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmullw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmullw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmullw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmullw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmullw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD5, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmullw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmullw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmullw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD5, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmullw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmullw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmullw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD5, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmullw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmullw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmullw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD5, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmullw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmullw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmullw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD5, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmullw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movq_xmmm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.WX, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movq_xmmm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovq_xmmm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.WX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovq_xmmm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovq_xmmm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD6, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovq_xmmm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movq2dq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movq2dq_xmm_mm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xD6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.N }),
				new InstrInfo(code[nameof(Code.Movq2dq_xmm_mm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movdq2q)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdq2q_mm_xmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xD6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Movdq2q_mm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmovmskb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovmskb_r32_mm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xD7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.N }),
				new InstrInfo(code[nameof(Code.Pmovmskb_r32_mm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmovmskb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovmskb_r64_mm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xD7, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.N }),
				new InstrInfo(code[nameof(Code.Pmovmskb_r64_mm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmovmskb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovmskb_r32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Pmovmskb_r32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmovmskb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovmskb_r64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD7, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Pmovmskb_r64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovmskb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovmskb_r32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD7, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovmskb_r32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovmskb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovmskb_r64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD7, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovmskb_r64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovmskb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovmskb_r32_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD7, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.RY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovmskb_r32_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovmskb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovmskb_r64_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD7, -1, VexVectorLength.L256, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.RY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovmskb_r64_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psubusb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubusb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xD8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psubusb_mm_mmm64)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psubusb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubusb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psubusb_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubusb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD8, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubusb_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubusb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD8, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubusb_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD8, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD8, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD8, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psubusw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubusw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xD9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psubusw_mm_mmm64)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psubusw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubusw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psubusw_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubusw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD9, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubusw_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubusw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD9, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubusw_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD9, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD9, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubusw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xD9, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pminub)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pminub_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xDA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pminub_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pminub)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pminub_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pminub_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpminub)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminub_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDA, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpminub_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminub)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminub_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDA, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpminub_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminub)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminub_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDA, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminub_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminub)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminub_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDA, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminub_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminub)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminub_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDA, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminub_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pand)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pand_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xDB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pand_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pand)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pand_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pand_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpand)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpand_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDB, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpand_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpand)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpand_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDB, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpand_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDB, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDB, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDB, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDB, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDB, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDB, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Paddusb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddusb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xDC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Paddusb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Paddusb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddusb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Paddusb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddusb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDC, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddusb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddusb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDC, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddusb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDC, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDC, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDC, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Paddusw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddusw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xDD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Paddusw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Paddusw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddusw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Paddusw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddusw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDD, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddusw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddusw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDD, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddusw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDD, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDD, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddusw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDD, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmaxub)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaxub_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xDE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pmaxub_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmaxub)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaxub_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmaxub_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxub)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxub_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDE, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxub_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxub)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxub_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDE, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxub_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxub)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDE, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxub)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDE, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxub)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDE, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pandn)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pandn_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xDF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pandn_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pandn)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pandn_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pandn_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpandn)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpandn_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDF, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpandn_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandn)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpandn_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDF, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpandn_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandnd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDF, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandnd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDF, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandnd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDF, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandnq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDF, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandnq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDF, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpandnq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xDF, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pavgb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pavgb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pavgb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pavgb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pavgb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pavgb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpavgb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpavgb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE0, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpavgb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpavgb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpavgb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE0, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpavgb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpavgb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpavgb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE0, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpavgb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpavgb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpavgb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE0, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpavgb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpavgb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpavgb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE0, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpavgb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psraw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psraw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xE1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psraw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psraw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psraw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psraw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsraw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE1, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsraw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsraw_ymm_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE1, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsraw_ymm_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE1, -1, EvexVectorLength.L128, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE1, -1, EvexVectorLength.L256, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE1, -1, EvexVectorLength.L512, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psrad)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrad_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xE2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psrad_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psrad)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psrad_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psrad_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrad_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE2, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrad_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrad_ymm_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE2, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrad_ymm_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE2, -1, EvexVectorLength.L128, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE2, -1, EvexVectorLength.L256, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrad)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE2, -1, EvexVectorLength.L512, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE2, -1, EvexVectorLength.L128, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE2, -1, EvexVectorLength.L256, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsraq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE2, -1, EvexVectorLength.L512, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pavgw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pavgw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xE3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pavgw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pavgw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pavgw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pavgw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpavgw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpavgw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE3, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpavgw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpavgw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpavgw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE3, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpavgw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpavgw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpavgw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE3, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpavgw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpavgw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpavgw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE3, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpavgw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpavgw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpavgw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE3, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpavgw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmulhuw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmulhuw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xE4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pmulhuw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmulhuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmulhuw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmulhuw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmulhuw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE4, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmulhuw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhuw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmulhuw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE4, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmulhuw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE4, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhuw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE4, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhuw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE4, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmulhw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmulhw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xE5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pmulhw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmulhw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmulhw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmulhw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmulhw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE5, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmulhw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmulhw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE5, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmulhw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE5, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE5, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE5, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvttpd2dq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvttpd2dq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvttpd2dq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2dq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvttpd2dq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvttpd2dq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2dq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvttpd2dq_xmm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vcvttpd2dq_xmm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2dq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2dq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvttpd2dq)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtdq2pd)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtdq2pd_xmm_xmmm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xE6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtdq2pd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2pd)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtdq2pd_xmm_xmmm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xE6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtdq2pd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2pd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtdq2pd_ymm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xE6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtdq2pd_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2pd)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Broadcast64_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L128, TupleType.Half_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2pd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L256, TupleType.Half_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtdq2pd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L512, TupleType.Half_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtqq2pd)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtqq2pd)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtqq2pd)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Cvtpd2dq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Cvtpd2dq_xmm_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xE6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Cvtpd2dq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2dq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtpd2dq_xmm_xmmm128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xE6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtpd2dq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2dq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtpd2dq_xmm_ymmm256)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xE6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtpd2dq_xmm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2dq)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2dq)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtpd2dq)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xE6, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movntq)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movntq_m64_mm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xE7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.P }),
				new InstrInfo(code[nameof(Code.Movntq_m64_mm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movntdq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movntdq_m128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.M, LegacyOpKind.VX }),
				new InstrInfo(code[nameof(Code.Movntdq_m128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovntdq_m128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE7, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmovntdq_m128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdq)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovntdq_m256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE7, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.M, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmovntdq_m256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntdq_m128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE7, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.M, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntdq_m128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdq)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntdq_m256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE7, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.M, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntdq_m256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdq)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntdq_m512_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE7, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.M, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntdq_m512_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psubsb)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubsb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xE8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psubsb_mm_mmm64)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psubsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubsb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psubsb_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubsb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE8, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubsb_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubsb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE8, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubsb_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE8, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE8, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsb)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE8, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psubsw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubsw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xE9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psubsw_mm_mmm64)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psubsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psubsw_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubsw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE9, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubsw_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubsw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE9, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubsw_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE9, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE9, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubsw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xE9, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pminsw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pminsw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xEA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pminsw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pminsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pminsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pminsw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpminsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminsw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEA, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpminsw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminsw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEA, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpminsw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEA, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEA, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEA, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Por)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Por_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xEB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Por_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Por)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Por_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Por_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpor)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpor_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEB, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpor_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpor)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpor_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEB, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpor_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpord)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpord_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEB, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpord_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpord)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpord_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEB, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpord_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpord)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpord_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEB, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpord_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vporq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vporq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEB, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vporq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vporq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vporq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEB, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vporq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vporq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vporq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEB, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vporq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Paddsb)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddsb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xEC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Paddsb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Paddsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddsb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Paddsb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddsb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEC, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddsb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddsb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEC, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddsb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEC, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEC, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsb)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEC, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Paddsw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddsw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xED, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Paddsw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Paddsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xED, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Paddsw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddsw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xED, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddsw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddsw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xED, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddsw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xED, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xED, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddsw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xED, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmaxsw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaxsw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xEE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pmaxsw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmaxsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaxsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmaxsw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxsw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEE, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxsw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxsw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEE, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxsw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEE, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEE, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEE, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pxor)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pxor_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xEF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pxor_mm_mmm64)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pxor)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pxor_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pxor_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpxor)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpxor_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEF, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpxor_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpxor)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpxor_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEF, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpxor_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpxord)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEF, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpxord)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEF, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpxord)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEF, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpxorq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEF, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpxorq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEF, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpxorq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xEF, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Lddqu)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Lddqu_xmm_m128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xF0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Lddqu_xmm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vlddqu)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vlddqu_xmm_m128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xF0, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vlddqu_xmm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vlddqu)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vlddqu_ymm_m256)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0xF0, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vlddqu_ymm_m256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psllw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psllw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xF1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psllw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psllw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psllw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF1, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psllw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF1, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllw_ymm_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF1, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllw_ymm_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF1, -1, EvexVectorLength.L128, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF1, -1, EvexVectorLength.L256, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF1, -1, EvexVectorLength.L512, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pslld)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pslld_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xF2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pslld_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pslld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pslld_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF2, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pslld_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpslld_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF2, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpslld_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpslld_ymm_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF2, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpslld_ymm_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF2, -1, EvexVectorLength.L128, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF2, -1, EvexVectorLength.L256, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpslld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF2, -1, EvexVectorLength.L512, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psllq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psllq_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xF3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psllq_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psllq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psllq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF3, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psllq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF3, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllq_ymm_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF3, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllq_ymm_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF3, -1, EvexVectorLength.L128, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF3, -1, EvexVectorLength.L256, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF3, -1, EvexVectorLength.L512, TupleType.Mem128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmuludq)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmuludq_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xF4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pmuludq_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmuludq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmuludq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF4, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmuludq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmuludq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmuludq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF4, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmuludq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmuludq)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmuludq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF4, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmuludq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmuludq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_2xUInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF4, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmuludq)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_2xUInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF4, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmuludq)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_2xUInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF4, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmaddwd)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaddwd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xF5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pmaddwd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmaddwd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaddwd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmaddwd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddwd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaddwd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF5, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaddwd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddwd)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaddwd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF5, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaddwd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddwd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF5, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddwd)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF5, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddwd)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF5, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psadbw)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psadbw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xF6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psadbw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psadbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psadbw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psadbw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsadbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsadbw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsadbw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsadbw)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsadbw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsadbw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsadbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsadbw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF6, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsadbw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsadbw)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsadbw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF6, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsadbw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsadbw)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsadbw_zmm_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF6, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsadbw_zmm_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Maskmovq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Maskmovq_rDI_mm_mm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xF7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.rDI, LegacyOpKind.P, LegacyOpKind.N }),
				new InstrInfo(code[nameof(Code.Maskmovq_rDI_mm_mm)], CodeInfo.Maskmovq, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Maskmovdqu)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Maskmovdqu_rDI_xmm_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF7, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.rDI, LegacyOpKind.VX, LegacyOpKind.RX }),
				new InstrInfo(code[nameof(Code.Maskmovdqu_rDI_xmm_xmm)], CodeInfo.Maskmovq, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vmaskmovdqu)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaskmovdqu_rDI_xmm_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF7, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.rDI, VexOpKind.VX, VexOpKind.RX }),
				new InstrInfo(code[nameof(Code.VEX_Vmaskmovdqu_rDI_xmm_xmm)], CodeInfo.Maskmovq, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psubb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psubb_mm_mmm64)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psubb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psubb_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsubb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF8, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubb_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF8, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubb_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF8, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubb_xmm_k1z_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF8, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubb_ymm_k1z_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF8, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubb_zmm_k1z_zmm_zmmm512)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psubw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xF9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psubw_mm_mmm64)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psubw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psubw_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsubw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF9, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubw_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF9, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubw_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF9, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubw_xmm_k1z_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF9, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubw_ymm_k1z_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xF9, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubw_zmm_k1z_zmm_zmmm512)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psubd)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xFA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psubd_mm_mmm64)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psubd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psubd_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsubd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFA, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubd_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFA, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubd_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFA, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFA, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFA, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psubq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubq_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xFB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psubq_mm_mmm64)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psubq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psubq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psubq_xmm_xmmm128)], CodeInfo.Clear_reg_regmem, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsubq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFB, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubq_xmm_xmm_xmmm128)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsubq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFB, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsubq_ymm_ymm_ymmm256)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFB, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFB, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsubq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFB, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.Clear_reg_reg_regmem, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Paddb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xFC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Paddb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Paddb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Paddb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpaddb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFC, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFC, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFC, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFC, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFC, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Paddw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xFD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Paddw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Paddw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Paddw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpaddw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFD, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFD, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFD, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFD, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFD, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Paddd)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0xFE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Paddd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MMX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Paddd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Paddd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Paddd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpaddd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFE, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpaddd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFE, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpaddd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFE, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFE, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpaddd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F, 0xFE, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Ud0)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ud0_r16_rm16)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xFF, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Ud0_r16_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ud0)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ud0_r32_rm32)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xFF, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Ud0_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL386)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Ud0)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ud0_r64_rm64)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xFF, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Ud0_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.X64)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pshufb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pshufb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x00, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pshufb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pshufb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pshufb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x00, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pshufb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpshufb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpshufb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x00, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpshufb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpshufb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x00, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpshufb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x00, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x00, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x00, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Phaddw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phaddw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x01, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Phaddw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Phaddw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phaddw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x01, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Phaddw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vphaddw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphaddw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x01, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vphaddw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vphaddw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphaddw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x01, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vphaddw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Phaddd)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phaddd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x02, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Phaddd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Phaddd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phaddd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x02, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Phaddd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vphaddd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphaddd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x02, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vphaddd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vphaddd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphaddd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x02, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vphaddd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Phaddsw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phaddsw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x03, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Phaddsw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Phaddsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phaddsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x03, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Phaddsw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vphaddsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphaddsw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x03, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vphaddsw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vphaddsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphaddsw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x03, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vphaddsw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmaddubsw)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaddubsw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x04, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pmaddubsw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmaddubsw)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaddubsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x04, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmaddubsw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddubsw)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaddubsw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x04, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaddubsw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddubsw)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaddubsw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x04, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaddubsw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddubsw)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x04, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddubsw)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x04, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaddubsw)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x04, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Phsubw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phsubw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x05, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Phsubw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Phsubw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phsubw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x05, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Phsubw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vphsubw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphsubw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x05, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vphsubw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vphsubw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphsubw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x05, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vphsubw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Phsubd)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phsubd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x06, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Phsubd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Phsubd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phsubd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x06, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Phsubd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vphsubd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphsubd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x06, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vphsubd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vphsubd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphsubd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x06, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vphsubd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Phsubsw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phsubsw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x07, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Phsubsw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Phsubsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phsubsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x07, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Phsubsw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vphsubsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphsubsw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x07, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vphsubsw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vphsubsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphsubsw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x07, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vphsubsw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psignb)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psignb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x08, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psignb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psignb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psignb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x08, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psignb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsignb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsignb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x08, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsignb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsignb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsignb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x08, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsignb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psignw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psignw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x09, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psignw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psignw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psignw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x09, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psignw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsignw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsignw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x09, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsignw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsignw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsignw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x09, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsignw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Psignd)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psignd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x0A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Psignd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Psignd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psignd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Psignd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsignd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsignd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0A, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsignd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsignd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsignd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsignd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmulhrsw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmulhrsw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x0B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pmulhrsw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmulhrsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmulhrsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmulhrsw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhrsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmulhrsw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmulhrsw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhrsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmulhrsw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmulhrsw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhrsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0B, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhrsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0B, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulhrsw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0B, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermilps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpermilps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermilps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpermilps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermilpd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpermilpd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermilpd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpermilpd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0D, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0D, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0D, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vtestps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vtestps_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vtestps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vtestps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vtestps_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vtestps_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vtestpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vtestpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vtestpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vtestpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vtestpd_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x0F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vtestpd_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pblendvb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pblendvb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x10, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pblendvb_xmm_xmmm128)], CodeInfo.R_XMM0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x10, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x10, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x10, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovuswb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovuswb_xmmm64_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x10, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovuswb_xmmm64_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovuswb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovuswb_xmmm128_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x10, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovuswb_xmmm128_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovuswb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovuswb_ymmm256_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x10, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovuswb_ymmm256_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsravw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x11, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsravw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsravw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x11, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsravw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsravw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x11, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsravw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusdb)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusdb_xmmm32_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x11, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusdb_xmmm32_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusdb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusdb_xmmm64_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x11, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusdb_xmmm64_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusdb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusdb_xmmm128_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x11, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusdb_xmmm128_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x12, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x12, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x12, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusqb)], memSize[nameof(MemorySize.Packed16_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusqb_xmmm16_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x12, -1, EvexVectorLength.L128, TupleType.Eighth_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusqb_xmmm16_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusqb)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusqb_xmmm32_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x12, -1, EvexVectorLength.L256, TupleType.Eighth_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusqb_xmmm32_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusqb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusqb_xmmm64_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x12, -1, EvexVectorLength.L512, TupleType.Eighth_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusqb_xmmm64_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtph2ps)], memSize[nameof(MemorySize.Packed64_Float16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtph2ps_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x13, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtph2ps_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.F16C)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtph2ps)], memSize[nameof(MemorySize.Packed128_Float16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtph2ps_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x13, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtph2ps_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.F16C)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtph2ps)], memSize[nameof(MemorySize.Packed64_Float16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtph2ps_xmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x13, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtph2ps_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtph2ps)], memSize[nameof(MemorySize.Packed128_Float16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtph2ps_ymm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x13, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtph2ps_ymm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtph2ps)], memSize[nameof(MemorySize.Packed256_Float16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x13, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusdw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusdw_xmmm64_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x13, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusdw_xmmm64_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusdw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusdw_xmmm128_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x13, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusdw_xmmm128_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusdw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusdw_ymmm256_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x13, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusdw_ymmm256_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blendvps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Blendvps_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x14, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Blendvps_xmm_xmmm128)], CodeInfo.R_XMM0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vprorvd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x14, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprorvd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x14, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprorvd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x14, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprorvq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x14, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprorvq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x14, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprorvq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x14, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusqw)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusqw_xmmm32_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x14, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusqw_xmmm32_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusqw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusqw_xmmm64_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x14, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusqw_xmmm64_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusqw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusqw_xmmm128_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x14, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusqw_xmmm128_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blendvpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Blendvpd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x15, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Blendvpd_xmm_xmmm128)], CodeInfo.R_XMM0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vprolvd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x15, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprolvd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x15, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprolvd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x15, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprolvq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x15, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprolvq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x15, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprolvq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x15, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusqd)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusqd_xmmm64_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x15, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusqd_xmmm64_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusqd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusqd_xmmm128_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x15, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusqd_xmmm128_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovusqd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovusqd_ymmm256_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x15, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovusqd_ymmm256_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x16, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpermps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x16, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x16, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x16, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x16, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Ptest)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ptest_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x17, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Ptest_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vptest)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vptest_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x17, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vptest_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vptest)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vptest_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x17, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vptest_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vbroadcastss_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x18, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vbroadcastss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.AVX2_Check)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vbroadcastss_ymm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x18, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vbroadcastss_ymm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.AVX2_Check)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastss_xmm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x18, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastss_xmm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastss_ymm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x18, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastss_ymm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastss_zmm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x18, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastss_zmm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vbroadcastsd_ymm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x19, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vbroadcastsd_ymm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.AVX2_Check)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastf32x2)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x19, -1, EvexVectorLength.L256, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastf32x2)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x19, -1, EvexVectorLength.L512, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastsd_ymm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x19, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastsd_ymm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastsd_zmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x19, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastsd_zmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastf128)], memSize[nameof(MemorySize.Float128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vbroadcastf128_ymm_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vbroadcastf128_ymm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastf32x4)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastf32x4_ymm_k1z_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1A, -1, EvexVectorLength.L256, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastf32x4_ymm_k1z_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastf32x4)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastf32x4_zmm_k1z_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1A, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastf32x4_zmm_k1z_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastf64x2)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastf64x2_ymm_k1z_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1A, -1, EvexVectorLength.L256, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastf64x2_ymm_k1z_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastf64x2)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastf64x2_zmm_k1z_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1A, -1, EvexVectorLength.L512, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastf64x2_zmm_k1z_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastf32x8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastf32x8_zmm_k1z_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1B, -1, EvexVectorLength.L512, TupleType.Tuple8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastf32x8_zmm_k1z_m256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcastf64x4)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcastf64x4_zmm_k1z_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1B, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcastf64x4_zmm_k1z_m256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pabsb)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pabsb_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x1C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pabsb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pabsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pabsb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pabsb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpabsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpabsb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpabsb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpabsb_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpabsb_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsb_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1C, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsb_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsb_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1C, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsb_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsb)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsb_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1C, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsb_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pabsw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pabsw_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x1D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pabsw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pabsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pabsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pabsw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpabsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpabsw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpabsw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpabsw_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpabsw_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsw_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1D, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsw_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsw_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1D, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsw_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsw_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1D, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsw_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pabsd)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pabsd_mm_mmm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x1E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q }),
				new InstrInfo(code[nameof(Code.Pabsd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pabsd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pabsd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pabsd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpabsd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpabsd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpabsd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpabsd_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpabsd_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsd_xmm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsd_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsd_ymm_k1z_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsd_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsd)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsd_zmm_k1z_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsd_zmm_k1z_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsq_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1F, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsq)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsq_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1F, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsq_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpabsq)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpabsq_zmm_k1z_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x1F, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpabsq_zmm_k1z_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovsxbw)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovsxbw_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x20, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovsxbw_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbw)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxbw_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x20, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxbw_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbw)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxbw_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x20, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxbw_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbw)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxbw_xmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x20, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxbw_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbw)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxbw_ymm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x20, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxbw_ymm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbw)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxbw_zmm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x20, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxbw_zmm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovswb)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovswb_xmmm64_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x20, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovswb_xmmm64_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovswb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovswb_xmmm128_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x20, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovswb_xmmm128_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovswb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovswb_ymmm256_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x20, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovswb_ymmm256_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovsxbd)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovsxbd_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x21, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovsxbd_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbd)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxbd_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x21, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxbd_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbd)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxbd_ymm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x21, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxbd_ymm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbd)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxbd_xmm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x21, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxbd_xmm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbd)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxbd_ymm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x21, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxbd_ymm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbd)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxbd_zmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x21, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxbd_zmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsdb)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsdb_xmmm32_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x21, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsdb_xmmm32_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsdb)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsdb_xmmm64_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x21, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsdb_xmmm64_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsdb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsdb_xmmm128_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x21, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsdb_xmmm128_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovsxbq)], memSize[nameof(MemorySize.Packed16_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovsxbq_xmm_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x22, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovsxbq_xmm_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbq)], memSize[nameof(MemorySize.Packed16_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxbq_xmm_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x22, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxbq_xmm_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbq)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxbq_ymm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x22, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxbq_ymm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbq)], memSize[nameof(MemorySize.Packed16_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxbq_xmm_k1z_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x22, -1, EvexVectorLength.L128, TupleType.Eighth_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxbq_xmm_k1z_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbq)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxbq_ymm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x22, -1, EvexVectorLength.L256, TupleType.Eighth_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxbq_ymm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxbq)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxbq_zmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x22, -1, EvexVectorLength.L512, TupleType.Eighth_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxbq_zmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsqb)], memSize[nameof(MemorySize.Packed16_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsqb_xmmm16_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x22, -1, EvexVectorLength.L128, TupleType.Eighth_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsqb_xmmm16_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsqb)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsqb_xmmm32_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x22, -1, EvexVectorLength.L256, TupleType.Eighth_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsqb_xmmm32_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsqb)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsqb_xmmm64_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x22, -1, EvexVectorLength.L512, TupleType.Eighth_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsqb_xmmm64_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovsxwd)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovsxwd_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x23, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovsxwd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwd)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxwd_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x23, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxwd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxwd_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x23, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxwd_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwd)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxwd_xmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x23, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxwd_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxwd_ymm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x23, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxwd_ymm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwd)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxwd_zmm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x23, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxwd_zmm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsdw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsdw_xmmm64_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x23, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsdw_xmmm64_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsdw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsdw_xmmm128_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x23, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsdw_xmmm128_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsdw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsdw_ymmm256_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x23, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsdw_ymmm256_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovsxwq)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovsxwq_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x24, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovsxwq_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwq)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxwq_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x24, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxwq_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwq)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxwq_ymm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x24, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxwq_ymm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwq)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxwq_xmm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x24, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxwq_xmm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwq)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxwq_ymm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x24, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxwq_ymm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxwq)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxwq_zmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x24, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxwq_zmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsqw)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsqw_xmmm32_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x24, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsqw_xmmm32_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsqw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsqw_xmmm64_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x24, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsqw_xmmm64_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsqw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsqw_xmmm128_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x24, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsqw_xmmm128_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovsxdq)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovsxdq_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x25, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovsxdq_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxdq)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxdq_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x25, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxdq_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxdq)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovsxdq_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x25, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovsxdq_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxdq)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxdq_xmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x25, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxdq_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxdq)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxdq_ymm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x25, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxdq_ymm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsxdq)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsxdq_zmm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x25, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsxdq_zmm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsqd)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsqd_xmmm64_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x25, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsqd_xmmm64_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsqd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsqd_xmmm128_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x25, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsqd_xmmm128_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovsqd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovsqd_ymmm256_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x25, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovsqd_ymmm256_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmb_kr_k1_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmb_kr_k1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmb_kr_k1_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmb_kr_k1_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmb_kr_k1_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmb_kr_k1_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmw_kr_k1_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmw_kr_k1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmw_kr_k1_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmw_kr_k1_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmw_kr_k1_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmw_kr_k1_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmb_kr_k1_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmb_kr_k1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmb_kr_k1_ymm_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmb_kr_k1_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmb_kr_k1_zmm_zmmm512)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmb_kr_k1_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmw_kr_k1_xmm_xmmm128)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmw_kr_k1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmw_kr_k1_ymm_ymmm256)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmw_kr_k1_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmw_kr_k1_zmm_zmmm512)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x26, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmw_kr_k1_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmd_kr_k1_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmd_kr_k1_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmd_kr_k1_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmd_kr_k1_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmd_kr_k1_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmd_kr_k1_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmq_kr_k1_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmq_kr_k1_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmq_kr_k1_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmq_kr_k1_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestmq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestmq_kr_k1_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestmq_kr_k1_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmd_kr_k1_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmd_kr_k1_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmd_kr_k1_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmq_kr_k1_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmq_kr_k1_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vptestnmq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x27, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vptestnmq_kr_k1_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmuldq)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmuldq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x28, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmuldq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmuldq)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmuldq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x28, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmuldq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmuldq)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmuldq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x28, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmuldq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmuldq)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_2xInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x28, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmuldq)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_2xInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x28, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmuldq)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_2xInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x28, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2b)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2b_xmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x28, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VX, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2b_xmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2b)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2b_ymm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x28, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VY, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2b_ymm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2b)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2b_zmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x28, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VZ, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2b_zmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2w)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2w_xmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x28, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2w_xmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2w)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2w_ymm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x28, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VY, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2w_ymm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2w)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2w_zmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x28, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VZ, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2w_zmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pcmpeqq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpeqq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x29, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pcmpeqq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpeqq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x29, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpeqq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpeqq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x29, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpeqq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x29, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqq_kr_k1_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x29, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqq_kr_k1_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpeqq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x29, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpeqq_kr_k1_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovb2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovb2m_kr_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x29, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VK, EvexOpKind.RX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovb2m_kr_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovb2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovb2m_kr_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x29, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VK, EvexOpKind.RY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovb2m_kr_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovb2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovb2m_kr_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x29, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VK, EvexOpKind.RZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovb2m_kr_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovw2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovw2m_kr_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x29, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VK, EvexOpKind.RX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovw2m_kr_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovw2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovw2m_kr_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x29, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VK, EvexOpKind.RY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovw2m_kr_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovw2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovw2m_kr_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x29, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VK, EvexOpKind.RZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovw2m_kr_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movntdqa)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movntdqa_xmm_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Movntdqa_xmm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdqa)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovntdqa_xmm_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2A, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmovntdqa_xmm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdqa)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmovntdqa_ymm_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmovntdqa_ymm_m256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdqa)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntdqa_xmm_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2A, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VX, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntdqa_xmm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdqa)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntdqa_ymm_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2A, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VY, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntdqa_ymm_m256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vmovntdqa)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vmovntdqa_zmm_m512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2A, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VZ, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vmovntdqa_zmm_m512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastmb2q)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastmb2q_xmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x2A, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastmb2q_xmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastmb2q)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastmb2q_ymm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x2A, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VY, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastmb2q_ymm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastmb2q)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastmb2q_zmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x2A, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VZ, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastmb2q_zmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Packusdw)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Packusdw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Packusdw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpackusdw)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpackusdw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpackusdw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackusdw)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpackusdw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpackusdw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackusdw)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2B, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackusdw)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2B, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpackusdw)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2B, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaskmovps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaskmovps_xmm_xmm_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmaskmovps_xmm_xmm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaskmovps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaskmovps_ymm_ymm_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmaskmovps_ymm_ymm_m256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vscalefps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vscalefps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vscalefps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vscalefpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vscalefpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vscalefpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaskmovpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaskmovpd_xmm_xmm_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmaskmovpd_xmm_xmm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaskmovpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaskmovpd_ymm_ymm_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vmaskmovpd_ymm_ymm_m256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vscalefss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vscalefsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaskmovps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaskmovps_m128_xmm_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.M, VexOpKind.HX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmaskmovps_m128_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaskmovps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaskmovps_m256_ymm_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.M, VexOpKind.HY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmaskmovps_m256_ymm_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaskmovpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaskmovpd_m128_xmm_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.M, VexOpKind.HX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vmaskmovpd_m128_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vmaskmovpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmaskmovpd_m256_ymm_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x2F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.M, VexOpKind.HY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vmaskmovpd_m256_ymm_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovzxbw)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovzxbw_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x30, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovzxbw_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbw)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxbw_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x30, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxbw_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxbw_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x30, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxbw_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbw)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxbw_xmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x30, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxbw_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxbw_ymm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x30, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxbw_ymm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbw)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxbw_zmm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x30, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxbw_zmm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovwb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovwb_xmmm64_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x30, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovwb_xmmm64_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovwb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovwb_xmmm128_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x30, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovwb_xmmm128_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovwb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovwb_ymmm256_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x30, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovwb_ymmm256_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovzxbd)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovzxbd_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x31, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovzxbd_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbd)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxbd_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x31, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxbd_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbd)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxbd_ymm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x31, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxbd_ymm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbd)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxbd_xmm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x31, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxbd_xmm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbd)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxbd_ymm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x31, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxbd_ymm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbd)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxbd_zmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x31, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxbd_zmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovdb)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovdb_xmmm32_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x31, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovdb_xmmm32_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovdb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovdb_xmmm64_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x31, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovdb_xmmm64_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovdb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovdb_xmmm128_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x31, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovdb_xmmm128_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovzxbq)], memSize[nameof(MemorySize.Packed16_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovzxbq_xmm_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x32, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovzxbq_xmm_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbq)], memSize[nameof(MemorySize.Packed16_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxbq_xmm_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x32, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxbq_xmm_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbq)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxbq_ymm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x32, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxbq_ymm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbq)], memSize[nameof(MemorySize.Packed16_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxbq_xmm_k1z_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x32, -1, EvexVectorLength.L128, TupleType.Eighth_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxbq_xmm_k1z_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbq)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxbq_ymm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x32, -1, EvexVectorLength.L256, TupleType.Eighth_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxbq_ymm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxbq)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxbq_zmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x32, -1, EvexVectorLength.L512, TupleType.Eighth_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxbq_zmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovqb)], memSize[nameof(MemorySize.Packed16_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovqb_xmmm16_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x32, -1, EvexVectorLength.L128, TupleType.Eighth_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovqb_xmmm16_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovqb)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovqb_xmmm32_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x32, -1, EvexVectorLength.L256, TupleType.Eighth_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovqb_xmmm32_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovqb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovqb_xmmm64_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x32, -1, EvexVectorLength.L512, TupleType.Eighth_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovqb_xmmm64_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovzxwd)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovzxwd_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x33, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovzxwd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwd)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxwd_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x33, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxwd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwd)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxwd_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x33, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxwd_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwd)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxwd_xmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x33, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxwd_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwd)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxwd_ymm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x33, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxwd_ymm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwd)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxwd_zmm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x33, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxwd_zmm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovdw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovdw_xmmm64_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x33, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovdw_xmmm64_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovdw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovdw_xmmm128_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x33, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovdw_xmmm128_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovdw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovdw_ymmm256_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x33, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovdw_ymmm256_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovzxwq)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovzxwq_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x34, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovzxwq_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwq)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxwq_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x34, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxwq_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwq)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxwq_ymm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x34, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxwq_ymm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwq)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxwq_xmm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x34, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxwq_xmm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwq)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxwq_ymm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x34, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxwq_ymm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxwq)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxwq_zmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x34, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxwq_zmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovqw)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovqw_xmmm32_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x34, -1, EvexVectorLength.L128, TupleType.Quarter_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovqw_xmmm32_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovqw)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovqw_xmmm64_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x34, -1, EvexVectorLength.L256, TupleType.Quarter_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovqw_xmmm64_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovqw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovqw_xmmm128_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x34, -1, EvexVectorLength.L512, TupleType.Quarter_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovqw_xmmm128_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmovzxdq)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmovzxdq_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x35, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmovzxdq_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxdq)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxdq_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x35, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxdq_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxdq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmovzxdq_ymm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x35, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmovzxdq_ymm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxdq)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxdq_xmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x35, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxdq_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxdq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxdq_ymm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x35, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxdq_ymm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovzxdq)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovzxdq_zmm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x35, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovzxdq_zmm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovqd)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovqd_xmmm64_k1z_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x35, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovqd_xmmm64_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovqd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovqd_xmmm128_k1z_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x35, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovqd_xmmm128_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovqd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovqd_ymmm256_k1z_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x35, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovqd_ymmm256_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x36, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpermd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x36, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x36, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x36, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x36, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pcmpgtq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpgtq_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x37, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pcmpgtq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpgtq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x37, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpgtq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtq)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpgtq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x37, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpgtq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x37, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtq_kr_k1_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtq)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x37, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtq_kr_k1_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpgtq)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x37, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpgtq_kr_k1_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pminsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pminsb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x38, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pminsb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpminsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminsb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x38, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpminsb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminsb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x38, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpminsb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x38, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x38, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsb)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x38, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2d)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2d_xmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x38, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VX, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2d_xmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2d)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2d_ymm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x38, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VY, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2d_ymm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2d)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2d_zmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x38, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VZ, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2d_zmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2q)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2q_xmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x38, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2q_xmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2q)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2q_ymm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x38, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VY, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2q_ymm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovm2q)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovm2q_zmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x38, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VZ, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovm2q_zmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pminsd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pminsd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x39, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pminsd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpminsd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminsd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x39, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpminsd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminsd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x39, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpminsd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsd)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsq)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminsq)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovd2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovd2m_kr_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VK, EvexOpKind.RX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovd2m_kr_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovd2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovd2m_kr_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VK, EvexOpKind.RY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovd2m_kr_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovd2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovd2m_kr_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VK, EvexOpKind.RZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovd2m_kr_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovq2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovq2m_kr_xmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VK, EvexOpKind.RX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovq2m_kr_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovq2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovq2m_kr_ymm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VK, EvexOpKind.RY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovq2m_kr_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpmovq2m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmovq2m_kr_zmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x39, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VK, EvexOpKind.RZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmovq2m_kr_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pminuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pminuw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pminuw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpminuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminuw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3A, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpminuw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminuw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminuw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpminuw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminuw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3A, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminuw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminuw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminuw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3A, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminuw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminuw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminuw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3A, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminuw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastmw2d)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastmw2d_xmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x3A, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VX, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastmw2d_xmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastmw2d)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastmw2d_ymm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x3A, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VY, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastmw2d_ymm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastmw2d)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastmw2d_zmm_kr)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x3A, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VZ, EvexOpKind.RK }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastmw2d_zmm_kr)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pminud)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pminud_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pminud_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpminud)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminud_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpminud_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminud)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpminud_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpminud_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminud)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3B, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminud)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3B, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminud)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3B, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminuq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3B, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminuq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3B, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpminuq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3B, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmaxsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaxsb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmaxsb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxsb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxsb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxsb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxsb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3C, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3C, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsb)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3C, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmaxsd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaxsd_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmaxsd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxsd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxsd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxsd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxsd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3D, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3D, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsd)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3D, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3D, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsq)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3D, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxsq)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3D, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmaxuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaxuw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmaxuw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxuw_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxuw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxuw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxuw_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxuw_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3E, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxuw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3E, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxuw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3E, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmaxud)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmaxud_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmaxud_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxud)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxud_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxud_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxud)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaxud_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaxud_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxud)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3F, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxud)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3F, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxud)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3F, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxuq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3F, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxuq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3F, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaxuq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x3F, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pmulld)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pmulld_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x40, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Pmulld_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpmulld)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmulld_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x40, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmulld_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulld)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmulld_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x40, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmulld_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulld)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x40, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulld)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x40, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmulld)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x40, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmullq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x40, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmullq)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x40, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmullq)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x40, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Phminposuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Phminposuw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x41, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Phminposuw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vphminposuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vphminposuw_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x41, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vphminposuw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vgetexpps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetexpps_xmm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x42, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetexpps_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vgetexpps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetexpps_ymm_k1z_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x42, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetexpps_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vgetexpps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x42, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vgetexppd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetexppd_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x42, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetexppd_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vgetexppd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetexppd_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x42, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetexppd_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vgetexppd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x42, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgetexpss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x43, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgetexpsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x43, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vplzcntd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vplzcntd_xmm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x44, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vplzcntd_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vplzcntd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vplzcntd_ymm_k1z_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x44, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vplzcntd_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vplzcntd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vplzcntd_zmm_k1z_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x44, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vplzcntd_zmm_k1z_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vplzcntq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vplzcntq_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x44, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vplzcntq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vplzcntq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vplzcntq_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x44, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vplzcntq_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vplzcntq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vplzcntq_zmm_k1z_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x44, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vplzcntq_zmm_k1z_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlvd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlvd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlvd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlvd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlvq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlvq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsrlvq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsrlvq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsrlvq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x45, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsravd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x46, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsravd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsravd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x46, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsravd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x46, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x46, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x46, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x46, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x46, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsravq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x46, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllvd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllvd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllvd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllvd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllvq_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllvq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpsllvq_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vpsllvq_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpsllvq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x47, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrcp14ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp14ps_xmm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp14ps_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrcp14ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp14ps_ymm_k1z_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp14ps_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrcp14ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp14ps_zmm_k1z_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp14ps_zmm_k1z_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrcp14pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp14pd_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp14pd_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrcp14pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp14pd_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp14pd_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrcp14pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp14pd_zmm_k1z_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp14pd_zmm_k1z_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrcp14ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrcp14sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrt14ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrt14ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrt14ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrt14pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrt14pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrt14pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrsqrt14ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4F, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrsqrt14sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4F, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpbusd)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x50, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpbusd)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x50, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpbusd)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x50, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpbusds)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x51, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpbusds)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x51, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpbusds)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x51, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpwssd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Broadcast128_2xInt16)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x52, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpwssd)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Broadcast256_2xInt16)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x52, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpwssd)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Broadcast512_2xInt16)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x52, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdpbf16ps)], memSize[nameof(MemorySize.Packed128_2xBFloat16)], memSize[nameof(MemorySize.Broadcast128_2xBFloat16)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x52, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BF16)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdpbf16ps)], memSize[nameof(MemorySize.Packed256_2xBFloat16)], memSize[nameof(MemorySize.Broadcast256_2xBFloat16)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x52, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BF16)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vdpbf16ps)], memSize[nameof(MemorySize.Packed512_2xBFloat16)], memSize[nameof(MemorySize.Broadcast512_2xBFloat16)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x52, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.AVX512_BF16)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vp4dpwssd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x52, -1, EvexVectorLength.L512, TupleType.Tuple1_4X, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZP3, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_4VNNIW)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.ReadP3, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpwssds)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Broadcast128_2xInt16)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x53, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpwssds)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Broadcast256_2xInt16)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x53, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpdpwssds)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Broadcast512_2xInt16)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x53, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VNNI)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vp4dpwssds)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x53, -1, EvexVectorLength.L512, TupleType.Tuple1_4X, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZP3, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_4VNNIW)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.ReadP3, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntb_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x54, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntb_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BITALG)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntb_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x54, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntb_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BITALG)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntb_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x54, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntb_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_BITALG)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntw_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x54, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntw_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BITALG)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntw_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x54, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntw_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BITALG)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntw_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x54, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntw_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_BITALG)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntd_xmm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x55, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntd_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VPOPCNTDQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntd_ymm_k1z_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x55, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntd_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VPOPCNTDQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntd_zmm_k1z_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x55, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntd_zmm_k1z_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VPOPCNTDQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntq_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x55, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VPOPCNTDQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntq_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x55, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntq_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VPOPCNTDQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpopcntq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpopcntq_zmm_k1z_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x55, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpopcntq_zmm_k1z_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VPOPCNTDQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpbroadcastd_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x58, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpbroadcastd_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpbroadcastd_ymm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x58, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpbroadcastd_ymm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x58, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x58, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x58, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpbroadcastq_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x59, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpbroadcastq_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpbroadcastq_ymm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x59, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpbroadcastq_ymm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti32x2)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x59, -1, EvexVectorLength.L128, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti32x2)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x59, -1, EvexVectorLength.L256, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti32x2)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x59, -1, EvexVectorLength.L512, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x59, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x59, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x59, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti128)], memSize[nameof(MemorySize.Int128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vbroadcasti128_ymm_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x5A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vbroadcasti128_ymm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti32x4)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcasti32x4_ymm_k1z_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x5A, -1, EvexVectorLength.L256, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcasti32x4_ymm_k1z_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti32x4)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcasti32x4_zmm_k1z_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x5A, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcasti32x4_zmm_k1z_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti64x2)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcasti64x2_ymm_k1z_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x5A, -1, EvexVectorLength.L256, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcasti64x2_ymm_k1z_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti64x2)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcasti64x2_zmm_k1z_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x5A, -1, EvexVectorLength.L512, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcasti64x2_zmm_k1z_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti32x8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcasti32x8_zmm_k1z_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x5B, -1, EvexVectorLength.L512, TupleType.Tuple8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcasti32x8_zmm_k1z_m256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vbroadcasti64x4)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vbroadcasti64x4_zmm_k1z_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x5B, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_Vbroadcasti64x4_zmm_k1z_m256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandb_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x62, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandb_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandb_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x62, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandb_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandb_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x62, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandb_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandw_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x62, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandw_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandw_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x62, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandw_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandw_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x62, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandw_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressb_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x63, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressb_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressb_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x63, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressb_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressb_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x63, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressb_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressw_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x63, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressw_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressw_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x63, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressw_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressw_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x63, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressw_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x64, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x64, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x64, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x64, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x64, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x64, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vblendmps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x65, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vblendmps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x65, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vblendmps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x65, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vblendmpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x65, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vblendmpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x65, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vblendmpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x65, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x66, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x66, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x66, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x66, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x66, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpblendmw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x66, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vp2intersectd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vp2intersectd_kp1_xmm_xmmm128b32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x68, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast, new[] { EvexOpKind.VKP1, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vp2intersectd_kp1_xmm_xmmm128b32)], CodeInfo.KP1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VP2INTERSECT)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vp2intersectd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vp2intersectd_kp1_ymm_ymmm256b32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x68, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast, new[] { EvexOpKind.VKP1, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vp2intersectd_kp1_ymm_ymmm256b32)], CodeInfo.KP1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VP2INTERSECT)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vp2intersectd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vp2intersectd_kp1_zmm_zmmm512b32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x68, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast, new[] { EvexOpKind.VKP1, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vp2intersectd_kp1_zmm_zmmm512b32)], CodeInfo.KP1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.AVX512_VP2INTERSECT)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vp2intersectq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vp2intersectq_kp1_xmm_xmmm128b64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x68, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast, new[] { EvexOpKind.VKP1, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vp2intersectq_kp1_xmm_xmmm128b64)], CodeInfo.KP1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VP2INTERSECT)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vp2intersectq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vp2intersectq_kp1_ymm_ymmm256b64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x68, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast, new[] { EvexOpKind.VKP1, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vp2intersectq_kp1_ymm_ymmm256b64)], CodeInfo.KP1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VP2INTERSECT)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vp2intersectq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vp2intersectq_kp1_zmm_zmmm512b64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x68, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast, new[] { EvexOpKind.VKP1, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vp2intersectq_kp1_zmm_zmmm512b64)], CodeInfo.KP1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.AVX512_VP2INTERSECT)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshldvw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x70, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshldvw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x70, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshldvw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x70, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshldvd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x71, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshldvd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x71, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshldvd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x71, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshldvq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x71, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshldvq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x71, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshldvq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x71, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshrdvw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x72, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshrdvw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x72, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshrdvw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x72, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtneps2bf16)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x72, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BF16)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtneps2bf16)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x72, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BF16)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcvtneps2bf16)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x72, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.AVX512_BF16)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtne2ps2bf16)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x72, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BF16)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtne2ps2bf16)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x72, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BF16)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtne2ps2bf16)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x72, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.AVX512_BF16)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshrdvd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x73, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshrdvd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x73, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshrdvd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x73, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshrdvq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x73, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshrdvq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x73, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshrdvq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x73, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2b)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x75, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2b)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x75, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2b)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x75, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2w)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x75, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2w)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x75, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2w)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x75, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2d)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x76, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2d)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x76, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2d)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x76, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2q)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x76, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2q)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x76, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2q)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x76, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x77, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x77, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x77, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x77, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x77, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermi2pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x77, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastb)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpbroadcastb_xmm_xmmm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x78, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpbroadcastb_xmm_xmmm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastb)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpbroadcastb_ymm_xmmm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x78, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpbroadcastb_ymm_xmmm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastb)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_xmmm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x78, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_xmmm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastb)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_xmmm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x78, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_xmmm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastb)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_xmmm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x78, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_xmmm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastw)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpbroadcastw_xmm_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x79, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpbroadcastw_xmm_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastw)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpbroadcastw_ymm_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x79, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vpbroadcastw_ymm_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastw)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x79, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastw)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x79, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastw)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_xmmm16)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x79, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_xmmm16)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7A, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_r32)], CodeInfo.Read_Reg8_OpM1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7A, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_r32)], CodeInfo.Read_Reg8_OpM1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7A, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_r32)], CodeInfo.Read_Reg8_OpM1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7B, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_r32)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7B, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_r32)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7B, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_r32)], CodeInfo.Read_Reg16_OpM1, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7C, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_r32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7C, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_r32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7C, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.Rd }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_r32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_r64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7C, -1, EvexVectorLength.L128, TupleType.None, OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.Rq }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_r64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_r64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7C, -1, EvexVectorLength.L256, TupleType.None, OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.Rq }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_r64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpbroadcastq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_r64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7C, -1, EvexVectorLength.L512, TupleType.None, OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.Rq }),
				new InstrInfo(code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_r64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2b)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7D, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2b)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7D, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2b)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7D, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2w)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7D, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2w)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7D, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2w)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7D, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2d)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2d)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2d)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2q)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2q)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2q)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7F, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7F, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7F, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7F, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7F, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermt2pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x7F, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Invept)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invept_r32_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x80, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gd, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Invept_r32_m128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVEPT)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Invept)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invept_r64_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x80, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Invept_r64_m128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVEPT)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Invvpid)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invvpid_r32_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x81, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gd, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Invvpid_r32_m128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVVPID)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Invvpid)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invvpid_r64_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x81, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Invvpid_r64_m128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVVPID)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Invpcid)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invpcid_r32_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x82, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gd, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Invpcid_r32_m128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVPCID)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Invpcid)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invpcid_r64_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x82, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Invpcid_r64_m128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVPCID)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(3, mnemonic[nameof(Mnemonic.Vpmultishiftqb)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x83, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmultishiftqb)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x83, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmultishiftqb)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x83, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vexpandps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vexpandps_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x88, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vexpandps_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vexpandps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vexpandps_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x88, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vexpandps_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vexpandps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vexpandps_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x88, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vexpandps_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vexpandpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vexpandpd_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x88, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vexpandpd_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vexpandpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vexpandpd_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x88, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vexpandpd_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vexpandpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vexpandpd_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x88, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vexpandpd_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandd_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x89, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandd_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandd_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x89, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandd_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandd_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x89, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandd_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandq_xmm_k1z_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x89, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandq_xmm_k1z_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandq_ymm_k1z_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x89, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandq_ymm_k1z_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpexpandq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpexpandq_zmm_k1z_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x89, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpexpandq_zmm_k1z_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcompressps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcompressps_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8A, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcompressps_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcompressps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcompressps_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8A, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcompressps_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcompressps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcompressps_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8A, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcompressps_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcompresspd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcompresspd_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8A, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vcompresspd_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcompresspd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcompresspd_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8A, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vcompresspd_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vcompresspd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcompresspd_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8A, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vcompresspd_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressd_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8B, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressd_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressd_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8B, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressd_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressd_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8B, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressd_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressq_xmmm128_k1z_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8B, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressq_xmmm128_k1z_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressq_ymmm256_k1z_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8B, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressq_ymmm256_k1z_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpcompressq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcompressq_zmmm512_k1z_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8B, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WZ, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcompressq_zmmm512_k1z_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaskmovd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaskmovd_xmm_xmm_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaskmovd_xmm_xmm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaskmovd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaskmovd_ymm_ymm_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaskmovd_ymm_ymm_m256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaskmovq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaskmovq_xmm_xmm_m128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaskmovq_xmm_xmm_m128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaskmovq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaskmovq_ymm_ymm_m256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaskmovq_ymm_ymm_m256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8D, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8D, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8D, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermw_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8D, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermw_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermw_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8D, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermw_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermw_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8D, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermw_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaskmovd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaskmovd_m128_xmm_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.M, VexOpKind.HX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaskmovd_m128_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaskmovd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaskmovd_m256_ymm_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.M, VexOpKind.HY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaskmovd_m256_ymm_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaskmovq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaskmovq_m128_xmm_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.M, VexOpKind.HX, VexOpKind.VX }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaskmovq_m128_xmm_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmaskmovq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpmaskmovq_m256_ymm_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.M, VexOpKind.HY, VexOpKind.VY }),
				new InstrInfo(code[nameof(Code.VEX_Vpmaskmovq_m256_ymm_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufbitqmb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufbitqmb_kr_k1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BITALG)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufbitqmb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufbitqmb_kr_k1_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_BITALG)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshufbitqmb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x8F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshufbitqmb_kr_k1_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_BITALG)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpgatherdd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpgatherdd_xmm_vm32x_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.VM32X, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vpgatherdd_xmm_vm32x_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpgatherdd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpgatherdd_ymm_vm32y_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.VM32Y, VexOpKind.HY }),
				new InstrInfo(code[nameof(Code.VEX_Vpgatherdd_ymm_vm32y_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpgatherdq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpgatherdq_xmm_vm32x_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.VM32X, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vpgatherdq_xmm_vm32x_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpgatherdq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpgatherdq_ymm_vm32x_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.VM32X, VexOpKind.HY }),
				new InstrInfo(code[nameof(Code.VEX_Vpgatherdq_ymm_vm32x_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherdd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherdd_xmm_k1_vm32x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM32X }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherdd_xmm_k1_vm32x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherdd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherdd_ymm_k1_vm32y)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VY, EvexOpKind.VM32Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherdd_ymm_k1_vm32y)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherdd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherdd_zmm_k1_vm32z)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VZ, EvexOpKind.VM32Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherdd_zmm_k1_vm32z)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherdq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherdq_xmm_k1_vm32x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM32X }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherdq_xmm_k1_vm32x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherdq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherdq_ymm_k1_vm32x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VY, EvexOpKind.VM32X }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherdq_ymm_k1_vm32x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherdq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherdq_zmm_k1_vm32y)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x90, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VZ, EvexOpKind.VM32Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherdq_zmm_k1_vm32y)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(3, mnemonic[nameof(Mnemonic.Vpgatherqd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpgatherqd_xmm_vm64x_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.VM64X, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vpgatherqd_xmm_vm64x_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpgatherqd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpgatherqd_xmm_vm64y_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.VM64Y, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vpgatherqd_xmm_vm64y_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpgatherqq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpgatherqq_xmm_vm64x_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.VM64X, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vpgatherqq_xmm_vm64x_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpgatherqq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpgatherqq_ymm_vm64y_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.VM64Y, VexOpKind.HY }),
				new InstrInfo(code[nameof(Code.VEX_Vpgatherqq_ymm_vm64y_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherqd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM64X }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherqd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64y)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM64Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64y)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherqd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherqd_ymm_k1_vm64z)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VY, EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherqd_ymm_k1_vm64z)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherqq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherqq_xmm_k1_vm64x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM64X }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherqq_xmm_k1_vm64x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherqq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherqq_ymm_k1_vm64y)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VY, EvexOpKind.VM64Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherqq_ymm_k1_vm64y)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpgatherqq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpgatherqq_zmm_k1_vm64z)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x91, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VZ, EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vpgatherqq_zmm_k1_vm64z)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(3, mnemonic[nameof(Mnemonic.Vgatherdps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgatherdps_xmm_vm32x_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.VM32X, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vgatherdps_xmm_vm32x_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgatherdps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgatherdps_ymm_vm32y_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.VM32Y, VexOpKind.HY }),
				new InstrInfo(code[nameof(Code.VEX_Vgatherdps_ymm_vm32y_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgatherdpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgatherdpd_xmm_vm32x_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.VM32X, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vgatherdpd_xmm_vm32x_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgatherdpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgatherdpd_ymm_vm32x_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.VM32X, VexOpKind.HY }),
				new InstrInfo(code[nameof(Code.VEX_Vgatherdpd_ymm_vm32x_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vgatherdps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherdps_xmm_k1_vm32x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM32X }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherdps_xmm_k1_vm32x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherdps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherdps_ymm_k1_vm32y)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VY, EvexOpKind.VM32Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherdps_ymm_k1_vm32y)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherdps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherdps_zmm_k1_vm32z)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VZ, EvexOpKind.VM32Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherdps_zmm_k1_vm32z)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherdpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherdpd_xmm_k1_vm32x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM32X }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherdpd_xmm_k1_vm32x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherdpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherdpd_ymm_k1_vm32x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VY, EvexOpKind.VM32X }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherdpd_ymm_k1_vm32x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherdpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherdpd_zmm_k1_vm32y)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x92, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VZ, EvexOpKind.VM32Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherdpd_zmm_k1_vm32y)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(3, mnemonic[nameof(Mnemonic.Vgatherqps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgatherqps_xmm_vm64x_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.VM64X, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vgatherqps_xmm_vm64x_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgatherqps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgatherqps_xmm_vm64y_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.VM64Y, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vgatherqps_xmm_vm64y_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgatherqpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgatherqpd_xmm_vm64x_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.VM64X, VexOpKind.HX }),
				new InstrInfo(code[nameof(Code.VEX_Vgatherqpd_xmm_vm64x_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgatherqpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgatherqpd_ymm_vm64y_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.VM64Y, VexOpKind.HY }),
				new InstrInfo(code[nameof(Code.VEX_Vgatherqpd_ymm_vm64y_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.ReadWrite }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vgatherqps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM64X }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherqps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64y)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM64Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64y)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherqps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherqps_ymm_k1_vm64z)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VY, EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherqps_ymm_k1_vm64z)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherqpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherqpd_xmm_k1_vm64x)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VX, EvexOpKind.VM64X }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherqpd_xmm_k1_vm64x)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherqpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherqpd_ymm_k1_vm64y)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VY, EvexOpKind.VM64Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherqpd_ymm_k1_vm64y)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vgatherqpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherqpd_zmm_k1_vm64z)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x93, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VZ, EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherqpd_zmm_k1_vm64z)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub132ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub132ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub132ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub132ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub132pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub132pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub132pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub132pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub132pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x96, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd132ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd132ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd132ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd132ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd132pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd132pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd132pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd132pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd132pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x97, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd132ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd132ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd132ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd132ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd132pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd132pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd132pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd132pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x98, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd132ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x99, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd132ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd132sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x99, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd132sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x99, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd132sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x99, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub132ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub132ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub132ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub132ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub132pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub132pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub132pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub132pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9A, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.V4fmaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_V4fmaddps_zmm_k1z_zmmp3_m128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x9A, -1, EvexVectorLength.L512, TupleType.Tuple1_4X, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZP3, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_V4fmaddps_zmm_k1z_zmmp3_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_4FMAPS)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.ReadP3, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub132ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9B, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub132ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub132sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9B, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub132sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9B, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub132sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9B, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.V4fmaddss)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_V4fmaddss_xmm_k1z_xmmp3_m128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x9B, -1, EvexVectorLength.L128, TupleType.Tuple1_4X, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HXP3, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_V4fmaddss_xmm_k1z_xmmp3_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_4FMAPS)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadP3, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd132ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd132ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd132ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd132ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd132pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd132pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd132pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd132pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9C, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd132ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9D, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd132ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd132sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9D, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd132sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd132sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9D, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub132ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub132ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub132ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub132ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub132pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub132pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub132pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub132pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub132ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub132ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub132sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub132sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9F, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub132sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x9F, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterdd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterdd_vm32x_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA0, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32X, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterdd_vm32x_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterdd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterdd_vm32y_k1_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA0, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Y, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterdd_vm32y_k1_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterdd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterdd_vm32z_k1_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA0, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Z, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterdd_vm32z_k1_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterdq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA0, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32X, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterdq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA0, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32X, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterdq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterdq_vm32y_k1_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA0, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Y, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterdq_vm32y_k1_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterqd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterqd_vm64x_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA1, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64X, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterqd_vm64x_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterqd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterqd_vm64y_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA1, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Y, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterqd_vm64y_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterqd)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterqd_vm64z_k1_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA1, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterqd_vm64z_k1_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterqq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterqq_vm64x_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA1, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64X, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterqq_vm64x_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterqq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterqq_vm64y_k1_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA1, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Y, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterqq_vm64y_k1_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vpscatterqq)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpscatterqq_vm64z_k1_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA1, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpscatterqq_vm64z_k1_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterdps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterdps_vm32x_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA2, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32X, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterdps_vm32x_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterdps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterdps_vm32y_k1_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA2, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Y, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterdps_vm32y_k1_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterdps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterdps_vm32z_k1_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA2, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Z, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterdps_vm32z_k1_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterdpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA2, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32X, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterdpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA2, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32X, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterdpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterdpd_vm32y_k1_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA2, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Y, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterdpd_vm32y_k1_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterqps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterqps_vm64x_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA3, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64X, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterqps_vm64x_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterqps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterqps_vm64y_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA3, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Y, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterqps_vm64y_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterqps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterqps_vm64z_k1_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA3, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterqps_vm64z_k1_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterqpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterqpd_vm64x_k1_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA3, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64X, EvexOpKind.VX }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterqpd_vm64x_k1_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterqpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterqpd_vm64y_k1_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA3, -1, EvexVectorLength.L256, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Y, EvexOpKind.VY }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterqpd_vm64y_k1_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(2, mnemonic[nameof(Mnemonic.Vscatterqpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterqpd_vm64z_k1_zmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA3, -1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z, EvexOpKind.VZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterqpd_vm64z_k1_zmm)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.OpMaskRegReadWrite)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub213ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub213ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub213ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub213ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub213pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub213pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub213pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub213pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub213pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA6, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd213ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd213ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd213ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd213ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd213pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd213pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd213pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd213pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd213pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA7, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd213ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd213ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd213ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd213ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd213pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd213pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd213pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd213pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA8, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd213ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA9, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd213ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd213sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA9, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd213sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA9, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd213sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xA9, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub213ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub213ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub213ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub213ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub213pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub213pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub213pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub213pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAA, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.V4fnmaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xAA, -1, EvexVectorLength.L512, TupleType.Tuple1_4X, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZP3, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_4FMAPS)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.ReadP3, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub213ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAB, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub213ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub213sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAB, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub213sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAB, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub213sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAB, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.V4fnmaddss)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xAB, -1, EvexVectorLength.L128, TupleType.Tuple1_4X, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HXP3, EvexOpKind.M }),
				new InstrInfo(code[nameof(Code.EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_4FMAPS)] }, new[] { OpInfo.ReadWrite, OpInfo.ReadP3, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd213ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd213ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd213ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd213ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd213pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd213pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd213pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd213pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAC, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd213ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAD, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd213ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd213sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAD, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd213sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAD, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd213sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAD, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub213ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub213ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub213ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub213ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub213pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub213pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub213pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub213pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAE, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub213ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAF, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub213ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub213sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAF, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub213sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAF, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub213sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xAF, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmadd52luq)], memSize[nameof(MemorySize.Packed128_UInt52)], memSize[nameof(MemorySize.Broadcast128_UInt52)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB4, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_IFMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmadd52luq)], memSize[nameof(MemorySize.Packed256_UInt52)], memSize[nameof(MemorySize.Broadcast256_UInt52)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB4, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_IFMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmadd52luq)], memSize[nameof(MemorySize.Packed512_UInt52)], memSize[nameof(MemorySize.Broadcast512_UInt52)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB4, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_IFMA)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmadd52huq)], memSize[nameof(MemorySize.Packed128_UInt52)], memSize[nameof(MemorySize.Broadcast128_UInt52)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB5, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_IFMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmadd52huq)], memSize[nameof(MemorySize.Packed256_UInt52)], memSize[nameof(MemorySize.Broadcast256_UInt52)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB5, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_IFMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpmadd52huq)], memSize[nameof(MemorySize.Packed512_UInt52)], memSize[nameof(MemorySize.Broadcast512_UInt52)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB5, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_IFMA)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub231ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub231ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub231ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub231ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub231pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub231pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsub231pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsub231pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmaddsub231pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB6, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd231ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd231ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd231ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd231ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd231pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd231pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubadd231pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubadd231pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsubadd231pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB7, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd231ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd231ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd231ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd231ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd231pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd231pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd231pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd231pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB8, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd231ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB9, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd231ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmadd231sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB9, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmadd231sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB9, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmadd231sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xB9, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub231ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub231ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub231ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub231ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub231pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub231pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub231pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub231pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBA, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub231ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBB, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub231ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsub231sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBB, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsub231sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBB, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfmsub231sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBB, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd231ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd231ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd231ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd231ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd231pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd231pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd231pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd231pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBC, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd231ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBD, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd231ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmadd231sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBD, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmadd231sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBD, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmadd231sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBD, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub231ps_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub231ps_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub231ps_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub231ps_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub231pd_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub231pd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub231pd_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub231pd_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBE, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub231ss_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBF, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub231ss_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsub231sd_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBF, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsub231sd_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBF, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfnmsub231sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xBF, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.RoundingControl | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpconflictd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpconflictd_xmm_k1z_xmmm128b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC4, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpconflictd_xmm_k1z_xmmm128b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpconflictd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpconflictd_ymm_k1z_ymmm256b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC4, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpconflictd_ymm_k1z_ymmm256b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpconflictd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpconflictd_zmm_k1z_zmmm512b32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC4, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpconflictd_zmm_k1z_zmmm512b32)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpconflictq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpconflictq_xmm_k1z_xmmm128b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC4, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vpconflictq_xmm_k1z_xmmm128b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpconflictq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpconflictq_ymm_k1z_ymmm256b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC4, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vpconflictq_ymm_k1z_ymmm256b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vpconflictq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpconflictq_zmm_k1z_zmmm512b64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC4, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vpconflictq_zmm_k1z_zmmm512b64)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512CD)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vgatherpf0dps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherpf0dps_vm32z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC6, 1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherpf0dps_vm32z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vgatherpf0dpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherpf0dpd_vm32y_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC6, 1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherpf0dpd_vm32y_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vgatherpf1dps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherpf1dps_vm32z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC6, 2, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherpf1dps_vm32z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vgatherpf1dpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherpf1dpd_vm32y_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC6, 2, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherpf1dpd_vm32y_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vscatterpf0dps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterpf0dps_vm32z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC6, 5, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterpf0dps_vm32z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vscatterpf0dpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterpf0dpd_vm32y_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC6, 5, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterpf0dpd_vm32y_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vscatterpf1dps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterpf1dps_vm32z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC6, 6, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterpf1dps_vm32z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vscatterpf1dpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterpf1dpd_vm32y_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC6, 6, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM32Y }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterpf1dpd_vm32y_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vgatherpf0qps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherpf0qps_vm64z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC7, 1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherpf0qps_vm64z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vgatherpf0qpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherpf0qpd_vm64z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC7, 1, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherpf0qpd_vm64z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vgatherpf1qps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherpf1qps_vm64z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC7, 2, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherpf1qps_vm64z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vgatherpf1qpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgatherpf1qpd_vm64z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC7, 2, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vgatherpf1qpd_vm64z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vscatterpf0qps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterpf0qps_vm64z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC7, 5, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterpf0qps_vm64z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vscatterpf0qpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterpf0qpd_vm64z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC7, 5, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterpf0qpd_vm64z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vscatterpf1qps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterpf1qps_vm64z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC7, 6, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterpf1qps_vm64z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Vscatterpf1qpd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vscatterpf1qpd_vm64z_k1)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC7, 6, EvexVectorLength.L512, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.NonZeroOpMaskRegister, new[] { EvexOpKind.VM64Z }),
				new InstrInfo(code[nameof(Code.EVEX_Vscatterpf1qpd_vm64z_k1)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512PF)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Sha1nexte)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sha1nexte_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xC8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sha1nexte_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SHA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vexp2ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC8, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vexp2pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xC8, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Sha1msg1)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sha1msg1_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xC9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sha1msg1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SHA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Sha1msg2)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sha1msg2_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xCA, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sha1msg2_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SHA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vrcp28ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCA, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrcp28pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCA, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Sha256rnds2)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sha256rnds2_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xCB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sha256rnds2_xmm_xmmm128)], CodeInfo.R_XMM0, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SHA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vrcp28ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCB, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrcp28sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCB, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Sha256msg1)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sha256msg1_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xCC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sha256msg1_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SHA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrt28ps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCC, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vrsqrt28pd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCC, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.WriteVmm, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Sha256msg2)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sha256msg2_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xCD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Sha256msg2_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SHA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vrsqrt28ss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCD, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrsqrt28sd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCD, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512ER)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Gf2p8mulb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Gf2p8mulb_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Gf2p8mulb_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vgf2p8mulb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgf2p8mulb_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCF, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vgf2p8mulb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgf2p8mulb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgf2p8mulb_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCF, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vgf2p8mulb_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgf2p8mulb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCF, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgf2p8mulb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCF, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgf2p8mulb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xCF, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Aesimc)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aesimc_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDB, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Aesimc_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Vaesimc)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaesimc_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDB, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaesimc_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)], cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Aesenc)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aesenc_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Aesenc_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaesenc)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaesenc_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDC, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaesenc_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)], cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesenc)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaesenc_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDC, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vaesenc_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesenc)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesenc_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDC, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesenc_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesenc)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesenc_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDC, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesenc_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesenc)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesenc_zmm_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDC, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesenc_zmm_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Aesenclast)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aesenclast_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDD, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Aesenclast_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaesenclast)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaesenclast_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDD, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaesenclast_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)], cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesenclast)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaesenclast_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDD, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vaesenclast_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesenclast)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesenclast_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDD, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesenclast_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesenclast)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesenclast_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDD, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesenclast_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesenclast)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesenclast_zmm_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDD, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesenclast_zmm_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Aesdec)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aesdec_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Aesdec_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaesdec)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaesdec_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDE, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaesdec_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)], cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesdec)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaesdec_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDE, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vaesdec_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesdec)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesdec_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDE, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesdec_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesdec)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesdec_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDE, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesdec_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesdec)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesdec_zmm_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDE, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesdec_zmm_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Aesdeclast)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aesdeclast_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX }),
				new InstrInfo(code[nameof(Code.Aesdeclast_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaesdeclast)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaesdeclast_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDF, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vaesdeclast_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)], cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesdeclast)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaesdeclast_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDF, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vaesdeclast_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesdeclast)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesdeclast_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDF, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesdeclast_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesdeclast)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesdeclast_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDF, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesdeclast_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vaesdeclast)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vaesdeclast_zmm_zmm_zmmm512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xDF, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ }),
				new InstrInfo(code[nameof(Code.EVEX_Vaesdeclast_zmm_zmm_zmmm512)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.VAES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movbe)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movbe_r16_m16)], MandatoryPrefix.None, OpCodeTableKind.T0F38, 0xF0, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gw, LegacyOpKind.Mw }),
				new InstrInfo(code[nameof(Code.Movbe_r16_m16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVBE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movbe)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movbe_r32_m32)], MandatoryPrefix.None, OpCodeTableKind.T0F38, 0xF0, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Md }),
				new InstrInfo(code[nameof(Code.Movbe_r32_m32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVBE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movbe)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movbe_r64_m64)], MandatoryPrefix.None, OpCodeTableKind.T0F38, 0xF0, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Mq }),
				new InstrInfo(code[nameof(Code.Movbe_r64_m64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVBE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Crc32)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Crc32_r32_rm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF0, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Crc32_r32_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Crc32)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Crc32_r64_rm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF0, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eb }),
				new InstrInfo(code[nameof(Code.Crc32_r64_rm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movbe)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movbe_m16_r16)], MandatoryPrefix.None, OpCodeTableKind.T0F38, 0xF1, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mw, LegacyOpKind.Gw }),
				new InstrInfo(code[nameof(Code.Movbe_m16_r16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVBE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movbe)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movbe_m32_r32)], MandatoryPrefix.None, OpCodeTableKind.T0F38, 0xF1, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Md, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Movbe_m32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVBE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movbe)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movbe_m64_r64)], MandatoryPrefix.None, OpCodeTableKind.T0F38, 0xF1, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Mq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Movbe_m64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVBE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Crc32)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Crc32_r32_rm16)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF1, -1, OperandSize.Size16, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ew }),
				new InstrInfo(code[nameof(Code.Crc32_r32_rm16)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Crc32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Crc32_r32_rm32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF1, -1, OperandSize.Size32, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Crc32_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Crc32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Crc32_r64_rm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF1, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Crc32_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Andn)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Andn_r32_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF2, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Hd, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Andn_r32_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Andn)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Andn_r64_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF2, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Hq, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Andn_r64_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Blsr_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF3, 1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Hd, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Blsr_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Blsr_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF3, 1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Hq, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Blsr_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsmsk)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Blsmsk_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF3, 2, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Hd, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Blsmsk_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.CF, RflagsBits.OF | RflagsBits.ZF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsmsk)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Blsmsk_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF3, 2, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Hq, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Blsmsk_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.CF, RflagsBits.OF | RflagsBits.ZF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsi)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Blsi_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF3, 3, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Hd, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Blsi_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsi)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Blsi_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF3, 3, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Hq, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Blsi_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Bzhi)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Bzhi_r32_rm32_r32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF5, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Ed, VexOpKind.Hd }),
				new InstrInfo(code[nameof(Code.VEX_Bzhi_r32_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Bzhi)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Bzhi_r64_rm64_r64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF5, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Eq, VexOpKind.Hq }),
				new InstrInfo(code[nameof(Code.VEX_Bzhi_r64_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Wrussd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrussd_m32_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xF5, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Md, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Wrussd_m32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Wrussq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrussq_m64_r64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xF5, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Mq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Wrussq_m64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(3, mnemonic[nameof(Mnemonic.Pext)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Pext_r32_r32_rm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0xF5, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Hd, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Pext_r32_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pext)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Pext_r64_r64_rm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0xF5, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Hq, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Pext_r64_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pdep)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Pdep_r32_r32_rm32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF5, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Hd, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Pdep_r32_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pdep)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Pdep_r64_r64_rm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF5, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Hq, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Pdep_r64_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Wrssd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrssd_m32_r32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Md, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Wrssd_m32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Wrssq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Wrssq_m64_r64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF6, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Mq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Wrssq_m64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.CET_SS)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Adcx)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adcx_r32_rm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xF6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Adcx_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ADX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adcx)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adcx_r64_rm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xF6, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Adcx_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.CF, RflagsBits.None, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ADX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adox)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adox_r32_rm32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0xF6, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd, LegacyOpKind.Ed }),
				new InstrInfo(code[nameof(Code.Adox_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.OF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ADX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Adox)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Adox_r64_rm64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0xF6, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq, LegacyOpKind.Eq }),
				new InstrInfo(code[nameof(Code.Adox_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.OF, RflagsBits.None, RflagsBits.OF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ADX)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Mulx)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Mulx_r32_r32_rm32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF6, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Hd, VexOpKind.Ed }),
				new InstrInfo(code[nameof(Code.VEX_Mulx_r32_r32_rm32)], CodeInfo.Mulx, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Mulx)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Mulx_r64_r64_rm64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF6, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Hq, VexOpKind.Eq }),
				new InstrInfo(code[nameof(Code.VEX_Mulx_r64_r64_rm64)], CodeInfo.Mulx, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Bextr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Bextr_r32_rm32_r32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF7, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Ed, VexOpKind.Hd }),
				new InstrInfo(code[nameof(Code.VEX_Bextr_r32_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Bextr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Bextr_r64_rm64_r64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF7, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Eq, VexOpKind.Hq }),
				new InstrInfo(code[nameof(Code.VEX_Bextr_r64_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Shlx)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Shlx_r32_rm32_r32)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xF7, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Ed, VexOpKind.Hd }),
				new InstrInfo(code[nameof(Code.VEX_Shlx_r32_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Shlx)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Shlx_r64_rm64_r64)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xF7, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Eq, VexOpKind.Hq }),
				new InstrInfo(code[nameof(Code.VEX_Shlx_r64_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Sarx)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Sarx_r32_rm32_r32)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0xF7, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Ed, VexOpKind.Hd }),
				new InstrInfo(code[nameof(Code.VEX_Sarx_r32_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Sarx)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Sarx_r64_rm64_r64)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0xF7, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Eq, VexOpKind.Hq }),
				new InstrInfo(code[nameof(Code.VEX_Sarx_r64_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Shrx)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Shrx_r32_rm32_r32)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF7, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Ed, VexOpKind.Hd }),
				new InstrInfo(code[nameof(Code.VEX_Shrx_r32_rm32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Shrx)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Shrx_r64_rm64_r64)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF7, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Eq, VexOpKind.Hq }),
				new InstrInfo(code[nameof(Code.VEX_Shrx_r64_rm64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Movdir64b)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdir64b_r16_m512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xF8, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gw_mem, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Movdir64b_r16_m512)], CodeInfo.Movdir64b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVDIR64B)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movdir64b)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdir64b_r32_m512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xF8, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd_mem, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Movdir64b_r32_m512)], CodeInfo.Movdir64b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVDIR64B)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movdir64b)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdir64b_r64_m512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0xF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq_mem, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Movdir64b_r64_m512)], CodeInfo.Movdir64b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVDIR64B)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Enqcmds)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enqcmds_r16_m512)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0xF8, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gw_mem, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Enqcmds_r16_m512)], CodeInfo.Movdir64b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ENQCMD)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Enqcmds)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enqcmds_r32_m512)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0xF8, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd_mem, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Enqcmds_r32_m512)], CodeInfo.Movdir64b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ENQCMD)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Enqcmds)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enqcmds_r64_m512)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0xF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq_mem, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Enqcmds_r64_m512)], CodeInfo.Movdir64b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ENQCMD)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.Privileged)),
				(2, mnemonic[nameof(Mnemonic.Enqcmd)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enqcmd_r16_m512)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF8, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, new[] { LegacyOpKind.Gw_mem, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Enqcmd_r16_m512)], CodeInfo.Movdir64b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ENQCMD)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Enqcmd)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enqcmd_r32_m512)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF8, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Gd_mem, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Enqcmd_r32_m512)], CodeInfo.Movdir64b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ENQCMD)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Enqcmd)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Enqcmd_r64_m512)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0xF8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Gq_mem, LegacyOpKind.M }),
				new InstrInfo(code[nameof(Code.Enqcmd_r64_m512)], CodeInfo.Movdir64b, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.ZF, RflagsBits.OF | RflagsBits.SF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.ENQCMD)] }, new[] { OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movdiri)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdiri_m32_r32)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Md, LegacyOpKind.Gd }),
				new InstrInfo(code[nameof(Code.Movdiri_m32_r32)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVDIRI)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Movdiri)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Movdiri_m64_r64)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0xF9, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Mq, LegacyOpKind.Gq }),
				new InstrInfo(code[nameof(Code.Movdiri_m64_r64)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.MOVDIRI)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpermq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermq_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x00, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpermq_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x00, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x00, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermpd_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x01, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpermpd_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x01, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x01, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpblendd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpblendd_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x02, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpblendd_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpblendd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpblendd_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x02, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpblendd_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Valignd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x03, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Valignd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x03, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Valignd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x03, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Valignq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x03, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Valignq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x03, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Valignq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x03, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermilps_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x04, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpermilps_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermilps_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x04, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpermilps_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x04, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x04, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x04, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermilpd_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x05, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpermilpd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermilpd_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x05, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpermilpd_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x05, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x05, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpermilpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x05, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vperm2f128)], memSize[nameof(MemorySize.Packed256_Float128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vperm2f128_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x06, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vperm2f128_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Roundps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Roundps_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x08, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Roundps_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vroundps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vroundps_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x08, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vroundps_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vroundps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vroundps_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x08, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vroundps_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrndscaleps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x08, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrndscaleps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x08, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrndscaleps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x08, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Roundpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Roundpd_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x09, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Roundpd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vroundpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vroundpd_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x09, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vroundpd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vroundpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vroundpd_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x09, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vroundpd_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrndscalepd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x09, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrndscalepd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x09, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vrndscalepd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x09, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Roundss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Roundss_xmm_xmmm32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0A, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Roundss_xmm_xmmm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vroundss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vroundss_xmm_xmm_xmmm32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0A, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vroundss_xmm_xmm_xmmm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrndscaless)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0A, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Roundsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Roundsd_xmm_xmmm64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0B, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Roundsd_xmm_xmmm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vroundsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vroundsd_xmm_xmm_xmmm64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0B, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vroundsd_xmm_xmm_xmmm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrndscalesd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0B, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Blendps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Blendps_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0C, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Blendps_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vblendps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vblendps_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vblendps_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vblendps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vblendps_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vblendps_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Blendpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Blendpd_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0D, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Blendpd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vblendpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vblendpd_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vblendpd_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vblendpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vblendpd_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vblendpd_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pblendw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pblendw_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0E, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pblendw_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vpblendw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpblendw_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpblendw_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpblendw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpblendw_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpblendw_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Palignr)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Palignr_mm_mmm64_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F3A, 0x0F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.P, LegacyOpKind.Q, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Palignr_mm_mmm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Palignr)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Palignr_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0F, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Palignr_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSSE3)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vpalignr)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpalignr_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpalignr_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpalignr)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpalignr_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpalignr_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpalignr)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpalignr)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpalignr)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x0F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pextrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrb_r32m8_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x14, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RdMb, LegacyOpKind.VX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrb_r32m8_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pextrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrb_r64m8_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x14, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RqMb, LegacyOpKind.VX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrb_r64m8_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpextrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpextrb_r32m8_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x14, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.RdMb, VexOpKind.VX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpextrb_r32m8_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpextrb_r64m8_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x14, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.RqMb, VexOpKind.VX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpextrb_r64m8_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpextrb_r32m8_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x14, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.RdMb, EvexOpKind.VX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpextrb_r32m8_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpextrb_r64m8_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x14, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.RqMb, EvexOpKind.VX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpextrb_r64m8_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pextrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrw_r32m16_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x15, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.RdMw, LegacyOpKind.VX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrw_r32m16_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pextrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrw_r64m16_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x15, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.RqMw, LegacyOpKind.VX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrw_r64m16_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpextrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpextrw_r32m16_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x15, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.RdMw, VexOpKind.VX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpextrw_r32m16_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpextrw_r64m16_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x15, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.RqMw, VexOpKind.VX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpextrw_r64m16_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpextrw_r32m16_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x15, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.RdMw, EvexOpKind.VX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpextrw_r32m16_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrw)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpextrw_r64m16_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x15, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_2, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.RqMw, EvexOpKind.VX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpextrw_r64m16_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pextrd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrd_rm32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x16, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.VX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrd_rm32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pextrq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pextrq_rm64_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x16, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.VX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pextrq_rm64_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpextrd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpextrd_rm32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x16, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Ed, VexOpKind.VX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpextrd_rm32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpextrq_rm64_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x16, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Eq, VexOpKind.VX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpextrq_rm64_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpextrd_rm32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x16, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.Ed, EvexOpKind.VX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpextrd_rm32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpextrq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpextrq_rm64_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x16, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_8, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.Eq, EvexOpKind.VX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpextrq_rm64_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Extractps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Extractps_rm32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x17, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Ed, LegacyOpKind.VX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Extractps_rm32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Extractps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Extractps_r64m32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x17, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.Eq, LegacyOpKind.VX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Extractps_r64m32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vextractps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vextractps_rm32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x17, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Ed, VexOpKind.VX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vextractps_rm32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vextractps_r64m32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x17, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Eq, VexOpKind.VX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vextractps_r64m32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextractps_rm32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x17, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.Ed, EvexOpKind.VX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextractps_rm32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextractps_r64m32_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x17, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_4, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.Eq, EvexOpKind.VX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextractps_r64m32_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinsertf128)], memSize[nameof(MemorySize.Float128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vinsertf128_ymm_ymm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x18, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vinsertf128_ymm_ymm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinsertf32x4)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x18, -1, EvexVectorLength.L256, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinsertf32x4)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x18, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinsertf64x2)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x18, -1, EvexVectorLength.L256, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinsertf64x2)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x18, -1, EvexVectorLength.L512, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractf128)], memSize[nameof(MemorySize.Float128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vextractf128_xmmm128_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x19, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.WX, VexOpKind.VY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vextractf128_xmmm128_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractf32x4)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x19, -1, EvexVectorLength.L256, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractf32x4)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x19, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractf64x2)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x19, -1, EvexVectorLength.L256, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractf64x2)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x19, -1, EvexVectorLength.L512, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinsertf32x8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1A, -1, EvexVectorLength.L512, TupleType.Tuple8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinsertf64x4)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1A, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractf32x8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1B, -1, EvexVectorLength.L512, TupleType.Tuple8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextractf64x4)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1B, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtps2ph)], memSize[nameof(MemorySize.Packed64_Float16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtps2ph_xmmm64_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.WX, VexOpKind.VX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtps2ph_xmmm64_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.F16C)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtps2ph)], memSize[nameof(MemorySize.Packed128_Float16)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vcvtps2ph_xmmm128_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.WX, VexOpKind.VY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vcvtps2ph_xmmm128_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.F16C)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtps2ph)], memSize[nameof(MemorySize.Packed64_Float16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1D, -1, EvexVectorLength.L128, TupleType.Half_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtps2ph)], memSize[nameof(MemorySize.Packed128_Float16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1D, -1, EvexVectorLength.L256, TupleType.Half_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vcvtps2ph)], memSize[nameof(MemorySize.Packed256_Float16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1D, -1, EvexVectorLength.L512, TupleType.Half_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpud)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpud_kr_k1_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpud)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpud_kr_k1_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpud)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpud_kr_k1_zmm_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpuq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1E, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpuq_kr_k1_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpuq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1E, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpuq_kr_k1_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpuq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1E, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpuq_kr_k1_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1F, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpd_kr_k1_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpd)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1F, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpd_kr_k1_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpd)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1F, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpd_kr_k1_zmm_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1F, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpq_kr_k1_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpq)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1F, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpq_kr_k1_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpq)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x1F, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpq_kr_k1_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pinsrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pinsrb_xmm_r32m8_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x20, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.RdMb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pinsrb_xmm_r32m8_imm8)], CodeInfo.Read_Reg8_OpM1_imm, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pinsrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pinsrb_xmm_r64m8_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x20, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.RqMb, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pinsrb_xmm_r64m8_imm8)], CodeInfo.Read_Reg8_OpM1_imm, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r32m8_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x20, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.RdMb, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r32m8_imm8)], CodeInfo.Read_Reg8_OpM1_imm, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r64m8_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x20, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.RqMb, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r64m8_imm8)], CodeInfo.Read_Reg8_OpM1_imm, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r32m8_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x20, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.RdMb, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r32m8_imm8)], CodeInfo.Read_Reg8_OpM1_imm, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrb)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r64m8_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x20, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_1, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.RqMb, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r64m8_imm8)], CodeInfo.Read_Reg8_OpM1_imm, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Insertps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Insertps_xmm_xmmm32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x21, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Insertps_xmm_xmmm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vinsertps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vinsertps_xmm_xmm_xmmm32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x21, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vinsertps_xmm_xmm_xmmm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinsertps)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinsertps_xmm_xmm_xmmm32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x21, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinsertps_xmm_xmm_xmmm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pinsrd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pinsrd_xmm_rm32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x22, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Ed, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pinsrd_xmm_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pinsrq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pinsrq_xmm_rm64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x22, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.Eq, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pinsrq_xmm_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpinsrd_xmm_xmm_rm32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x22, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Ed, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpinsrd_xmm_xmm_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpinsrq_xmm_xmm_rm64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x22, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Eq, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpinsrq_xmm_xmm_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrd)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpinsrd_xmm_xmm_rm32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x22, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Ed, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpinsrd_xmm_xmm_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpinsrq)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpinsrq_xmm_xmm_rm64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x22, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar_8, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.Eq, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpinsrq_xmm_xmm_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshuff32x4)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x23, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshuff32x4)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x23, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshuff64x2)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x23, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshuff64x2)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x23, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpternlogd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x25, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpternlogd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x25, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpternlogd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x25, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpternlogq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x25, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpternlogq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x25, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpternlogq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x25, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgetmantps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x26, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgetmantps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x26, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgetmantps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x26, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgetmantpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x26, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgetmantpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x26, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vgetmantpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x26, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgetmantss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x27, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgetmantsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x27, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kshiftrb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kshiftrb_kr_kr_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x30, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Kshiftrb_kr_kr_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kshiftrw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kshiftrw_kr_kr_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x30, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Kshiftrw_kr_kr_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kshiftrd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kshiftrd_kr_kr_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x31, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Kshiftrd_kr_kr_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kshiftrq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kshiftrq_kr_kr_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x31, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Kshiftrq_kr_kr_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kshiftlb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kshiftlb_kr_kr_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x32, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Kshiftlb_kr_kr_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kshiftlw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kshiftlw_kr_kr_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x32, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Kshiftlw_kr_kr_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kshiftld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kshiftld_kr_kr_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x33, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VK, VexOpKind.RK, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Kshiftld_kr_kr_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Kshiftlq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Kshiftlq_kr_kr_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x33, -1, VexVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VK, VexOpKind.RK, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Kshiftlq_kr_kr_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinserti128)], memSize[nameof(MemorySize.Int128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vinserti128_ymm_ymm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x38, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vinserti128_ymm_ymm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinserti32x4)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x38, -1, EvexVectorLength.L256, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinserti32x4)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x38, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinserti64x2)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x38, -1, EvexVectorLength.L256, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinserti64x2)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x38, -1, EvexVectorLength.L512, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextracti128)], memSize[nameof(MemorySize.Int128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vextracti128_xmmm128_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x39, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.WX, VexOpKind.VY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vextracti128_xmmm128_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextracti32x4)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x39, -1, EvexVectorLength.L256, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextracti32x4)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x39, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextracti64x2)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x39, -1, EvexVectorLength.L256, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextracti64x2)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x39, -1, EvexVectorLength.L512, TupleType.Tuple2, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WX, EvexOpKind.VZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinserti32x8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3A, -1, EvexVectorLength.L512, TupleType.Tuple8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vinserti64x4)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3A, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextracti32x8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3B, -1, EvexVectorLength.L512, TupleType.Tuple8, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vextracti64x4)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3B, -1, EvexVectorLength.L512, TupleType.Tuple4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.WY, EvexOpKind.VZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpub)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3E, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpub_kr_k1_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpub)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3E, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpub_kr_k1_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpub)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3E, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpub_kr_k1_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3E, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpuw_kr_k1_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpuw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3E, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpuw_kr_k1_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpuw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3E, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpuw_kr_k1_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpb_kr_k1_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpb)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpb_kr_k1_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpb)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpb_kr_k1_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3F, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpw_kr_k1_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpw)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3F, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpw_kr_k1_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmpw)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x3F, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpcmpw_kr_k1_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Dpps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Dpps_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x40, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Dpps_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vdpps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vdpps_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x40, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vdpps_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vdpps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vdpps_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x40, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vdpps_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Dppd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Dppd_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x41, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Dppd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vdppd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vdppd_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x41, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vdppd_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Mpsadbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Mpsadbw_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x42, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Mpsadbw_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_1)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vmpsadbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x42, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vmpsadbw)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x42, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vdbpsadbw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x42, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vdbpsadbw)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x42, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vdbpsadbw)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x42, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512BW)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufi32x4)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x43, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufi32x4)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x43, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufi64x2)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x43, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vshufi64x2)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x43, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pclmulqdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pclmulqdq_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x44, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pclmulqdq_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PCLMULQDQ)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vpclmulqdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x44, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PCLMULQDQ)], cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpclmulqdq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x44, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.VPCLMULQDQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpclmulqdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x44, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VPCLMULQDQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpclmulqdq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x44, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.VPCLMULQDQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpclmulqdq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x44, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.VPCLMULQDQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vperm2i128)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vperm2i128_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x46, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vperm2i128_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(5, mnemonic[nameof(Mnemonic.Vpermil2ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm2)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x48, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is5X, VexOpKind.I2 }),
				new InstrInfo(code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm2)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(5, mnemonic[nameof(Mnemonic.Vpermil2ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm2)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x48, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is5Y, VexOpKind.I2 }),
				new InstrInfo(code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm2)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(5, mnemonic[nameof(Mnemonic.Vpermil2ps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm2)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x48, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is5X, VexOpKind.WX, VexOpKind.I2 }),
				new InstrInfo(code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm2)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(5, mnemonic[nameof(Mnemonic.Vpermil2ps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm2)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x48, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is5Y, VexOpKind.WY, VexOpKind.I2 }),
				new InstrInfo(code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm2)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(5, mnemonic[nameof(Mnemonic.Vpermil2pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm2)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x49, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is5X, VexOpKind.I2 }),
				new InstrInfo(code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm2)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(5, mnemonic[nameof(Mnemonic.Vpermil2pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm2)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x49, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is5Y, VexOpKind.I2 }),
				new InstrInfo(code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm2)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(5, mnemonic[nameof(Mnemonic.Vpermil2pd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm2)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x49, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is5X, VexOpKind.WX, VexOpKind.I2 }),
				new InstrInfo(code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm2)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(5, mnemonic[nameof(Mnemonic.Vpermil2pd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm2)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x49, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is5Y, VexOpKind.WY, VexOpKind.I2 }),
				new InstrInfo(code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm2)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vblendvps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vblendvps_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x4A, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vblendvps_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vblendvps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vblendvps_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x4A, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vblendvps_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vblendvpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vblendvpd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x4B, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vblendvpd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vblendvpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vblendvpd_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x4B, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vblendvpd_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpblendvb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpblendvb_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x4C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vpblendvb_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpblendvb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpblendvb_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x4C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vpblendvb_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrangeps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x50, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrangeps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x50, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrangeps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x50, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrangepd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x50, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrangepd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x50, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrangepd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x50, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrangess)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x51, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vrangesd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x51, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfixupimmps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x54, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfixupimmps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x54, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfixupimmps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x54, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfixupimmpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x54, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfixupimmpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x54, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfixupimmpd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x54, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfixupimmss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x55, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfixupimmsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x55, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vreduceps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x56, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vreduceps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x56, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vreduceps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x56, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vreducepd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x56, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vreducepd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x56, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vreducepd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x56, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vreducess)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x57, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vreducesd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x57, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.SuppressAllExceptions | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubaddps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5E, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubaddps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5E, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubaddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubaddpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubaddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5F, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubaddpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x5F, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pcmpestrm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpestrm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x60, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pcmpestrm_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pcmpestrm64)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpestrm64_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x60, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pcmpestrm64_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpestrm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpestrm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x60, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpestrm_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpestrm64)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpestrm64_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x60, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpestrm64_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pcmpestri)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpestri_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x61, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pcmpestri_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Pcmpestri64)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpestri64_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x61, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pcmpestri64_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpestri)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpestri_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x61, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpestri_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpestri64)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpestri64_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x61, -1, VexVectorLength.L128, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpestri64_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pcmpistrm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpistrm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x62, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pcmpistrm_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpistrm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpistrm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x62, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpistrm_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Pcmpistri)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pcmpistri_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x63, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Pcmpistri_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SSE4_2)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vpcmpistri)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vpcmpistri_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x63, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vpcmpistri_xmm_xmmm128_imm8)], CodeInfo.PcmpXstrY, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.AF | RflagsBits.PF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfpclassps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x66, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfpclassps_kr_k1_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfpclassps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x66, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfpclassps_kr_k1_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfpclassps)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x66, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfpclassps_kr_k1_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfpclasspd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x66, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfpclasspd_kr_k1_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfpclasspd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x66, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfpclasspd_kr_k1_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfpclasspd)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x66, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfpclasspd_kr_k1_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfpclassss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfpclassss_kr_k1_xmmm32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x67, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfpclassss_kr_k1_xmmm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vfpclasssd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vfpclasssd_kr_k1_xmmm64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x67, -1, EvexVectorLength.L128, TupleType.Tuple1_Scalar, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W | OpCodeFlags.OpMaskRegister, new[] { EvexOpKind.VK, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vfpclasssd_kr_k1_xmmm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512DQ)] }, new[] { OpInfo.WriteForce, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x68, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x68, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x68, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x68, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x69, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x69, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x69, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x69, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmmm32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6A, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmmm32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6A, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6B, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmaddsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6B, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmmm32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6E, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmmm32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6E, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfmsubsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x6F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshldw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x70, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshldw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x70, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshldw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x70, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshldd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x71, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshldd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x71, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshldd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x71, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshldq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x71, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshldq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x71, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshldq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x71, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshrdw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x72, -1, EvexVectorLength.L128, TupleType.Full_Mem_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshrdw)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x72, -1, EvexVectorLength.L256, TupleType.Full_Mem_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshrdw)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x72, -1, EvexVectorLength.L512, TupleType.Full_Mem_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshrdd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x73, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshrdd)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x73, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshrdd)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x73, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshrdq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x73, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshrdq)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x73, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpshrdq)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x73, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512_VBMI2)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x78, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x78, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x78, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x78, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x79, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x79, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x79, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x79, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7A, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7A, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7B, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmaddsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7B, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7C, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7C, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7D, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7D, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.Is4Y, VexOpKind.WY }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7E, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7E, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vfnmsubsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0x7F, -1, VexVectorLength.LIG, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.LIG | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.Is4X, VexOpKind.WX }),
				new InstrInfo(code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.FMA4)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Sha1rnds4)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Sha1rnds4_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.T0F3A, 0xCC, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Sha1rnds4_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SHA)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Gf2p8affineqb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Gf2p8affineqb_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Gf2p8affineqb_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineqb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCE, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineqb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCE, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineqb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCE, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineqb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCE, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineqb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCE, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Gf2p8affineinvqb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Gf2p8affineinvqb_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Gf2p8affineinvqb_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCF, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VX, VexOpKind.HX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCF, -1, VexVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.VY, VexOpKind.HY, VexOpKind.WY, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Broadcast128_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCF, -1, EvexVectorLength.L128, TupleType.Full_128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VX, EvexOpKind.HX, EvexOpKind.WX, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Broadcast256_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCF, -1, EvexVectorLength.L256, TupleType.Full_256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VY, EvexOpKind.HY, EvexOpKind.WY, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512VL)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vgf2p8affineinvqb)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Broadcast512_UInt64)],
				new EvexOpCodeInfo(code[nameof(Code.EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xCF, -1, EvexVectorLength.L512, TupleType.Full_512, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W | OpCodeFlags.Broadcast | OpCodeFlags.OpMaskRegister | OpCodeFlags.ZeroingMasking, new[] { EvexOpKind.VZ, EvexOpKind.HZ, EvexOpKind.WZ, EvexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.EVEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AVX512F)], cpuid[nameof(CpuidFeature.GFNI)] }, new[] { OpInfo.WriteVmm, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Aeskeygenassist)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Aeskeygenassist_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xDF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.VX, LegacyOpKind.WX, LegacyOpKind.Ib }),
				new InstrInfo(code[nameof(Code.Aeskeygenassist_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.None)),
				(3, mnemonic[nameof(Mnemonic.Vaeskeygenassist)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Vaeskeygenassist_xmm_xmmm128_imm8)], MandatoryPrefix.P66, OpCodeTableKind.T0F3A, 0xDF, -1, VexVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG, new[] { VexOpKind.VX, VexOpKind.WX, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Vaeskeygenassist_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AES)], cpuid[nameof(CpuidFeature.AVX)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Rorx)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Rorx_r32_rm32_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F3A, 0xF0, -1, VexVectorLength.LZ, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { VexOpKind.Gd, VexOpKind.Ed, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Rorx_r32_rm32_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Rorx)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Rorx_r64_rm64_imm8)], MandatoryPrefix.PF2, OpCodeTableKind.T0F3A, 0xF0, -1, VexVectorLength.LZ, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { VexOpKind.Gq, VexOpKind.Eq, VexOpKind.Ib }),
				new InstrInfo(code[nameof(Code.VEX_Rorx_r64_rm64_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.BMI2)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacssww)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacssww_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x85, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacssww_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacsswd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x86, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacssdql)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x87, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacssdd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x8E, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacssdqh)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x8F, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacsww)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacsww_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x95, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacsww_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacswd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacswd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x96, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacswd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacsdql)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x97, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacsdd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x9E, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmacsdqh)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0x9F, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmov)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xA2, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmov)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymmm256_ymm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xA2, -1, XopVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VY, XopOpKind.HY, XopOpKind.WY, XopOpKind.Is4Y }),
				new InstrInfo(code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymmm256_ymm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmov)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xA2, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.Is4X, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcmov)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xA2, -1, XopVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VY, XopOpKind.HY, XopOpKind.Is4Y, XopOpKind.WY }),
				new InstrInfo(code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpperm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpperm_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xA3, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpperm_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpperm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpperm_xmm_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xA3, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.Is4X, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpperm_xmm_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmadcsswd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xA6, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpmadcswd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xB6, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Is4X }),
				new InstrInfo(code[nameof(Code.XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotb_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xC0, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vprotb_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotw_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xC1, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vprotw_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotd_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xC2, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vprotd_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotq_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xC3, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vprotq_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcomb)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xCC, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcomw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xCD, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcomd)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xCE, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcomq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xCF, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcomub)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xEC, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcomuw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xED, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcomud)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xEE, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(4, mnemonic[nameof(Mnemonic.Vpcomuq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8)], MandatoryPrefix.PNP, OpCodeTableKind.XOP8, 0xEF, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX, XopOpKind.Ib }),
				new InstrInfo(code[nameof(Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blcfill)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blcfill_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 1, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed }),
				new InstrInfo(code[nameof(Code.XOP_Blcfill_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blcfill)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blcfill_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 1, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Eq }),
				new InstrInfo(code[nameof(Code.XOP_Blcfill_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsfill)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blsfill_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 2, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed }),
				new InstrInfo(code[nameof(Code.XOP_Blsfill_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsfill)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blsfill_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 2, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Eq }),
				new InstrInfo(code[nameof(Code.XOP_Blsfill_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blcs)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blcs_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 3, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed }),
				new InstrInfo(code[nameof(Code.XOP_Blcs_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blcs)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blcs_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 3, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Eq }),
				new InstrInfo(code[nameof(Code.XOP_Blcs_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Tzmsk)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Tzmsk_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 4, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed }),
				new InstrInfo(code[nameof(Code.XOP_Tzmsk_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Tzmsk)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Tzmsk_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 4, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Eq }),
				new InstrInfo(code[nameof(Code.XOP_Tzmsk_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blcic)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blcic_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 5, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed }),
				new InstrInfo(code[nameof(Code.XOP_Blcic_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blcic)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blcic_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 5, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Eq }),
				new InstrInfo(code[nameof(Code.XOP_Blcic_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsic)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blsic_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 6, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed }),
				new InstrInfo(code[nameof(Code.XOP_Blsic_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blsic)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blsic_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 6, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Eq }),
				new InstrInfo(code[nameof(Code.XOP_Blsic_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.T1mskc)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_T1mskc_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 7, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed }),
				new InstrInfo(code[nameof(Code.XOP_T1mskc_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.T1mskc)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_T1mskc_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x01, 7, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Eq }),
				new InstrInfo(code[nameof(Code.XOP_T1mskc_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blcmsk)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blcmsk_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x02, 1, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed }),
				new InstrInfo(code[nameof(Code.XOP_Blcmsk_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blcmsk)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blcmsk_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x02, 1, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Eq }),
				new InstrInfo(code[nameof(Code.XOP_Blcmsk_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blci)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blci_r32_rm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x02, 6, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed }),
				new InstrInfo(code[nameof(Code.XOP_Blci_r32_rm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Blci)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Blci_r64_rm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x02, 6, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Eq }),
				new InstrInfo(code[nameof(Code.XOP_Blci_r64_rm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.AF | RflagsBits.PF, RflagsBits.SF | RflagsBits.ZF | RflagsBits.CF, RflagsBits.OF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Llwpcb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Llwpcb_r32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x12, 0, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Rd }),
				new InstrInfo(code[nameof(Code.XOP_Llwpcb_r32)], CodeInfo.Llwpcb, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LWP)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Llwpcb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Llwpcb_r64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x12, 0, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Rq }),
				new InstrInfo(code[nameof(Code.XOP_Llwpcb_r64)], CodeInfo.Llwpcb, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LWP)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Slwpcb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Slwpcb_r32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x12, 1, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Rd }),
				new InstrInfo(code[nameof(Code.XOP_Slwpcb_r32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LWP)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Slwpcb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Slwpcb_r64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x12, 1, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Rq }),
				new InstrInfo(code[nameof(Code.XOP_Slwpcb_r64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LWP)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vfrczps)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vfrczps_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x80, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vfrczps_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vfrczps)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vfrczps_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x80, -1, XopVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VY, XopOpKind.WY }),
				new InstrInfo(code[nameof(Code.XOP_Vfrczps_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vfrczpd)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vfrczpd_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x81, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vfrczpd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vfrczpd)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vfrczpd_ymm_ymmm256)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x81, -1, XopVectorLength.L256, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VY, XopOpKind.WY }),
				new InstrInfo(code[nameof(Code.XOP_Vfrczpd_ymm_ymmm256)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vfrczss)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vfrczss_xmm_xmmm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x82, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vfrczss_xmm_xmmm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vfrczsd)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vfrczsd_xmm_xmmm64)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x83, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vfrczsd_xmm_xmmm64)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotb_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x90, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vprotb_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotb_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x90, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vprotb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotw_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x91, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vprotw_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotw_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x91, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vprotw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotd_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x92, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vprotd_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotd)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotd_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x92, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vprotd_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotq_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x93, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vprotq_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vprotq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vprotq_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x93, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vprotq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshlb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshlb_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x94, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshlb_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshlb)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshlb_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x94, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshlb_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshlw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshlw_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x95, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshlw_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshlw)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshlw_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x95, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshlw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshld_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x96, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshld_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshld)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshld_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x96, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshld_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshlq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshlq_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x97, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshlq_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshlq)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshlq_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x97, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshlq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshab)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshab_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x98, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshab_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshab)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshab_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x98, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshab_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshaw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshaw_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x99, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshaw_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshaw)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshaw_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x99, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshaw_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshad)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshad_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x9A, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshad_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshad)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshad_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x9A, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshad_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshaq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshaq_xmm_xmmm128_xmm)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x9B, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX, XopOpKind.HX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshaq_xmm_xmmm128_xmm)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Vpshaq)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vpshaq_xmm_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0x9B, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.VX, XopOpKind.HX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vpshaq_xmm_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphaddbw)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphaddbw_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xC1, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphaddbw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphaddbd)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphaddbd_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xC2, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphaddbd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphaddbq)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphaddbq_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xC3, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphaddbq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphaddwd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphaddwd_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xC6, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphaddwd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphaddwq)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphaddwq_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xC7, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphaddwq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphadddq)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphadddq_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xCB, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphadddq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphaddubw)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphaddubw_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xD1, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphaddubw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphaddubd)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphaddubd_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xD2, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphaddubd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphaddubq)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphaddubq_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xD3, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphaddubq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphadduwd)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphadduwd_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xD6, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphadduwd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphadduwq)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphadduwq_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xD7, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphadduwq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphaddudq)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphaddudq_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xDB, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphaddudq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphsubbw)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphsubbw_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xE1, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphsubbw_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphsubwd)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphsubwd_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xE2, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphsubwd_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Vphsubdq)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Vphsubdq_xmm_xmmm128)], MandatoryPrefix.PNP, OpCodeTableKind.XOP9, 0xE3, -1, XopVectorLength.L128, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { XopOpKind.VX, XopOpKind.WX }),
				new InstrInfo(code[nameof(Code.XOP_Vphsubdq_xmm_xmmm128)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.XOP)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Bextr)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Bextr_r32_rm32_imm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOPA, 0x10, -1, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Gd, XopOpKind.Ed, XopOpKind.Id }),
				new InstrInfo(code[nameof(Code.XOP_Bextr_r32_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Bextr)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Bextr_r64_rm64_imm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOPA, 0x10, -1, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Gq, XopOpKind.Eq, XopOpKind.Id }),
				new InstrInfo(code[nameof(Code.XOP_Bextr_r64_rm64_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.SF | RflagsBits.AF | RflagsBits.PF, RflagsBits.ZF, RflagsBits.OF | RflagsBits.CF, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TBM)] }, new[] { OpInfo.Write, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Lwpins)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Lwpins_r32_rm32_imm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOPA, 0x12, 0, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed, XopOpKind.Id }),
				new InstrInfo(code[nameof(Code.XOP_Lwpins_r32_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LWP)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Lwpins)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Lwpins_r64_rm32_imm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOPA, 0x12, 0, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Ed, XopOpKind.Id }),
				new InstrInfo(code[nameof(Code.XOP_Lwpins_r64_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LWP)] }, new[] { OpInfo.Read, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Lwpval)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Lwpval_r32_rm32_imm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOPA, 0x12, 1, XopVectorLength.L0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64 | OpCodeFlags.WIG32, new[] { XopOpKind.Hd, XopOpKind.Ed, XopOpKind.Id }),
				new InstrInfo(code[nameof(Code.XOP_Lwpval_r32_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LWP)] }, new[] { OpInfo.Read, OpInfo.CondRead, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Lwpval)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)],
				new XopOpCodeInfo(code[nameof(Code.XOP_Lwpval_r64_rm32_imm32)], MandatoryPrefix.PNP, OpCodeTableKind.XOPA, 0x12, 1, XopVectorLength.L0, OpCodeFlags.Mode64 | OpCodeFlags.W, new[] { XopOpKind.Hq, XopOpKind.Ed, XopOpKind.Id }),
				new InstrInfo(code[nameof(Code.XOP_Lwpval_r64_rm32_imm32)], CodeInfo.None, encoding[nameof(EncodingKind.XOP)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.LWP)] }, new[] { OpInfo.Read, OpInfo.CondRead, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Pi2fw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pi2fw_mm_mmm64)], 0x0C, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pi2fw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOWEXT)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pi2fd)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pi2fd_mm_mmm64)], 0x0D, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pi2fd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pf2iw)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pf2iw_mm_mmm64)], 0x1C, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pf2iw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOWEXT)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pf2id)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pf2id_mm_mmm64)], 0x1D, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pf2id_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfrcpv)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfrcpv_mm_mmm64)], 0x86, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfrcpv_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.GEODE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfrsqrtv)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfrsqrtv_mm_mmm64)], 0x87, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfrsqrtv_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.GEODE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfnacc)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfnacc_mm_mmm64)], 0x8A, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfnacc_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOWEXT)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfpnacc)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfpnacc_mm_mmm64)], 0x8E, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfpnacc_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOWEXT)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfcmpge)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfcmpge_mm_mmm64)], 0x90, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfcmpge_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfmin)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfmin_mm_mmm64)], 0x94, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfmin_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfrcp)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfrcp_mm_mmm64)], 0x96, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfrcp_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfrsqrt)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfrsqrt_mm_mmm64)], 0x97, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfrsqrt_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfsub)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfsub_mm_mmm64)], 0x9A, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfsub_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfadd)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfadd_mm_mmm64)], 0x9E, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfadd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfcmpgt)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfcmpgt_mm_mmm64)], 0xA0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfcmpgt_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfmax)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfmax_mm_mmm64)], 0xA4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfmax_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfrcpit1)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfrcpit1_mm_mmm64)], 0xA6, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfrcpit1_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfrsqit1)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfrsqit1_mm_mmm64)], 0xA7, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfrsqit1_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfsubr)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfsubr_mm_mmm64)], 0xAA, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfsubr_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfacc)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfacc_mm_mmm64)], 0xAE, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfacc_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfcmpeq)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfcmpeq_mm_mmm64)], 0xB0, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfcmpeq_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfmul)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfmul_mm_mmm64)], 0xB4, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfmul_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pfrcpit2)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pfrcpit2_mm_mmm64)], 0xB6, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pfrcpit2_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pmulhrw)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pmulhrw_mm_mmm64)], 0xB7, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pmulhrw_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pswapd)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pswapd_mm_mmm64)], 0xBB, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pswapd_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOWEXT)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.None)),
				(2, mnemonic[nameof(Mnemonic.Pavgusb)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)],
				new D3nowOpCodeInfo(code[nameof(Code.D3NOW_Pavgusb_mm_mmm64)], 0xBF, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64),
				new InstrInfo(code[nameof(Code.D3NOW_Pavgusb_mm_mmm64)], CodeInfo.None, encoding[nameof(EncodingKind.D3NOW)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.D3NOW)] }, new[] { OpInfo.ReadWrite, OpInfo.Read }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Rmpadjust)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rmpadjust)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x01FE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Rmpadjust)], CodeInfo.Rmpadjust, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SNP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Rmpupdate)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Rmpupdate)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x01FE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Rmpupdate)], CodeInfo.Rmpupdate, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SNP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Psmash)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Psmash)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x01FF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Psmash)], CodeInfo.Psmash, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SNP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Pvalidate)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pvalidatew)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x01FF, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pvalidatew)], CodeInfo.Pvalidate, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SNP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Pvalidate)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pvalidated)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x01FF, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pvalidated)], CodeInfo.Pvalidate, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SNP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Pvalidate)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Pvalidateq)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x01FF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Pvalidateq)], CodeInfo.Pvalidate, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.PF | RflagsBits.CF, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SNP)] }, Array.Empty<OpInfo>(), InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Serialize)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Serialize)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01E8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Serialize)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SERIALIZE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xsusldtrk)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xsusldtrk)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x01E8, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xsusldtrk)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.XbeginXabortXend)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TSXLDTRK)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Xresldtrk)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Xresldtrk)], MandatoryPrefix.PF2, OpCodeTableKind.T0F, 0x01E9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Xresldtrk)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.XbeginXabortXend)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.TSXLDTRK)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Invlpgb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invlpgbw)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01FE, -1, OperandSize.None, AddressSize.Size16, OpCodeFlags.Mode16 | OpCodeFlags.Mode32, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Invlpgbw)], CodeInfo.Invlpgb, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVLPGB)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Invlpgb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invlpgbd)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01FE, -1, OperandSize.None, AddressSize.Size32, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Invlpgbd)], CodeInfo.Invlpgb, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVLPGB)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Invlpgb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Invlpgbq)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01FE, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Invlpgbq)], CodeInfo.Invlpgb, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVLPGB)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(0, mnemonic[nameof(Mnemonic.Tlbsync)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Tlbsync)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x01FF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Tlbsync)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INVLPGB)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode | InstrInfoFlags.Privileged)),
				(1, mnemonic[nameof(Mnemonic.Prefetchw)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.PrefetchReserved3_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, 3, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.PrefetchReserved3_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PREFETCHW)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetch)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.PrefetchReserved4_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, 4, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.PrefetchReserved4_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PREFETCHW)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetch)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.PrefetchReserved5_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, 5, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.PrefetchReserved5_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PREFETCHW)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetch)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.PrefetchReserved6_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, 6, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.PrefetchReserved6_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PREFETCHW)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(1, mnemonic[nameof(Mnemonic.Prefetch)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.PrefetchReserved7_m8)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0x0D, 7, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, new[] { LegacyOpKind.Mb }),
				new InstrInfo(code[nameof(Code.PrefetchReserved7_m8)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.PREFETCHW)] }, new[] { OpInfo.NoMemAccess }, InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Ud0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Ud0)], MandatoryPrefix.None, OpCodeTableKind.T0F, 0xFF, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Ud0)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Exception)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.INTEL286)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Vmgexit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Vmgexit)], MandatoryPrefix.PF3, OpCodeTableKind.T0F, 0x01D9, -1, OperandSize.None, AddressSize.None, OpCodeFlags.Mode16 | OpCodeFlags.Mode32 | OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Vmgexit)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Call)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SVM)] }, Array.Empty<OpInfo>(), InstrInfoFlags.None)),
				(0, mnemonic[nameof(Mnemonic.Getsecq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new LegacyOpCodeInfo(code[nameof(Code.Getsecq)], MandatoryPrefix.PNP, OpCodeTableKind.T0F, 0x37, -1, OperandSize.Size64, AddressSize.None, OpCodeFlags.Mode64, Array.Empty<LegacyOpKind>()),
				new InstrInfo(code[nameof(Code.Getsecq)], CodeInfo.None, encoding[nameof(EncodingKind.Legacy)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.SMX)] }, Array.Empty<OpInfo>(), InstrInfoFlags.SaveRestore)),
				(1, mnemonic[nameof(Mnemonic.Ldtilecfg)], memSize[nameof(MemorySize.Tilecfg)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Ldtilecfg_m512)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x49, 0, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Ldtilecfg_m512)], CodeInfo.Tilerelease, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_TILE)] }, new[] { OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(0, mnemonic[nameof(Mnemonic.Tilerelease)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tilerelease)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x49C0, -1, -1, VexVectorLength.L128, OpCodeFlags.Mode64, Array.Empty<VexOpKind>()),
				new InstrInfo(code[nameof(Code.VEX_Tilerelease)], CodeInfo.Tilerelease, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_TILE)] }, Array.Empty<OpInfo>(), InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Sttilecfg)], memSize[nameof(MemorySize.Tilecfg)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Sttilecfg_m512)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x49, 0, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.M }),
				new InstrInfo(code[nameof(Code.VEX_Sttilecfg_m512)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_TILE)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(1, mnemonic[nameof(Mnemonic.Tilezero)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tilezero_tmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x49, -1, 0, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.VT }),
				new InstrInfo(code[nameof(Code.VEX_Tilezero_tmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_TILE)] }, new[] { OpInfo.Write }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Tileloaddt1)], memSize[nameof(MemorySize.Tile)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tileloaddt1_tmm_sibmem)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x4B, -1, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.VT, VexOpKind.Sibmem }),
				new InstrInfo(code[nameof(Code.VEX_Tileloaddt1_tmm_sibmem)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_TILE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Tilestored)], memSize[nameof(MemorySize.Tile)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tilestored_sibmem_tmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x4B, -1, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.Sibmem, VexOpKind.VT }),
				new InstrInfo(code[nameof(Code.VEX_Tilestored_sibmem_tmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_TILE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(2, mnemonic[nameof(Mnemonic.Tileloadd)], memSize[nameof(MemorySize.Tile)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tileloadd_tmm_sibmem)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x4B, -1, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.VT, VexOpKind.Sibmem }),
				new InstrInfo(code[nameof(Code.VEX_Tileloadd_tmm_sibmem)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_TILE)] }, new[] { OpInfo.Write, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Tdpbf16ps)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tdpbf16ps_tmm_tmm_tmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x5C, -1, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.VT, VexOpKind.RT, VexOpKind.HT }),
				new InstrInfo(code[nameof(Code.VEX_Tdpbf16ps_tmm_tmm_tmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_BF16)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Tdpbuud)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tdpbuud_tmm_tmm_tmm)], MandatoryPrefix.PNP, OpCodeTableKind.T0F38, 0x5E, -1, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.VT, VexOpKind.RT, VexOpKind.HT }),
				new InstrInfo(code[nameof(Code.VEX_Tdpbuud_tmm_tmm_tmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_INT8)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Tdpbusd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tdpbusd_tmm_tmm_tmm)], MandatoryPrefix.P66, OpCodeTableKind.T0F38, 0x5E, -1, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.VT, VexOpKind.RT, VexOpKind.HT }),
				new InstrInfo(code[nameof(Code.VEX_Tdpbusd_tmm_tmm_tmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_INT8)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Tdpbsud)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tdpbsud_tmm_tmm_tmm)], MandatoryPrefix.PF3, OpCodeTableKind.T0F38, 0x5E, -1, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.VT, VexOpKind.RT, VexOpKind.HT }),
				new InstrInfo(code[nameof(Code.VEX_Tdpbsud_tmm_tmm_tmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_INT8)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
				(3, mnemonic[nameof(Mnemonic.Tdpbssd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)],
				new VexOpCodeInfo(code[nameof(Code.VEX_Tdpbssd_tmm_tmm_tmm)], MandatoryPrefix.PF2, OpCodeTableKind.T0F38, 0x5E, -1, -1, VexVectorLength.L128, OpCodeFlags.Mode64, new[] { VexOpKind.VT, VexOpKind.RT, VexOpKind.HT }),
				new InstrInfo(code[nameof(Code.VEX_Tdpbssd_tmm_tmm_tmm)], CodeInfo.None, encoding[nameof(EncodingKind.VEX)], flowControl[nameof(FlowControl.Next)], RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, RflagsBits.None, new[] { cpuid[nameof(CpuidFeature.AMX_INT8)] }, new[] { OpInfo.ReadWrite, OpInfo.Read, OpInfo.Read }, InstrInfoFlags.ProtectedMode)),
			};

			if (result.Length != code.Values.Length)
				throw new InvalidOperationException();
			if (result.Select(a => a.opCodeInfo.Code).ToHashSet<EnumValue>().Count != code.Values.Length)
				throw new InvalidOperationException();
			Array.Sort(result, (a, b) => a.opCodeInfo.Code.Value.CompareTo(b.opCodeInfo.Code.Value));
			return result.Select(a => new InstructionDef(a.opCount, a.mnemonic, a.mem, a.bcst, a.opCodeInfo, a.instrInfo)).ToArray();
		}
	}
}
